###
# Package : studbookR
# 
# Copyright (c) 2011 Frank PG Princee, All rights reserved
#

###
#' @title Variance effective size 
#' @param N number of individuals
#' @param k mean number of offspring per individual (non-breeders included)
#' @param k.var sample variance of offspring in individuals
#' @return variance effective population size for single sex
#' @author Frank PG Princee
#' @family Ne
#' @export
#'
Ne.variance <- function(N,k,k.var)
{
  return( (N * k - 1) / (k - 1 + k.var/k) )
}


###
#' @title Effective size for unequal sex-ratio
#' @param Nm Number of males
#' @param Nf Number of females
#' @return Ne
#' @author Frank PG Princee
#' @family Ne 
#' @export
#' 
Ne.sexratio <-  function(Nm,Nf)
{
  return( (4 * Nm * Nf) / (Nm + Nf) )
}



### 
#' @title Variance effective size and generation time
#' @note Lande and Barrowclough (1987)
#' @family Ne 
#' @export
#' 
Ne.overlap <- function(Nme,Tm,Nfe,Tf)
{
  warning("algorithm needs to be checked")
  T <- (Tm + Tf)/2
  return( 4 * ( T/(Nme*Tm) + T/(Nfe*Tf) )^(-1) )
}


###
#' @title Ne from genetic drift
#' @param dH Loss in gene diversity
#' @return Ne
#' @author Frank PG Princee
#' @family Ne
#' @export
#' 
Ne.drift <- function(dH)
{
  Ne <- 0.5 * (1/dH)
  return(Ne)
}

###
#' @title Effect of numbers and sex-ratio on Ne
#' @description \code{Ne.effect} estimates Ne for different sex-ratio and numbers
#' @param males maximum number of males (default: 2)
#' @param females maximum number of females (default:2) 
#' @author Frank PG Princee
#' @family Ne
#' @export
#
Ne.effect <- function(males=2,females=2)
{
  Ne <- rep(0,males)
  N  <- rep(0,males)
  
  for (m in 1:males) {
    Ne[m] <- Ne.sexratio(m,females) 
    N[m]  <- m+females
  }
  studbook.openOutput("Ne-sexratio")
  plot(1:males,Ne,type='l',xlab='Number of males',ylab="Ne")
  lines(1:males,N,lty='dashed')
  studbook.closeOutput()
  invisible(data.frame(N,Ne))
}


###
#' @title Estimate variance effective size per sex in generation group or living population
#' @inheritParams Ne.living
#' @param sex \code{"female" or "male"} (default: "female")
#' @param group Maternal generation number of parent or NA = living population (default:\code{"generation"})
#' @return list with N, Ne, breeders, kids, k and k.var 
#' @author Frank PG Princee
#' @family Ne
#' @keywords internal
#'
Ne.group <- function(pedigree,sex=c("female","male"),group=NA, adult=0,breeders_only=FALSE) 
{
  Ne <- 0
 
  sex <- match.arg(sex)
  if (sex == "female") {
    parent <- "DAM"
  } else {
    parent <- "SIRE"
  }
  sex   <- toupper(substr(sex,1,1))
  
  offspring <- subset(pedigree, is.na(DEATHDATE) | (!is.na(DEATHDATE) & AGE >=adult) )  # Living or those survived unti adulthood
 
  breeder   <- table(offspring[!is.na(offspring[,parent] ),parent])  # Get number of offspring per dam/sire
  breeder <- as.data.frame(breeder)
  colnames(breeder) <- c('ID','KIDS')

  for (x in 1:nrow(breeder)) {
    idx <- which(pedigree$ID == as.character(breeder[x,'ID']))
    if (length(idx) > 0 ) {
      pedigree[idx,'KIDS'] <- breeder[x,'KIDS']
    }
  }
 
  rm(breeder)
  
  adults <- subset(pedigree,SEX == sex)
  ped <<- adults
  if (is.na(group)) {
    if (breeders_only) {
      adults <- subset(adults,is.na(DEATHDATE) & KIDS > 0)
    } else {
      adults <- subset(adults,is.na(DEATHDATE) & (KIDS > 0  | AGE >= adult) ) 
    }
    living <- nrow(adults)
  } else {
    adults <- subset(adults,G == group & AGE >= adult)  # Subset for generations and animal has at least reached minimum age
    living <- nrow(adults[is.na(adults$DEATHDATE),])
  }
  
  N     <- nrow(adults)   
  bred  <- nrow(adults[adults$KIDS > 0,])
  kids  <- sum(adults$KIDS )
  
  k     <- mean(adults$KIDS)
  k.var <- var(adults$KIDS)
  
  Ne    <- Ne.variance(N,k,k.var)
  Ne_N  <- Ne/N
  
  return(c(N=N,bred=bred,kids=kids,k=k,k.var=k.var,Ne=Ne,Ne_N=Ne_N,Living=living))
}

###
#' @title Variance effective size in living population
#' @param pedigree pedigree data.
#' @param adult list with male and female ages at sexual maturity (default: list(male=0,female=0)).
#' @param breeders_only (default: FALSE)
#' @author Frank PG Princee
#' @family Ne
#' @keywords internal
#'
Ne.living <- function(pedigree,adult=list( male=0,female=0 ),breeders_only=FALSE)
{
  males   <- Ne.group(pedigree,sex="male",group=NA,adult=adult$male,breeders_only)
  females <- Ne.group(pedigree,sex="female",group=NA,adult=adult$female,breeders_only)
  
  total <- c(N=(males[["N"]] + females[["N"]]),Ne=Ne.sexratio(males[["Ne"]],females[["Ne"]]) )

  return(c(males=males,females=females,total=total))  
}



###
#' @title Variance effective size in generation groups
#' @inheritParams Ne.living
#' @author Frank PG Princee
#' @family Ne 
#' @keywords internal

Ne.generations <- function(pedigree,adult=list( male=0,female=0 ),breeders_only=FALSE)
{                          
  max.G <- max(pedigree$G)
  
  # single sex c(N=N,bred=bred,kids=kids,k=k,k.var=k.var,Ne=Ne)
  
  generations <- matrix(ncol=20,nrow=max.G+1)
  rownames(generations) <- 0:max.G
  colnames(generations) <- c("Nm","bred.m","kids.m","km","km.var","Nme","Nme/Nm","Living.m",
                             "Nf","bred.f","kids.f","kf","kf.var","Nfe","Nfe/Nf","Living.f",
                             "N","Ne","Ne/N","Living")
  
  
  for (x in 1:nrow(generations)) {
    G <- x-1 
    males   <- Ne.group(pedigree,sex="male",group=G,adult=adult$male,breeders_only)
    females <- Ne.group(pedigree,sex="female",group=G,adult=adult$female,breeders_only)  
    Ne <- Ne.sexratio(males[["Ne"]],females[["Ne"]])
    N  <- males[["N"]] + females[["N"]]
    Living <- males[["Living"]] + females[["Living"]]
    generations[x,] <- c(males,females,N,Ne,Ne/N,Living)
  }
  return(generations)
}



###
#' @title Estimate variance effective size per sex in generation group/living population
#' @param group \code{"generation"} or \code{"living"} (default: \code{"living"})
#' @param census.date Census date of living population: \code{"YYYY-MM-DD"} or NULL for most recent
#' date of birth (default: \code{NULL}) 
#' @param taxon Subspecies name (default: NA)
#' @param adult List with ages of male and female at sexual maturity to include potential breeders (default: \code{list(male=0,female=0)})
#' @param breeders_only TRUE: include only breeders in calculations. FALSE: include adult non-breeders.  (default:FALSE) 
#' @author Frank PG Princee
#' @family Ne
#' @export
#'
studbook.Ne <- function(group=c("living","generation"),census.date=NULL,taxon=NA,
                        adult= list(male=0,female=0),breeders_only=FALSE)
{
  pedigree <- studbook.csv('pedigree.csv',field_sep='\t')
  if (is.null(pedigree)) {
    stop("No pedigree data")
  }  
  
  group <- match.arg(group)
  
  pedigree[pedigree$SEX==1,'SEX'] <- "M"
  pedigree[pedigree$SEX==2,'SEX'] <- "F"
  pedigree[pedigree$SEX==3,'SEX'] <- NA
  
  if (!is.na(taxon)) {
    pedigree <- subset(pedigree, TAXON == taxon)
  }
  
  pedigree <- within(pedigree, {
    KIDS <-  0
    AGE  <- NaN
    BREEDER <- ifelse(BREEDER == "no",FALSE,TRUE)
    })
  
  if (is.null(census.date)) {
    census.date <- max(pedigree$DEATH,pedigree$BIRTH,na.rm=TRUE)  # Use as 'census' date
  }
  
  cat("Census date:",census.date,"\n")
  census.date <- tis::jul(census.date)
  
  for (x in 1: nrow(pedigree)) {
    birth <- tis::jul(pedigree[x,'BIRTHDATE'])
    if (is.na(pedigree[x,'DEATHDATE'])) {
      death <- census.date 
    } else {
      death <- tis::jul(pedigree[x,'DEATHDATE'])
    }
    pedigree[x,'AGE'] <- death - birth
  }
  
  # status 0=censored, 1=dead
  
  
  if (group == "living") {
    studbook.screenTitle(paste("Ne(v) in living population on",as.Date(census.date)))
    
    living <- Ne.living(pedigree,adult,breeders_only)
    print(living,digits=2)
    
  } else {
    studbook.screenTitle("Ne(v) in generation groups")
    generations <- Ne.generations(pedigree,adult,breeders_only)
    generations <- generations[!is.na(generations[,"Ne"]),]
    print(generations,digits=2)
    
    studbook.openOutput("Ne(v)")
    matplot(generations[,c('N','Ne')],type='l',lty=,col='black',
            xlab='Generations',ylab='Size',xaxt='n')  #No x labels
    axis(side=1,at=1:nrow(generations),labels=rownames(generations))
    studbook.title("Ne(v) per generation")
    studbook.closeOutput()
    
    matplot(generations[,"Ne/N"],type='l',col='black',xlab='Generations',ylab="Ne/N ratio",xaxt='n')
    axis(side=1,at=1:nrow(generations),labels=rownames(generations))
    studbook.title("Ne/N per generation")
    studbook.closeOutput()
  }
}








