###
#' @title Select age distribution data
#' @description \code{age.file} select age data from list with available age distributions
#' @details The word \code{"first"} to select the oldest age distribution and \code{"last"} 
#' to selects the most recent age distribution. Use a question mark (\code{"?"}) to select
#' from a list of available files. 
#' @param select select date \code{"first","last","?"}. 
#' @param sex sex group; valid names are \code{"none","female","female","all"} 
#' (default: \code{"unknown"}).
#' @return date
#' @author Frank PG Princee
#' @family age distribution
#' @keywords internal
#' 
age.file <- function(select=c("last","first","?"),
                     sex=c("none","female","male","all"))
{
  select <- match.arg(select)
  sex    <- match.arg(sex)
  
  pattern <- "^age-[[:digit:]]{8}"
  
  if (sex != "none") {
    pattern <- paste(pattern,sex,sep="-")
  } 
 
  files  <- studbook.files(pattern=pattern)
  if (is.null(files)) {
    stop("No data on age distribution")
  }
  
  #Extract dates from files
  date.matches <- regexpr("[[:digit:]]{8}",files)
  dates        <- regmatches(files,date.matches)
  dates        <- unique(dates)
  dates        <- sort(dates)
  
  if (select == "first") {
    return(min(dates))
  } else if (select == "last") {
    return(max(dates))
  } else {
    number <- menu(dates,graphics = FALSE,title="Select date number (0=cancel):")
    
    if (number == 0) {
      return(NULL)
    }
    
    return(dates[number])
  }
  
  return(NULL)
}


###
#' @title Import age distribution 
#' @description \code{age.read} reads age class data from tab delimited files. 
#' @details A matrix with columns \strong{M}(ales), \strong{F}(emales) and \strong{U}(nknown) and 
#' a row per age class is created from the tab delimited data files. The file names follow:
#' \code{age-{YYYYMMDD}-{sex}.csv}, indicating ISO date and sex, respectively, 
#' e.g. \code{age-2150821-male.csv}.  
#'  
#' Format of the import file \code{age-YYYYMMDD-sex.csv}:
#' \tabular{lll}{
#' \strong{Field name}(*) \tab \strong{Type} \tab \strong{Comment} \cr
#' CLASS        \tab integer     \tab Age class starting at 0 \cr  
#' COUNT        \tab integer     \tab Number of individuals \cr
#' }
#' (*) The exact names of column headers in age data files are not important.
#' 
#' The age distribution can be truncated for older age classes that have zero counts. 
#' An error is generated when one of the three data files is missing.
#' 
#' @param date monitoring date in ISO 8601 "YYYYMMDD" format;  
#' or "choose".  
#' @param  truncate TRUE will truncate older age classes that are 0 (default TRUE). 
#' @return matrix
#' @author Frank Princee
#' @family age distribution
#' @keywords internal
#' 
age.read <- function(date="",truncate=TRUE) 
{
  if (studbook.empty()) {
    studbook()
  }
  
  # Read data and create data frame
  tmp.male     <- studbook.read(paste('age-',date,sep=""),sex='male')
  tmp.female   <- studbook.read(paste('age-',date,sep=""),sex='female')
  tmp.unknown  <- studbook.read(paste('age-',date,sep=""),sex='unknown')
  
  if (is.null(tmp.male) || is.null(tmp.female) || is.null(tmp.unknown)) {
    return(NULL)
  }
  
  
  if (truncate) {
    #Get last age classes > 0
    last.male     <- which(tmp.male[,2] > 0)
    last.female   <- which(tmp.female[,2] > 0)
    last.unknown  <- which(tmp.unknown[,2] > 0)
    
    if (is.empty(last.unknown)) {
      last.unknown <- 0
    }
    
    classes <- max (max(last.male), max(last.female), max(last.unknown))
  } else {
    classes <- max(nrow(tmp.male),nrow(tmp.female),nrow(tmp.unknown),na.rm=TRUE)
  }
  ages <- matrix( ncol=3,nrow=classes )
  colnames(ages)   <- c('male','female','unknown')
  rownames(ages)   <- 0:(classes-1) 
  ages[,'male']    <- tmp.male[1:classes,2]
  ages[,'female']  <- tmp.female[1:classes,2]
  ages[,'unknown'] <- tmp.unknown[1:classes,2]
  
  return(ages)
}


###
#' @title Distribute unknown sex
#' 
#' @description \code{age.unknown} distributes individuals of unknown sex over females 
#' and males.
#' @details The female sex ratio \eqn{F/(M+F)} per age class is used to distribute unknown
#' individuals.
#'
#' @param ages matrix with columns \code{M(ales)},\code{F(emales)} and \code{U(known)}.
#' @param sexratio female sex ratio F/(M+F) (default: 0.5).
#' @seealso \code{\link{age.read}}
#' @author Frank Princee
#' @family age distribution
#' @keywords internal
#'  
age.unknown <- function(ages,sexratio=0.5)
{
  ages[,'male']    <- ages[,'male']   + (ages[,'unknown'] - (sexratio * ages[,'unknown']) ) 
  ages[,'female']  <- ages[,'female'] + (sexratio * ages[,'unknown'])
  ages[,'unknown'] <- NA
  attr(ages,'distributed') <- TRUE
  return(ages)
}

###
#' @title Age proportion
#' @description \code{age proportion} calculates the proportion individuals per age class.
#' @param group vector with number of individuals per class.
#' @author Frank PG Princee
#' @keywords internal
#' @family age distribution
#' 
age.proportion <- function(group)
{
  total      <- sum(group)
  proportion <- group/total
  return(proportion)
}

###
#' @title Create a stage matrix from age(class)-based data
#' @description \code{create.stages} creates a stage stage matrix from age classes
#' @details Parameter \code{stage.list} is a list in which stages are linked
#' to single or range of  age classes e.g. infant=0, juvenile=1:3. 
#' The last stage does not require an end e.g. old=10 will include all individuals 
#' in age class 10 and older.
#' @param stage.list list with definition of stages to group age classes. 
#' Age classes start at 0 (and will be internally adjusted to row indexes). 
#' @param ages a matrix or data frame with age class per row and counts in columns.
#' @return matrix 
#' @author Frank Princee
#' @family age distribution
#' @export
#' 
create.stages <- function(stage.list,ages)
{ 
  # Prepare matrix
  classes <- nrow(ages)
  stage.matrix <- matrix(0,nrow=length(stage.list),ncol=ncol(ages))
  rownames(stage.matrix) <- names(stage.list)
  colnames(stage.matrix) <- colnames(ages)
  
  for (x in 1:length(stage.list)) {
    # Add last classes to last stage if only start has been defined
    if (x == length(stage.list) & length(stage.list[[x]]) < 2) {
      stage.list[[x]] <- stage.list[[x]]:(classes - 1)
    }
    stage.list[[x]] <- stage.list[[x]] + 1                 # Adjust age class to row index
    
    if (length(stage.list[[x]]) < 2) {
      stage.matrix[x,] <- ages[stage.list[[x]],]
    } else {
      stage.matrix[x,] <- colSums(ages[stage.list[[x]],])  # Sum columns for defined rows
    }
  }
  return(stage.matrix)
}



###
#' @title Plot age pyramid
#' @description \code{age.plot} plots an age pyramid from age class or stage data.  
#' @details The class numbers of stage names are stored in the row names of 
#' matrix \code{ages}. The names of the two groups are stored in the column names
#' 
#' The function \code{pyramid} in package \code{plotrix} is used. 
#' 
#' @param ages matrix with age class or stage data for two groups. 
#' @param main_title title to plot above graph (default: \code{"Age distribution"}) 
#' @param color colours of left and right side of pyramid (default: \code{('black','black')} ).
#' @author Frank Princee
#' @family age distribution
#' @keywords internal
#'  
age.plot <- function(ages,main_title="Age distribution",color=c('black','black')) 
{  
  if (!studbook.plot()) {
    return(NULL)
  }

  studbook.openOutput('age_distribution') 
  
  size <- max(ages[,1],ages[,2])   
  axis.numbers <- seq(0,size,5)
  if (max(axis.numbers < size)) {
    axis.numbers <- seq(0,size+5,5)
  }
  size <- max(axis.numbers)
  studbook.caption(main_title)

  plotrix::pyramid.plot(ages[,1],ages[,2], main="",
                        xlim=c(size,size),top.labels =NA,
                        labels=rownames(ages),labelcex=0.70,gap=5.0,
                        unit=firstCap(colnames(ages)[1:2]),
                        laxlab=c(0,axis.numbers),raxlab=c(0,axis.numbers),
                        lxcol=color[1],rxcol=color[2]) 
  
  #studbook.title(main_title)
  
  studbook.closeOutput()
}


###
#' @title Print and plot age distribution
#' 
#' @description \code{studbook.age} creates age pyramid from age classes.
#' 
#' @details This function imports data, distributes unknown sex over males and
#' females according given ratio \code{F/M+F}, and plots an age pyramid.
#' @param date monitoring date in "YYYYMMDD" format, or \code{"last","first","?"} 
#' (default: \code{"last"}).
#' @param sexratio sex ratio (F/M+F) used to distribute unknown sex (default 0.5)m
#' @param stage.list list with stage definitions. 
#' @param truncate remove older classes that are 0 (default TRUE).
#' @param color colours of left and right side of pyramid (default: \code{('black','black')} ). 
#' @author Frank PG Princee
#' @examples
#' \dontrun{
#' studbook.age()
#' studbook.age(sexratio=0.3,truncate=FALSE)
#' studbook.age(date="20143112",stage.list=list(infant=0,juvenile=1,adult=3:9,senescent=10))
#' }
#' @family age distribution
#' @export
#' 
studbook.age <- function(date="last",sexratio=0.5,stage.list=NULL,truncate=TRUE,
                         color=c('black','black'))
{
  if (studbook.empty()) {
    studbook()
  }

  if (is.element(date,c("first","last","?"))) {
    date <- age.file(date)
    if (is.null(date)) {
      stop("No date selected.")
    }
  }
  
  studbook.screenTitle(paste("Age distribution on",as.Date(date,"%Y%m%d")))
  ages <- age.read(date,truncate)
  
  if(is.null(ages)) { 
    stop("No age data on females, males and/or unknown sex.")
  }
  
  unknown <- sum(ages[,'unknown'])
  
  attr(ages,'date')        <- as.Date(date,"%Y%m%d")
  attr(ages,'distributed') <- FALSE
  attr(ages,'unknown') <- unknown
  attr(ages,'sexratio') <- sexratio
  cat("Number of individuals of unknown sex:",unknown,".\n")
  
  if (!is.na(sexratio) & unknown > 0 ) {
    cat("Unknown distributed according sex-ratio F/(M+F):",sexratio,".\n") 
    ages <- age.unknown(ages,sexratio)
    ages <- ages[,c('male','female')]
  } else {
    cat("Unknown excluded from age distribution\n")
  }
  
  if (!is.null(stage.list)) {
    ages <- create.stages(stage.list,ages)
    print(ages)
  } else {
    print(t(ages))   # Transpose x -> y
  }
  age.plot(ages,color=color) 
}
