
###
#' @title Default settings for animal model
#' @description \code{.defaultAnimalModel} creates and initializes variables in the environment 
#' \code{.StudbookEnv}
#' @details This function is called when is the package \code{studbookR} is loaded. 
#' The following variables are created in the environment \code{.StudbookEnv}:
#' \describe{
#'  \item{distribution}{List with available distributions e.g. \code{"gaussian}, \code{"poisson"}.}
#'  \item{family}{Distribution family set to \code{"gaussian"}.}
#'  \item{nitt}{Number of iterations set to 13,000.}
#'  \item{thin}{Number of thinning set to 10.}
#'  \item{burnin}{Number of burning iterations set to 3,000.}
#'  \item{Effect.fixed}{Fixed effect set to \code{NULL}.}
#'  \item{Effect.random}{Random effect set to \code{"~ animal"}.}
#'  \item{Prior}{Prior set to NULL}
#'  \item{Prior_model}{Prior model set to NULL}
#'  \item{Model}{MCMCglmm results set to NULL}
#'  \item
#' }
#' @author Frank PG Princee
#' 
.defaultAnimalModel <- function()
{
  assign("distribution",c("gaussian", "poisson", "categorical", "multinomial", 
                          "ordinal", "threshold", "exponential", "geometric", 
                          "cengaussian", "cenpoisson", "cenexponential", 
                          "zipoisson", "zapoisson", "ztpoisson", "hupoisson",
                          "zibinomial"),.StudbookEnv)
  
  assign("family","gaussian",.StudbookEnv)
  assign("nitt",13000,.StudbookEnv)
  assign("thin",10,.StudbookEnv)
  assign("burnin",3000,.StudbookEnv)
  
  assign("Effect.fixed", NULL,.StudbookEnv)                       # Fixed
  assign("Effect.random",stats::formula("~ animal"),.StudbookEnv) # Random 
  assign("Prior",NULL,.StudbookEnv)                               # Prior (default to improper)
  assign("Prior.model","improper",.StudbookEnv)                   # Model name e.g. weak, observed
  assign("Model",NULL,.StudbookEnv)                               # MCMCglmm results
  assign("Variances",NULL,.StudbookEnv)                           # List with variances
  assign("EBV.list",NULL,.StudbookEnv)                            # Data frame with estimated breeding values
} 


###
#' @title Set number of iterations, thinning and burnin
#' @description \code{animalmodel.setRuns} will set iterations, thinning and burnin of MCMCglmm. 
#' @param nitt number of iterations (default: \code{13000}).
#' @param thin number of thinning (default: \code{10}).
#' @param burnin number of burn'in (default: \code{3000}).
#' @author Frank PG Princee
#' @family animal model 
#' @examples
#' \dontrun{
#' animalmodel.setRuns(20000,25,5000)
#' studbook.get("nitt")
#' studbook.get("thin")
#' studbook.get("burnin")
#' }
#' @export
#' 
animalmodel.setRuns <- function(nitt=13000,thin=10,burnin=3000)
{
  studbook.set("nitt",nitt)
  studbook.set("thin",thin)
  studbook.set("burnin",burnin)
  cat("Sample size=",(nitt-burnin)/thin,"\n")
}


###
#' @title Set fixed effect
#' @description \code{animalmodel.setFixed} is used to set fixed effects in the 
#' animal model.
#' @details The contents of parameter \code{effect} is combined with the trait as formula e.g. 
#' \code{SIZE ~ DAM_AGE + YEAR} which is stored in variable \code{Effect.fixed}.   
#' @param effect a string with a single or multiple fixed effects combined e.g. \code{"DAM_AGE + YEAR"}
#' (default: \code{"1"}). Each effect needs to be a valid data field.
#' @author Frank PG Princee
#' @family animal model 
#' 
#' @examples
#' \dontrun{
#' animalmodel.setFixed()                 # Fixed effect againsts "1"
#' animalmodel.setFixed('YEAR')           # Fixed effect against YEAR
#' animalmodel.setFixed('DAM_AGE + YEAR') # Fixed effect against DAM_AGE and YEAR
#' }
#' @export
#' 
animalmodel.setFixed <- function(effect = "1")
{
  effect <- paste(studbook.get("phenotype.active"),"~",effect)
  studbook.set("Effect.fixed",stats::formula(effect))
}


###
#' @title Set random effect
#' @description \code{animalmodel.setRandom} is used to set random effects in the animal model. 
#' @details The field names "animal" and "ID" are reserved in \code{MCMCglmm}. A formula is created from
#' the random effects e.g. \code{~ animal} or \code{~ animal + ID + YEAR}.
#' @param effect string with random effect(s) e.g. \code{"animal + ID"}. Each effect needs to be a valid data field.
#' (default: \code{"animal"}).
#' @note the Prior is reset to NULL
#' @author Frank PG Princee
#' 
#' @family animal model 
#' @examples
#' \dontrun{
#' animalmodel.setRandom()              # Random effect is  "animal" 
#' animalmodel.setRandom("animal + ID") # Random effects are "animal" and "ID"
#' }
#' @export
#' 
animalmodel.setRandom <- function(effect="animal")
{     
  effect <- paste("~ ",effect)
  studbook.set("Prior",NULL)
  studbook.set("Prior.model","improper")
  studbook.set("Effect.random",formula(effect))
}


###
#' @title Select supported trait distributions
#' @description \code{animalmodel.selectFamily} selects from a list of trait distributions 
#' that are supported by \code{MCMCglmm}.
#' @return selected distribution or \code{NULL} when no selecton has been made. 
#' @author Frank PG Princee
#' @family animal model 
#' @keywords internal
#' 
animalmodel.selectFamily <- function()
{
  if (!interactive()) {
    return(NULL)
  }
  number <- menu(studbook.get("distribution"),graphics = FALSE,
                 title="Select distribution number:")
  
  if (number == 0) {
    return(NULL)
  }
  
  family <- studbook.get("distribution")
  return( family[number])
}


###
#' @title Set distribution family of trait
#' 
#' @description \code{animalmodel.setFamily} sets the distribution of a trait in the 
#' animal model. 
#' 
#' @details See the function \code{\link{MCMCglmm}} 
#' @note This function is equivalent to \code{studbook.set("family",family))}.
#' @param family trait distribution as supported by MCMCglmm. Enter "?" to select from list.
#' (default: \code{"gaussian"}). 
#' @author Frank PG Princee
#' @family animal model 
#' @examples
#' \dontrun{
#' animalmodel.setFamily("?") 
#' }
#' @export
#' 
animalmodel.setFamily <- function(family="gaussian")
{
  if (family == "?") {
    family <- animalmodel.selectFamily()
  }
  
  if (!is.null(family)) {
    if (is.element(family,studbook.get("distribution"))) {
      studbook.set("family",family)
    } else {
      cat("Distribution",family,"not a valid option.\n")
    }
  }
}

###
#' @title Select prior
#' @description \code{animalmodel.setPrior} selects a prior model.
#' @param prior.model prior model \code{"weak","improper","Fisher","fixed","flat","observed","user"} 
#' (default:\code{"weak"}).
#' @param user.var base variance to create prior when prior.model is \code{"user"}.
#' @param belief belief in prior (\code{nu} in MCMCglmm model) (default: 1).
#' @author Frank PG Princee
#' @family animal model 
#' @seealso \code{\link{prior.create}}
#' @export
#' 
animalmodel.setPrior <- function(prior.model=c("weak","improper","Fisher", "fixed","flat","observed","user"), user.var=NA,belief=1)
{
  prior.model <- match.arg(prior.model)
  
  # Assign observed variance when no user variance is provided
  if (prior.model == "user" && is.na(user.var)) {
    prior.model <- "observed"
  }
    
  effects <- length(all.vars(studbook.get("Effect.random")))

  
  if (prior.model == 'observed') {
    # Create prior based on observed variance 
    studbook.set("Prior", prior.create(prior.var=phenotype.variance(),random.vars=effects)) 
    studbook.set("Prior.model","observed")
  } else if (prior.model == "Fisher") {
    # Create 'Fisher' prior   
    studbook.set("Prior",prior.Fisher(effects))                              
    studbook.set("Prior.model","Fisher")
  } else if ( prior.model == 'fixed') {
    studbook.set("Prior",prior.fixed(effects))
    studbook.set("Prior.model", "fixed")
  } else if (prior.model == "flat") {      
    # Flat prior
    studbook.set("Prior",prior.flat(effects))                                
    studbook.set("Prior.model","flat")
  } else if (prior.model == "weak") {
    # Create weak prior
    studbook.set("Prior",prior.weak(effects))
    studbook.set("Prior.model","weak")
  } else if (prior.model == "user") {
    studbook.set("Prior", prior.create(prior.var=user.var,random.vars=effects,belief=belief)) 
    studbook.set("Prior.model","user")
  } else {
    # Default: improper in MCMCglmm
    studbook.set("Prior",NULL)                                
    studbook.set("Prior.model","improper")
  }
}


###
#' @title Clean phenotype fields
#' @description \code{animalmodel.cleanPhenotype} removes rows with NA values in
#' fields that are used in fixed and random effects
#' @author Frank PG Princee
#' @family animal model 
#' @keywords internal
#' 
animalmodel.cleanPhenotype <- function()
{
  vars  <- all.vars(studbook.get("Effect.fixed"))
  for (v in vars) {   
    if (v == phenotype.getActive()) {
      next
    }
    phenotype.clean(v)
  }
  
  vars <- all.vars(studbook.get("Effect.random"))
  for (v in vars) {
    if (is.element(v,c("animal","ID","dom"))){
      next
    }
    phenotype.clean(v)
  }
}


###
#' @title Extracts phenotypic, environmental and additive variances from results
#' 
#' @description \code{animalmodel.variances} Computes mean variances 
#' @details This function calls \code{posterior.mode} and creates a data frame with results
#' The results are stored in the environmental variable \code{"Variances"}
#' @author Frank PG Princee
#' @family animal model 
#' @keywords internal  
#' 
animalmodel.variances <- function()
{
  Model <- studbook.get("Model") 
  
  #replace names of random effect variances to symbols
  var.names <- coda::varnames(Model$VCV)

  for (x in 1:length(var.names)) {
    if (var.names[x] == "animal") {
      var.names[x] <- "VA" 
    } else if ( var.names[x] == "ID") {
      if (is.element("VA",var.names)) {    # Expects 'animal' as first random variable
        var.names[x] <- "VPE"              # General or permanent environment (Falconer: V[Eg])
      } else {
        var.names[x] <- "VI"
      }
    } else if (var.names[x] == "dom") {
      var.names[x] <- 'VD'
    } else if (var.names[x] == "DAM" | var.names[x] == 'MOTHER') {
      var.names[x] <- "VM"
    } else if (var.names[x] == "units") {
      var.names[x] <- "VR"
    } else {
      var.names[x] <- paste("V[",var.names[x],"]",sep="")
    }
  }
  
  Variances <- rep(0,ncol(Model$VCV))
  names(Variances) <- var.names
 
  for (x in 1:ncol(Model$VCV)) {
    Variances[x] <- mean(Model$VCV[,x])
  }
  studbook.set("Variances",Variances)
}


###
#' @title Estimates heritability
#' 
#' @description \code{animalmodel.heritability} estimates mean heritability \eqn{h2} 
#' from additive \code{VA} and phenotypic \code{VP} variances: \eqn{h2 = VA/VP}. 
#'  
#' @return list with mean credibility range of h2
#' \describe{
#'  \item{mean}{mean}
#'  \item{se}{standard error}
#'  \item{cv}{coefficient of variation}
#'  \item{lower}{lower range of credibility}
#'  \item{upper}{upper range of credibility}
#' }
#' @author Frank PG Princee
#' @family animal model 
#' @keywords internal  
#' 
animalmodel.heritability <- function()
{
  Model <- studbook.get("Model")
  VA <- Model$VCV[,'animal']
  VP <- 0
  for (x in 1:ncol(Model$VCV) )  {  
    VP <- VP + Model$VCV[,x]
  }
  
  h2.runs     <- VA / VP
  h2.mean     <- mean(h2.runs)
  h2.median   <- median(h2.runs)
  h2.mode     <- MCMCglmm::posterior.mode(h2.runs)
  h2.credible <- coda::HPDinterval(h2.runs,0.95)
  h2.mcse     <- mcmcse::mcse(h2.runs)
  
  VA.mean     <- mean(VA)
  h2.cv       <- cv.mean(phenotype.mean(),as.numeric(VA.mean)) # Postma (2014)
  
  lower.value  <- as.numeric(h2.credible["var1","lower"])
  upper.value  <- as.numeric(h2.credible["var1","upper"])
  
  studbook.openOutput("heritability")
  plot(h2.runs,main="",xlab="Heritability",trace=FALSE)
  abline(v=h2.mean,col='black',lty='dashed')
  abline(v=h2.median,col='black',lty='dotdash')
  abline(v=h2.mode["var1"],col='black',lty='longdash')
  abline(v=c(lower.value,upper.value),col='black',lty='dotted')
  legend.txt <- c("mean","median","mode","credible")
  ltypes <-  c(2,4,5,3,3)
  legend("topright",legend=legend.txt,bty='n',lwd=1,lty=ltypes,cex=0.8)
  studbook.title("Density distribution of heritability")
  studbook.closeOutput()
  
  return(c(mean=h2.mean,se=h2.mcse$se,cv=h2.cv,
           median=h2.median,mode=as.numeric(h2.mode["var1"]),
           lower=lower.value,upper=upper.value))
}


###
#' @title Estimates repeatability from MCMC animal model results
#' @description \code{animalmodel.repeatability} estimates repeatability from data on variances in MCMCglmm
#' 
#' @param variances (co-)variance matrix with results from MCMCglmm with at least columns ID and units.
#' @return list with mean credibility range of repeatability
#' \describe{
#'  \item{mean}{posterior mean}
#'  \item{se}{Standard error}
#'  \item{lower}{lower range of credibility}
#'  \item{upper}{upper range of credibility}
#' }
#' @author Frank Princee
#' @family animal model 
#' @keywords internal 
#'  
animalmodel.repeatability <- function()
{
  VP <- 0
  Model <- studbook.get("Model")
  for (x in 1:ncol(Model$VCV) )  {  
    VP <- VP + Model$VCV[,x]
  }
  
  if (is.element("animal",coda::varnames(Model$VCV)) ) {
    r.runs <- ( Model$VCV[,"ID"] + Model$VCV[,"animal"]) / VP
  } else {
    r.runs <- Model$VCV[,"ID"] / VP
  }
  
  r.mean      <- mean(r.runs)
  r.median    <- median(r.runs)
  r.mode      <- MCMCglmm::posterior.mode(r.runs)
  r.credible  <- coda::HPDinterval(r.runs,0.95)
  r.mcse      <- mcmcse::mcse(r.runs)
  
  lower.value <- as.numeric(r.credible["var1","lower"])
  upper.value <- as.numeric(r.credible["var1","upper"])
  
  return(c(mean=r.mean,se=r.mcse$se,median=r.median,mode=r.mode,
           lower=lower.value,upper=upper.value))
}


###
#' @title Clear pedigree and phenotype data to save memory
#' @author Frank PG Princee
#' @family animal model 
#' @export
#' 
animalmodel.clear <- function()
{
  assign("studbook.pedigree",NULL,.GlobalEnv)
  assign("studbook.phenotype",NULL,.GlobalEnv)  
  pedigree.setPruned(FALSE)
  .defaultAnimalModel()
}


###
#' @title Initialize animal model
#' @description \code{animalmodel.init} imports pedigree and phenotype data.
#' @details Pedigree data of the active studbook is imported. Users are asked to select a 
#' phenotype (trait).
#' @note use this function whenever a new trait is to be selected
#' @author Frank Princee
#' @family animal model 
#' @seealso \code{\link{phenotype}} \code{\link{phenotype.repeats}}, \code{\link{pedigree.read}}
#' @export
#' 
animalmodel.init <- function()
{
  phenotype()
  assign("Effect.fixed",NULL,.StudbookEnv)
  assign("Effect.random",NULL,.StudbookEnv)
  pedigree.read()
}


###
#' @title Plot estimated heritability vs sample size
#' 
#' @description \code{animalmodel.sampleSize} plots estimated heritability vs sample size
#' @author Frank Princee
#' @family animal model 
#' @keywords internal
#' 
animalmodel.sampleSize <- function()
{
    VP <- 0
    Model <- studbook.get("Model")
    for (x in 1:ncol(Model$VCV)) {
      VP <- VP + Model$VCV[,x]
    }
  
    h2.runs <- Model$VCV[,"animal"] / VP
    mcmcse::estvssamp(h2.runs,main='Estimated heritability vs sample size')
    
}



###
#' @title Diagnose MCMC results of animal model
#' @description \code{animalmodel.diagnose} prints and plot various results of MCMCglmm using
#' diagnostics in package \code{coda}.
#' @author Frank Princee 
#' @family animal model
#' @export
#' 
animalmodel.diagnose <- function()
{
  oldpar <- par(ask=TRUE,mar=c(1,1,1,1))
  
  Model <- studbook.get("Model")
 
  if (is.element("animal",coda::varnames(Model$VCV)) ) {
     animalmodel.sampleSize()
  }
  
  print(summary(Model$VCV))
  
  plot(Model$VCV)
  acf(Model$VCV)
  
  cat("[Autocorrelation]\n")
  corr_data <- coda::autocorr(Model$VCV)
  print(corr_data)
  
  cat("[Effective size]\n")
  eff.mean <- as.data.frame(coda::effectiveSize(Model$Sol))
  colnames(eff.mean) <- "Effective size of mean"
  print(eff.mean)
  
  eff.var <- as.data.frame(coda::effectiveSize(Model$VCV))
  colnames(eff.var) <- "Effective size of variance"
  print(eff.var)
  cat("** Effective size preferably > 1000\n\n")
  
  cat("[Heidelberger and Welch's convergence diagnostic]\n")
  print(coda::heidel.diag(Model$VCV))
  cat("(p-values must be > 0.05)\n\n")

  cat("[Raftery and Lewis's diagnostic]\n")
  print(coda::raftery.diag(Model$Sol))
  cat("\n")
  print(coda::raftery.diag(Model$VCV))
  cat("\n")
  
  cat("[Geweke's convergence diagnostic]\n")
  print(coda::geweke.diag(Model$Sol))
  cat("\n")
  coda::geweke.plot(Model$Sol)
  
  print(coda::geweke.diag(Model$VCV))
  cat("\n")
  coda::geweke.plot(Model$VCV)
  
  par(oldpar)
}


###
#' @title List animal model settings
#' @description Display list of settings for running the animal model.
#' @author Frank PG Princee
#' @family animal model
#' @export
#' 
animalmodel.settings <- function()
{
  cat("\n")
  cat("[Settings]\n")
  cat("-----------------------------------------------------------------\n")
  cat("Studbook                :",studbook.getName(),"\n")
  cat("Life history feature    :",phenotype.remarks(),"\n")
  cat("Trait analysed          :",phenotype.getActive(),"\n")
  if (phenotype.getRepeats() > 1) {
    cat("Minimum repeats         :",phenotype.getRepeats(),"\n")
  }
  cat("Total observations      :",nrow(studbook.phenotype),"\n") 
  cat("Number of individuals   :",length(unique(studbook.phenotype[,'animal'])),"\n") 
  cat("Mean value of trait     :",phenotype.mean(),"\n")
  cat("Variance (Vp[obs])      :",phenotype.variance(),"\n")
  cat("Fixed effect(s)         : ")
  print(studbook.get("Effect.fixed"),showEnv=FALSE)
  cat("Random effect(s)        : ")
  print(studbook.get("Effect.random"),showEnv=FALSE)
  cat("Prior model             :",studbook.get("Prior.model"),"\n")
#  cat("Prior                   : ")
#  print(studbook.get("Prior"),showEnv=FALSE)
  cat("Trait distribution      :",studbook.get('family'),"\n") 
  cat("Iterations              :",studbook.get('nitt'),"\n")
  cat("Thinning                :",studbook.get('thin'),"\n")
  cat("Burn in                 :",studbook.get('burnin'),"\n\n")
}
  

###
#' @title Print summary of MCMCglmm results
#' @param has_pedigree TRUE: no pedigree used
#' @param repeated TRUE: repeatability has been tested
#' @author Frank PG Princee
#' @family animal model
#' @keywords internal
#' 
animalmodel.results <- function(has_pedigree=TRUE,repeated=FALSE)
{
  cat("[Results]\n")
  cat("-----------------------------------------------------------------\n")
  cat("Variances:\n")
  Model <- studbook.get("Model")
  animalmodel.variances()
  
  variance <- studbook.get("Variances")

  cat("VP                      :",sum(variance),"\n")
  if (is.element("VA",names(variance)) ){
    cat("VA                      :",variance['VA'],"\n")
  }

  
  if (is.element("VD",names(variance))) {
    cat("VD                      :",variance['VD'],"\n")
  }
  
  if (is.element("VI",names(variance))) {
    cat("VI                      :",variance['VI'],"\n")
  }
  
  if (is.element("VPE",names(variance))) {
    cat("VPE                     :",variance['VPE'],"\n")
  }
  

  if (is.element("VM",names(variance))) {
    cat("VM                      :",variance['VM'],"\n")         
  }
  
  for (x in 1:length(variance)) {
    if (!is.element(names(variance)[x],c("VA","VD","VI","VM","VPE","VR")) ) {
      cat(sprintf("%-24s:",names(variance)[x]),variance[x],"\n")
    }
  }
  
  cat("VR                      :",variance['VR'],"\n")
  cat("\n")
  if (has_pedigree) {  
    heritability  <- animalmodel.heritability()
    cat("VA/VP[obs]              :",variance["VA"]/phenotype.variance(),"\n")
    cat("Heritability - mean     :",heritability['mean'],"\n")
    cat("             - median   :",heritability['median'],"\n")
    cat("             - mode     :",heritability['mode'],"\n")
    cat("Credible interval (95%) :",heritability['lower'],"-",heritability['upper'],"\n")
    cat("Standard error          :",heritability['se'],"\n")
    cat("z-ratio h2/se           :",(heritability['mean']/heritability['se']),"\n")
    cat("CV_A                    :",heritability['cv'],"%\n")
  }
  
  if (repeated) {
    repeatability <- animalmodel.repeatability()
    cat("Repeatability - mean    :",repeatability['mean'], "\n")
    cat("              - median  :",repeatability['median'],"\n")
    cat("              - mode    :",repeatability['mode'],"\n")
    cat("Credible interval (95%) :",repeatability['lower'],"-",repeatability['upper'],"\n")
    cat("Standard error          :",repeatability['se'],"\n")
    cat("z-ratio r/se            :",(repeatability['mean']/repeatability['se']),"\n")
  }
  cat("DIC                     :",Model$DIC,"\n")
  cat("-----------------------------------------------------------------\n\n")
}

  
###
#' @title  Markov chain Monte Carlo animal model
#' 
#' @description \code{animalmodel.mcmc} runs the MCMC implementation of the 
#' 
#' @details This function is a wrapper to the package \code{MCMCglmm} 
#' 
#' @param prior_model the prior defined by the system is used when no prior_model is passed (default:NULL).
#' @param ebv save iteration resuls for breeding values and other random effects (default: FALSE).
#' @param verbose show MH diagnostocs (default: FALSE).
#' @note the function \code{\link{animal.init}} needs to called before 
#' invoking this function.
#' @author Frank PG Princee
#' @family animal model
#' @examples
#' \dontrun{studbook()
#' animalmodel.init()
#' animalmodel.mcmc()
#' animalmodel.evaluate()
#' }
#' @seealso \code{\link{MCMCglmm}}
#' @export
#' 
animalmodel.mcmc <- function(prior_model = NULL,start=NULL,ebv=FALSE,verbose=FALSE)
{
  TRAIT <- phenotype.getActive()
 
  #IMPLEMENT filter on minimum repeats in Phenotype.data
  distribution <- studbook.get('family')
  nitt    <- studbook.get('nitt')
  thin    <- studbook.get('thin')
  burnin  <- studbook.get('burnin')
  
  if (is.null(studbook.get("Effect.fixed")) ) {
    assign("Effect.fixed",formula(paste(TRAIT," ~ 1")),.StudbookEnv)
  }
  
  if (is.null(studbook.get("Effect.random")) ) {
    assign("Effect.random",formula("~ animal"),.StudbookEnv) 
  }
  
  
  if (!is.null(prior_model)) {
     animalmodel.setPrior(prior_model)
  }


  animalmodel.cleanPhenotype() # Remove rows with NA for selected fixed and random effects
  
  pedigree_used <- FALSE                           
  repeated <- FALSE
  ginverse <- NULL
  
  random.vars <- all.vars(studbook.get("Effect.random"))

  if (is.element("animal",random.vars) ) {
    pedigree_used <- TRUE
    
    pedigree.prune(studbook.phenotype[,'animal'])   
  
    invA <- MCMCglmm::inverseA(pedigree=studbook.pedigree[,1:3])$Ainv    #column 1:3 are id/dam/sire 
    
    if (is.element("dom",random.vars) ) {
      listD <- nadiv::makeD(studbook.pedigree) 
      ginverse <- list(animal=invA,dom=listD$Dinv)
      rm(listD)
    } else {
      ginverse <- list(animal=invA)
    }
    rm(invA)
  } 
   
  if (is.element("ID",all.vars(studbook.get("Effect.random")) ) ){
    repeated <- TRUE
  }
  
  if (!phenotype.isRepeatable()) {
    repeated <- FALSE
  }
  
  available.distr <- studbook.get("distribution")

  #  3:categorical, 4:multinomial, 
  # 12:zipoisson,  13:zapoisson
  # 14:hupoisson   15:zibinomial
  
  if ( is.element(distribution,available.distr[c(3:4,12:16)])) {
      rcov <- "~us(trait):units"
  } else {
      rcov <- "~units"
  }
  
  # Clear memory
  studbook.set("Model",NULL)     
  studbook.set("Variances",NULL)
  studbook.set("EBV.list",NULL)  
  
  cat("\n")
  cat("MCMCglmm animal model\n")
  cat("=================================================================\n")
  animalmodel.settings()
  
  #Build test for repeatability with single observations
  start.time <- Sys.time()
  
  Model <- MCMCglmm::MCMCglmm(fixed=studbook.get("Effect.fixed"),
                              random=studbook.get("Effect.random"),
                              rcov=formula(rcov),
                              family=distribution,
                              data=studbook.phenotype,start=start,
                              prior=studbook.get("Prior"),
                              ginverse=ginverse,
                              nitt=nitt,thin=thin,burnin=burnin,
                              pr=ebv,
                              verbose=verbose) 

  studbook.set("Model",Model)
  end.time <- Sys.time()
  difftime(end.time,start.time)
  animalmodel.results(pedigree_used,repeated)
  
 
  
}


###
#' @title Run REML animal model 
#' 
#' @details column \code{ID} is the unique animal id, while PE is used to
#' estimate the permanent environment V[PE].
#' @param effects fixed and random effects. If NULL then TRAIT  ~ (1|animal) is used (default: NULL).
#' @param family.distr distribution of trait  @seealso family (default: NULL = gaussian).
#' @param REML REML (TRUE) or log-likelihood (FALSE)  (default:TRUE). 
#' @author Frank PG Princee
#' @family animal model
#' @export
#' 
animalmodel.reml <- function(effects=NULL,family.distr=NULL,REML=TRUE)
{
  if (!pedigree.isPruned() & studbook.get("phenotype.format") == 'factor') {
    pedigree.prune(studbook.phenotype[,'animal'])
  }
  
  if (studbook.get("phenotype.format")  == 'factor') {
    phenotype.integer()
  }
  
  if (studbook.get("pedigree.format") == 'factor') {
    pedigree.integer()
  }
  
  
  # Use gsub("[+]"," ",getFixed + random effects)
  if (is.null(effects)) {
     effects <- paste(phenotype.getActive(),"~ (1|animal)")
  } else {
     effects <- paste(phenotype.getActive(),"~",effects)
  }
  
  studbook.set("Model",NULL)     # Clear memory
  studbook.set("Variances",NULL) 
  studbook.set("EBV.list",NULL)  
  
  cat("Model:",effects,"\n")
  start.time <- Sys.time()
 
  #Field animal must exists
  if (is.null(family.distr) ) {   # Work around problem of glmer rejecting gaussian
    Model <- pedigreemm::pedigreemm(formula = formula(effects),
                                  data=studbook.phenotype,
                                  REML=REML,
                                  pedigree=list(animal=studbook.pedigree),             # ID is integer index to pedigree
                                  control=lme4::lmerControl(check.nobs.vs.nlev="ignore", 
                                                           check.nobs.vs.nRE="ignore")) 
                                 
  } else {
    Model <- pedigreemm::pedigreemm(formula = formula(effects),
                                  data=studbook.phenotype,
                                  family=family.distr,
                                  REML=REML,
                                  pedigree=list(animal=studbook.pedigree),
                                  control=lme4::lmerControl(check.nobs.vs.nlev="ignore",
                                                      check.nobs.vs.nRE="ignore"))
                                  
                                  
    
  }
  end.time <- Sys.time()
  print(difftime(end.time,start.time),.showEnv=FALSE)
  cat("\n")
 
  print(summary(Model))
  cat("\n")
  variance <- lme4::VarCorr(Model)
  variance <- as.data.frame(variance)
  
  
  VA  <- variance[variance$grp == 'animal','vcov']
  VR  <- variance[variance$grp == 'Residual','vcov']
  VP   <- sum(variance[,'vcov'])
  CV_A <- cv.mean(phenotype.mean(),as.numeric(VA)) # Postma (2014)
    
    
  cat("VP               :",VP,"\n")
  cat("heritability     :",VA/VP,"\n")
  cat("CV[A]            :",CV_A,"\n")
  
  if (is.element('ID',variance$grp)) {
    VPE <- variance[variance$grp == 'ID','vcov']
    cat("repeatability    :",(VA + VPE)/VP,"\n")
  }
  studbook.set("Model",Model)
  
  Variances <- c(VA,VR,VP)
  names(Variances) <- c("VA","VR")
  
  studbook.set("Variances",Variances)
  # Breeding values
  ebv.reliability()
  cat("AIC              :",AIC(Model),"\n")
  cat("BIC              :",BIC(Model),"\n")
  cat("-----------------------------------------------------------------\n\n")

}
