#' @title Interbirth interval
#' 
#' @description \code{studbook.birthInterval} calculates distribution and basic statistics 
#' of interbirth interval. 
#' 
#' @details Data are imported from the tab delimited text file \code{birth_interval.female.csv} 
#' in the project folder. 
#' 
#' @param unit Display time unit in \code{"day", "week", "month" or "year"} (default: \code{"month"})
#' @param truncate Proportion of data that needs to be truncated from start and end of distribution. 
#' @param max_interval Maximum interval in days (default NA)
#' @param explore Explore effect of sample size and truncation (default: FALSE)
#' @param bootstrap Bootstrap on mean (default: FALSE)
#' @param file_name Basic name of data file (default: \code{"birth_interval"})
#' @author Frank Princee
#' @examples
#' \dontrun{
#' studbook.birthInterval()
#' studbook.birthInterval(unit='days',truncate=0.01)
#' }
#' @export
#' 
studbook.birthInterval <- function(unit='month',truncate=0.0,max_interval=NA,
                                   explore=FALSE,bootstrap=FALSE,
                                   file_name="birth_interval")
{

  raw.data <- studbook.read(file_name,sex='female')[,'INTERVAL'] 
  if (is.null(raw.data)) { 
    stop("No birth interval data.")
  }
  
  studbook.screenTitle('Inter-birth interval')
  # Restrict raw data 
  if (is.na(max_interval)) {
    interbirth.data <- raw.data
  } else {
    interbirth.data <- subset(raw.data,raw.data <= max_interval)
  }
  
  rm(raw.data)
  
  interbirth.data <- sort(interbirth.data)
  
  if (truncate > 0.0) {
    interbirth.data <- sample.truncate(interbirth.data,truncate,'both')
  }
  studbook.summary(interbirth.data,long=FALSE,title='Inter-birth interval')    
  
  
  studbook.openOutput('birth_interval')
  
  if (studbook.plot()) {
    
    
    if (explore) {
      distribution.sample(interbirth.data,c(10,50,100,250),
                          main_title='Random samples of birth interval', 'days') 
      
      
      distribution.exploreEnds(interbirth.data,levels=c(0.1,0.05,0.01),
                               side='left',"Inter-birth interval","days") 
      distribution.exploreEnds(interbirth.data,levels=c(0.1,0.05,0.01),
                               side='right',"Inter-birth interval","days") 
    } 
    
    if (unit == 'day') {
      interbirth.unit <- interbirth.data
    } else if (unit == 'week') {
      interbirth.unit <- floor(interbirth.data/7.0)
    } else if (unit == 'month') {
      interbirth.unit <- floor(interbirth.data/30.5)
    } else if (unit == 'year') {
      interbirth.unit <- floor(interbirth.data/365.25)
    } else {
      interbirth.unit <- interbirth.data
    }
    xlabel <- paste0(firstCap(unit),"s")
    
    interbirth.max <- max(interbirth.unit)
    
    distribution.initGraphs(FALSE)
    distribution.selectGraph('histogram',TRUE) 
    distribution.selectGraph('density',FALSE)
    distribution.plot(interbirth.unit,main_title='Inter-birth interval',
                      xlabel=xlabel,bar_breaks=c(0:interbirth.max),
                      leftPoints=min(interbirth.unit) )
  }
  
  # Display summary when graphic mode is off
  if (bootstrap) {
    studbook.bootstrap(interbirth.data,statistic=mean.fun,
                       main_title='Inter-birth interval')
  }
  studbook.closeOutput()  
}
