###
#' @title Annual births
#' @description \code{studbook.births} plots and analyses annual births in population.
#' 
#' @details This function plots annual births in males, females and total population per year and
#' analyses differences in sex--ratio, and estimates and plots \emph{per capita birth rates},
#' births per female, and female births per female.  
#' 
#' Assign \code{NULL} to arguments \code{first} and/or \code{last} to include the oldest and
#' most recent data, respecitively. 
#' 
#' @param first First year of death census (default = NULL) 
#' @param last Last year of death (default = NULL)
#' @param analyse options: \code{"sexratio"} and/or \code{"birthrate"} (default: both) 
#' @author Frank PG Princee
#' @examples 
#' \dontrun{
#' studbook.births()                     # all years in data-set
#' studbook.births(first=1977)           # births in 1977 and onwards
#' studbook.births(first=1977,last=1996) # births from 1977 until 1996 (20 years)
#' }
#' @family census
#' @export
#' 
studbook.births <- function(first=NULL, last=NULL,analyse=c('sexratio','birthrate')) 
{
  studbook.screenTitle('Census of births')
  
  birth.ts <- census.read('birth',first,last)
  
  if (is.null(birth.ts)) {
    stop("No birth census data.")
  }
  analyse <- match.arg(analyse,several.ok=TRUE)
  
  print(birth.ts)
  census.plot(birth.ts,'births')
  
  # Analyse sex-ratio
  if (is.element("sexratio",analyse)) {
    census.sexratio(birth.ts,'birth',TRUE)
  }
  
  # Calculate and analyse birth rates
  if (is.element("birthrate",analyse)) {
    start_year <- start(birth.ts)
    end_year   <- end(birth.ts)
    
    census.ts <- census.read('census',start_year[1],end_year[1])
    if (is.null(census.ts)) {
      stop("No census data") 
    }
    
    #Births/population = per capita birth rate
    birth_rate <- data.frame("BIRTHS" = birth.ts[,'ALL'],"CENSUS" = census.ts[,'ALL'])
    birth_rate <- ts(birth_rate,start=start_year,frequency=1) # Create time-series
    census.rate( birth_rate,'birth rate','birth_rate')
    
    
    #Births/female population
    birth_rate <- data.frame("BIRTHS" = birth.ts[,'ALL'],"FEMALE" = census.ts[,'FEMALES'])
    birth_rate <- ts(birth_rate,start=start_year,frequency=1)
    census.rate(birth_rate,'births per female','birth_per_female')
    
    #Female births/female population
    birth_rate <- data.frame("FEMALE_BIRTHS" = birth.ts[,'FEMALES'],"FEMALE" = census.ts[,'FEMALES'])
    birth_rate <- ts(birth_rate,start=start_year,frequency=1)
    census.rate(birth_rate,'female births per female','female_birth_female') 
  }
}


