###
# Common function to be used with function boot
###


###
#' @title Calculate mean from bootstrap run
#' 
#' @param data vector with original data
#' @param idx  vector with sampled data elements
#' @author Frank Princee
#' @export
#' @note Function designed for use with package \code{boot}
#' 
mean.fun <- function(data,idx) 
{
  return(mean(data[idx]))
}


###
#' @title Calculate median from bootstrap run
#' 
#' @param data vector with original data
#' @param idx  vector with sampled data elements
#' @author Frank Princee
#' @export
#' @note Function designed for use with package \code{boot}
#' 
median.fun <- function(data,idx) 
{
  return(median(data[idx]))
}


###
#' @title Estimate 95 quantile from bootstrap runs
#' 
#' @param data vector with original data
#' @param idx  vector with sampled data elements
#' @author Frank Princee
#' @note Function designed for use with package \code{boot}
#' @export
#' 
quantile.fun <- function(data,idx) 
{
  return(quantile(data[idx],.95))
}

###
#' @title Bootstrap dataset
#' 
#' @description \code{studbook.bootstrap} carries out bootstrap analysis on data-set,  
#' prints summary statistics on bootstrap, and plots bootstrap distribution of results
#' as a histogram.
#' @param data data object
#' @param runs number of bootstrap runs (default 999)
#' @param boot.fun function to be applied in bootstrap
#' @param main_title main title
#' @author Frank PG Princee
#' @export
#' 
studbook.bootstrap <- function(data,statistic=mean.fun,runs=999,main_title="mean")
{
  
  cat("Bootstrap on",main_title," [runs = ",runs,"]\n")
  
  result.boot <- boot::boot(data,statistic=statistic,R=runs)  
 
  print(result.boot)
  
  
  ci    <- boot::boot.ci(result.boot,type=c("norm","basic", "perc"))
  stdev <- sd(result.boot$t) 
  avg   <- mean(result.boot$t)  
  print(ci)
  
  if (studbook.plot()) {
    sub_title <- paste("[runs = ",runs,"]")
    caption   <- paste("Bootstrap on",main_title)
    comment   <- paste("Number of runs:",runs)
    
    studbook.caption(caption,comment)
    
    hist(result.boot$t,main="",border='black',col='grey60',density=50,
         angle=45,xlab="Bootstrap value" ,ylab="density",freq=FALSE )
    studbook.title(main_title,sub_title)
    #abline(v=ci,lty="dashed")
  }
}




