###
#' @title Estimate reproductive lifespan
#'
#' @description \code{studbook.breeding} reads data on age at breeding for males and females, 
#' and estimates reproductive lifespan.
#' 
#' @details Data are imported from the tab delimited text files \code{reproductive-female.csv} 
#' and \code{reproductive-male.csv} in the project folder. 
#' 
#' Format of reproductive data file:
#' \tabular{lll}{
#' \bold{Field name} \tab \bold{Type} \tab \bold{Comment} \cr
#' DATA        \tab integer     \tab Age at reproduction in days\cr          
#' }
#' @param truncate proportion of left and right truncation of data  (default: 0)
#' @param explore Explore effect of sample size and truncation (default: FALSE)
#' @param bootstrap: bootstrap on mean (default: FALSE)
#' @param file_name Baisc name of data file (default: \code{"reproductive"})
#' @author Frank PG Princee
#' @examples
#' \dontrun{
#' studbook.breeding()     # No trunctation
#' studbook.breeding(0.05) # 5% truncation at left and right side
#' }
#' @export 
#'
studbook.breeding <- function(truncate=0.0,explore=FALSE,bootstrap=FALSE,file_name="reproductive")
{
  
  male   <-studbook.read(file_name,sex='male')[,'DATA']
  female <-studbook.read(file_name,sex='female')[,'DATA']
  
  if (is.null(male) || is.null(female)) {
    stop("No reproductive data on males and/or females")
  }
  
  studbook.screenTitle("Reproductive lifespan")
  male   <- sort(male)
  female <- sort(female)
  
  studbook.openOutput('reproductive')
  
  if (explore) {
    distribution.sample(female,c(10,50,100,250),main_title='Random samples of reproductive age', 'days') 
    distribution.exploreEnds(female,levels=c(0.1,0.025,0.05,0.01),
                             side='left',"Reproduction","Age in days") 
    distribution.exploreEnds(female,levels=c(0.1,0.025,0.05,0.01),
                             side='right',"Reproduction","Age in days") 
  } 
  
  male   <- sample.truncate(male,truncate,'both') 
  female <- sample.truncate(female,truncate,'both') 
  
  
  studbook.summary( male,long=FALSE,title="males")
  studbook.summary( female,long=FALSE,title="females")
  
  distribution.initGraphs(FALSE)
  distribution.selectGraph('histogram',TRUE)
  
  max.data <- max(max(male),max(female) )
  age_breaks <- seq(0,max.data + 366,365.25)
  
  naturalhistory.plot(male,  'Reproductive lifespan (males)',age_breaks)
  naturalhistory.plot(female,'Reproductive lifespan (females)',age_breaks)
  
  #DAYS
  naturalhistory.compareSexes(male,female)    
  
  # Bootstrap of mean
  if (bootstrap) {
    studbook.bootstrap(data=male,statistic=mean.fun,main_title="Reproductive lifespan (males)")
    studbook.bootstrap(data=female,statistic=mean.fun,main_title="Reproductive lifespan (females)")
  }
  studbook.closeOutput()
  
}


