
###
#' @title Compute min and max in sex group of census data
#' @param census.data Data frame (tseries) with census data on sex groups
#' @param groups sex groups e.g. \code{'ALL', 'MALES', 'FEMALES'}.
#' @return vector with min and max
#' @author Frank PG Princee
#' @family census
#' @keywords internal
#' 
census.ylim <- function(census.data,groups)
{
  ymin <- min(census.data[,groups],na.rm=TRUE)
  ymax <- max(census.data[,groups],na.rm=TRUE)
  ymin <- ifelse(ymin < 0,0,ymin)  # minimum
  return(c(ymin,ymax))
}


###
#' @title Strip census with zero counts at start of census data
#' 
#' @description \code{census.leadingZero} strips zero counts from start of census data. 
#' @param  census.data census counts either as time series or vector
#' @return row number 
#' @note Calling function should handle exception last year = end of data_set
#' @author Frank Princee
#' @family census
#' @note calling function should handle exception last year = end of data_set
#' @keywords internal
#' 
census.leadingZero <- function(census.data)    
{
  zero <- which(census.data == 0)
  len  <- length(zero)
  
  if (is.ts(census.data)) {
    start_year <- start(census.data)
    start_row  <- ifelse( len < 1,start_year[1], start_year[1] + len)        
  } else {
    start_row  <- ifelse (len < 1,1,zero[1])
  }
  
  invisible(start_row)
}


###
#' @title Read census-style data for males, females, and unknown sex
#' 
#' @description \code{census.read} reads census and census event data 
#' for \code{all}, \code{males} and \code{females}.  
#' 
#' Format of census-style data files:
#' \tabular{lll}{
#' \bold{Field name} \tab \bold{Type} \tab \bold{Comment} \cr
#' <interval>(*)   \tab integer     \tab Date of census (iso86 format) \cr  
#' COUNT        \tab integer     \tab Number of individuals \cr
#' }
#' (*) The column name <interval> reflects the census interval e.g. column name "annual".
#' 
#' @param analysis Type of census i.e. \code{"birth", "census", "death", "emigration" or "immigration"}  
#' @param first first year of census. NULL refers to first year in data (default NULL) 
#' @param last last year of census. NULL refers to last years data (default NULL)
#' @return Time series for years of selected sex groups
#' @seealso \code{link{studbook.read}}
#' @author Frank Princee
#' @family census
#' @keywords internal
#' 
census.read <- function(analysis,first=NULL,last=NULL) 
{
  # Read data and create data frame
  
  # SPARKS 1.6 data
  if (studbook.get("program_mode") == "SPARKS") {
    if (analysis == "immigration-all" || analysis == "immigration-wildborn") {
      analysis <- "immigration"
    }
    census.data <- sparks.census(analysis)
    census.data <- within(census.data,SEXRATIO <- FEMALES/(MALES+FEMALES))
  } else {
    
    tmp.all     <- studbook.read(analysis)
    tmp.male    <- studbook.read(analysis,sex='male')
    tmp.female  <- studbook.read(analysis,sex='female')
    
    if (is.null(tmp.all) || is.null(tmp.male) || is.null(tmp.female) ){
      warning("No census data available")
      return(FALSE)
    }
    
    census.data <-  data.frame("DATE"=base::as.Date(tmp.all[,1]),"ALL"=tmp.all[,2],
                               "MALES"=tmp.male[,2],"FEMALES"=tmp.female[,2],
                               "SEXRATIO"=rep(NA,nrow(tmp.all)))
    
    census.data <- within(census.data,SEXRATIO <- FEMALES/(MALES+FEMALES))
    
    attr(census.data,"interval") <- names(tmp.all)[1]
  }
  
  attr(census.data,"census_date") <- format(census.data[1,'DATE'],"%d-%b")    #day-month
  # Convert data frame to time series and set window
  frequency <- 1     # Assumes annual
  
  # Default time window
  period <- range(tis::year(census.data[,'DATE']))
  
  census.ts <- ts(data=census.data[,2:5],start=period[1],frequency=frequency)
  
  if (is.numeric(first)) {
    period[1] <- first
  }
  
  if (is.numeric(last))  {
    period[2] <- last
  }
  
  
  #Trailing zero counts
  if (is.null(first)) {
    period[1] <- census.leadingZero(census.ts[,'ALL'])
  }
  census.ts <- window(census.ts,start=period[1],end=period[2])
  invisible(census.ts) 
}


###
#' @title Plot census-like data. 
#' @details Census dates (year) are stored in row names
#' @param census.data Census(-like) data
#' @param census_type Type of census e.g. \code{"census"} or \code{"births"}
#' @param project.year Year that projection starts (default: NULL)
#' @param conf.int Plots confidence interval for single sex and total when provided
#' (default:FALSE)
#' @author Frank PG Princee
#' @family census
#' @keywords internal
#' 
census.plot <- function(census.data,census_type,project.year=NULL,conf.int=FALSE) 
{
  if (!studbook.plot()) {
    return(NULL)
  }
  
  census.interval <- studbook.get("census.interval")
  census.interval <- firstCap(census.interval)
  
  # Group names
  group.names  <- colnames(census.data)
 
  census.years <- nrow(census.data)
  ytxt <- 'N'
  
  #"solid", "dashed", "dotted", "dotdash", "longdash", or "twodash"
  
  ltypes <- c(1,2,6)
  
  studbook.openOutput(census_type)
  
  
  #Plot total population
  if ( is.element("ALL",colnames(census.data)) ) {
    
    if (conf.int) {
      ylim <- census.ylim(census.data,c("ALL","ALL.LOWER","ALL.UPPER"))
    } else {
      ylim <- NULL
    }
    main_title <- paste(census_type,"(total)") 
    studbook.caption(main_title,"")
    
    plot(census.data[,'ALL'], ylim=ylim,
         xlab=census.interval, ylab=ytxt,main="",
         col='black',type='l',lwd=1,lty=1)
    
    if (!is.null(project.year)) {
      abline(v=project.year,lty='dotted')
    }
    
    if (conf.int) {
      lines(census.data[,"ALL.LOWER"],lty='dashed')
      lines(census.data[,"ALL.UPPER"],lty='dashed')
    }
    studbook.title(main_title)    
  }
  
 
  
  #Plot all groups separate
  groups <- c("ALL","MALES","FEMALES")
  if (sum(is.element(groups,colnames(census.data))) == 3) {
    studbook.caption(census_type,"Overview of all sex groups.")
    
    plot(census.data[,groups],main="", plot.type='single',
         xlab=census.interval,ylab=ytxt,
         type='l',lty=ltypes,lwd=1)
    
    if (!is.null(project.year)) {
      abline(v=project.year,lty='dotted')
    }
    studbook.title(census_type)
    legend("topleft","(x,y)",bty='n',group.names[1:3],lwd=1,lty=ltypes,cex=0.6)
  }
  
  #Plot males and females
  groups <- c("MALES","FEMALES")
  if (sum(is.element(groups,colnames(census.data))) == 2) { # Males and females exist
    studbook.caption(census_type,"(males and females)")
    plot(census.data[,groups],main="", plot.type='single',
         xlab=census.interval,ylab=ytxt,type='l',lty=c(2,6),lwd=1)
    
    if (!is.null(project.year)) {
      abline(v=project.year,lty='dotted')
    }
    
    studbook.title(census_type)
    legend("topleft","(x,y)",bty='n',group.names[2:3],lwd=1,lty=c(2,6),cex=0.6)
  }
  
  #Plot males
  if (is.element("MALES",colnames(census.data))) {
    
    if (conf.int) {
      ylim <- census.ylim(census.data,c("MALES","MALES.LOWER","MALES.UPPER"))
    } else {
      ylim <- NULL
    }
    
    main_title <- paste(census_type,"(males)") 
    studbook.caption(main_title,"")
    plot(census.data[,'MALES'],ylim=ylim,
         xlab=census.interval, ylab=ytxt,main="",
         col='black',type='l',lwd=1,lty=1)
    if (!is.null(project.year)) {
      abline(v=project.year,lty='dotted')
    }
    
    if (conf.int) {
      lines(census.data[,"MALES.LOWER"],lty='dashed')
      lines(census.data[,"MALES.UPPER"],lty='dashed')
    }
    studbook.title(main_title)    
  }
  
  
  #Plot females
  if (is.element("FEMALES",colnames(census.data))) {
    if (conf.int) {
      ylim <- census.ylim(census.data,c("FEMALES","FEMALES.LOWER","FEMALES.UPPER"))
    } else {
      ylim <- NULL
    }
    main_title <- paste(census_type,"(females)") 
    studbook.caption(main_title,"")
    plot(census.data[,'FEMALES'],ylim=ylim,
         xlab=census.interval,ylab=ytxt,main="",
         col='black',type='l',lwd=1,lty=1)
    if (!is.null(project.year)) {
      abline(v=project.year,lty='dotted')
    }
    
    if (conf.int) {
      lines(census.data[,"FEMALES.LOWER"],lty='dashed')
      lines(census.data[,"FEMALES.UPPER"],lty='dashed')
    }
    studbook.title(main_title)     
  }
  studbook.closeOutput()
}


###
#' @title Calculates geometric mean of census rates
#'
#' @param rates.data census rates
#' @param weighted vector with weighted data or NULL (default: NULL)
#' @param alpha Signficance level (default: 0.05)
#' @param interval Type of interval: \code{"confidence" or "prediction"} 
#' (default: \code{"confidence"})
#' @author Frank PG Princee
#' @family census
#' @seealso \code{\link{geometric.Mean}}
#' @keywords internal
#' 
census.geometricMean <-function(rates.data,weighted = NULL,alpha=0.05,
                                interval=c("confidence","prediction"))
{
  n <- length(rates.data)
  geomean <- data.frame("mean"=rep(NA,n),"lower"=rep(NA,n),"upper"=rep(NA,n))
  end     <- n 
  
  for (c in 1:end)  {
    if ( c < end && is.finite(rates.data[c])  )  {
      if (!length(weighted)) {
        geomean[c,1:3] <- geometric.Mean(rates.data[c:end],alpha=alpha,interval=interval)                 # over last years census year, year + 1, year + 2
      } else {
        geomean[c,1:3] <- geometric.Mean(rates.data[c:end],weighted[c:end])  
      }
    } 
  }
  return(geomean)
}



###
#' @title  Finite rate of change from census data
#' 
#' @description Calculates finite rate of change (\eqn{\lambda}) from census data 
#' @param census.data Census(-like) data
#' @author Frank PG Princee
#' @family census
#' @keywords internal
#' 
census.lambda <- function(census.data)
{
  n <- length(census.data)
  lambda <- c(NA,(census.data[2:n]/census.data[1:n-1])) # Add one for first class 
  lambda[is.infinite(lambda)] <- NA 
  return(lambda)
}

###
#' @title Instantaneous rates
#' 
#' @description Calculates instantaneous rate(s) (\eqn{r}) from finite rate(s) \eqn{\lambda}.
#' 
#' @param lambda single value or vector with lambda value(s).
#' @return single value or vector with r 
#' @author Frank PG Princee
#' @family census
#' @keywords internal
#' 
census.r <- function(lambda)
{       
  r <- log(lambda)
  r[is.infinite(r)] <- NA
  return(r)
}


###
#' @title Growth rates
#'
#' @description \code{census.growth} calculates finite (\eqn{\lambda}) and 
#' instantaneous rates \eqn{r} of change from census.data.
#' @param census.data Census data
#' @param group male or female as text
#' @author Frank PG Princee
#' @family census
#' @keywords internal
#' 
census.growth <- function(census.data,group)
{
  census.interval <- studbook.get("census.interval")
  census.interval <- firstCap(census.interval)
  
  main_title <- paste("Finite and instantaneous rates of change (",group,")")
  studbook.screenTitle(main_title)
  
  
  lambda  <- census.lambda(census.data)
  geomean <- census.geometricMean(lambda)
  r       <- census.r(lambda)    
  
  growth.data           <- data.frame(census.data,lambda,geomean,r)
  colnames(growth.data) <- c('N','LAMBDA','GEOMEAN','GEOMEAN_LWR','GEOMEAN_UPR','r')
  growth.data           <- ts(growth.data,start=start(census.data),frequency=1)
  
  print(growth.data)
  cat("\nSummary\n")
  print(summary(growth.data[,c('LAMBDA','r')]))
  cat("\n")
  
  
  if (!studbook.plot()) {
    return(NULL)  
  }
  
  limits.y <- range(growth.data[,'LAMBDA'],na.rm=TRUE)
  
  if (limits.y[1] > 1.0) {
    limits.y[1] <- 1.0
  }
  
  if (limits.y[2] < 1.0) {
    limits.y[2] <- 1.0
  }
  
  ylow  <- min(growth.data[,c('LAMBDA','GEOMEAN_LWR')],na.rm=TRUE)
  yhigh <- max(growth.data[,c('LAMBDA','GEOMEAN_UPR')],na.rm=TRUE)
  
  n <- nrow(growth.data)
  growth.data[n,c("GEOMEAN_LWR","GEOMEAN_UPR")] <- c(NA,NA)   # Based on two observations 

  studbook.openOutput('growth')
  main_title <- paste("Finite rates of change (",group,")")
  comment <- "Dashed line is geometric mean."
  
  studbook.caption(main_title,comment)

  plot(growth.data[,'LAMBDA'],
       xlab=census.interval,ylab=expression(lambda),ylim=c(ylow,yhigh), 
       type='l',col='black',lwd=1,lty='solid',main='')
  points(growth.data[,'LAMBDA'],pch=17,cex=0.8)
  
  lines(growth.data[,'GEOMEAN'],lty='dashed',lwd=2)
  lines(growth.data[,'GEOMEAN_LWR'],lty='dashed',lwd=1)
  lines(growth.data[,'GEOMEAN_UPR'],lty='dashed',lwd=1)
  
  abline(h=1.0,lty='solid')
  studbook.title(main_title)
  
  limits.y <- range(growth.data[,'r'],na.rm=TRUE)

  if (limits.y[1] > 0.0) {
    limits.y[1] <- 0.0
  }
  
  if (limits.y[2] < 0.0) {
    limits.y[2] <- 0.0
  }
  
  
  main_title <- paste("Instantaneous rate of change (",group,")")
  comment <- ""
  studbook.caption(main_title,comment)
  
  #ylim=limits.y,
  plot(growth.data[,'r'],
       xlab=census.interval,ylab=expression(r), 
       type='l',col='black',lwd=1,lty='solid',main='')
  abline(h=0.0,lty='solid')
  studbook.title(main_title)    
  
  studbook.closeOutput()
}


###
#' @title Calculate rates between census and births or deaths
#' 
#' @description \code{census.rate} is a generic function to calculate census
#' related rates e.g. birth rate, death rate. 
#' 
#' @param census.rate ts  Time series with columns \code{year, value1, value2}
#' @param main_title text Title of plot 
#' @param graph_name text File name of graph (default rate) 
#' @author Frank PG Princee
#' @family census
#' @keywords internal
#' 
census.rate <- function(census.ts,main_title="",graph_name="rate")
{
  census.interval <- studbook.get("census.interval")
  census.interval <- firstCap(census.interval)
  
  studbook.screenTitle(main_title)
  column <- colnames(census.ts)
  
  rate <- rep(NA,length(census.ts))
  rate <- census.ts[,1] / census.ts[,2] 
  rate[is.infinite(rate)] <- NA
  
  geomean <- census.geometricMean(rate)
  
  rates.ts <- data.frame(census.ts,rate,geomean[1:3])
  colnames(rates.ts) <- c(column,'RATE','GEOMEAN','GEOMEAN_LWR','GEOMEAN_UPR')
  rates.ts <- ts(rates.ts,start=start(census.ts),frequency=1)
  
  print(rates.ts)
  cat('\n')
  
  rates.ts <- na.contiguous(rates.ts)  #Get longest stretch without NAs
  
  rate_label <- paste(column[1],"per",column[2])
  rate_label <- tolower(rate_label)
  rate_label <- chartr("_"," ",rate_label)
  rate_label <- firstCap(rate_label)
  
  studbook.summary(rates.ts[,'RATE'],long=TRUE,CV=TRUE,title=rate_label)
  
  #Use weighted against census size as this has stochastic effect on rates
  w <- length(column) 
  values.mean(rates.ts[,'RATE'], rates.ts[,w],column[w])
  
  if (studbook.plot()) {
    n <- nrow(rates.ts)
    rates.ts[n,c("GEOMEAN_LWR","GEOMEAN_UPR")] <- c(NA,NA)   # Based on two observations 
    
    
    ylow  <- min(rates.ts[,c('RATE','GEOMEAN_LWR')],na.rm=TRUE)
    yhigh <- max(rates.ts[,c('RATE','GEOMEAN_UPR')],na.rm=TRUE)
    studbook.openOutput(paste(graph_name,sep='-'))
    caption <- studbook.caption(main_title,"Dashed line is geometric mean.")   
    plot(rates.ts[,'RATE'], xlab=census.interval,ylab=rate_label, 
         ylim=c(ylow,yhigh),
         main="",type='l',col='black',lwd=1,lty='solid')
    
    points(rates.ts[,'RATE'],pch=17,cex=0.8)
    lines(rates.ts[,'GEOMEAN'],lty='dashed',lwd=2)
    lines(rates.ts[,'GEOMEAN_LWR'],lty='dashed',lwd=1)
    lines(rates.ts[,'GEOMEAN_UPR'],lty='dashed',lwd=1)
    studbook.title(main_title) 
    studbook.closeOutput()
  }
}


###
#' @title Run test for annual differences in sex-ratio
#' @details Uses runs test as implemented in package \emph{tseries}.
#' @param census.ts Time series of census.data
#' @param census_type Text with name of census type (e.g. births)
#' @param alpha Significance level (default: 0.05)
#' @author Frank PG Princee
#' @family census
#' @keywords internal
#' 
sexratio.runsTest <- function(census.ts,census_type,alpha=0.05)
{
  cat("\n")
  studbook.screenTitle("Runs test (tseries")
  
  years       <- nrow(census.ts)
  newcol      <- rep(NA,years)
  census.run  <- data.frame("RATIO"=newcol,"RUN"=newcol,"P"=newcol)
  census.run  <- ts(census.run,start=start(census.ts),frequency=1)
  
  runs.text <- ""
  
  for (year in 1:years) {
    # There seem to be some issues with counts written as character
    census.ts[year,'FEMALES'] <- as.integer(census.ts[year,"FEMALES"])
    census.ts[year,'MALES']   <- as.integer(census.ts[year,"MALES"])
    census.run[year,'RATIO']  <- census.ts[year,'FEMALES'] / (census.ts[year,'FEMALES'] + census.ts[year,'MALES'])
    
    if (is.nan(census.run[year,'RATIO'])) {  # Skip 
      next
    }
    
    # Test whether ratio differs significantly from equal
    n <- sum(census.ts[year,c('MALES','FEMALES')])
    rs <- binom.test(x=census.ts[year,c('FEMALES','MALES')],n=n,p=1/2)
    census.run[year,'P'] <- rs$p.value
    
    if (census.run[year,'P'] >= 0.95) { # Male and female counts do not differ 
      census.ts[year,'FEMALES'] <- NA
      census.ts[year,'MALES']   <- NA
      runs.text <- paste(runs.text,".",sep="")
      next
    }       
    
    if (census.ts[year,'FEMALES'] >= census.ts[year,'MALES']) {
      census.ts[year,'FEMALES'] <- 0.55
      census.ts[year,'MALES']   <- NA 
      census.run[year,'RUN']     <- 'F'    
    } else {  
      census.ts[year,'FEMALES'] <- NA
      census.ts[year,'MALES']   <- 0.45
      census.run[year,'RUN']     <- 'M'
    }
    runs.text <- paste(runs.text,census.run[year,'RUN'],sep="")
    
  }
  runs <- as.vector(census.run[,'RUN'])
  runs <- na.omit(runs)
  cat("sex-ratio runs: [",runs.text,"]\n",sep="")
  cat(". = equal")
  rs1 <- tseries::runs.test(as.factor(runs))
  print(rs1)
  cat("H0: sequences are not random is ")
  cat(ifelse(rs1$p.value < alpha,"rejected","not rejected"),"\n") 
  
  if (studbook.plot()) {
    studbook.openOutput( paste(census_type,'runtest',sep='-') )
    
    main_title <- paste('sex-ratio sequence at',census_type,sep=' ')
    studbook.caption(main_title,"Check for non-random sequences.")   
    
    plot(census.ts[,'FEMALES'],type = "p",pch="F", ylim = c(0,1), 
         yaxt='n', xlab = "Date", ylab="")
    points(census.ts[,'MALES'], pch = "M")
    abline(h = 0.5,lty=3,lwd=2)
    studbook.title(main_title)
    studbook.closeOutput()
  }
  
}



###
#' @title Calculate sex-ratio in census-style data.
#' 
#' @param census.ts Time series with census data of males and females
#' @param census_type text with type of census e.g. census, births
#' @param randomness Test distribution pattern of sex-ratio 
#' @note Sex-ratio is expressed as proportion of females (Caughly, 1977)
#' @author Frank PG Princee
#' @family census
#' @keywords internal
#' 
census.sexratio <- function(census.ts,census_type,randomness=FALSE)
{
  census.interval <- studbook.get("census.interval")
  census.interval <- firstCap(census.interval)
  
  studbook.screenTitle(paste('Sex-ratio at',census_type))
  
  male.total   <- sum(census.ts[,'MALES'])
  female.total <- sum(census.ts[,'FEMALES'])
  equal.sex    <- (male.total + female.total) / 2

  #Extract longest contiguous block from geometric means
  census.ts <- na.contiguous(census.ts) 
  cat('\n')
  
  studbook.summary(census.ts[,'SEXRATIO'],long=TRUE,CV=FALSE,'Sex-ratio')
  
  #Use weighted as sample size has stochastic effect on sex-ratio
  values.mean(census.ts[,'SEXRATIO'], census.ts[,'FEMALES'] + census.ts[,'MALES'],'CENSUS')
  
  cat('\n')
  cat("Total number of males  : ",male.total,'\n')
  cat("Total number of females: ",female.total,'\n')
  cat("Proportion of females  : ",female.total/(male.total + female.total),'\n')
  
  rs <- binom.test(x=c(male.total,female.total),p=1/2)                          #p = equal sex-ratio
  cat("\n")
  cat("Binomial test over totals:\n")
  cat("--------------------------------------------------\n")
  cat("p =",rs$p.value,"\n")
  cat("H0: Equal sex-ratio is ")
  cat(ifelse(rs$p.value < 0.05,"rejected","not rejected"))
  cat("\n")
  
  limits.y <- range(census.ts[,'SEXRATIO'],na.rm=TRUE)
  if (limits.y[1] > 0.5) {
    limits.y[1] <- 0.5}
  
  if (limits.y[2] < 0.5) {
    limits.y[2] <- 0.5
  }
  
  if (studbook.plot()) {
    studbook.openOutput( paste(census_type,'sexratio',sep='-') )
    main_title <- paste('sex-ratio at',census_type,sep=' ')
    studbook.caption(main_title)                  
    plot(census.ts[,'SEXRATIO'],ylim=limits.y, 
         xlab=census.interval,ylab="Proportion of females", main="",
         type='l',col='black',lwd=1,lty='solid')
    studbook.title(main_title) 
    abline(h=0.5,lty='solid')
    
    studbook.closeOutput()
  }
  
  #Runs test 
  if (randomness) {
    sexratio.runsTest(census.ts[,c('FEMALES','MALES')],census_type)
  }        
  
}


