###
#' @title Contingency two-table test
#' 
#' @description \code{studbook.twotable} carries out Fisher`s or Pearson's chi-square test. 
#' @details The following criteria for sample sizes are used to determine which test is approperiate:
#' \itemize{
#' \item N <=20 and samples 20 < N <= 40 with lowest sample < 5: Fisher's test.
#' \item 20 < N <= 40 with lowest sample >= 5: Pearson`s Chi square with no continuity correction.
#' \item N > 40:  Pearson`s Chi square with continuity correction.
#' }
#' Order in data_set : by row: i.e. groupI+, groupI-, groupII+, groupII-
#' 
#' Order in name_list: \code{list("Group"=c("I","II"),"Variable"=("+","-"),)}
#' 
#'  @param data_set Data
#'  @param name_list list of groups and variables
#'  @param main_title Main title
#'  @param alpha significance level (default 0.05)
#'  @param hypothesis one or two sided
#'  @author Frank PG Princee
#'  @seealso \code{\link{accept.X2}} 
#'  @references Siegel S. and Castellan, Jr, N.J. (1988) \emph{Nonparametric Statistics for the Behavioural Sciences}. Second edition. 
#'  New York: McGraw-Hill. Page 399.
#'  @export
#'  
studbook.twotable <- function(data_set,name_list=NULL,main_title="Same proportion",alpha=0.05,hypothesis='two.sided')
{
  N     <- sum(data_set)
  N.low <- min(data_set)
  
  if (is.null(name_list) ) {
    name_list <- list("Group"=c("I","II"),"Variable"=c("+","-"))
  }
  table2x2 <- matrix(data_set,nrow=2,ncol=2,byrow=FALSE,dimnames=name_list)
  
  if (N <= 20) {
    result <- fisher.test(table2x2,alternative=hypothesis)
  } else if (N %in% 21:40) {                                                      #Siegler and N.J. Castellan, Jr. (1988)
    if (N.low < 5) {
      result <- fisher.test(table2x2,alternative=hypothesis)
    } else {
      result <- prop.test(table2x2,correct=FALSE)
    }
  } else {
    result <- prop.test(table2x2,correct=TRUE)       
  }
  
  cat("----------------------------------------------------\n")
  print(table2x2)
  print(result)
  accept.X2(main_title,result$p.value,alpha)  
}


###
#' @title Accept null hypothesis
#' 
#' @description \code{accept.X2} checks whether the chi-square null hypothesis is accepted.
#' 
#' @details The null hypothesis (H0) is: \code{samples are from same distribution}
#' @param H0 Text with null hypothesis
#' @param X2p estimated chi square  
#' @param alpha significance level (default 0.05)
#' @author Frank PG Princee
#' @note Make sure to pass 1 - pchisq()
#' @keywords internal
#' 
accept.X2 <- function(H0="H0: Samples are from the same distribution", X2p,alpha=0.05)
{
  cat(H0,"=> ")
  if (X2p > alpha) {   
    cat("not rejected\n")
  }
  else {
    cat("rejected\n")
  }
  cat("[alpha = ",alpha,"]\n")
}


###
#' @title Contingency test
#' @description Contingency test with two samples
# 
#' @param samples Contingency tables with two samples
#' @param main_title Title
#' @param alpha criterium
#' @author Frank PG Princee
#' @export
#' 
studbook.X2 <- function(samples,main_title,alpha=0.05)
{
  X2 <- chisq.test(samples)
  cat("Chi-square:",main_title,"\n")
  cat("---------------------------------------------------------------------\n")
  print(samples)
  cat("---------------------------------------------------------------------\n")
  print(X2)

  if (is.nan(X2$statistic)) {
    stop("Error in computing Chi-Square: check data for potential expected values of 0.")
  }

  cat("X2 = ", X2$statistic, ",  p = ",X2$p.value," (df = ",X2$parameter,")\n")
  accept.X2(X2p=X2$p.value,alpha=alpha)
  cat("\n")
}                          


###
#' @title Chi square log rank test for testing survival
#'   
#' @details Numbers at risk (N) and number of observations (O).
#' 
#' \code{X2 = ( (O1 - E1)^2/E1) + ((O2 -E2)^2 / E2)}
#' Degrees of freedom is 1!   
#' @param samples matrix with columns N1,O1,N2,O2
#' @param main_title Title
#' @param alpha criterium (default=0.05) 
#' @author Frank PG Princee 
#' @references V. Bewick, L. Cheek, and J. Ball. Statistics review 12: Survival analysis. 
#' Critical Care, 8(5):389–394, 2004.
#' 
#' @examples
#' samples <- matrix(ncol=5,nrow=12)
#' colnames(samples) <-c('class','N1','O1','N2','O2')
#' samples[1,]  <- c(1, 7,0, 8,2)
#' samples[2,]  <- c(4, 7,0, 6,1)
#' samples[3,]  <- c(5, 7,0, 5,1)
#' samples[4,]  <- c(6, 7,0, 4,0)
#' samples[4,]  <- c(8, 7,1, 3,0)
#' samples[5,]  <- c(9, 6,0, 3,1)
#' samples[6,]  <- c(12,6,1, 1,0)
#' samples[7,]  <- c(15,6,0, 1,0)
#' samples[8,]  <- c(22,4,0, 1,1)
#' samples[9,]  <- c(25,4,0, 0,0)
#' samples[10,] <- c(37,3,1, 0,0)
#' samples[11,] <- c(55,2,1, 0,0)
#' samples[12,] <- c(72,1,0, 0,0)
#' comment(samples) <- "Test data in Bewick et al. 2004"
#' attr(samples,"X2") <-  4.5
#' attr(samples,"p")  <-  0.032
#' studbook.logRank(samples,"Bewick et al.,2004")
#' @export
#'
studbook.logRank <- function(samples,main_title="",alpha=0.05)
{
  classes   <- length(samples[,1])
  
  for (c in 1:classes) {
    if (is.na(samples[c,'N1']) ) samples[c,'N1'] <- 0
    if (is.na(samples[c,'O1']) ) samples[c,'O1'] <- 0
    if (is.na(samples[c,'N2']) ) samples[c,'N2'] <- 0
    if (is.na(samples[c,'O2']) ) samples[c,'O2'] <- 0
  }
  
  chi_data  <- matrix(ncol=3,nrow=classes)
  colnames(chi_data) <-c('N','O','E2')
  
  
  #Nj = N1j + N2j
  chi_data[,"N"]  <- samples[,"N1"] + samples[,"N2"]
  #Oj = O1j + O2j
  chi_data[,"O"]  <- samples[,"O1"] + samples[,"O2"]
  
  #Calculate expected values for second group
  #E2j = (Oj / Nj) * N2j
  chi_data[,"E2"] <- (chi_data[,"O"] / chi_data[,"N"]) * samples[,"N2"]
  
  O2 <- sum(samples[,"O2"])
  E2 <- sum(chi_data[,"E2"])
  O1 <- sum(samples[,"O1"])
  E1 <- sum(chi_data[,"O"]) - E2
  
  X2 <- ( ( O1 - E1)^2 / E1 ) + ( (O2 -E2)^2 / E2 )
  
  if (is.nan(X2)) {
    stop("Error in computing Chi-Square: check data for potential expected values of 0.")
  }
  
  cat("\n")
  studbook.screenSection(paste("Logrank test:",main_title))
  print(samples)
  cat("---------------------------------------------------------------\n")
  pc <- 1.0 - pchisq(X2,1)
  cat("X2 = ", X2, ",  p = ",pc," (df = 1)\n")
  accept.X2(X2p=pc,alpha=alpha)
  cat("\n")
}


###
#' @title Test whether sample variance differs from parametric variance
#' @details $H_0: sample var = population var for normal distributions.
#' @param sample.var double Sampling variance
#' @param n integer Sample size
#' @param pop.var double Parametric variance
#' @param alpha double Significance level (default: 0.05)
#' @param do_print logical Print results (default:TRUE)
#' @return TRUE when not rejected
#' @references Sokal and Rohlf, 2012
#' @export
#' 
popvar.test <- function(sample.var,n,pop.var,alpha=0.05,do_print=TRUE)
{
  
  if (is.nan(sample.var) | is.nan(pop.var)) { 
    return(NA)
  }
  
  if (sample.var == 0 || pop.var == 0) {
    return(NA)
  }
  
  
  X2 <- (n-1)* sample.var/pop.var
  
  X2_lower <- qchisq(alpha/2,n-1)
  X2_upper <- qchisq(1-alpha/2,n-1)
  if (X2 >= X2_lower & X2 <= X2_upper) {
    result <- TRUE
  } else {
    result <- FALSE
  }
  
  if (do_print) {
    cat("X2 = ",X2,"\n",sep="")
    cat("X2[",alpha/2,",",n-1,"] = ",X2_lower,"\n",sep="")
    cat("X2[",1-(alpha/2),",",n-1,"] = ",X2_upper,"\n",sep="")
    cat("H0: sample variance does not differ from parametric variance is ")
    cat(ifelse(result,"not rejected","rejected"),"\n")
  }
  return(result)
}


