###  WORLDCLIM
# Functions to handle data from Worldclim raster files
#  www.worldclim.org
#  Resolutions: 
#  -   30 seconds   (0.93 x 0.93) =   0.86 km2 ~ 1
#  -   2.5 minutes  (1.85 x 1.85) =   3.42 km2 ~ 5
#  -   5.0 minutes  (3.70 x 3.70) =  13.7 km2 ~ 10
#  -  10.0 minutes  (18.6 x 18.6) = 346 km2
#


###
# Worldclim variables: bio1-19, tmean,tmax,prec, and alt 

.defaultWorldClim <- function()
{
  
  assign("worldclim.files",list("bio1" ="Mean annual temperature", 
                              "bio2" ="Mean diurnal range (mean max temp - min temp)",
                              "bio3" ="Isothermality (bio2/bio7) * 100",
                              "bio4" ="Temperature seasonality (standard deviation * 100)",
                              "bio5" ="Max temperature of warmest month",
                              "bio6" ="Min temperature of coldest month",
                              "bio7" ="Temperature annual range (bio5-bio6)",                     
                              "bio8" ="Mean temperature of wettest quarter",
                              "bio9" ="Mean temperature of driest quarter",
                              "bio10"="Mean temperature of warmest quarter",
                              "bio11"="Mean temperature of coldest quarter",
                              "bio12"="Total (annual) precipitation",
                              "bio13"="Precipitation of wettest month",
                              "bio14"="Precipitation of driest month",
                              "bio15"="Precipitation seasonality (coefficient of variation)",
                              "bio16"="Precipitation of wettest quarter",
                              "bio17"="Precipitation of driest quarter",
                              "bio18"="Precipitation of warmest quarter",
                              "bio19"="Precipitation of coldest quarter",
                              "tmean"="Mean monthly temperature",
                              "tmax" ="Maximum monthly temperature",
                              "tmin" ="Minimum monthly temperature",
                              "prec" ="Monthly precipitation",
                              "alt"  ="altitude"),
          .StudbookEnv)
  
  
  # World clim path 2.5 arc ~ 5km^2 data 
  # note: getData creates wc2-5 subdir)
  assign("worldclim.path","~/pml/gis/climate/wc2-5",.StudbookEnv)   # Wordclim 2.5m  files
}


###
#' @title Set path to worldclim files
#' @param path Name of path 
#' @author Frank PG Princee
#' @export
#' 
worldclim.setPath <- function(path)
{
  assign("worldclim.path",path,.StudbookEnv)
}

###
#' @title Import worldclim raster
#' @description \code{worldclim.load} creates a raster object from a "worldclim" file
#' 
#' @details The following climate files are available: 
#' \describe{
#' \item{bio1}{Mean annual temperature} 
#' \item{bio2}{Mean diurnal range (mean max temp - min temp)}
#' \item{bio3}{Isothermality ( (bio2/bio7) * 100 ) }
#' \item{bio4}{Temperature seasonality (standard deviation * 100)}
#' \item{bio5}{Max temperature of warmest month}
#' \item{bio6}{Min temperature of coldest month}
#' \item{bio7}{Temperature annual range (bio5-bio6)}                     
#' \item{bio8}{Mean temperature of wettest quarter}
#' \item{bio9}{Mean temperature of driest quarter}
#' \item{bio10}{Mean temperature of warmest quarter}
#' \item{bio11}{Mean temperature of coldest quarter}
#' \item{bio12}{Total (annual) precipitation}
#' \item{bio13}{Precipitation of wettest month}
#' \item{bio14}{Precipitation of driest month}
#' \item{bio15}{Precipitation seasonality (coefficient of variation)}
#' \item{bio16}{Precipitation of wettest quarter}
#' \item{bio17}{Precipitation of driest quarter}
#' \item{bio18}{Precipitation of warmest quarter}
#' \item{bio19}{Precipitation of coldest quarter}
#' \item{tmean}{Mean monthly temperature}
#' \item{tmax}{Maximum monthly temperature}
#' \item{tmin}{Minimum monthly temperature}
#' \item{prec}{Monthly precipitation}
#' \item{alt}{Altitude}
#' }
#' The full path name to the bioclim files is stored in the global variable
#' \code{bioclim.path}. The raster data are stored in the global variable 
#' \code{studbook.raster}.
#' 
#' @param field name of worldclim file without extension (default "bio1")
#' @author Frank PG Princee
#' @seealso \code{\link{worldclim.setPath}}
#' @export
#' 
worldclim.load <- function(field="bio1",month=1) 
{
   studbook.screenTitle(get("worldclim.files",.StudbookEnv)[[field]])
   
   # Get climate parameters
   month_fields <- c("tmean","tmin","tmax","prec")
   if (is.element(field,month_fields)) {
     field <- paste0(field,month)
   }
  
   worldclim.file <- paste0(get("worldclim.path",.StudbookEnv),"/",field,".bil")
   if (file.exists(worldclim.file)) {
       raster.load(worldclim.file)   
   } else {
       stop(paste(worldclim.file,"does not exist. Download data using raster::getData()"))
   }
   
   
   # Detemine palette
   
   # Precipitation
   ppt_fields <- c('bio12','bio13','bio14','bio15','bio16','bio17','bio18','bio19','prec')
   if (is.element(field,ppt_fields) ) {
     create.palette(9,'Blues')
   } else if (field == 'alt') {
     create.palette(9,'YlOrBr') 
   } else {
     create.palette(11,'RdBu',reverse=TRUE) #reverse
   }
   
   
   
   # Temperature
   tmp_fields <- c("bio1","bio5","bio6","bio8","bio9","bio10","bio11")
   if (is.element(field,tmp_fields)) {
     fun <- function(x){x/10}
     raster.modify(fun)
   }
}


###
#' @title Plot active bioclim data
#' @author Frank PG Princee
#' @export
#' 
worldclim.plot <- function()
{
   field <- raster.names()
   print(field)
   raster.plot(get("worldclim.files",.StudbookEnv)[[field[1]]])
}


###
#' @title Update sites frame with bioclim data
#' @param field Short name of both bioclim raster and sites column (default: "bio1")
#' @param max_delta Number of steps to extract nearest value (default: 10)
#' @author Frank PG Princee
#' 
#' @export
#' 
sites.worldclim <- function(field='bio1',max_delta=10)
{
  worldclim.load(field) 
  sites.update(field,max_delta)
}



###
# Koeppen-Geiger climate zones in Peel et al.(2007)
# Codes in vector order of raster values

.defaultKoeppen <- function()
{
  assign("koeppen.zone", c("Af","Am","Aw","BWh","BWk","BSh","BSk","Csa","Csb","Csc",
                           "Cwa","Cwb","Cwc","Cfa","Cfb","Cfc","Dsa","Dsb","Dsc","Dsd",
                           "Dwa","Dwb","Dwc","Dwd","Dfa","Dfb","Dfc","Dfd","ET","EF"),
         .StudbookEnv)
  
  
  assign("koeppen.palette",c("#0000FF","#0078FF","#46AAFA","#FF0000","#FF9696","#F5A500",
                             "#FFDC64","#FFFF00","#C8C800","#969600","#96FF96","#64C864",
                             "#329632","#C8FF50","#64FF50","#32C800","#FF00FF","#C800C8",
                             "#963296","#966496","#AAAFFF","#5A78DC","#4B50B4","#320087",
                             "#00FFFF","#37C8FF","#007D7D","#00465F","#B3B3B3","#666666"),
         .StudbookEnv)
  
  assign("koeppen.selected",1:30,.StudbookEnv)
  assign("koeppen.file","~/pml/climate/koeppen/w001001.adf",.StudbookEnv)
}


###
#' @title Set Koeppen climate file
#' @param file_name Full path name of file
#' @author Frank PG Princee
#' @family Koeppen climate
#' @export
#' 
koeppen.setFile <- function(file_name="")
{
  assign("koeppen.file",file_name,.StudbookEnv)
}

###
#' @title Select Koeppen climate zones
#' 
#' @description \code{koeppen.select} sets the Koeppen-Geiger climatic zones 
#' to be included. List according to Peel et al. (2007).
#' @details The numerical values of thermal climatic zones are entered as a
#' numerical sequence in the range 1-x. The following zones are recognized:
#' \enumerate{
#' \item  1. Af  : Rainforest
#' \item  2. Am  : Monsoon
#' \item  3. Aw  : Savannah
#' \item  4. BWh : Hot desert
#' \item  5. BWk : Cold desert 
#' \item  6. BSh : Hot steppe
#' \item  7. BSk : Cold steppe
#' \item  8. Csa : Temperate dry and hot summer
#' \item  9. Csb : Temperate dry and warm summer 
#' \item 10. Csc : Temperate dry and cold summer
#' \item 11. Cwa : Temperate dry winter and hot summer
#' \item 12. Cwb : Temperate dry winter and warm summer
#' \item 13. Cwc : Temperate dry winter and cold summer
#' \item 14. Cfa : Temperate no dry season and hot summer
#' \item 15. Cfb : Temperate no dry season and warm summer
#' \item 16. Cfc : Temperate no dry season and cold summer
#' \item 17. Dsa : Cold dry and hot summer
#' \item 18. Dsb : Cold dry and warm summer
#' \item 19. Dsc : Cold dry and cold summer
#' \item 20. Dsd : Cold dry summer and very cold winter 
#' \item 21. Dwa : Cold dry winter and hot summer
#' \item 22. Dwb : Cold dry winter and warm summer
#' \item 23. Dwc : Cold dry winter and cold summer
#' \item 24. Dwd : Cold dry and very cold winter
#' \item 25. Dfa : Cold no dry season and hot summer
#' \item 26. Dfb : Cold no dry season and warm summer
#' \item 27. Dfc : Cold no dry season and cold summer
#' \item 28. Dfd : Cold no sry season and very cold winter
#' \item 29. ET  : Tundra
#' \item 30. EF  : Frost
#' }
#' 
#'The selected range is stored in the global variable \code{climate.zones}.
#'
#'@param zones Numeric value(s) or name(s) of zones (default 1:30) 
#'@note The rare zone \code{As} is not included in the data map of Peel et al. 
#'
#'@author Frank PG Princee
#'@family Koeppen climate
#'@export
#' 
koeppen.select <- function(zones=1:30) 
{
  if (zones[1] == "?") {
    print(get("koeppen.zone",.StudbookEnv))
  } else {
    assign("koeppen.selected",zones,.StudbookEnv)
  }
}

###
#' @title Zone is selected
#' @description Check whether zone is selected
#' @author Frank PG Princee
#' @seealso \code{\link{koeppen.select}}
#' @family Koeppen climate
#' @keywords internal
#' 
inKoeppen <- function(zone)
{
  return( is.element(zone,get("koeppen.selected",.StudbookEnv) ) )
}  


###
#' @title Load FAO thermal climate raster from file
#' @family Koeppen climate
#' @export

koeppen.load <- function()
{
  raster.load(get("koeppen.file",.StudbookEnv))
  create.palette(palette = get("koeppen.palette",.StudbookEnv))
}


###
#' @title Plot climate raster
#' @author Frank PG Princee
#' @family Koeppen climate
#' 
#' @export 
koeppen.plot <- function()
{
  raster.plot('Koeppen-Geiger climate zones')
}


###
#' @title Update sites frame with Koeppen-Geiger climate data
#' @param max_delta Number of steps to extract nearest value
#' @author Frank PG Princee
#' @family Koeppen climate
#' @export
#' 
sites.koeppen <- function(max_delta=10)
{
  koeppen.load() 
  sites.update('koeppen',max_delta)
}


###
# Functions to handle Universal Thermal Scale (Trewartha)
#


.defaultTrewartha <- function()
{
  assign("thermal.scale",c("severely hot","very hot","hot","warm","mild","cool",
                           "cold","very cold","severely cold","excessively cold"),
         .StudbookEnv)
}       

###
#' @title Determine universal thermal scale
#' @param temperature real Temperature
#' @details U
#' \enumerate{
#' \item i: severely hot >= 35
#' \item h: very hot [28,35)
#' \item a: hot [23,28)
#' \item b: warm [18,23)
#' \item l: mild [10,18)
#' \item k: cool (0,10)
#' \item o: cold (-10,0]
#' \item c: very cold (-25,-10]
#' \item d: severely cold (-40,-25]
#' \item e: excessively cold <= -40
#' }
#' @return scale 1-10 (see thermal.getscale)
#' @author Frank PG Princee
#' @export
#' 
thermal.getScale <- function(temperature)
{  
  if (temperature >= 35) {
    scale <- 1
  } else if (temperature >= 28 & temperature < 35) {
    scale <- 2
  } else if (temperature >= 23 & temperature < 28) {
    scale <- 3
  } else if (temperature >= 18 & temperature < 23) {
    scale <- 4
  } else if (temperature >= 10 & temperature < 18) {
    scale <- 5
  } else if (temperature > 0 & temperature < 10) {
    scale <- 6
  } else if (temperature > -10 & temperature <= 0) {
    scale <- 7
  } else if (temperature > -25 & temperature <= -10) {
    scale <- 8
  } else if (temperature > -40 & temperature <= -25) {
    scale <- 9
  } else if (temperature <= -40) {
    scale <- 10
  } else {
    scale <- 0 # will generate an error
  }
  return( get("thermal.scale",.StudbookEnv)[scale]) 
}


###
# Functions to handle FAO thermal climate zones
# Thermal climate zones (FAO) 
# Global Agro--ecological Assessment for Agriculture in the 21st Century: Methodology and Results
# Fisher et al. 2002)
#


.defaultThermal <- function()
{
  assign("thermal.zone",c("Tropics","Subtropics (summer rain)","Subtropics (winter rain)",
                          "Temperate (oceanic)","Temperate (sub-continental)","Temperate (continental)",
                          "Boreal (oceanic)","Boreal (sub-continental)","Boreal (continental)",
                          "Artic","Inland water bodies"),.StudbookEnv)
  
  assign("thermal.palette",c("#FF0000","#FFAA00","#FFD37E","#D4FFBD","#55FF00","#3AFF00",
                             "#FFBEE7","#00C5FF","#004DA9","#B2B2B2","#BEEAFF"), .StudbookEnv)
  
  
  assign("thermal.selected",1:8,.StudbookEnv)
  assign("thermal.file","~/pml/climate/thermal/w001001.adf",.StudbookEnv)
}

###
#' @title Set thermal climate file
#' @param file_name Full path name of file
#' @author Frank PG Princee
#' @family thermal climate
#' @export
#' 
thermal.setFile <- function(file_name="")
{
  assign("thermal.file",file_name,.StudbookEnv)
}

###
#' @title Select thermal climate zones
#' 
#' @description \code{climate.select} sets the thermal climatic zones to be included
#' @details The numerical values of thermal climatic zones are entered as a
#' numerical sequence in the range 1-8. The following zones are recognized:
#' \enumerate{
#' \item  1. Tropics
#' \item  2. Subtropics (summer rain)
#' \item  3. Subtropics (winter rain)
#' \item  4. Temperate (oceanic)
#' \item  5. Temperate (sub-continental)
#' \item  6. Temperate (continental)
#' \item  7. Boreal (oceanic)
#' \item  8. Boreal (sub-continental)
#' \item  9. Boreal (continental)
#' \item 10. Artic
#' \item 11. Inland water bodies
#'}
#'The selected range is stored in variable \code{selected} of environment \code{.StudbookEnv}.
#'
#' @param zones Numerical value of zones (default 1:8) 
#' @author Frank PG Princee#' 
#' @family thermal climate
#' @export
#' 
thermal.select <- function(zones=1:8) 
{
  if (zones[1] == "?") {
    print(get("thermal.zone",.StudbookEnv) )   
  } else {
    assign("thermal.selected",zones,.StudbookEnv)
  }
}

###
#' @title Check whether thermal zone is selected
#' @description \code{inThermal} checks whether thermal zone is selected
#' @author Frank PG Princee
#' @family thermal climate
#' @keywords internal
#' 
inThermal <- function(zone)
{
  return( is.element(zone,get("thermal.selected",.StudbookEnv) ) )
}  


### 
#' @title Load FAO thermal climate raster from file
#' @note The FAO thermal climate raster mat need to be downloaded from
#' @family thermal climate
#' @export
#' 
thermal.load <- function()
{
  raster.load(get("thermal.file",.StudbookEnv))
  create.palette(palette = get("thermal.palette",.StudbookEnv))
}

###
#' @title Plot thermal climate raster
#' @author Frank PG Princee
#' @family thermal climate
#' @export 
#' 
thermal.plot <- function()
{
  raster.plot('Thermal climate zones')
}


###
#' @title Update sites frame with Thermal climate data
#' @param max_delta Number of steps to extract nearest value
#' @author Frank PG Princee
#' @family thermal climate
#' @export
#' 
sites.thermal <- function(max_delta=10)
{
  thermal.load() 
  sites.update('thermal',max_delta)
}











