###
#' @title Estimate hazard ratio for partial unit
#' @param beta.coef Regression coefficient for covariate
#' @param i unit increase; can be fractions (default: 1)
#' @return adjusted HR for partial or multiple units
#' @author Frank Princee
#' @family Cox regression
#' @keywords internal
#' 
HR.unit <- function(beta.coef,i=1)
{
  HR <- exp(beta.coef  * i)
  return(HR)
}


###
#' @title Estimate survival rate from hazard ratio
#' @param S[x] Survival rate for unit x (at time t)
#' @param HR hazard ratio (default: NA)
#' @inheritParams HR.unit
#' @return S[x+i] Survival rate for x + i units (at time t)
#' @author Frank PG Princee
#' @family Cox regression
#' @keywords internal
#' 
Sx.from.HR <-  function(Sx,HR)
{
  return(Sx^HR)
}


###
#' @title Summary of Cox results
#' @param cox.rs coxph or coxme.object
#' @author Frank PG Princee
#' @family Cox regression
#' @keywords internal
#' 
cox.result <- function(cox.rs)
{
  print(summary(cox.rs))
  if (!is.element(class(cox.rs)[1],c("coxph.null","coxme")) ) {
     temp <- survival::cox.zph(cox.rs)
     cat("\nTherneau-Grambsch test for constant HR\n")
     print(temp)
    # plot(temp)
  }
  
  if (is.null(coef(cox.rs))) {
    resid.type <- 'martingale'
  } else {
    resid.type <- 'dfbeta'
  }
  
  #cox.resid <- resid(cox.rs,type="dfbeta")
  
  #if (is.null(dim(cox.resid))) {
  #  plot(cox.resid,ylab=names(coef(cox.rs)))
  #  abline(h=0, lty=2)
  #} else {
  #  n <- dim(dfbeta)[2]
  #  op <- par(par(mfrow=c(2, 2)))
  #  for (j in 1:n ) {
  #    plot(cox.resid[,j],ylab=names(coef(cox.rs))[j])
  #    abline(h=0,lty=2)   
  #  }
  #  par(op)
  #}
}


###
#' @title Create adjusted survival curves from Cox model
#' @details This function creates a new data frame that is used by the survfit.coxph function
#' in package survival. 
#' @param survival.data Data frame with raw data
#' @param cox.rs coxph.object
#' @param prediction vector with prediction values. When first element is "p" other 
#' elements are percentiles
#' @author Frank PG Princee
#' @family Cox regression
#' @keywords internal
#' 
cox.predict <- function(survival.data,cox.rs,prediction=NULL)
{
  
  if (class(cox.rs)[1] == "coxph.null") {
    cox.fit <- survival::survfit(cox.rs)
    print(cox.fit)
    return(cox.fit)
  }
  
  fields <- all.vars(cox.rs$formula)   
  field <- fields[2]  # First term in formula
  
  if (is.numeric(survival.data[,field])) {
   
    
    if (is.element("LITTER_ID",colnames(survival.data))) {
      if (!is.factor(survival.data$LITTER_ID)) {
        survival.data <- subset(survival.data,LITTER_ID > -1)
        survival.data$LITTER_ID <- as.factor(survival.data$LITTER_ID)
      }
      temp.data <- survival.data[unique(survival.data$LITTER_ID),field]
    } else {
      temp.data <- survival.data[,field]
    }
    
    if (is.null(prediction)) {
      percentiles <- quantile(temp.data,c(0.1,0.25,0.5,0.75,0.9),na.rm=TRUE)
      prediction <- as.numeric(percentiles)
      prediction <- unique(prediction)   # Avoid duplicated values
      
    } else if (prediction[1] == "p") {   # percentiles
      prediction[1] <- NA
      prediction <- na.omit(prediction)
      percentiles <- quantile(temp.data,as.numeric(prediction),na.rm=TRUE)
      prediction <- as.numeric(percentiles)
      prediction <- unique(prediction) # Avoid duplicated values
     
    } else {  
      prediction <- sort(prediction)
      field.range <- range(temp.data,na.rm=TRUE)
      if (min(prediction) < field.range[1]) {
        warning("requested prediction lower than observed")
      }
      if (max(prediction) > field.range[2]) {
        warning("requested prediction higher than observed")
      } 
    }
    
    
  } else {
    if (is.character(survival.data[,field])) {
       survival.data[,field] <- as.factor(survival.data[,field])
    }
    prediction <- sort(unique(survival.data[,field]))
  }    
  
  
  predict.frame <- data.frame(field = prediction)
  
  colnames(predict.frame) <- field
 
  cox.fit <- survival::survfit(cox.rs,newdata=predict.frame)

  digits.old <- options(digits=4) 
  
  colnames(cox.fit$surv) <- as.character(prediction)
  predictions <- data.frame(cox.fit$time,cox.fit$surv)
  colnames(predictions) <- c("age",as.character(prediction))
  
  cat("* Survival predictions\n")
  cat("Median survival\n")
  print(cox.fit)
  
  cat("\n")
  n <- nrow(predictions)
  cat("* First and last two age groups in predicted values\n")
  print(predictions[c(1:2,n-1,n),])
  cat("\n")  
  
  options(digits.old)
  return(cox.fit)
}


###
#' @title Cox Proportial Hazard Regression for survival and fecundity
#' @description \code{studbook.cox} estimates survival and fecundity using the 
#' Cox Proportial Hazard Regression model. 
#'  
#' @param survival.data Survival data
#' @param subset Subset of survival data
#' @param term string with fixed terms
#' @param stratum Field to stratify (default:NA)
#' @param cluster optional cluster 
#' @return survfit result
#' @author Frank Princee
#' @keywords internal
#' @family Con regression
#' @seealso \code{\link{studbook.survival}}
#' 
cox.regression <- function(survival.data,subset,term,stratum=NA,cluster="NONE")
{  
  survival.object <- with(survival.data, survival::Surv(TIME,EVENT))
  formula.cox <- paste("survival.object ~",term) 
  
  if (term == "1") {
    cluster <- "NONE"
  }
  
  if (cluster != "NONE") {
      cluster <- "NONE"
  }
  
  if (!is.na(stratum)) {
    formula.cox <- paste(formula.cox,"+ strata(",stratum,")")
  }
  
  if (is.recurrent(survival.data)) {
     formula.cox <- paste(formula.cox,"+ survival::cluster(ID)")
  } else {
    if (cluster != "NONE") {
       formula.cox <- paste(formula.cox,"+ survival::cluster(",cluster,")") 
    } 
  }
  
  cat("Formula  :",formula.cox,"\n\n")
  coxph.rs <- survival::coxph(as.formula(formula.cox),data=survival.data,subset=subset,
                              control=survival::coxph.control(eps=1e-11,iter.max=40))
  return(coxph.rs)    
}

