###
#' @title Annual deaths
#' @description \code{studbook.deaths} analyses deaths.
#' 
#' @details This function plots annual deaths in males, females and total population per year and
#' analyses differences in sex--ratio, and estimates and plots \emph{per capita death rates}
#' and birth/death rates.  
#' 
#' Assign \code{NULL} to arguments \code{first} and/or \code{last} to include the oldest and
#' most recent data, respecitively. 
#' 
#' @param first First year of death census (default = NULL) 
#' @param last Last year of death (default = NULL)
#' @param analyse options: \code{"sexratio"}, \code{"deathrate"} and/or 
#' \code{"birthdeath"} (default: all)
#' @author Frank PG Princee
#' @examples
#' \dontrun{
#' studbook.deaths()                     # all deaths in data-set
#' studbook.deaths(first=1977)           # deaths in 1977 and onwards
#' studbook.deaths(first=1977,last=1997) # deaths from 1977 until 1996 (20 years)
#' }
#' @family census
#' @export
#' 
studbook.deaths<-function(first=NULL,last=NULL,analyse=c('sexratio','deathrate','birthdeath')) 
{
  studbook.screenTitle('Census of deaths') 
  death.ts   <- census.read('death',first,last)
  
  if (is.null(death.ts)) {
    stop("No death census data")
  }
  analyse <- match.arg(analyse,several.ok=TRUE)
  print(death.ts)
  census.plot(death.ts,'deaths')
  census.sexratio(death.ts,'death')
  
  # Death and birth/death rates
  start_year <- start(death.ts)
  end_year   <- end(death.ts)
  
  census.ts  <- census.read('census',start_year[1],end_year[1])
  birth.ts   <- census.read('birth',start_year[1],end_year[1]) 
  
  if (is.element("deathrate",analyse)) {
    if (!is.null(census.ts)) { # Deaths/population = per capita death rate
      death_rate <- data.frame("DEATHS" = death.ts[,'ALL'],"CENSUS" = census.ts[,'ALL'])
      death_rate <- ts(death_rate,start=start_year,frequency=1)
      census.rate(death_rate,'Death rate','death_rate')
    }
  }
  
  if (is.element("birthdeath",analyse)) {
    #Birth/death rate
    if (!is.null(birth.ts)) {
      death_rate <- data.frame("BIRTHS" = birth.ts[,'ALL'],"DEATH" = death.ts[,'ALL'],
                               "CENSUS" = census.ts[,'ALL'])
      death_rate <- ts(death_rate,start=start_year,frequency=1)
      census.rate(death_rate,'Birth/death rate','birth_death_rate')           
    }
  }
  return(TRUE) 
}

