
.defaultDistribution <- function()
{
  assign("histogram",TRUE,.StudbookEnv)
  assign("barplot",FALSE,.StudbookEnv)
  assign("density",TRUE,.StudbookEnv)
  assign("ecdf",TRUE,.StudbookEnv)
  assign("qq",TRUE,.StudbookEnv)
  assign("dist.type","frequency",.StudbookEnv)                           
}

###
#' @title Select graph
#' @details Switches selected graphs on or off
#' \describe{
#' \item{histgram}{Plots histogram (counts) of data}
#' \item{barplot}{Plots barplot}
#' \item{density}{Plots density distribution of data}
#' \item{ecdf}{Emperical Cumulative Density Function of data}
#' \item{qq}{Quantile-Quantile plot of data}
#' \item{dist.type}{frequency,proportion or percentage in graphs}
#' }
#' 
#' @param flag TRUE to plot graph
#' @author Frank PG Princee
#' @family data distribution
#' @export
#' 
distribution.selectGraph <- function(name,value)
{
  assign(name,value,.StudbookEnv)
}    

###
#' @title Select or deselect all graph
#' @details Switches all types of graphs on or off
#' @param flag TRUE to plot graph
#' @author Frank PG Princee
#' @family data distribution
#' @export
#'
distribution.initGraphs <- function(flag)
{
  assign("histogram",flag,.StudbookEnv)
  assign("barplot",  flag,.StudbookEnv)
  assign("density",  flag,.StudbookEnv)
  assign("ecdf",     flag,.StudbookEnv)
  assign("qq",       flag,.StudbookEnv)            
}


###
#' @title Get graph selection#' 
#' @param name name of graph
#' @return TRUE or FALSE
#' @author Frank PG Princee
#' @family data distribution
#' @export
#' 
distribution.getGraph<-function(name)
{
  return(get(name,.StudbookEnv))
}


###
#' @title Select type of distribution 
#' @param name name of type \code{'frequency','percentage','proportional'} (default: frequency)
#' @author Frank PG Princee
#' @family data distribution
#' @export
#' 
distribution.setType <- function(name=c('frequency','percentage','proportional'))
{
  name <- match.arg(name)
  assign("dist.type",name,.StudbookEnv)
}


###
#' @title Gettype of distribution 
#' @return \code{'frequency','percentage','proportional'}
#' @author Frank PG Princee
#' @family data distribution
#' @export
#' 
distribution.getType <- function()
{
  return(get("dist.type",.StudbookEnv))
}

###
#' @title Get number of graphs
#' @author Frank PG Princee
#' @family data distribution
#' @keywords internal
#' 
distribution.countGraphs <- function()
{
  x <- 0
  x <- ifelse(get("histogram",.StudbookEnv),x + 1, x)
  x <- ifelse(get("barplot",  .StudbookEnv),x + 1, x)
  x <- ifelse(get("density",  .StudbookEnv),x + 1, x)
  x <- ifelse(get("ecdf",     .StudbookEnv),x + 1, x)
  x <- ifelse(get("qq",       .StudbookEnv),x + 1, x)
}




###
#' @title Plot data
#' @description \code{distribution.plot} presents distribution data in different graphs.
#' @author Frank PG Princee
#' @family data distribution
#' @keywords internal
#' 
distribution.plot <- function(data,main_title='Data',xlabel="X",bar_breaks="Sturges",
                              leftPoints=NA,rightPoints=NA,quantiles=c(0.25,0.5,0.75),
                              labels=FALSE)
{

  if (!studbook.plot()) {
    invisible(NULL)
  }
  mean <- mean(data)
  quantiles <- quantile(data,quantiles,na.rm=TRUE)

  # Plot histogram
  if (distribution.getGraph('histogram') ) {
    h<- hist(data,breaks=bar_breaks,plot=FALSE) 
    ylabel <- distribution.getType() 
    sub_title <- paste("N =",length(data))  
    if (ylabel == 'proportion') {
      h$counts <- h$counts / sum(h$counts)
    } else if (ylabel == 'percentage') {
      h$counts <- (h$counts / sum(h$counts)) * 100.0
    } 
    
    xlabel <- firstCap(xlabel)
    ylabel <- firstCap(ylabel)
   
    studbook.caption(main_title)
    
    plot(h,main="",xlab=xlabel,ylab=ylabel,border='black',
         col='grey60',density=50,angle=45,bty='o')
    studbook.title(main_title)     
  }
  
  # Barplot
  if (distribution.getGraph('barplot') ) {
    studbook.caption(main_title)
    display.table <- table(data)
    ylabel <- distribution.getType() 
    sub_title <- paste("N =",length(data))  
    
    if (ylabel == 'proportion') {
      display.table <- prop.table(display.table)
    } else if (ylabel == 'percentage') {
      display.table <- prop.table(display.table) * 100
    } else {
      display.table <- table(data)
    }
    
    xlabel <- firstCap(xlabel)
    ylabel <- firstCap(ylabel)
    
    barplot(display.table,border='black',col='grey60',density=50,angle=45,
            space=0,main="",xlab=xlabel,ylab=ylabel)
    studbook.title(main_title,sub_title) 
  }
  
  
  if (distribution.getGraph("density")) {
    #ESTIMATED DENSITY FUNCTION
    
    if (is.na(leftPoints)) {
      
      if (is.na(rightPoints)) {
        density_data <- density(data)
      } else {
        density_data <- density(data,to=rightPoints)
      }
      
    } else {
      if (is.na(rightPoints)) {
        density_data <- density(data,from=leftPoints)
      } else {
        density_data <- density(data,from=leftPoints,to=rightPoints)
      }
    }
    cat("N =",density_data$n,'bandwidth =',density_data$bw,'\n')
    rrd <- density_data
    print(sum(rrd$y[-1]*diff(rrd$x)))
    
    xlabel <- firstCap(xlabel)
    ylabel <- firstCap(ylabel)
    
    studbook.caption(main_title,"Density probability of data.") 
    plot(density_data,type='h',main="",xlab=xlabel,ylab='Density',
         col='grey80')
    lines(density_data)
   
    abline(v=mean,lty="dashed",lwd=1)  
    
    studbook.title(main_title=main_title) 
    if (!studbook.isOption('output','publish') ) {
      plot.mean(mean)
      plot.quantiles(quantiles)
      legend("topright","(x,y)",c("mean","percentiles"),lwd=2,
             lty=c('dashed','dotted'),bty='n')
    }
  }
  
  if (distribution.getGraph("ecdf")) {
    #EMPIRICAL CUMULATIVE DISTRIBUTION FUNCTION
    studbook.caption(main_title,"Empirical distribution.")
    xlabel <- firstCap(xlabel)
   
    plot(ecdf(data),pch=20,verticals=TRUE,main="",xlab=xlabel,
         ylab="Cumulative frequency")
    studbook.title(main_title)
    
    if (!studbook.isOption('output','publish')) {
      plot.mean(mean)
      plot.quantiles(quantiles)
      legend("topright","(x,y)",c("mean","percentiles"),lwd=2,
             lty=c('dashed','dotted'),bty='n')
    }
    
  }
  
  if (distribution.getGraph("qq")) {
    #NORMAL Q-Q PLOT
    studbook.caption(main_title,"Compare data with normal distribution.") 
    qqnorm(data,main="",pch=20)
    qqline(data,col="black",lty="dashed")
    studbook.title(main_title)
  }
  
}


###
###
#' @title Create stripchart of left and/or right truncated data
#' @description \code{distribution.exploreEnds} creates a stripchart with single or multiple
#' truncated copies of original data-set
#' @param data vector with numerical data
#' @param levels vector levels of truncation (default: \code{c(0.1,0.025,0.050,0.01)})
#' @param side Truncation of \code{"left"} and/or \code{"right"} (default: \code{c("right")})
#' @param main_title Main title for figures
#' @param xlabel Label for x-axis
#' @author Frank PG Princee
#' @family data distribution
#' @export
#' 
distribution.exploreEnds <- function(data,levels=c(0.1,0.025,0.05,0.01),side=c("right","left"),
                                     main_title="Truncation",xlabel)
{
  if (!studbook.plot()) {
    return(NULL)
  }
  
  side <- match.arg(side,several.ok=TRUE)
  
  n <- length(data)
  levels <- sort(levels)
  truncated.data <- NA
  for ( level in levels){
    s  <- floor( n * level) 
    if (side == 'right') {
      truncated <- data[(n-s):n]
    }
    
    if (side == 'left') {
      truncated <- data[1:s]
    }
    
    tmp_data <- data.frame("value"=truncated,
                           "group"=rep(as.factor(level*100),length(truncated)) )
    truncated.data <- rbind(truncated.data,tmp_data)
    
  }
  comment <- paste("Explore which data will be excluded when",side,"truncation is applied.") 
  studbook.caption(main_title,comment)
  xlabel <- firstCap(xlabel)
  
  stripchart(value ~ group,data=truncated.data,main='',
             xlab=xlabel,ylab=paste('Percentage of',side,'truncation'))
  studbook.title(main_title)
}


###
#' @title Create stripchart of a single or multiple random samples of data 
#' @description \code{distribution.sample} creates a stripchart with a single or multiple 
#' random sample(s) of data-set
#' @param data vector with numerical data
#' @param sizes single value or vector with multiple sample sizes (default: \code{c(10,25,50,100,250)})
#' @param main_title Main title for figures
#' @param xlabel Label for x-axis
#' @author Frank PG Princee
#' @family data distribution
#' @export
#' 
distribution.sample <- function(data,sizes=c(10,25,50,100,250), main_title='Random sample',
                                xlabel)
{
  if (!studbook.plot()) {
    return(NULL)
  }
  sizes <- sort(sizes)
  n <- length(data)
  samples <- length(sizes)
  
  sample.data <- data.frame(value=data,group=rep('original',n))
  
  for ( size in sizes){
    sample_data <- sample(data,size,replace=TRUE)     
    tmp_sample <- data.frame(value=sample_data,group=rep(as.factor(size),size))
    sample.data <- rbind(sample.data,tmp_sample)                            # Add to column
  }
  
  xlabel <- firstCap(xlabel)
  comment <- paste("Compare distribution of random samples with original data; (n=",n,").",sep="")
  studbook.caption(main_title,comment)
  stripchart(value ~ group,data=sample.data,main='',xlab=xlabel,ylab='Number or random draws')
  studbook.title(main_title)
}


###
#' @title Fit and test normal distribution
#' @description \code{distribution.normal} tests whether normal distribution fits data
#' @details Normal distribution. The shapiro test is used
#' @param data Vector with observations
#' @author Frank PG Princee
#' @family data distribution
#' @export 
#' 
distribution.normal <- function(data)
{
  if (length(data) > 5000) {
    message("Sample size > 5000: sampled 5000 without replacement ")
    data <- sample(data,5000,replace=FALSE)
  }
  results <- shapiro.test(data)
  print(results)
  cat("p < 0.05 means that observed data do not follow normal distribution.\n\n")
}


###
#' @title Fit and test discrete distribution
#' @description \code{distribution.discrete} tests whether statistical distribution fits data
#' @details Discrete distribution
#' @param data Vector with observations
#' @param test Name of distribution \code{"binomial"}, \code{"nbinomial"} or \code{"poisson"}  (default: 'binomial')
#' @param params List with additional parameters for the distribution:
#' \code{lambda} for Poisson and \code{prob} and \code{size} for "bionomal
#' @author Frank PG Princee
#' @family data distribution
#' @export 
#' 
distribution.discrete <- function(data,test=c('binomial','nbinomial','poisson'),params=NULL)
{
  test <- match.arg(test)
  
  data.gf <- vcd::goodfit(data,type=test,method= "MinChisq",par=params) 
  summary(data.gf)
  cat("\n")
  if (studbook.plot()) {
    plot(data.gf,main=paste("Count data vs",test,"distribution"))
  }
}



###
#' @title Analyse distribution data
#' @description \code{distribution.analyse} plots and tests distribution of data 
#' @details This is an wrapper function for plotting and testing distribution data
#' @param data vector with data
#' @param main_title Main title in plots etc. (default:\code{"Data"})
#' @param xlabel Label at x-axis (default: \code{"X"})
#' @param bar_breaks Algorithm, function or number of breaks. See help for \code{"hist"}. Default(\code{"Sturges"})
#' @param leftPoints left most points at which density is estimated (default:NA)
#' @param rightPoints right most points (default:NA)
#' @param CV TRUE estimate coefficient of variation
#' @param quantiles Vector with quantiles to plotted on graph
#' @param test Test if data are distributed according \code{"binomial", "normal",
#' "nbinomial","poisson"} (default: \code{"normal"})
#' @author Frank PG Princee 
#' @family data distribution
#' @export
#' 
distribution.analyse <- function(data,main_title='Data',xlabel="X",bar_breaks="Sturges",
                                 leftPoints=NA,rightPoints=NA, CV=TRUE,
                                 quantiles=c(0.25,0.5,0.75),
                                 test=c('normal','binomial','nbinomial','poisson'))
{
  test <- match.arg(test)
  if (studbook.plot()) {
    distribution.plot(data,main_title,xlabel,bar_breaks,leftPoints,rightPoints,quantiles)
  } 
  
  if (test == 'normal') {
    distribution.normal(data)
  } else { 
    distribution.discrete(data,test)
  }
  
  studbook.summary(data,long=TRUE,CV=CV,title=xlabel)
}
