###
# Functions to show estimated breeding values (EBV)
# Requires variables set in studbook_animalmodel


###
#' @title Get estimated breeding values from animal model
#' @details Estimated breeding values are obtained from MCMCglmm or pedigreemm objects
#' (depending on which model was used). 
#' @author Frank Princee
#' @note REML - see Vazquez(2010); MCMC - see WAMWiki website
#' @family breeding value
#' @export
#' 
ebv.get <- function()
{
   Model <- studbook.get("Model")
  
   studbook.set("EBV.list",NULL)
   
   
   if (class(Model) == "MCMCglmm") {
     cat("Extracting breeding value from MCMCglmm\n")
     ebv.list <- summary(Model$Sol)$statistics[,1]        # Summary of mcmc; column No 1: Mean    
     idx <- grep("animal[.]",labels(ebv.list))
     ebv.list <- ebv.list[idx]
     rm(idx)
     ebv.list <- data.frame("value"=as.numeric(ebv.list),
                            "animal"=gsub("animal[.]","",labels(ebv.list)),
                            "G"=rep(NA,length(ebv.list)) )
     
     # Extract fields animal and G from pedigree file                       
     ped.small <- studbook.csv("pedigree.csv")[,c("ID","G")]
     ped.idx <- match(ebv.list$animal,ped.small$ID)
     ebv.list[,'G'] <- ped.small[ped.idx,"G"]
     rm(ped.small)
     rm(ped.idx)
   } else {
     ebv <- lme4::ranef(Model)$animal
     ebv.list <- data.frame("value"=as.numeric(ebv[[1]]), 
                            "animal"=labels(ebv)[[1]],
                            "G"=rep(NA,nrow(ebv)))
   }
   studbook.set("EBV.list",ebv.list)
}


###
#' @title Variance in estimated breeding value
#' @return variance
#' @family breeding value
#' @keywords internal
#' 
ebv.variance <- function()
{
  return(var(studbook.get("EBV.list")[,'value']))
}


###
#' @title Reliability in estimated breeding value
#' @param Variance in estimated breeding value
#' @return reliability
#' @author Frank PG Princee
#' @family breeding value
#' @export
#' 
ebv.reliability <- function()
{
  if (is.null(studbook.get("EBV.list"))) {
      ebv.get()
  }
  VEBV <- ebv.variance()
  VA   <- studbook.get("Variances")['VA']
  r2   <- VEBV/VA
  cat("\n[Breeding values]\n")
  cat("Variance EBV     :",VEBV,"\n")
  cat("Reliability EBV  :",r2,"\n")
  cat("Prediction error :",(1-r2)*VA,"\n")  
  cat("-----------------------------------------------------------------\n")
}



###
#' @title Display breeding values
#' @param best Number of highest breeding values
#' @author Frank PG Princee
#' @family breeding value
#' @export
#' 
ebv.best <- function(best=10)
{
  ebv.list <- studbook.get("EBV.list")
  best <- head(ebv.list[order(ebv.list$value,decreasing=TRUE),])  # Highest first
  cat("Highest breeding values for phenotype",phenotype.getActive(),"\n")
  cat(createLine(40,"-"),"\n")
  print(best)
  cat("\n")
}


###
#' @title Mean breeding value per generation
#' @param G.min minimum generation to be included (default: NA)
#' @param G.max maximum generation to be included (default: NA)
#' @author Frank PG Princee
#' @family breeding value
#' @export
#' 
ebv.generation <- function(G.min=NA,G.max=NA)
{
  TRAIT <- tolower(phenotype.getActive())
  
  if (is.na(G.min)) {
    G.min <- min(studbook.get("EBV.list")[,'G'])
    G.min <- as.integer(G.min)
  }
  if (is.na(G.max)) {
    G.max <- max(studbook.get("EBV.list")[,'G'])
    G.max <- as.integer(G.max)
  }
  
  text <- paste("Breeding value for",TRAIT,"in generation groups")
  studbook.openOutput("ebv-generation")
  generations <- boxplot(value ~ G,data=studbook.get("EBV.list"),
                         subset=(G >= G.min & G <= G.max), main="",
                         xlab="Generation",ylab="Breeding values")
  studbook.title(text)
  studbook.closeOutput()
  
  cat(text,"\n")
  
  
  rownames(generations$stats) <- c("extreme low","lower","median","upper","extreme upper") 
  colnames(generations$stats) <- G.min:G.max
  print(generations$stats)
  cat("\n")
  ebv.mean <- aggregate(value ~ G,data=studbook.get("EBV.list"),mean)
  print(ebv.mean)
  studbook.set("confidence",FALSE)
  studbook.set("prediction",FALSE)
  regression.linear(data_X=subset(studbook.get("EBV.list")[,c("G","value")], G >= G.min & G <= G.max),diagnostic=2,
                    main_title="Mean breeding value per generation",
                    xlabel="generation",ylabel="mean breeding value",
                    base_name=paste('generation_ebv',phenotype.getActive())) 
}