###
# Package : studbookR
# 
# Copyright (c) 2011 Frank PG Princee, All rights reserved
#
# Functions to compute lambda, r and R0 
# from standard ecology books e.g. Caughley(1977), Sinclair(2006), Krebs(2009)
# 

###
#' @title Estimate maximum intrinsic rate of change (\eqn{r_{max}}) from adult weight. 
#' @description \code{growth.rm} calculates the maximum intrinisc rate of change (\eqn{r_m}) 
#' from adult weight.
#' @details The maximum intrinsic rate is estimated as \eqn{r_{max}=1.5(weight^{-0.36}}
#' @param weight Weight in kilogrammes.
#' @return Intrinsic rate
#' @author Frank PG Princee
#' @examples
#' growth.rm(100)
#' @references Sinclair
#' @family ecology
#' @export
#' 
growth.rm <- function(weight)
{
  return(1.5*(weight^-0.36))
}


###
#' @title Convert \eqn{r} to \eqn{\lambda}
#' @description \code{growth.r} calculates the intrinsic rate of change (\eqn{r}) 
#' from the finite rate of change (\eqn{\lambda}).
#' @details Equation: \eqn{r = log(\lambda)}
#' @param lambda Finite rate of change
#' @author Frank PG Princee
#' @family ecology
#' @export 
#' 
growth.r <- function(lambda)
{
  return( log(lambda) )
}

###
#' @title Convert \eqn{\lambda} from \eqn{r}
#' @description \code{growth.lambda} calculates the finite rate of change (\eqn{\lambda})
#' from the intrinsic rate of change (\eqn{r}).
#' @details Equation: \eqn{\lambda = ln(r)}
#' @param r intrinsic rate of change
#' @author Frank PG Princee
#' @family ecology
#' @export
#' 
growth.lambda <- function(r)
{
  return( exp(r) )
}


###
#' @title Exponential growth 
#' @description \code{growth.expontential} calculates population size according the
#' exponential growth model.
#' @details Equation: \eqn{N_t=e^{rt}N_0}
#' @param N0 population size at t=0
#' @param r intrinsic (exponential) rate of change
#' @param t time units since t=0
#' @author Frank PG Princee
#' @family ecology
#' @export
#' 
growth.exponential <- function(N0,r,t)
{
  return(exp(r*t)*N0)
}

###
#' @title Logistic growth 
#' @description \code{growth.logistic} calculates population size according the
#' logistic growth model.
#' @details Equation: \eqn{N_t= K / (1+ (K/N_0 - 1) e^{-rt}) }
#' @param N0 population size at t=0
#' @param K Carrying capacity
#' @param r intrinsic (exponential) rate of change
#' @param t time units since t=0
#' @author Frank PG Princee
#' @references Roughgarden (1979, p305)
#' @family breeding value
#' @export
#' 
growth.logistic <- function(N0,K,r,t)
{
  Nt <- K / ( 1 + (K/N0 - 1)*exp(-r*t))
  return(Nt)
}


###
#' @title Log equation in discrete time
#' @author Frank PG Princee
#' @family ecology
#' @export
#  
growth.lograte <- function(N,K)
{
  dN <- r*(1 - N/K)*N
  return(dN)
}


###
#' @title Plot exponential and logistic model for hypothetical populations
#' @details Either \code{r} or \code{lambda} must be provided
#' @param N0 Population size at start (time=0)
#' @param r Intrinsic rate of change (default: NA)
#' @param lambda Finite rate of change (default:NA)
#' @param time Length of time in whole units     
#' @author Frank PG Princee
#' @family ecology
#' @keywords internal   
#' 
plot.models <- function(N0,r=NA,lambda=NA,K,time)
{
  if (!studbook.plot()) {
    return(NULL)
  }
  
  if ( is.na(r) && is.na(lambda) ) {
    stop("Either r or lambda needs to be provided")
  }
  
  
  if (is.na(r)) {
    r <- growth.r(lambda)
  }
  
  pop           <-matrix(nrow=time+1,ncol=3)
  colnames(pop) <- c('time','NE','NL')
  pop[,1]       <- 0:time
  pop[,2:3]     <- 0
  pop[1,2:3]    <- N0
  
  for (t in 2:(time+1) ) {
    pop[t,'NE'] <- growth.exponential(N0,r,(t-1))
    pop[t,'NL'] <- growth.logistic(N0,K,r,(t-1))
  }
  
  K_2 <- N0 + ((K - N0) * 0.50)
  #Check were logistic is 1/2 the graph
  for (t in 0:time+1) {
    if (pop[t,'NL'] > K_2 ) {
      x.NL <- t
      break
    } 
  }
  
  
  K_3 <- N0 + ((K - N0) * 0.75)
  #Check were exponential is 3/4 the graph
  for (t in 0:time+1) {
    if (pop[t,'NE'] > K_3 ) {
      x.NE <- t
      break
    } 
  }
  
  studbook.openOutput('growth.model')
  plot(pop[,'time'],pop[,'NL'],type='l',xlab='Time',ylab='N',
       ylim=c(N0,1.1*K),lty=1)
  lines(pop[,'time'],pop[,'NE'],type='l',lty=1)
  abline(h=K,lty=2)
  text((time/2),K,"K",pos=3)
  text(x.NL,K_2,"Logistic")
  text(x.NE,K_3,"Exponential")
  studbook.title('Exponential and logistic growth')
  studbook.closeOutput()
}


###
#' @title Create population based on exponential growth
#' @details population[1] = N0
#' @param population data frame
#' @param K carrying capacity
#' @param r intrinsic rate of chance
#' @family ecology
#' @keywords internal
#' 
population.exponential <- function(population,K=NA,r)
{
  period <- length(population)
  for (t in 2:period) {
    population[t] <- growth.exponential(population[1],r,(t-1))              # t[1] is time 0
    if (!is.na(K) && population[t] > K) {
      population[t] <- K
    }
  }
  invisible(population)
}



###
#' @title Create population based on logistic growth
#' @details population[1] = N0
#' @param population data frame
#' @param K carrying capacity
#' @param r intrinsic rate of chance
#' @family ecology
#' @keywords internal
#
population.logistic <- function(population,K,r)
{
  period <- length(population)
  for (t in 2:period) {
    population[t] <- growth.logistic(population[1],K,r,(t-1))               # t[1] is time 0
  }
  invisible(population)
}


###
#' @title Test exponential growth model on census data
#' @details Exponential growth is tested by linear regression of ln(N)
#' ln(Nt) = a + rt
#' a= ln(N0)
#' @param census.data Matrix of census data
#' @param sex Name of sex
#' @author Frank PG Princee
#' @family ecology
#' @export
#' 
studbook.exponential <- function(census.data,sex='all')
{
  if (!studbook.plot()) {
    return(NULL)
  }
  main_title <- 'Exponential growth curve'
  studbook.screenTitle(main_title)
  
  studbook.set('confidence',TRUE)
  studbook.set('prediction',FALSE)
  regression.linear(census.data[,1:2],
                    data_W=NULL,log="y",diagnostic=0,
                    main_title=paste("Log linear regression (",sex,")"),
                    xlabel='Year',ylabel='N',
                    base_name=paste("exponential",sex,sep='.')) 
}



###
#' @title Logistic growth is tested nonlinear regression model
#' 
#' @param census.data data frame with columns year and count
#' @param sex group 
#' @param project.years number of years in projection; negative years for back to the future predictions (default:0)
#' @param add_line plots carrying capacity and half carrying capacity A/r.
#' @author Frank PG Princee
#' @references Nonlinear Regression and Nonlinear Least Squares in R, Fox & Weisberg (2010)
#' @family ecology
#' @export
#' 
studbook.logistic <- function(census.data,sex=c("all","female","male","unknown"),
                              project.years=0,add_line=TRUE)
{
  main_title <- 'Logistic growth curve'
  studbook.screenTitle(main_title)
  
  last <- nrow(census.data)  # Last element to include in model

  # Back to the future
  if (project.years < 0 ) {
    last <- last + project.years # Remember is negative
    project.years <- abs(project.years)
  }
  
  last_year <- census.data[last,'DATE']
 
  logistic.model <- COUNT ~ K/(1 + exp(-(A  + r * DATE)))
  
  # Find initial parameters
  logistic.start <- getInitial(COUNT ~ SSlogis(DATE,Ks,xmid,scale),data=census.data[1:last,])
  
  model.start <- c( K = as.numeric(logistic.start['Ks']),
                    A = as.numeric( -(logistic.start['xmid']/logistic.start['scale']) ),    #xmid/scale
                    r = as.numeric(1/logistic.start['scale'])
  )
  
  
  census.nls  <- nls(logistic.model,start=model.start,data=census.data[1:last,])
  
  K           <- coef(census.nls)['K']
  A           <- -coef(census.nls)['A']   #Note that model calculates -A
  r           <- coef(census.nls)['r']
  
  lK          <- logLik(census.nls)
  aic         <- -2 * as.numeric(lK) + 2 * attr(lK,'df')
 
  K.half      <- car::deltaMethod(census.nls,"-A/r")                  
  r.max       <- car::deltaMethod(census.nls,"r")
  
  
  #Create data frame for prediction
  
  census.years  <- range(census.data[1:last,'DATE'])
  predict.years <- data.frame("DATE"=seq(from=census.years[1],to=census.years[2]+project.years,by=1))
  predict.fit   <- predict(census.nls,newdata=predict.years)
  N0            <- predict.fit[1]      
  
  cat("Census data          :",census.years[1],"-",census.years[2],"\n")
  cat("Carrying capacity (K):",K,"\n")
  cat("Half K reached (-A/r):",K.half$Estimate,"(SE:",K.half$SE,")\n")
  cat("r[max]               :",r.max$Estimate,"(SE:",r.max$SE,")\n")
  cat("N[0]                 :",N0,"(year:",census.data[1,'DATE'],")\n")  
  cat("AIC                  :",aic,"(df:",attr(lK,'df'),")\n")
  cat("r[max] = intrinsic rate of change\n")
  cat("\n")       
  
  rs <- chisq.test(data.frame('OBSERVED'=census.data[1:last,'COUNT'],
                              'EXPECTED'=predict.fit[1:last]))   # Do not include extrapolated
  cat("\n")
  cat("Pearson's Chi-Square test:\n")
  cat("--------------------------------------------------\n")
  
  X2 <- rs$statistic
  df <- rs$parameter 
  cat("X2 = ", rs$statistic, ",  p =",rs$p.value," (df=",rs$parameter,")\n")
  cat("H0: Samples are from the same distribution => ")
  
  if (rs$p.value < 0.95) {
    cat("rejected\n")
  } else {
    cat("not rejected\n")
  }
  cat("\n")
  
  
  if (studbook.plot()) {
    #Graphic plot
    
    if (add_line) {
      ylim <- c(0,K)
    } else {
      ylim <- NULL
    }
    
    studbook.openOutput(paste("logistic",sex,sep='.'))
    main_title <- paste("Logistic growth (",sex,")")
    studbook.caption(main_title,"Fit logistic growth model to census data.")
    plot(census.data,pch=21,cex=0.75,xlim=c(census.years[1],census.years[2]+project.years),
         ylim=ylim, main='',xlab='Year',ylab='N')
    lines(predict.years[,'DATE'],predict.fit,lty='solid')
   
    abline(h=0,lty=3,lwd=1.5)
    abline(h=K,lty=3,lwd=1.5)
    segments(x0=0,y0=0.5*K,x1=K.half$Estimate,y1=0.5*K,lty=2,lwd=1)
    segments(x0=K.half$Estimate,y0=0,x1=K.half$Estimate,y1=0.5*K,lty=2,lwd=1)
  
    # Population projection
    if (project.years > 0) {
      abline(v=census.years[2],lty='dotted')
    }
    
    # Back to the future
    if (last < nrow(census.data)) {
      abline(v=last_year,lty='dotted') 
    }
    
    studbook.title(main_title)    
    studbook.closeOutput()
  }
  
  invisible(TRUE)
}


###
#' @title Ecological models
#' @description \code{studbook.ecology} applies ecological models to census data
#' @details Census is stored as data frame (SSLogis does not work on ts objects)
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female").
#' @param model Ecological model: \code{"exponential"} or \code{"logistic"} (default: "logistic").
#' @param last  Last year in analysis, NULL is last year in data set (default:NULL).
#' @param last  Last year in analysis, NULL is last year in data set (default:NULL).
#' @param project.years Number of years to project after last census. (default:0).
#' @param file_name File name with census data (default: \code{"census"}).
#' @param add_line plot carrying capacity and mid-K lines in logistic model (default:TRUE).
#' @author Frank PG Princee
#' @family ecology
#' @export
#' 
studbook.ecology <- function(sex=c("all","female","male","unknown"),
                             model=c('logistic','exponential'),
                             first=NULL,last=NULL,project.years=0,
                             file_name="census",add_line=TRUE)
{
  if (studbook.empty()) {
    studbook()
  }
  
  sex       <- match.arg(sex)
  model     <- match.arg(model)
  
  census.data <- studbook.read(file_name,sex=sex)     
  
  if (is.null(census.data)) {
    stop("No census data")
  } 
  
  attr(census.data,"interval")    <- names(census.data)[1]
  attr(census.data,"census_date") <- format(base::as.Date(census.data[1,1],"%d-%b"))  #day-month
  colnames(census.data)           <- c('DATE','COUNT')
  census.data[,'DATE']            <- tis::year(base::as.Date(census.data[,'DATE']))  # Assumes annual
 
  # Default time window
  
  period <- range(census.data[,'DATE'])
  if (is.numeric(first)) {
    period[1] <- first
  }
  
  if (is.numeric(last)) {
    period[2] <- last
  }
  
  #Trailing zero counts
  if (is.null(first)) {
    period[1] <- census.leadingZero(census.data[,'COUNT'])
  }
  
  census.data <- census.data[census.data$DATE >= period[1] & census.data$DATE <= period[2],]
  
  if (model == 'logistic') {
    studbook.logistic(census.data,sex,project.years,add_line=add_line) 
  } else {
    studbook.exponential(census.data,sex) 
  }
  
 invisible(TRUE)
}





