###
# Package : studbookR
# 
# Copyright (c) 2011 Frank PG Princee, All rights reserved
#

###
#' @title Reproductive age classes
#' 
#' @details Extract first class of reproduction and last class of reproduction 
#' i.e. m[x] > 0. 
#'
#' @note Element 1 = class 0.
#' 
#' @param mx fecundity rates
#' @return vector with first and last class
#' @author Frank Princee
#' @family fecundity
#' @keywords internal
#' 
fecundity.range <-  function(mx)
{
  classes <- length(mx)
  first   <- 0    # First age class = class 0!
  last    <- classes
  
  
  
  # Get first class
  for (x in 1:classes) {
    first <- x
    if (mx[x] > 0) { 
      break
    } 
  }
  
  #Get last class
  for (x in classes:1 ) {
    last <- x
    if (mx[x] > 0) {
      break
    } 
  }
  return(c(first,last))
}


###
#' @title Variance of mx
#' 
#' @details Variance can be Poisson (see Keilman and Pham 2002). 
#' @param mx double Fecundity rate
#' @param Nx integer Individuals at risk
#' @param distribution Probability distribution (default: "Poisson")
#' @return double
#' @author Frank PG Princee
#' @family fecundity
#' @keywords internal
#' 
mx.var <- function(mx,Nx,distribution="Poisson")
{ 
  if (distribution == "binomial" & mx <= 1) {
    dist.var <- qx.var(mx,Nx)  
  } else {
    dist.var <- mx/Nx # Poisson
  }
  return(dist.var)
}  


###
#' @title Coefficient of variation of mx
#' 
#' Variance is Poisson
#' 
#' @param mx double Fecundity rates
#' @param Nx integer Individuals at risk
#' @return double
#' @author Frank PG Princee
#' @family fecundity
#' @keywords internal
#' 
mx.cv <- function(mx,Nx)
{
  svar <- mx.var(mx,Nx)
  return(cv.mean(mx,svar))
}  


###
#' @title Confidence limits of mx
#' @description \code{mx.cf} Calculates 95% confidence limits of mx
#' @param births Number of births
#' @param Nx integer Individuals at risk
#' @param significance double Significane level of t-test (default: 0.05)
#' @return lower and upper limits [0,inf]
#' @author Frank PG Princee
#' @family fecundity
#' @keywords internal
#' 
mx.cf <- function(births,Nx,significance=0.05)
{
  if (is.na(births) | is.na(Nx)) {
    return(c(lower=NA,upper=NA))
  }
  
  if (Nx < 2) {
    return(c(lower=NA,upper=NA))
  }
  
  mx.limit <- poisson.test(births,Nx)$conf.int  #Uses Garwood(1936)
  if (mx.limit[1] < 0.0) {
    mx.limit[1] <- 0.0
  }
  return(c(lower=mx.limit[[1]],upper=mx.limit[[2]]))  
}


###
#' @title Radix of cohort 
#' @details The radix allows to more easily to compare Bx values when numbers at risk per age classes 
#' differ between samples.
#' @param life_table fecundity table as created by fecundity.table
#' @param cohort Size of cohort at start (default: 1000) 
#' @author Frank PG Princee
#' @family fecundity
#' @keywords internal
#' 
fecundity.radix <- function(life_table,cohort=1000)
{
  cohort_size <- cohort
  ratio <- cohort/life_table[1,'N']
  
  for (c in 1:nrow(life_table) ) {
    life_table[c,"N"] <- round(ratio * life_table[c,"N"])
    life_table[c,"bred"] <- round(ratio * life_table[c,"bred"])
    life_table[c,"Bx"] <- round(life_table[c,"Nx"] * life_table[c,"mx"])
  }
  return(life_table)
}




###
#' @title Analyse bootstrap data on fecundity
#' 
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param boot Type of bootstrap results: \code{"risk"} or \code{"birth"} (default: "risk")
#' @param groups  Which groups/classes to analyse (default: NA)
#' @author Frank PG Princee
#' @seealso \code{link{lifetable.analyseBootstrap}}
#' @family fecundity
#' @export
#' 
fecundity.analyse <- function(sex=c("male","female","unknown","all"),
                              boot=c("risk","birth"),groups=c(NA))
{
  if (studbook.empty()) {
    studbook()
  }
  sex  <- match.arg(sex)
  boot <- match.arg(boot) 
  
  file_name <- paste('bootstrap-Mx',boot,sex,sep="-")
  file_name <- paste(file_name,".csv.gz",sep='') 
  
  mx <- studbook.read('fecundity',sex=sex)
  if (is.null(mx)) {
    return(FALSE)
  }
  rownames(mx) <- 0:(nrow(mx)-1)
  
  studbook.openOutput('bootstrap_Mx')
  n <- mx$N
  rm(mx)
  
  rate       <- expression(m[x])
  main_title <- paste('Bootstrap of',sex,rate,"(",boot,")");    
  lifetable.analyseBootstrap(file_name,groups,main_title,ylabel=rate)
  studbook.closeOutput()
  
  return(TRUE)
}


###
#' @title Compare fecundity rates between sexes
#' @param class_width Width of class in number of days (and decimals) (default: 365.25 = 1 year)
#' @param file_name Base name of both male and female fecundity data file (default: "fecundity")
#' @author Frank PG Princee 
#' @family fecundity
#' @export
#' 
fecundity.sexes <- function(class_width=365.25,file_name='fecundity')
{
  main_title <- 'Fecundity between sexes'
  
  male       <- studbook.read(file_name,sex='male')
  female     <- studbook.read(file_name,sex='female')
  
  if (is.null(male) || is.null(female)) {
    stop("No male and/or female mortality data.")
  }
  rownames(male)   <- 0:(nrow(male)-1)
  rownames(female) <- 0:(nrow(female)-1)
  
  male   <- male[,c('N','B','OBSERVED')]
  female <- female[,c('N','B','OBSERVED')]
  
  attr(male,"sample")   <- 'males'
  attr(female,"sample") <- 'females'
  lifetable.compare(male,female,main_title)    
  
  lifetable.plotGroups(male[,'OBSERVED'],female[,'OBSERVED'],
                       main_title=main_title,
                       ylabel=expression(m[x]),file_name='sexes_mx')             
  return(TRUE)
}



###
#' @title Import fecundity rates
#' @description \code{fecundity.table} imports PML fecundity tables
#' 
#' @details Fecundity input files have the following data format
#' \tabular{lll}{
#' \bold{Field name} \tab \bold{Type} \tab \bold{Comment} \cr
#' CLASS             \tab integer     \tab Age class starting at 0 \cr  
#' N                 \tab real        \tab Number of individuals at risk \cr
#' BRED              \tab integer     \tab Number of individuals that reproduced \cr
#' B                 \tab integer     \tab Number of births (same sex as parents is optional)\cr
#' OBSERVED          \tab real        \tab Observed fecundity rate mx = B/N\cr
#' ------            \tab ----        \tab -----\cr
#' MEAN              \tab real        \tab Mean mx in bootstrap samples\cr   
#' VAR               \tab real        \tab Variance of mx in bootstrap samples\cr 
#' LOWER             \tab real        \tab 2.5 percentile of mx in bootstrap samples\cr
#' UPPER             \tab real        \tab 97.5 percentile of mx in bootstrap samples\cr
#' }
#' 
#' 
#' @param sex Sex group; valid names are \code{"female","male","unknown","all"} 
#' (default: "female")
#' @param mx_field Column names in PML file \code{"OBSERVED","MEAN"}. The adjusted mean is 
#' estimated from deviation of observed and mean offspring (default: "OBSERVED") 
#' @param class_width Information on class width for X-axis label (default:"year")
#' @param resamples Information on used bootstrap resamples (default:999)
#' @author Frank PG Princee
#' @family fecundity
#' @export
#' 
fecundity.table <- function(sex=c("female","male","unknown","all"),
                            mx_field=c('OBSERVED','MEAN'))
{
  if (studbook.empty()) {
    studbook()
  }
  sex      <- match.arg(sex)
  
  fecundity  <- studbook.read('fecundity',sex=sex)
  
  if (is.null(fecundity)) {
    stop("No fecundity data")
  }
  rownames(fecundity) <- 0:(nrow(fecundity)-1)
  
  
  
  mx_field <- match.arg(mx_field)
  if (!is.element("MEAN",colnames(fecundity))) {  # No bootstrap data
    mx_field <- 'OBSERVED'
  }
  

  if (mx_field == "OBSERVED") {
    fecundity <- fecundity[,c('CLASS','N','B','BRED','OBSERVED')]
    fecundity <- within(fecundity,{
      UPPER <- NA
      LOWER <- NA
      VAR   <- NA
    }) 
    
    # Compute for added fields
    for (x in 1:nrow(fecundity)) {
      fecundity[x,'VAR'] <- round(mx.var(fecundity[x,'OBSERVED'],fecundity[x,'N'],'Poisson'),5)
      cf <- mx.cf(fecundity[x,"B"],fecundity[x,"N"])
      fecundity[x,c('LOWER','UPPER')] <- c(cf[["lower"]],cf[["upper"]])
    }
  } else {  # bootstrap (keep  mean, var, cv and percentiles)
    fecundity <- within(fecundity,{
      MEAN  <- OBSERVED - (MEAN-OBSERVED)  # Observed - bias
    })
    fecundity <- fecundity[,c('CLASS','N','B','BRED','MEAN','VAR','LOWER','UPPER')]
  } 
  
  fecundity <- as.matrix(fecundity)
  colnames(fecundity) <- c('class','Nx','Bx','bred','mx','var','lower','upper')
  return(fecundity)
}




###
#' @title Fecundity rates
#' @description \code{studbook.fecundity} imports PML fecundity tables
#' 
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param mx_field Column names in PML file \code{"OBSERVED","MEAN"}. The adjusted mean is 
#' estimated from deviation of observed and mean offspring (default: "OBSERVED") 
#' @param confidence TRUE plot confidence interval (default:TRUE)
#' @param radix Convert Nx to radix of 1,000 or 10,000 (default: NA) 
#' @param smooth Smoothing procedure or parametric model and additional parameters. Valid options are:
#' \code{"none"},\code{"PMx"},\code{"SPARKS"},\code{"Tukey"} and \code{"Hadwiger"} (default: 'none') 
#' @param smooth_params Optional parameters for smoothing procedure/parametric model (default: NULL)
#' @param class_width Information on class width for X-axis label (default:"year")
#' @author Frank PG Princee
#' @family fecundity
#' @export
#' 
studbook.fecundity <- function(sex=c("male","female","unknown","all"),
                               mx_field=c('OBSERVED','MEAN'),confidence=TRUE,radix=NA,
                               smooth=c("none","PMx","SPARKS","Tukey","Hadwiger"),
                               smooth_params=NULL,class_width='year')
{
  if (studbook.empty()) {
    studbook()
  }
  sex       <- match.arg(sex)
  mx_field  <- match.arg(mx_field)
  smooth    <- match.arg(smooth)
  fecundity <- fecundity.table(sex=sex,mx_field=mx_field)
  print(str(fecundity))
 
  if (is.null(fecundity)) {
    stop("No fecundity data")
  }
  
  if (mx_field == 'OBSERVED') {
    param_method <- 'observed' 
  } else {
    param_method <- 'bootstrap'
  }
 
  
  if (smooth == 'none') {
    model <- c(NA)  
  } else {
    model <- c(smooth,smooth_params)
  }
  
  if (!is.na(radix)) {
    fecundity <- fecundity.radix(fecundity,radix)
  }
  
  # Smoothing method
  smooth_comment <- smooth
  if (smooth != 'none') {
    model     <- c(smooth,smooth_params)
    result    <- model.smooth(fecundity[,c('Nx','mx')],c(smooth,smooth_params)) 
    fecundity <- cbind(fecundity,result)
    smooth_comment <- comment(result)
    colnames(fecundity)[ncol(fecundity)] <- "smoothed"
    confidence <- FALSE  # Do not combine smooth and bootstrap 
  }
  
  attr(fecundity,"sex")    <- sex
  attr(fecundity,"mx")     <- paste("Data column",mx_field)
  attr(fecundity,'smoothed') <- smooth_comment
  if (!is.na(radix)) {
    attr(fecundity,"radix") <- radix
  }
  
  
  studbook.screenTitle(paste('Fecundity rates in',sex,'(',param_method,')'))
  print(fecundity,digits=4)
  
  if (studbook.plot()) {
    studbook.openOutput(paste('fecundity',sex,sep='.'))
    main_title = paste( 'Fecundity rates (',sex,')' )
    lifetable.plot(fecundity,field_name='mx',confidence=confidence,
                   main_title=main_title,
                   class_width=class_width,max_y=NA,
                   ylabel=expression(m[x])) 
    studbook.closeOutput()  
  }
  
  return(TRUE)
}




