###
# Package : studbookR
# 
# Copyright (c) 2011 Frank PG Princee, All rights reserved
#


###
#' @title Assign inbreeding category to inbreeding coefficient
#' @description \code{inbreeding.level} assigns an inbreeding category to an
#' inbreeding coefficient.  
#' @details  Inbreeding classes for parents are:
#' - 0       : non-inbred
#' - 1/256   : third cousins
#' - 1/128   : second cousins (once removed)  
#' - 1/64    : second cousins
#' - 1/32    : first cousins (once removed)
#' - 1/16    : first cousins 
#' - 1/8     : half sibs
#' - 1/4     : full sibs
#' - 3/8     : full sibs 2nd
#' - 1/2     : full sibs 3rd
#' 
#' @param F double inbreeding coefficient
#' @return name
#' @author Frank PG Princee
#' @family inbreeding
#' @keywords internal
#' 
inbreeding.level <- function(F) 
{
  name <- 'inbred'
  if (F == 0.0) {
    name <- "non-inbred"
  } else if(F > 0.0 & F <= 1/256) {
    name <- "third cousins"
  } else if(F > 1/256 & F <= 1/128) {
    name <- "second cousins (once removed)"
  } else if(F > 1/128 & F <= 1/64) {
    name <- "second cousins"
  } else if(F > 1/64 & F <= 1/32) {
    name <- "first cousins (once removed)"
  } else if(F > 1/32 & F <= 1/16) {
    name <- "first cousins"
  } else if(F > 1/16 & F <= 1/8) {
    name <- "half sibs"
  } else if(F > 1/8 & F <= 1/4) {
    name <- "full sibs"
  } else if(F > 1/4 & F <= 3/8) {
    name <- "fulls sibs 2nd"
  } else if(F > 3/8 & F <= 1/2) {
    name <- "full sibs 3rd"
  } else {
    name <- "very high"
  }       
  return(name)
}



###
#' @title Create line from MLE S0 and B
#' @param fcoef Inbreeding coefficient
#' @param S0 survival at time 0
#' @param B regression coefficient haploid lethal equivalent 
#' @return fitted vector
#' @author Frank PG Princee
#' @family inbreeding
#' @keywords internal
#' 
lethaleqv.fit <- function(fcoef,S0,B)
{
  fit <- 1:length(fcoef)
  for (f in 1 : length(fcoef)) {
    fit[f] <- S0 * exp(-B*fcoef[f])
  }
  return(fit)
}


###
#' @title Inbreeding 2x2 contingency test
#' @details Wrapper to generic 2x2 contingency 
#' @param inbred.survived Number of inbred survivors 
#' @param inbred.dead Number of inbred deaths
#' @param noninbred.survived Number of noninbred survivors
#' @param noninbred.dead Number of noninbred deaths
#' @author Frank PG Princee
#' @family inbreeding
#' @export
#' 
inbreeding.test <- function(inbred.survived,inbred.dead,noninbred.survived,noninbred.dead,main_title='Inbreeding depression') 
{
  data_set  <- c(inbred.survived,noninbred.survived,inbred.dead,noninbred.dead)
  name_list <- list("Factor"=c("inbred","non-inbred"),"Survival"=c("Survived","Died"))
  studbook.twotable(data_set,name_list,main_title)
}

###
#' @title Select subset based on inbreeding of parent(s)
#' @param data either individual or litter survival data with fields DAM_F and SIRE_F
#' @param parents #' @param parents text Inbreeding status of parents: \code{"all"}, \code{"both"}, \code{inbred}, 
#' \code{dam},\code{sire},\code{"single"} and \code{"none"} (default:\code{"none"}) 
#' @author Frank PG Princee
#' @return data subset
#' @family inbreeding
#' @keywords internal
#' 
inbreeding.parents <- function(data,parents=c("none","inbred","dam","sire","both","single","all"))
{
  parents <- match.arg(parents)
  
  if (parents == "none") {
    data <- subset(data,DAM_F == 0 & SIRE_F == 0)
  } else if (parents == "inbred") {
    data <- subset(data,DAM_F > 0 | SIRE_F > 0)
  } else if (parents == "dam") {
    data <- subset(data,DAM_F > 0)
  } else if (parents == "sire") {
    data <- subset(data,SIRE_F > 0)
  } else if (parents == "both") {
    data <- subset(data,DAM_F > 0 & SIRE_F > 0)
  } else if (parents == "single") {
    data <- subset(data, (DAM_F > 0 & SIRE_F == 0) | (DAM_F == 0 & SIRE_F > 0) )
  }
  
  return(data)
}



###
#' @title Inbreeding depression
#' @param parents text Inbreeding status of parents: \code{"all"}, \code{"both"}, \code{"inbred"}, 
#' \code{"dam"},\code{"sire"},\code{"single"} and \code{"none"} (default:\code{"none"}) 
#' @param age Ages included default c(0,30)
#' @author  Frank PG Princee
#' @export
#' @family inbreeding
#' @note Frankham:2010
#' 
inbreeding.depression <- function(parents=c("none","inbred","dam","sire","both","single","all"),
                                  age=c(0,30))
{
  
  parents <- match.arg(parents)
  studbook.screenTitle(paste('Inbreeding depression in age group',age[1],"-",age[2]))
  
  cat("Inbreeding of parents:",parents,"\n")
  
  survival.data <- studbook.read('kmsurvival') 
  survival.data <- subset(survival.data,LITTER_ID > -1)  #Make sure that adjusted Chi use same data
  survival.data <- subset(survival.data, TIME >= age[1])
  
  survival.data <- survival.censorAge(survival.data,age[2])
  survival.data <- inbreeding.parents(survival.data,parents)
  survival.data <- within(survival.data, INBRED <- (F > 0) )
  
  inbred.births      <- nrow(survival.data[survival.data$INBRED,])
  inbred.survived    <- nrow(survival.data[survival.data$INBRED & survival.data$EVENT == 0,])
  noninbred.births   <- nrow(survival.data[!survival.data$INBRED,])
  noninbred.survived <- nrow(survival.data[!survival.data$INBRED & survival.data$EVENT == 0,])
  
  mean.F <- mean(survival.data$F)
  
  if (noninbred.births > 0 && inbred.births > 0) {
    
    fitness <- (inbred.survived/inbred.births ) / (noninbred.survived/noninbred.births)
    
    cat('Inbreeding depression ID:',1-fitness,"[F(mean) =",mean.F,"]\n\n")
    
    inbred.dead    <- inbred.births - inbred.survived
    noninbred.dead <- noninbred.births - noninbred.survived
    
    inbreeding.test(inbred.survived,inbred.dead,noninbred.survived,noninbred.dead,
                    'H0: inbred and non-inbred survival are the same.')
    cat("\n")
    
  } else {
    cat("Cannot estimate inbreeding depression ID and perform Chi-square test\n")
    if (noninbred.births == 0) {
      cat("- No noninbred offspring\n")
    }
    if (inbred.births == 0) {
      cat("- No inbred offspring\n")
    }
  }
  
  if (!is.singleton(survival.data)) {
    cat("Adjust for potential litter effect\n")
    cat("----------------------------------\n")
    
    litter.data <- survival.litters(survival.data)
    litter.data <- within(litter.data, INBRED <- (F > 0))
    
    donner.rs   <- aods3::drs(cbind(DEATHS,BIRTHS - DEATHS) ~ INBRED,
                              data=litter.data,type="d")
    aods3::print.drs(donner.rs)
    cat("\n")
  }
}


###
#' @title Cox regression on inbreeding effects
#' @param method Analysis method to be used: \code{"cox"} or \code{"kaplan"} (default: \code{"cox"})
#' @param sex text \code{"all"}, \code{"female"}, \code{"male"} or \code{"unknown"} 
#' (default: \code{"all"}).
#' @param parents text Inbreeding status of parents: \code{all}, \code{"both"}, \code{inbred}, 
#' \code{dam},\code{sire},\code{"single"} and \code{"none"} (default:\code{"none"}) 
#' @param stratum Field to stratify (default:NA)
#' @param age Ages included default c(0,Inf)
#' @param prediction vector with inbreeding coefficients (default:NULL)
#' @author Frank PG Princee
#' @family inbreeding
#' @seealso \code{\link{studbook.survival}}
#' @export
#' 
inbreeding.survival <- function(method=c("cox","kaplan"),
                                sex=c('all','female','male','unknown'),
                                parents=c("none","inbred","dam","sire","both","single","all"),
                                stratum=NA,age=c(0,Inf),prediction=NULL,ymin=0)
{
  
  method  <- match.arg(method)
  sex     <- match.arg(sex)
  parents <- match.arg(parents)
  
  #Inbred vs non--inbred
  studbook.survival(analysis="survival",method=method,sex=sex,unknown="include", 
                    group="INBRED",filter=list(param=NA,condition=parents),
                    stratum=stratum,age=age,ymin=ymin)
  
  if (method == 'cox') {
    #Inbreeding coefficient
    studbook.survival(analysis="survival",method=method,sex=sex,unknown="include", 
                      group="INBRED",filter=list(param='F',condition=parents,prediction=prediction),
                      stratum=stratum,age=age,ymin=ymin)
  }
}


###
#' @title Create survival in inbred groups 
#' @details Create survival in inbred groups from survival.data data must have 
#' been censored for age e.g. c(1,7) and inbreeding status of parents.
#' @param suvival data KM survival data
#' @return data frame with survival data
#' @author Frank PG Princee
#' @family inbreeding
#' @keywords internal
#
create.inbred <- function(survival.data)
{
  groups <- table(survival.data$F)
  n <- length(groups)
  lethal.data <- data.frame("F"=as.numeric(names(groups)),
                            "BIRTHS"=rep(0,n),"DEATHS"=rep(0,n),"SURVIVED"=rep(0,n))
  
  for (x in 1:nrow(survival.data)) {
    group <- which(lethal.data$F == survival.data[x,"F"]) 
    lethal.data[group,'BIRTHS'] <- lethal.data[group,'BIRTHS'] + 1
    if (survival.data[x,"EVENT"]) {
      lethal.data[group,'DEATHS'] <- lethal.data[group,'DEATHS'] + 1
    }
  }
  lethal.data <- within(lethal.data, SURVIVED <- BIRTHS-DEATHS)
  return(lethal.data)
}


###
#' @title Estimates lethal equivalents using linear regression
#' @description \code{lethaleqv.lm}
#' @details Zero surival is adjusted
#' @param lethal.data data frame with columns
#' @return lm object 
#' @author Frank PG Princee
#' @note Templeton:1983, Kalinowski:1998
#' @family inbreeding
#' @keywords internal
#' 
lethaleqv.lm <- function(lethal.data)
{
  if (!is.element('SURVIVED',colnames(lethal.data))) {
    lethal.data <- within(lethal.data,SURVIVED <- BIRTHS - DEATHS)
  }
  #Adjustment Templeton and Read [1998]
  lethal.data$BIRTHS[lethal.data$SURVIVED == 0] <- lethal.data$BIRTHS[lethal.data$SURVIVED==0] + 2
  lethal.data$SURVIVED[lethal.data$SURVIVED == 0] <- 1
  
  #Adjusted
  lethal.lm <- lm(log(SURVIVED/BIRTHS) ~ F,data=lethal.data,weights=lethal.data$BIRTHS)
  
  return(lethal.lm)
  
}


###
#' @title Lethal equivalents using Maximum Likelihood Estimator 
#' @param data data frame 
#' @keywords internal
#' 
#' @note Kalinowski:1998
#' @family inbreeding
#' @keywords internal  

lethaleqv.mle <- function(lethal.data,S0=0.5,B=3.0)
{
  
  if (B <= 0) {
    cat("MLE cannot be applied to negative lethal equivalents.\n")
    return(NULL)
  }
  
  # Increase B when small as optim might crash
  if (B <= 1) {
    B <- 3.0
  }
  
  ###
  # EMBEDDED Log likelihood function for lethal equivalents
  #
  lethaleqv.theta <- function(S0,B)
  {
    lnSum <- 0
    
    if (S0 <= 0 | S0 >= 1) {
      return(NA)
    }
    
    if (B < 0) {
      return(NA)
    }
    
    for(i in 1:length(lethal.data$F)) {
      Si <- S0 * exp(-B*lethal.data$F[i])
      Ns <- lethal.data$SURVIVED[i]
      Ni <- lethal.data$BIRTHS[i]
      
      sum_i <-(Ns * log(Si)) + (Ni - Ns)*log(1-Si)
      lnSum <- lnSum + sum_i 
    }
    return(-lnSum)   
  }  	
  
  ###
  
  lethal.mle  <- stats4::mle(lethaleqv.theta,start=list(S0=S0,B=B))
  
  return(lethal.mle)
}


###
#' @title Estimates lethal equivalents using Robust GLM
#' @description \code{lethaleqv.glmRob} uses a Robust generalised linear model to estimate 
#' \code{S0} and \code{B}
#' @details binomial distribution with logit link.
#' @param lethal.data data frame with columns \code{BIRTHS},\code{SURVIVED} and \code{F}.
#' @param link Bionomial link
#' @return glmRob object
#' @author Frank PG Princee
#' @keywords internal
#' @family inbreeding
#' @note Armstrong:2007  
#'   
lethaleqv.glmRob <- function(lethal.data,link)
{
  if (!is.element("DEATHS",colnames(lethal.data))) {
    lethal.data <- within(lethal.data,DEATHS <- BIRTHS - SURVIVED)
  }
  else if (!is.element("SURVIVED",colnames(lethal.data))) {
    lethal.data <- within(lethal.data,SURVIVED <- BIRTHS - DEATHS)
  } 
  
  
  lethal.glm  <- robust::glmRob(cbind(SURVIVED,DEATHS) ~ F,data=lethal.data,
                                family=binomial(link=link))
  
  return(lethal.glm)
}


###
#' @title Estimates lethal equivalents using GLM on inbred groups/litters
#' @description \code{lethaleqv.glm} uses a generalised linear model to estimate 
#' \code{S0} and \code{B}
#' @details binomial distribution with logit link.
#' @param lethal.data individual data or group data with columns \code{BIRTHS}, \code{DEATHS},\code{SURVIVED} and \code{F}.
#' @param link Bionomial link
#' @return glm object
#' @author Frank PG Princee
#' @family inbreeding
#' @keywords internal
#' @note Armstrong:2007  
#'   
lethaleqv.glm <- function(lethal.data,link)
{
  if (is.element("ID",colnames(lethal.data))) {
    lethal.glm  <- glm((EVENT == 0) ~ F,data=lethal.data,
                       family=binomial(link=link)) 
  } else {
    lethal.glm  <- glm(cbind(SURVIVED,DEATHS) ~ F,data=lethal.data,
                       family=binomial(link=link))
  }
  return(lethal.glm)
}


###
#' @title Estimates lethal equivalents using GLMER on inbred groups/litters
#' @description \code{lethaleqv.glm} uses a mixed generalised linear model to estimate 
#' \code{S0} and \code{B} 
#' @details binomial distribution with logit link.
#' @param lethal.data individual data with EVENT, F and LITTER_ID.
#' @return glmerMod object
#' @author Frank PG Princee
#' @family inbreeding
#' @keywords internal
#'   
lethaleqv.glmer <- function(lethal.data)
{
  lethal.glmer  <- lme4::glmer((EVENT == 0) ~ F + (1|LITTER_ID),data=lethal.data,
                                family=binomial("logit")) 
  return(lethal.glmer)
}



##
#' @title Estimates lethal equivalents using GEE GLM on individuals 
#' @description \code{lethaleqv.geeglm} uses a GEE version of generalised linear model to estimate 
#' \code{S0} and \code{B}
#' @details binomial distribution with logit link.
#' @param lethal.data data frame individual survival data 
#' @param link Bionomial link
#' @return geeglm object
#' @author Frank PG Princee
#' @family inbreeding
#' @keywords internal
#'   
lethaleqv.geeglm <- function(lethal.data,link)
{
  # Survival: EVENT == 0
  lethal.glm  <- geepack::geeglm((EVENT == 0) ~ F,id=LITTER_ID,data=lethal.data,
                                 family=binomial(link=link))
  return(lethal.glm)
}



###
#' @title Print results lethal equivalent analysis
#' @param object regression or MLE object
#' @param do_summary Print summary of regression results (default:FALSE)
#' @author Frank PG Princee
#' @family inbreeding
#' @keywords internal
#' 
lethaleqv.print <- function(object,do_summary=FALSE)
{
  
  method <- class(object)[1]
  

  if (method == "glmerMod") {
    coefficient <- stats4::coef(summary(object))
    interval <- lme4::confint.merMod(object,method="Wald")
  } else {
    coefficient <- stats4::coef(object)
    interval    <- stats4::confint(object)
  }
  
  cat("[",comment(object),"]\n\n")
  
  S0 <- rep(NaN,3)
  B  <- rep(NaN,3)
  
  
  if (method == "lm") {
    S0[1]   <- exp(coefficient[1])
    S0[2:3] <- exp(-interval[1,1:2])
    B[1]    <- -coefficient[2]
    B[2:3]  <- -interval[2,1:2]
  } else if (method == "mle") {
    #mle.object <<- object
    S0[1]   <- coefficient[1]
    S0[2:3] <- interval[1,1:2]
    B[1]    <- coefficient[2]
    B[2:3]  <- interval[2,1:2]
  } else if (is.element(method,c("glm","glmerMod","geeglm","glmRob"))) {
    S0[1]   <- plogis(coefficient[1])
    B[1]    <- log(-coefficient[2])
    S0[2:3] <- plogis(interval[1,1:2])
    B[2:3]  <- -interval[2,1:2]
    B[2:3]  <- ifelse(B[2:3] > 0,log(B[2:3]),-Inf)
    
    if (method == "glmerMod") {
      cat("Family/link                :",family(object)[["family"]],"/",family(object)[["link"]],"\n")
    } else {
      cat("Family/link                :",object$family[["family"]],"/",object$family[["link"]],"\n")
      cat("Residual deviance (df)     :",object$deviance,"(",object$df.residual,")\n")
      overdispersed <- object$deviance/object$df.residual # Crawley The R Book 
      cat("Overdispersion(*)          :",overdispersed,"\n")
    }
  }
  
  
  S0 <- round(S0,2)
  S0[2:3] <- sort(S0[2:3],na.last=FALSE)
  
  B  <- round(B,2) 
  B[2:3] <- sort(B[2:3],na.last=FALSE)
  B2 <- 2*B
  
  cat("Survivorship of non-inbred : ", S0[1], " [",S0[2],",",S0[3],"]\n",sep="")
  cat("Lethal equivalents diploid : ", B2[1]," [",B2[2],",",B2[3],"]\n",sep="")
  if (method != "glmRob") {
    cat('AIC:',AIC(object),'\n')
  }
  cat("---------------------------------------------------------------\n")
  cat("* Crawley (p273, 2007)\n\n")
  
  if (do_summary) {
    print(object)
    print(summary(object))
    cat("---------------------------------------------------------------\n\n")
  }
  
}


###
#' @title Plot lethal equivalents
#' @description \code{lethaleqvplot} plots results of multiple (optional) 
#' lethal equivalent methods.
#' 
#' @param lethal.matrix matrix with lethal equivalent results 
#' @author Frank PG Princee
#' @family inbreeding
#' @keywords internal
#' 
lethaleqv.plot <- function(lethal.matrix)
{
  sex <- comment(lethal.matrix)
  studbook.openOutput(paste("lethaleqv",sex=sex,sep='.'))
  
  main_title <- paste("Lethal equivalents (",sex,").")
  studbook.caption(main_title,"Size of circles indicate sample size.")
  
  cex_points <- weighted.points(lethal.matrix[,'births'],pow=0.25)
  plot(x=lethal.matrix[,'F'],y=lethal.matrix[,'observed'],
       pch=21,bg="grey80",cex=cex_points,
       xlim=c(0,max(lethal.matrix[,'F'])),ylim=c(0,1),
       main='',xlab='F',ylab='Survivorship')
  
  if (ncol(lethal.matrix) > 3 ) {
    n.lines <- ncol(lethal.matrix)
    matlines(x=lethal.matrix[,'F'],lethal.matrix[,(4:n.lines)],
             col='black',lty=2:(n.lines+1),lwd=1.5)
    
    if (!studbook.isOption("output","publish")) {
      legend.txt <- colnames(lethal.matrix)[4:n.lines]
      legend(x="topright",legend=legend.txt,col='black',
             lty=2:(n.lines+1),lwd=1.5,cex=0.8)
    }
  }
  studbook.title(main_title)    
  studbook.closeOutput()
  
}



###
#' @title Compute and plot lethal equivalents
#' 
#' @details 
#' \code{parents} refer to the inbreeding status of the parents:
#' - \code{"all"}: inbred and non-inbred parents  
#' - \code{"both"}: both parents are inbred, 
#' - \code{inbred}: at least one parent is inbred
#' - \code{dam}: dam inbred, sire noninbred 
#' - \code{sire}: sire inbred, dam noninbred
#' - \code{"single"}: single parent is inbred
#' - \code{"none"}: both parents are noninbred
#' 
#' \code{method} 
#' - \code{"lm"}: log linear model adjusted for zero survivals (groups)
#' - \code{"mle"}: maximum likelikhood estimation (groups)
#' - \code{"glm"}: generalised linear model (groups)
#' - \code{"glmer"}: mixed general linear model with litter as random effect (individuals)
#' - \code{"glmind"}: generalised linear model (individuals)
#' - \code{"geeglm"}: GEE version of generalised linear model (individuals)
#' - \code{"glmRob"}: Robust (sandwich) variance version of generalised linear model (litters) 
#' - \code{"none"}: Only plot observed data
#' 
#' 
#' @param sex text \code{"all"}, \code{"female"}, \code{"male"} or \code{"unknown"} 
#' (default: \code{"all"}).
#' @param parents text Inbreeding status of parents: \code{"all"}, \code{"both"}, \code{inbred}, 
#' \code{dam},\code{sire},\code{"single"} and \code{"none"} (default: \code{"none"}) 
#' @param age Neonatal ages for fitness, chi-square test and Cox regression (default: c(0,30))
#' @param method One or methods: \code{"lm"}, \code{"mle"}, \code{"glm"}, \code{"glmer"}, 
#' \code{"glmindv"},\code{"glmglee"} and/or \code{"glmRob"} (default: \code{"mle"})
#' @param link Binomial link model: \code{"clogclog"}, 
#' \code{"log"} or \code{"logit"} in GLM (default: "logit")  
#' @param do_summary Print summary of regression results (default:FALSE)
#' @author Frank PG Princee
#' @family inbreeding
#' @export
#' 
studbook.lethaleqv <- function(sex=c('all','female','male','unknown'),
                               parents=c("none","inbred","dam","sire","both","single","all"),
                               age=c(0,30),
                               method=c("mle","lm","glm","glmRob","glmer","glmindv","geeglm","none"),
                               link=c("logit","log","cloglog"),
                               do_summary = FALSE)
{
  sex     <- match.arg(sex)
  parents <- match.arg(parents)
  method  <- match.arg(method,several.ok=TRUE)
  link    <- match.arg(link)
  
  survival.data <- studbook.read('kmsurvival') 
  if (is.null(survival.data)) {
    stop("No inbreeding data")
  }
  survival.data <- subset(survival.data,LITTER_ID > -1)  #Make sure that adjusted Chi use same data
  survival.data <- subset(survival.data, TIME >= age[1])
  
  survival.data <- survival.censorAge(survival.data,age[2])
  survival.data <- inbreeding.parents(survival.data,parents)
  
  lethal.group  <- create.inbred(survival.data)
  
  # PML file
  #file_name <- paste("lethaleqv",parents,sep="_")
  #lethal.group <- studbook.read(file_name,sex=sex)  
  #lethal.group <- within(lethal.group,DEATHS <- BIRTHS - SURVIVED)
  
  # Examples from literature
  #lethal.group <- read.csv("~/Statistics/data/okapi.csv",sep="\t")
  #lethal.group <- read.csv("~/Statistics/data/lemur.csv",sep="\t")
  
  
  
  studbook.screenTitle(paste('Lethal equivalents in',sex,"(Inbred parents:",parents,")"))
  
  # Use linear regression to get start parameters for MLE and GLM
  # Optionally display and plot results
  
  lethal.lm  <- lethaleqv.lm(lethal.group)
  coef.lm    <- as.numeric(coef(lethal.lm))
  
  lethal.group <- within(lethal.group,observed <- (BIRTHS - DEATHS)/BIRTHS)
  lethal.matrix <- as.matrix(lethal.group[,c('F','BIRTHS','observed')])
  colnames(lethal.matrix) <- c('F','births','observed')
  
  # Display linear regression result
  if (is.element("lm",method))  {
    comment(lethal.lm) <- "Log-linear regression (Templeton and Read,1983,1998)"
    lethaleqv.print(lethal.lm,do_summary)
    
    fitted.lm  <- fitted(lethal.lm)
    
    old.names <- colnames(lethal.matrix)
    lethal.matrix <- cbind(lethal.matrix,exp(fitted.lm))
    colnames(lethal.matrix) <- c(old.names,"lm")
  }
  
  
  if (is.element("mle",method)) {
    lethal.mle <- lethaleqv.mle(lethal.group,S0=-coef.lm[1],B=-coef.lm[2]) #Linear is neg results
    
    if (is.null(lethal.mle)) {
      cat("Not able to estimate lethal equivalents through MLE method\n")
    } else {
      
      comment(lethal.mle) <- "Maximum likelihood method (Kalinoswki and Hedrick,1998)"
      coef.mle <- stats4::coef(lethal.mle)
      lethaleqv.print(lethal.mle,do_summary)
      fitted.mle <- lethaleqv.fit(lethal.group[,'F'],coef.mle['S0'],coef.mle['B'])
      old.names <- colnames(lethal.matrix)
      lethal.matrix <- cbind(lethal.matrix,fitted.mle)
      colnames(lethal.matrix) <- c(old.names,"mle")
    }
  }
  
  #### Inbreeding groups
  if (is.element("glm",method)) {
    lethal.glm <- lethaleqv.glm(lethal.group,link)
    comment(lethal.glm) <- "Generalised linear model (Amstrong and Cassey, 2007)"
    lethaleqv.print(lethal.glm,do_summary)
    
    fitted.glm <- predict(lethal.glm,type="response")
    
    old.names <- colnames(lethal.matrix)
    lethal.matrix <- cbind(lethal.matrix,fitted.glm)
    colnames(lethal.matrix) <- c(old.names,"glm")
  }
  
  #### Individuals
  if (is.element("glmer",method)) {
    lethal.glmer <- lethaleqv.glmer(survival.data)
    comment(lethal.glmer) <- "Mixed Generalised linear model with litter id as random effect"
    lethaleqv.print(lethal.glmer,do_summary)
  }
  
  
  #### Individuals
  if (is.element("glmindv",method)) {
    lethal.glm <- lethaleqv.glm(survival.data,link)
    comment(lethal.glm) <- "Generalised linear model - individuals (Amstrong and Cassey, 2007)"
    lethaleqv.print(lethal.glm,do_summary)
  }
  
  ### Robust individuals
  if (is.element("geeglm",method))  {
    lethal.gee <- lethaleqv.geeglm(survival.data,link)
    comment(lethal.gee) <- "GEE Generalised linear model (individuals)"
    lethaleqv.print(lethal.gee,do_summary)
    
  }
  
  #### Robust litters
  if (is.element("glmRob",method))  {
    lethal.litter <- survival.litters(survival.data) 
    lethal.glmR <- lethaleqv.glmRob(lethal.litter,link)
    comment(lethal.glmR) <- "Robust Generalised linear model (litters)"
    lethaleqv.print(lethal.glmR,do_summary)
    lethal.litter <- within(lethal.litter,observed <- (BIRTHS - DEATHS)/BIRTHS)
    litter.matrix <- as.matrix(lethal.litter[,c('F','BIRTHS','observed')])
    colnames(litter.matrix) <- c('F','births','observed')
    
    # Special plot for robust as it refers to litters
    if (studbook.plot()) {
      fitted.glmR <- predict(lethal.glmR,type="response")
      old.names <- colnames(litter.matrix)
      litter.matrix <- cbind(litter.matrix,fitted.glmR)
      colnames(litter.matrix) <- c(old.names,"glmRob")
      comment(litter.matrix) <- sex
      lethaleqv.plot(litter.matrix)
    }
  }
  
  if ( studbook.plot() ) {
  
    if (method[1] == "none" | ncol(lethal.matrix) > 3) {
      comment(lethal.matrix) <- sex
      lethaleqv.plot(lethal.matrix)
    }
  }
}



###
#' @title Estimate inbreeding depression, lethal equivalents and survival
#' @description \code{studbook.inbreeding} estimates 
#' @details Inbreeding depression ID
#' - Contingency test survival inbred, non-inbred after pre-defined age as used in lethal equivalents (default:30)
#' - Lethal equivalents 
#' @inheritParams studbook.lethaleqv
#' @author Frank PG Princee
#' @family inbreeding
#' @export 
#' 
studbook.inbreeding <- function(sex=c('all','female','male','unknown'),
                                parents=c("none","inbred","dam","sire","both","single","all"),
                                age=c(0,30),
                                method=c("lm","mle","glm","glmRob","glmer","glmindv","geeglm"),
                                link=c("logit","log","cloglog"))
{
  sex     <- match.arg(sex)
  parents <- match.arg(parents)
  method  <- match.arg(method,several.ok=TRUE)
  
  #Fitness and neonatal survival in inbred/noninbred
  inbreeding.depression(parents,age)
  
  #Survival curves
  inbreeding.survival(sex=sex,parents=parents,age=age)
  
  # Lethal equivalents
  studbook.lethaleqv(sex,parents,age,method,link,do_summary=FALSE)
}



