###
#' @title List data files
#' @param path Directory path
#' @param pattern File grep pattern (see R documentation)
#' @return vector with file names or NULL when no result
#' @author Frank PG Princee
#' @family data import
#' @keywords internal
#' 
studbook.files <- function(path=NULL,pattern=NULL)
{
  
  if (is.null(path)) {
    path <- paste(studbook.getPath(),studbook.getName(),sep="/")
    path <- paste(path,'export',sep='/') 
  }
  
  files <- list.files(path,pattern=pattern,full.names=FALSE)
  
  if (length(files) == 0) {
    return(NULL)
  } else {
    return(files)
  } 
}



###
#' @title Read comma separated value file
#' 
#' @description \code{studbook.csv} imports data from delimited text files using \code{read.table}. 
#' 
#' @details Files are assumed to be stored in the folder \code{<current_studbook_path>/export}.
#' Comments can be included using a hash \code{"#"}
#' 
#' @param file_name basic file name with extension
#' @param has_header TRUE if first line contains column headers (default)
#' @param col_classes required data type of columns. 
#' Default (value = \code{NA}) data are imported in the original data type.  
#' @param field_sep Field or column separator (default tab)
#' @param quote_char Quote character for text (default double quote) 
#' @return data frame or NULL when file does not exist
#' @author Frank PG Princee
#' @seealso \code{\link{studbook.read}} \code{\link{read.table}}
#' @examples
#' \dontrun{
#' census.data <- studbook.csv("census.male.csv")
#' print(census.data)
#' }
#' @family data import
#' @export
#' 
studbook.csv <- function(file_name="",has_header=TRUE,col_classes=NA,field_sep="\t",quote_char="\"")      # file_name: blank for screen devices
{
  if (studbook.empty()) {
    studbook()
  }
  csv_file <- paste(studbook.getPath(),studbook.getName(),'export',file_name,sep='/') 
  if (!file.exists(csv_file)) {
     warning(paste(file_name, "does not exists\n"))
     return(NULL)
  }
  
  data_set <- read.table(csv_file,header=has_header,as.is=TRUE,colClasses=col_classes,
                         sep=field_sep,quote=quote_char,comment.char='#',row.names=NULL)
  
  return(data_set)
}


###
#' @title Read data from PML result files 
#' @description \code{studbook.read} is a wrapper for function \code{studbook.csv} to import data from PML result files. 
#' @details The naming convention of csv files is \code{<analysis>.<sex>.csv}. 
#' Data files use the default settings of function \code{studbook.csv()}:
#' \itemize{
#' \item File has column headers.
#' \item Each record on a single line ended with a linefeed.
#' \item Fields are separated with a tab.
#' \item Text is embedded in double quotes. 
#' \item Comment lines start with character "#"
#' }
#' 
#' @param analysis name of analysis
#' @param sex  name of sex to be imported. Valid names are \code{"all","female","male","unknown"}.
#' @param col_classes required data type of columns. 
#' Default (value = \code{NA}) data are imported in the original data type.   
#' @return data frame or NULL when file does not exist
#' @author Frank PG Princee
#' @seealso \code{\link{studbook.csv}} \code{\link{read.table}}
#' @examples
#' \dontrun{
#' census.data <- studbook.read(analysis="census",sex="male")
#' print(census.data)
#' }
#' @family data import
#' @export
#' 
studbook.read <- function(analysis="",sex="all",col_classes=NA) 
{
  if (studbook.empty()) {
    studbook()
  }
  file_name <- paste(analysis,'-',sex,'.csv',sep='')
  return( studbook.csv(file_name,col_classes=col_classes) )
}

###
#' @title Extract comments from file
#' @param analysis name of analysis
#' @param sex Sex in data file (default: "all")
#' @author Frank PG Princee
#' @examples
#' \dontrun{
#' studbook.comments(analysis="census",sex="male")
#' }
#' @family data import
#' @export
#' 
studbook.comments <- function(analysis="",sex="all")
{
  if (studbook.empty()) {
    studbook()
  }
  file_name <- paste(analysis,"-",sex,'.csv',sep='')
  csv_file <- paste(studbook.getPath(),studbook.getName(),'export',file_name,sep='/') 
 
  if (!file.exists(csv_file)) {
     warning(paste(file_name, "does not exists\n"))
  }
  
  cat(createLine(60,"-"),"\n")
  cat(paste("File:",file_name,"\n"))
  cat(createLine(60,"-"),"\n")
  connect  <- file(csv_file, open = "r")
  while (length(oneLine <- readLines(connect, n = 1, warn = FALSE)) > 0) {
    oneLine <- gsub("###","",oneLine)
    if (substr(oneLine,1,1) == "#") {
      if (is.empty(grep("n/a",oneLine)) ) {
         cat(gsub("# ","",oneLine),"\n")
      }
    } 
  } 
  close(connect)
  
  cat(createLine(60,"-"),"\n")
}


