###
# Package : studbookR
# 17-05-2014: Added cohort
# 25-08-2013: Added minimum and maximum age criteria
# Copyright (c) 2011,2014 Frank PG Princee, All rights reserved
#
# Kaplan-Meier estimator to analyse survival in studbooks
# Optionally implement survplot functions in package rms

###
#' @title Create table with fitted survival data
#' @param kaplan.fit \code{survfit} object
#' @author Frank PG Princee
#' @family Kaplan Meier
#' @keywords internal
#' 
kaplan.summary <- function(kaplan.fit)
{  
  kaplan <- summary(kaplan.fit,censored=TRUE) 
  if (class(kaplan.fit) == "survfit") {
    kaplan.table <- data.frame("time"=kaplan$time,"risk"=kaplan$n.risk,
                               "event"=kaplan$n.event,"survivorship"=kaplan$surv)
  } else if (class(kaplan.fit) == "survfitr") {
    kaplan.table <- data.frame("time"=kaplan$time,"risk"=kaplan$AtRisk,
                               "event"=kaplan$n.event,"survivorship"=kaplan$survfunc)
  } else {
    # Nothing
  }
  
  export.file <- paste(studbook.getPath(),studbook.getName(),"export","survival_table.csv",sep='/') 
  write.table(kaplan.table,file=export.file,sep="\t",row.names=FALSE)
  
  cat("\n")
  n <- nrow(kaplan.table)
  cat("* Last age group in Kaplan-Meier estimator\n")
  print(kaplan.table[n,])
  cat("\n")  
}

###
#' @title Q-test for selecting rank test between two samples
#' @description \code{Q.test} 
#' @details The first two samples are tested when more than two samples exist
#' @param kaplan.fit survfit object
#' @note Martinez and Naranjo (2010)
#' @return  Q value (NA if no strata)
#' @author Frank PG Princee
#' @family Kaplan Meier
#' @keywords internal
#' 
Q.test <- function(kaplan.fit)
{
   if (is.null(kaplan.fit$strata)) {
     return(NA)
   }
  
   # Find out which of strata is lower curve by comparing median
   Sx.time <- quantile(kaplan.fit,c(0.4,0.8),conf.int=FALSE) # To retrieve S(t)=0.6 and S(t)=0.2
   
   lower <- ifelse(Sx.time[1,1] > Sx.time[2,1],2,1)
   upper <- ifelse(lower ==  1,2,1)  
   
   S06.time <- Sx.time[lower,1]  
   S02.time <- Sx.time[lower,2]
  

   Sx.upper   <- survival.getStratum(kaplan.fit,upper)
   S02.upper <- min(Sx.upper[Sx.upper$time <= S02.time,'surv'])   
   S06.upper <- min(Sx.upper[Sx.upper$time <= S06.time,'surv'])   
  
   Q <- (S06.upper - 0.6) - (S02.upper - 0.2)
   
   return(Q)
}



###
#' @title Kaplan Meier estimator for survival and fecundity
#' @description \code{studbook.kaplan} estimates survival and fecundity using the 
#' Kaplan-Meier product estimator.  
#' @param survival.data Survival data that corresponds with Surv object 
#' @param subset Subset of survival.data
#' @param term string with fixed terms
#' @param stratum Field to stratify (default: NULL)
#' @param rho The \code{rho}value in the G-rho family of Harington and Fleming (1982) test.
#' \code{rho=0} is the log-rank or Mantel-Haenszel test, \code{rho=1} is the Peto & Peto 
#' modification of the Gehan-Wilcoxon test. The latter is more approperiate for early deaths.
#' A Q test can be used to select for one of these test by entering \code{rho=NA} 
#' (default: rho=0) 
#' @param alpha Significance level (default: 0.05)
#' @return \code{survfit} or \code{survfitr} object
#' @author Frank Princee
#' @keywords internal
#' @family Kaplan Meier
#' @seealso studbook.survival
#' 
kaplan.meier <- function(survival.data,subset,term,stratum=NA,rho=0,alpha=0.05)
{  
  formula.km <- paste("survival.object ~",term)
  if (!is.na(stratum)) {
    formula.km <- paste(formula.km,"+ strata(",stratum,")")
  }
  cat("Formula  :",formula.km,"\n\n")
  
  survival.object <- with(survival.data, survival::Surv(TIME,EVENT))
  kaplan.fit <- survival::survfit(as.formula(formula.km),data=survival.data,
                                  subset=subset,na.action=na.omit)
  print(kaplan.fit,print.rmean=TRUE)
  cat("\n")
  kaplan.summary(kaplan.fit)
  
  # Standard survival implementation
  if (class(kaplan.fit) == "survfit" & !is.null(kaplan.fit$strata)) {
    if (is.na(rho)) {
      Q <- Q.test(kaplan.fit) 
      if (!is.na(Q)) {
        cat("Q test:",Q,"\n")
        rho <- ifelse(Q < 0,0,1 )
      } else {
        rho = 0
      }
    }
    
    if (rho == 0) {
      test_name <- "Mantel-Haenszel (logrank) test"
    } else if (rho == 1) {
      test_name <- "Gehan-Wilcoxon test (Peto & Peto)"
    } else {
      test_name <- "G-rho (Harington and Fleming)"
    }
  
    
    difftest  <- survival::survdiff(as.formula(formula.km),data=survival.data,subset=subset,
                                    na.action=na.omit,rho=rho)
    cat("[",test_name,"]\n")
    print(difftest)
    df <- (length(difftest$n) - 1)
    accept.X2(X2p= (1 - pchisq(difftest$chisq,df)),alpha=alpha)  
    cat("\n")
        
  } 
  return(kaplan.fit)
}
  
