###
# Interface to Leslie matrix as used in popbio
# See Caswell(2001) for calculations of birth-flow/pulse and pre- and postbreeding
###

###
#' @title Prepare matrix for prebreeding census
#' @param lx Age-specific survivorship
#' @param mx Age-specific fecundity rate
#' @return data frame with Px, Fx
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
prebreeding <- function(lx,mx)
{
   classes <- length(lx)
   Px <- rep(0,classes)
   
   # P1 = l(2)/l(1) : lx[3]/lx[2] 
   Px[1:(classes-2)] <- lx[3:(classes)] / lx[2:(classes-1)] 

   # Fi = l(1) * mi : lx[2] * mx
   Fx <- lx[2] * mx 
   return(data.frame("Px"=Px,"Fx"=Fx))
}

###
#' @title Prepare matrix for postbreeding census
#' @param lx Age-specific survivorship 
#' @param mx Age-specific fecundity rate
#' @return data frame with Px, Fx
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
postbreeding <-  function(lx,mx)
{
   classes <- length(lx)
   
   # Pi = l(i)/l(i-1)  = l[x]/l[x-1]
   Px <- rep(0,classes)
   Px[1:(classes-1)] <- lx[2:classes]/lx[1:(classes-1)]   

   #Fi = Pi * mi :  P[x] * m[x]
   
   Fx <- Px[1:classes] * mx[1:classes]                      
   return(data.frame("Px"=Px,"Fx"=Fx))
}


###
#' @title Prepare Leslie matrix for birth pulse
#' @param lx Age-specific survivorship
#' @param mx Age-specific fecundity rate
#' @param census Time of census before or after breeding season (birth-pulse only)
#' \code{"pre"} for prebreeding or \code{post} for postbreeding (default:"pre") 
#' @return data frame with Px, Fx
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
birth.pulse <- function(lx,mx,census=c("pre","post") )
{
  census <- match.arg(census)
  if (census == "pre") {
    vital.rates <- prebreeding(lx,mx)
  } else {
    vital.rates <- postbreeding(lx,mx)
  }
  return(vital.rates)
}


###
#' @title Prepare Leslie matrix for birth flow
#' @param lx Age-specific survivorship
#' @param mx Age-specific fecundity rate
#' @return data frame with lx,mx,Px, Fx
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
birth.flow <-  function(lx,mx)
{
  
  classes <- length(lx)
  Px      <- rep(0,classes)   
  Fx      <- rep(0,classes)
  
  # l(0.5) = ( l(0) + l(1) ) /2 : (lx[1] + lx[2])/2
  L0 <- (lx[1] + lx[2]) / 2      # Mid l0
  
 
  for (i in 1 : (classes-1))  {  # Note that vector starts with 1!
    
    # Pi = ( l(i) + l(i+1) ) / ( l(i-1) + l(i)) : (lx[i+1] + lx[i+2]) / lx[i] = lx[i+1])
    Px[i] <- (lx[i+1] + lx[i+2])/(lx[i] + lx[i+1])
    if (is.na(Px[i])) {
      Px[i] <- 0
    }
    
    #Fi = l(0.5) * (mi + P(i)m(i+1))
    
    Fx[i] <- L0 * ( (mx[i] + Px[i]*mx[i+1])/2 )
    if (is.na(Fx[i])) {
      Fx[i] <- 0
    }
  }
  return(data.frame("Px"=Px,"Fx"=Fx))
}


###
#' @title Clean until nonsingular i.e. |det(A)| > 0 
#' @param leslie.matrix Leslie matrix
#' @return matrix
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
leslieMatrix.clean <- function(leslie.matrix) 
{
  n <- nrow(leslie.matrix) 
  for (x in n:1) {
    if (abs(det(leslie.matrix[1:x,1:x])) > 0 ) {
      return( leslie.matrix[1:x,1:x])
    }
  }
  return(leslie.matrix[1,1])
}


###
#' @title Create Leslie matrix
#' @description Create Leslie matrix from survivorship and fecundity rates
#' @param lx Age-specific survivorship
#' @param mx Age-specific fecundity
#' @param season Breeding season: \code{"flow"} for birth-flow; \code{"pulse"} 
#' for birth-pulse. (default: "pulse")
#' @param census Time of census count before or after breeding season (birth-pulse only)
#' \code{"pre"} for prebreeding or \code{post} for postbreeding (default:"pre") 
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "unknown")
#' @return Leslie matrix
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
leslieMatrix.create <- function(lx,mx,season=c("pulse","flow"),census=c("pre","post"),
                                sex=c("unknown","female","male","all"))
{
  if (is.null(lx) | is.null(mx)) {
    stop("No survivorship and/or fecundity data provided.")
  }
  
  season <- match.arg(season)
  census <- match.arg(census)
  sex    <- match.arg(sex)
  
  if (season == "flow") {  
     vital.rates <- birth.flow(lx,mx)
  } else {
     vital.rates <- birth.pulse(lx,mx,census) 
  }
  
  
  
  classes <- nrow(vital.rates)
  vital.rates <- as.matrix(vital.rates)
  
  leslie.matrix <- matrix(0,nrow=classes,ncol=classes,
                          dimnames=list( 1:classes,1:classes) )
  
  leslie.matrix[1,] <- vital.rates[,'Fx']  # Fecundity rates in first row  
  

# Survival rates (Px) are placed on diagonal
  Px.diag <- diag(vital.rates[1:(classes-1),'Px'])
 
  leslie.matrix[2:classes,1:(classes-1)] <- Px.diag
  
  #Check whether right columns are > 0

  start0 <- 0:(nrow(leslie.matrix)-1)
  colnames(leslie.matrix) <- start0
  rownames(leslie.matrix) <- start0

  attr(leslie.matrix,"sex") <- sex
  attr(leslie.matrix,"season") <- season
  attr(leslie.matrix,"census") <- census
  
  return(leslie.matrix)
}

###
#' @title Basic characteristics of life tables
#' @details 
#' - damping ratio: the speed of convergence to the stable-age distribution. Proportional to generation time
#' @param leslie.matrix Leslie matrix 
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#'  
leslieMatrix.info <- function(leslie.matrix)
{
  
 studbook.screenTitle("Characteristics of Leslie matrix")
 
 R0      <- popbio::net.reproductive.rate(leslie.matrix)
 lambda  <- popbio::lambda(leslie.matrix)
 G       <- popbio::generation.time(leslie.matrix)
 dr      <- popbio::damping.ratio(leslie.matrix)
 
 cat("Sex                          :",attr(leslie.matrix,"sex"),"\n")
 cat("Breeding season              : birth-",attr(leslie.matrix,"season"),"\n",sep="")
 if (attr(leslie.matrix,"season") == "pulse" ) {
    cat("Census                       : ",attr(leslie.matrix,"census"),"breeding\n",sep="")
 }
 
 cat("Net reproductive rate     R0 =",R0,'\n')
 cat("Generation time            G ~",G,"\n")
 cat("Finite rate of change lambda ~",lambda,"\n" )
 cat("Intrinsic rate of change   r ~",log(R0)/G,"\n") 
 cat("\n")
 cat("Damping ratio            rho =",dr,"\n")
 cat("Convergence time           t =",dr * G,"\n\n")
 
 export.file <- paste(studbook.getPath(),studbook.getName(),"export/leslie_matrix.csv",sep='/')
 write.table(leslie.matrix,file=export.file,sep="\t")
}
 

Caswell.test <- function()
{
  lx <- c(1.0,0.8,0.5,0.1,0)
  mx <- c(0,2,6,3,0)
  return(cbind(lx,mx))
}





###
#' @title Estimate and plot stable age distribution from Leslie matrix
#' @param leslie.matrix Leslie matrix
#' @param observed Observed age distribution (default:NULL)
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
leslieMatrix.stable <- function(leslie.matrix,observed=NULL)
{
  sex <- comment(leslie.matrix)
  studbook.screenTitle(paste("Stable distribution of",sex,"population\n"))
  stable <- popbio::stable.stage(leslie.matrix)
  print(stable)
  studbook.openOutput(paste('stable',sex,sep="."))
  
  if (is.null(observed)) {
     barplot(stable,xlab='Age classes')
     studbook.title(paste(sex,'stable age distribution'))
  } else {
    N <- sum(observed)
    stable <- stable * N
    ages <- data.frame("observed"=observed,"stable"=stable)
    age.plot(ages)
    print(ages)
  }
  studbook.closeOutput()
}





###
#' @title Elasticity of population matrix
#' @details the functions elasticity and image2 in the package popbio are used. 
#' @param leslie.matrix Leslie matrix
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
leslieMatrix.elasticity <- function(leslie.matrix) 
{
  leslie.matrix <- leslieMatrix.clean(leslie.matrix)
  elasticity.rs <- popbio::elasticity(leslie.matrix)
  
  export.file <- paste(studbook.getPath(),studbook.getName(),"export/elasticity.csv",sep='/')
  write.table(elasticity.rs,file=export.file,sep="\t")

  studbook.openOutput(paste('elasticity',sex,sep="."))
  popbio::image2(elasticity.rs,round=3,cex=0.8)
  studbook.title("Elasticity")
  studbook.closeOutput()
}


###
#' @title Sensitivity of population matrix
#' @details the functions sensitivity and image2 in the package popbio are used. 
#' @param leslie.matrix Leslie matrix
#' @param colour Colour patterns (default: NULL))
#' @author Frank PG Princee
#' @family Leslie matrix
#' @keywords internal
#' 
leslieMatrix.sensitivity <- function(leslie.matrix)
{
  leslie.matrix <- leslieMatrix.clean(leslie.matrix)
  sensitivity.rs <- popbio::sensitivity(leslie.matrix)
  
  export.file <- paste(studbook.getPath(),studbook.getName(),
                       "export/sensitivity.csv",sep='/')
  write.table(sensitivity.rs,file=export.file,sep="\t")
  
  studbook.openOutput(paste('stable',sex,sep="."))
  popbio::image2(sensitivity.rs,round=3,cex=0.8)
  studbook.title("Sensitivity")
  studbook.closeOutput()
}



###
#' @title Create matrix from studbook lifetables
#' 
#' @param sex Sex group; valid names are \code{"female","male","unknown","all"} 
#' (default: "female")
#' @param field text Data field for \eqn{q_x} and \eqn{m_x}: \code{"OBSERVED","MEAN","LOWER","UPPER"} as
#' used in PML output (default: "OBSERVED").
#' @param season Type of birth season: \code{"pulse"} or \code{"flow"} (default: \code{"pulse"})
#' @param census Type of census" \code{"post"} or \code{"pre"}-breeding (default: \code{"post"})
#' @param show_info Show characteristics of matrix (default: TRUE)
#' @param stable estimate stable age distribution (default: FALSE)
#' @param sensitivity Sensitivity and elasticity analyses (default: FALSE)
#' @author Frank Princee
#' @family Leslie matrix
#' @export 
#' 
studbook.leslieMatrix <- function(sex=c("female","male","unknown","all"),
                                  field=c("OBSERVED","MEAN","LOWER","UPPER"),
                                  season=c("pulse","flow"),census=c("post","pre"),
                                  show_info=TRUE,stable=FALSE,sensitivity=FALSE)
{
  
  sex    <- match.arg(sex)
  field  <- match.arg(field)
  season <- match.arg(season)
  census <- match.arg(census)
  
  px <- 1 - studbook.read('mortality',sex=sex)[,field] # Vector with (1-qx) data
  lx <- mortality.lx(px)
  mx <- studbook.read('fecundity',sex=sex)[,field]
  
  leslie.matrix <- leslieMatrix.create(lx=lx,mx=mx,season=season,census=census,sex=sex)
  
  if (show_info) {
    leslieMatrix.info(leslie.matrix)
  }  
  
  if (stable) { 
   leslieMatrix.stable(leslie.matrix)
  }
  
  if (sensitivity) {
    leslieMatrix.sensitivity(leslie.matrix)
    leslieMatrix.elasticity(leslie.matrix)
  }
  
  return(leslie.matrix)
  
}


