###
# Package : studbookR
# 
# Copyright (c) 2011-2016 Frank PG Princee, All rights reserved
#


###
#' @title Lifespan 
#' @param sex Single or multiple sex groups. Valid codes are \code{"all","male","female"}.
#' @param truncate fraction  to be excluded from left need to be implemented
#' @param explore Explore effect of sample size and truncation (default: FALSE)
#' @param bootstrap: bootstrap on mean (default: FALSE)
#' @param file_name Base name of data file (default: \code{"lifespan"})
#' @author Frank PG Princee
#' @family natural history
#' @export
#' 
studbook.lifespan <- function(sex=c("all","male","female"), 
                              truncate=0.0, explore=FALSE,bootstrap=FALSE,
                              file_name="lifespan")
{
  studbook.screenTitle('Lifespan (age at death)')
  
  sex <- match.arg(sex,several.ok=TRUE)
  
  lifespan.male   <- NULL
  lifespan.female <- NULL
  
  for (group in sex) {
    studbook.screenSection(paste("Group:",group))
    
    #Read data	
    lifespan.data <- studbook.read(file_name,sex=group)[,'AGE']
    if (is.null(lifespan.data) ) {
      next
    }   
    lifespan.data <- sort(lifespan.data)  
    
    studbook.openOutput(paste('lifespan',group,sep="-"))
    
   
    if (group == "all" & explore) {
      studbook.screenSection("Explore effects of sample size and right truncation.")
      distribution.sample(lifespan.data,c(10,50,100,250),main_title='Random samples', 'Age in days') 
      distribution.exploreEnds(lifespan.data,levels=c(0.1,0.025,0.05,0.01),side='right',"Mortality and truncationx","Age in days") 
    } 
    
    lifespan.data <- sample.truncate(lifespan.data,truncate,'right') 
   
    # Summary statistics
    lifespan.sum <- data.frame("N"=rep(NA,5),"min."=rep(NA,5),
                               "p25"=rep(NA,5),"median"=rep(NA,5),
                               "mean"=rep(NA,5),"p75"=rep(NA,5),
                               "max."=rep(NA,5))
    
    rownames(lifespan.sum) <- c("all ages","1st week","1st month","1st year",
                                "> 1st year")
    lifespan.sum[1,] <- summary.line(lifespan.data)
    lifespan.sum[2,] <- summary.line(lifespan.data[lifespan.data <= 7])
    lifespan.sum[3,] <- summary.line(lifespan.data[lifespan.data <= 30])
    lifespan.sum[4,] <- summary.line(lifespan.data[lifespan.data <= 365])
    lifespan.sum[5,] <- summary.line(lifespan.data[lifespan.data > 365])
    
    print(lifespan.sum,digits=2)
    cat("\n")
    
    distribution.initGraphs(FALSE)
    distribution.selectGraph('histogram',TRUE)
    
    
    #DATA IN YEARS      
    distribution.setType('frequency') 
    naturalhistory.plot( lifespan.data,main_title="Mortality - age at death (all)",
                         breaks=seq(0,max(lifespan.data)+366,365.25))  
    distribution.setType('frequency') 
    naturalhistory.plot( lifespan.data[lifespan.data <=   7],
                         "Mortality in first week",breaks=c(0:7))
    distribution.setType('frequency') 
    naturalhistory.plot( lifespan.data[lifespan.data <=  28]/7,"Mortality <= 4 weeks",
                         breaks=c(0:4),unit='week')
    distribution.setType('frequency') 
    naturalhistory.plot( lifespan.data[lifespan.data <= 365]/30.25,"Mortality <= 1st year",
                         breaks=c(0:13),unit='month')
    distribution.setType('frequency') 
    naturalhistory.plot( lifespan.data[lifespan.data > 365 ] ,"Mortality > 1st year", 
                         breaks=seq(365,max(lifespan.data)+366,365.25) )

    #DATA IN DAYS 
    
    
    #Bootstrap on mean age at death
    if (bootstrap) {
      studbook.bootstrap(data= lifespan.data,statistic=meanfun,
                         main_title="bootstrap: age at death") 
      studbook.bootstrap(data= lifespan.data[lifespan.data <=   7],statistic=meanfun,
                         main_title="death age in the first week")
      studbook.bootstrap(data= lifespan.data[lifespan.data <=  28],statistic=meanfun,
                         main_title="death age in the first month")
      studbook.bootstrap(data= lifespan.data[lifespan.data <= 365],statistic=meanfun,
                         main_title="death age in the first year")
      studbook.bootstrap(data= lifespan.data[lifespan.data > 365] ,statistic=meanfun,
                         main_title="death age after the first year")
    }
    studbook.closeOutput()  
    
    if (group == "female") {
      lifespan.female <- lifespan.data
    }
    
    if (group == "male") {
      lifespan.male <- lifespan.data
    }
    
  } # sex groups

  if (!is.null(lifespan.male) & !is.null(lifespan.female)) {
    studbook.openOutput('lifespan-sexes')
    
    naturalhistory.compareSexes(lifespan.male[lifespan.male > 365],
                                lifespan.female[lifespan.female > 365])
    studbook.closeOutput()
  }
}













