


###
#' @title Determine age scale
#' @description \code{lifetable.ageScale} determines the scale for life tables from \code{age}. 
#'
#' @details The scale is set from day to year depending on the maximum \code{age} provided: 
#' \describe{
#' \item{0 < age < 31}{scale is 1 day}
#' \item{30 < age < 181}{scale in 7 days (week)}
#' \item{180 < age < 1095}{scale in 30 days (month)}
#' \item{age > 1094}{scale in 365 days (year)}
#' }
#' 
#' @param age Age in days
#' @return list with elements "scale" and "name"
#' 
#' @author Frank PG Princee
#' @family life table
#' @keywords internal
#' 
lifetable.ageScale <- function(age)
{
  if (age %in% 0:30) {
    scale <- 1
    name  <- "days"
  } else if (age %in% 31:180)   { # 6 months  
    scale <- 7
    name  <- "weeks"
  } else if (age %in% 181:1095) { #3 years     
    scale <- 30
    name <- "months"
  } else {
    scale <- 365
    name <- "years"
  }
  return(list('scale'=scale,'name'=name))
}



###
#' @title Trail zero age classes
#' @param group vector with individuals at risk
#' @return last class where value is zero
#' @author Frank PG Princee
#' @family life table
#' @keywords internal
#
lifetable.trailing <-  function(group)
{
  hit_zero <- FALSE
  classes = length(group)
  
  for (c in length(group):1) {
    if (hit_zero & !is.nan(group[c]) & group[c] > 0) {
      break 
    }
    
    if (is.nan(group[c]) | group[c] == 0) {
      classes  <- c - 1
      hit_zero <- TRUE
    }
  }   
  return(classes)
}



###
#' @title Plot lifetable
#' @description Barplot of mortality or fecundity rates (with smoothing or modelling)
#' @details data frame must hold N, qx or mx...
#' @param data frame 
#' @param field.name Name of column that holds parameter 
#' @param confidence Plot confidence interval of \eqn{q_x} or \eqn{m_x} when fields \code{lower} and \code{upper} are
#' availableor upper (default:FALSE)
#' @author Frank PG Princee
#' @family life table
#' @keywords internal
#' 
lifetable.plot <- function(data,field_name=NULL,confidence=TRUE,
                           main_title='Life table',class_width='year',
                           ylabel='rate',max_y=NA)
{
  
  if (!studbook.plot()) {
    return(NULL)
  }
  
  if (is.null(field_name)) {
    stop('No column name provided.')
  }
  
  if (!is.element(field_name,colnames(data))) {
    stop(paste("Column",field_name,"does not exist"))
  }
  
  if (field_name == 'lx') {
    confidence <- FALSE
  }
   
  xlabel <- paste('Age class (width=',class_width,')')
  
  Nc <- nrow(data)
  
  # Observed value 
  if (is.na(max_y)) {
    max_y = max(data[,field_name],na.rm=TRUE)  # NA is not considered as maximum value!
  }
  
  if (confidence) {
      max_y <- max(max_y,data[,'upper'],na.rm=TRUE)
  }
  
  if (is.element('smoothed',colnames(data))) {
    max_y <- max(max_y,data[,'smoothed'],na.rm=TRUE)
  }
  
  studbook.caption(main_title)
  barplot(data[,field_name],names.arg=rownames(data),ylim=c(0,max_y),
          space=0, border='black',col='grey80',density=50,angle=45,
          xlab=xlabel,ylab=ylabel)
  
  if (confidence) {
    data[,'class'] <- data[,'class'] + 0.5                    # mid-class
    plot.CI(data[,c('class',field_name,'lower','upper')])  
    data[,'class'] <- data[,'class'] - 0.5 # reset for use smooth
  }
  
  if (is.element('smoothed',colnames(data))) {
      lines(data[,'class']+0.5,data[,'smoothed'],type='b',lty='dashed',pch=4,lwd=1.5)
  }
  
  studbook.title(main_title)
}


###
#' @title Plot two life tables
#' @description \code{lifetable.plotGroups} plots life tables of two groups
#' @param sample1 Vector with life table data e.g. qx, mx 
#' @param sample2 Vector with life table data
#' @param max_y Maximum rate to be plotted
#' @param main_title Main title on plot
#' @param ylabel Label for y-axis
#' @param file_name Name to print file
#' @author Frank PG Princee
#' @family life table
#' @keywords internal
#
lifetable.plotGroups <- function(sample1,sample2,max_y=NA,main_title="Life tables",ylabel='rate',
                                 legend.txt =c("sample A","sample B"),file_name='life-tables')
{
  if (!studbook.plot()) {
    return(NULL)
  }
  classes1 <- length(sample1)
  classes2 <- length(sample2)
  classes  <- max(classes1,classes2)
  
  if (classes1 != classes2) {
    if (classes1 > classes2) {
      sample2 <- c(sample2,rep(NaN, (classes1-classes2) ))
    } else {
      sample1 <- c(sample1,rep(NaN, (classes2-classes1) ))
    }
  }
  
  if (is.na(max_y)) {
    max_y <- max(sample1,sample2,na.rm=TRUE)
  }

  studbook.openOutput(file_name)
  plot(0:(classes-1),sample1,ylim=c(0,max_y),type='b',pch=19,cex=0.8,lty=1,
       main='',xlab='Age class (years)',ylab=ylabel)
  lines(0:(classes-1),sample2,type='b',pch=21,cex=0.8,lty=1)
  legend("topright",legend=legend.txt,pch=c(19,21),cex=0.8,lty=1,bty="n")
  studbook.title(main_title)    
  
  studbook.closeOutput()
}




###
#' @title Plot distribution of bootstrap values 
#' @description Graphic distribution of bootstrap runs
#' @param data Bootstrap data
#' @param group_id group or class number in data set
#' @param main_title Main title on plot (default: "Bootstrap class")
#' @param xlabel Title on x-axis (default: "rate")
#' @author Frank PG Princee
#' @family life table
#' @keywords internal
#' 
lifetable.bootstrapDistribution <- function(data,group_id = 0,
                                            main_title = 'Bootstrap class',
                                            xlabel='rate')
{
  if (!studbook.plot()) {
    return(NULL)
  }
  max_rate <- max(data)
  
  if (max_rate == 0) {
    stop("No observations.")
  }
  
  main_title <- paste(main_title,'class',group_id)
  distribution.plot(data,main_title=main_title,xlabel=xlabel,
                    quantiles=c(0.025,0.975))
}


###
#' @title Analyse PML bootstrap results of life tables
#' @param file_name Full name of file to be imported 
#' @param groups  Which groups/classes to analyse (default: NA)
#' @param main_title Main title on plot
#' @param ylabel Label on y-axis (default:  "rate")
#' @author Frank PG Princee
#' @family life table
#' @export
#' 
lifetable.analyseBootstrap <- function(file_name,groups=c(NA),
                                       main_title = 'Bootstrap class',
                                       ylabel='rate')
{
  if (studbook.empty()) {
    studbook()
  }
    
  bt <- studbook.csv(file_name)
  
  if (is.null(bt)) {
    stop("No bootstrap data")
  }
 
  runs <- length(bt$VALUE[bt$ID == 0])
  
  values <- bt$VALUE[bt$ID == 0]
  hist(values)
  
  #Boxplot all bootstrap data
  
  boxplot(VALUE ~ ID, data=bt,notch=FALSE,xlab='Age class',
          ylab=ylabel,outline=FALSE)
  studbook.title(main_title,paste("(",runs,"runs)"))
  
  
  if (is.na(groups[1]) ) {
    return(TRUE)
  }
  
  #Check for Infinite = last class in groups e.g. c(Inf), c(0,Inf)
  for (c in length(groups) ) {
    if (is.infinite(groups[c]) ) {
      groups[c] <- max(bt$ID)
      lifetable.bootstrapDistribution(groups[c])
    }
  }
  return(TRUE)
}


###
#' @title Estimate r through Euler-Lotka equation
#' @param lxmx vector with lxmx for age classes (starts with 0)
#' @param r Initial value of r ( log(R0)/G ) 
#' @author Frank PG Princee 
#' @return r
#' @family life table
#' @keywords internal
#' 
Euler.Lotka <- function(lxmx,r)
{ 
   classes <- length(lxmx)
   
   accept        <- FALSE
   increment     <- 0.0
   init_step     <- abs(r/2.0)
   previous_sum  <- 1.0
   trials        <- 1
   
   repeat {
     if (accept) {
       break
     }
     c   <- 0;
     sum <- 0.0;
     r   <- r + increment;
     
     for (x in 1:classes) {
       
       if ( c < classes ) {
         sum  <- sum + exp(-r *   c ) * lxmx[x];
       }
       c <- c + 1
     }
     
     if ( abs(sum - 1.0) < 0.00001 ) {
       accept <- TRUE
     }
     
     if (sum > 1.0) {
       
       if ( previous_sum < 1.0 ) {       # Last increment to large
         init_step <- init_step / 2.0;   # Decrease increment
         r         <- r - increment;     # Set r to previous r
       }
       increment = init_step;
       
     } else {
       
       if ( previous_sum > 1.0 ){
         init_step <- init_step / 2.0;
         r         <- r - increment;
       }
       increment <- -(init_step);
       
     } # of sum > 1.000
     
     previous_sum <- sum;
     trails <- trials + 1;
     
     if (trails > 9999) {
       accept =TRUE;
     }
     
   }
   return(r)
}




###
#' @title Life table with combined lx and mx data
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param class_width Information on class width for X-axis label (default:"year")
#' @param file_name Base name of data file (default: \code{"lifetable"})
#' @author Frank PG Princee
#' @family life table
#' @export
#' 
studbook.lifetableFull <- function(sex=c("female","male","unknown","all"),
                                   class_width='year',file_name="lifetable")
{
  if (studbook.empty()) {
    studbook()
  }
  
  sex <- match.arg(sex)
  life_table <- studbook.read(file_name,sex=sex)

  if (is.null(life_table)) {
    stop("No lifetable data")
  }
  
  xlabel <- paste('Age class (width=',class_width,')')
  
  colnames(life_table) <- c("class","qx","px","lx","Lx","ex","mx","lxmx","vx","vxs","cx")
  studbook.screenTitle('Full lifetable')
  print(life_table)
 
  studbook.openOutput(paste('reprod_value',sex,sep='.'))
  main_title <- paste("Fisher's reproductive value in",sex)
  caption    <- studbook.caption(main_title)
  
  
  #Get last trailing zero element
  Vx.end  <- lifetable.trailing(life_table$vx)
  Vxs.end <- lifetable.trailing(life_table$vxs)
  

  Vx.max  <- max(life_table$vx,na.rm=TRUE)
  Vxs.max <- max(life_table$vxs,na.rm=TRUE)
  
  plot(vx ~ class,data=life_table,xlim=c(0,Vx.end),ylim=c(0,Vx.max),
       type='b',pch=20, xlab=xlabel,ylab=expression(V[x]))
  studbook.title(main_title)
  
  main_title <- paste("Expected future offspring in",sex)
  caption    <- studbook.caption(main_title)
  
  plot(vxs ~ class,data=life_table,xlim=c(0,Vxs.end),ylim=c(0,Vxs.max),
       type='b',pch=20,xlab=xlabel,ylab=expression(V[x]))
  studbook.title(main_title)
  
  main_title <- paste("Reproductive values",sex)
  caption    <- studbook.caption(main_title)
  
  V.end <- max(Vx.end,Vxs.end)
  V.max <- max(Vx.max,Vxs.max)
  
  plot(vxs ~ class,data=life_table,xlim=c(0,V.end),ylim=c(0,V.max),
       type='b',pch=21,cex=2/3,xlab=xlabel,ylab=expression(V[x]))
 
  points(life_table$class,life_table$vx,pch=20,xlim=c(0,V.end))
  lines(life_table$class,life_table$vx,lty=2,xlim=c(0,V.end))
  
  studbook.title(main_title)
  studbook.closeOutput()
  
  
  studbook.openOutput(paste('life_expectancy',sex,sep='.'))
  main_title <- paste("Life expectancy in",sex)
  plot(ex ~ class,data=life_table,ylim=c(0,max(life_table$ex)),
       type='b',pch=21,xlab=xlabel,ylab=expression(E[x]))
  studbook.title(main_title)
  studbook.closeOutput()
  
  studbook.openOutput(paste('stable_age',sex,sep='.'))
  main_title <- paste("Stable age class",sex)
  
  barplot(life_table$cx,names=life_table$class,
          xlab=xlabel,ylab=expression(C[x]))
  studbook.title(main_title)
  studbook.closeOutput()
  
  lxmx <- life_table$lx * life_table$mx
  lifetable.matrix <- data.frame("age"=round(life_table$class,0),"lx"=round(life_table$lx,3),
                                 "mx"=round(life_table$mx,3),"lxmx"=round(lxmx,3),
                                 "Vx"=round(life_table$vx,3),"Vxs"=round(life_table$vxs,3),
                                 "CR"="\\cr")
  
  # MASS::write.matrix(lifetable.matrix,sep=" &") 
  
  R0 <- sum(lifetable.matrix[,'lxmx'])
  G  <- sum(lifetable.matrix[,'age'] * lifetable.matrix[,'lxmx'])/R0
  r  <- log(R0)/G
  
  r_euler <- Euler.Lotka(lifetable.matrix[,'lxmx'],r)
  
  cat("\nSummary statistics:\n")
  cat("Net reproductive rate     R0 =",R0,'\n')
  cat("Generation time            G ~",G,"\n")
  cat("Intrinsic rate of change   r ~",r," (observed)\n")
  cat("                           r =",r_euler,"(Euler)\n")
  cat("Finite rate of change lambda ~",exp(r),"\n" )
  cat("\n")
}


studbook.fullLifeTable <- function(sex=c("female","male","unknown","all"),class_width='year')
{
  #read Nx, Dx,qx from mortality
  #read Nx, Bx,mx from fecundity
  
  #Create matrix Nq_x,Dx,qx,lx,Lx,Nm_x,Bx,mx
  #Calculate lx,Lx
  
  #for the moment
  studbook.lifetableFull(sex=sex,class_width=class_width)
}


####
# Functions to prepare data for logrank test 
###

###
#' @title Create matrix for two samples 
#' @description \code{lifetable.twoSamples} creates a matrix to hold basic 
#' mortality data or fecundity data of two population samples.
#' @details The first column must hold N, and second either Dx or Bx. The class 
#' number is stored in rownames. Data is truncated/lumped whenever required
#' @param sample1 Data of first sample
#' @param sample2 Data of second sample
#' @return matrix
#' @author Frank PG Princee
#' @family life table
#' @keywords internal
#' 
lifetable.twoSamples <-  function(sample1,sample2)
{
  
  n.sample1 <- nrow(sample1)
  n.sample2 <- nrow(sample2)
  classes   <- max(n.sample1,n.sample2)
    
 
  samples <- matrix(ncol=4,nrow=classes)
  colnames(samples) <- c('N1','O1','N2','O2')
  samples[,1:4] <- NA
    
  samples[1:n.sample1,c('N1','O1')] <- as.matrix(sample1[,1:2])
  samples[1:n.sample2,c('N2','O2')] <- as.matrix(sample2[,1:2])

  rownames(samples) <- 0:(classes-1)  
  return(samples)
}


###
#' @title Logrank test on two life tables
#' @description \code{lifetable.compare} applies the logrank test on two life tables (samples)
#' @param sample1 Data of first sample
#' @param sample2 Data of second sample
#' @param main_title Main title of information (default: "Logrank test") 
#' @author Frank PG Princee
#' @family life table
#' @keywords internal
#' 
lifetable.compare <- function(sample1,sample2,main_title="Logrank test")
{
  if (is.null(sample1) || is.null(sample2)) {
    stop("Two samples are required.")
  } 
  
  samples <- lifetable.twoSamples(sample1,sample2)
  
  attr(samples,"groups") <- paste("N1=",attr(sample1,"sample"), "N2=",attr(sample2,"sample"),sep=' ') 
  studbook.logRank(samples,main_title)
}






