
  

###
#' @title Binomial test of litter effect on neonatal survival
#' @description \code{litter.effect} tests for litter effect in neonatal survival
#' @param litter.data Data frame with size, deaths and survived per litter
#' @param alpha Significance level (default: 0.05) 
#' @author Frank PG Princee
#' @family natural history
#' @export
#'   
litter.effect <- function(alpha=0.05)
{
  litter.data <- litter.read(add_fields=TRUE) # Function in studbook.seasonality.R 
  if (is.null(litter.data)) {
    return(FALSE)
  }
  
  studbook.screenTitle("Litter size and neonatal survival")
  cat("Total litters:",nrow(litter.data),"\n")
  
  
  # Is survival of litter mates independent
  litter.max <- max(litter.data$SIZE)
  
  if (litter.max < 2) {
    return(FALSE)
  }
  
  size.survival <- data.frame("births"=rep(0,litter.max),
                              "survived"=rep(0,litter.max),
                              "S"=rep(0,litter.max))
  
  for (size in 1:litter.max) {
    litter<- litter.data[litter.data$SIZE == size,] # Litters of specified size
    
    survival.total <- sum(litter$SURVIVED)
    births.total   <- sum(litter$SIZE)
    Sv <- sum(survival.total)/sum(births.total)
    
    
    cat("Litter size  :",size,"\n")
    cat("Litters      :",nrow(litter),"\n")
    cat("Births       :",births.total,"\n")  
    cat("Survived     :",survival.total,"\n")
    cat("Survival rate:",Sv,"\n")
    
    size.survival[size,"births"]   <- births.total
    size.survival[size,"survived"] <- survival.total
    size.survival[size,"S"]        <- Sv
    
    if (size < 2) {
      cat("\n\n")
      next
    }
    
    survived <- table(litter$SURVIVED)               # Count number of survivors for each litter of "size"
    survival <- table(litter$SURVIVED)/nrow(litter)  # Estimate observed survival rate 
    
    for (x in 1 : length(survived)) {
      proportion <- dbinom(x-1,size,Sv)
      N <- round(proportion * nrow(litter),1)
      cat("survived     :",x-1,survived[x],survival[x],"(",N,proportion,")\n")
    }
    if (!is.na(survived[2]) && survived[2] >=0) {
      rs <- binom.test(survived[2],nrow(litter),dbinom(1,size,Sv)) # Number of single survivors
      print(rs)
      cat("H0: survival in litter is distributed binomially is",
          ifelse (rs$p.value < alpha,"rejected","not rejected"),"\n")
    }
    cat("\n\n")
  }
  
  cat("Survival and births per litter size\n")
  print(size.survival)
  cat("Pairwise Fisher exact test\n")
 
  rs <- fmsb::pairwise.fisher.test(size.survival$survived,
                                   size.survival$births,
                                   p.adjust.method="holm")
  print(rs)
}


###
#' @title Litter size
#' @description \code{studbook.litterSize} analyses litter (clutch) size data
#' @details The distribution of litter size and the mean (weighted and unweighted) is estimated.
#' This analyses includes a bootstrap of the mean value.
#' @param plot_type Plot litter distribution as \code{"frequency", "percentage", "proportion"} (default: \code{"proportion"})
#' @param bootstrap: bootstrap on mean (default: FALSE))
#' @author Frank PG Princee
#' @family natural history
#' @export
#' 
studbook.litterSize <- function(plot_type=c("frequency","percentage","proportion"),
                                bootstrap=FALSE)
{
  
  raw.data <- studbook.csv('littersize.csv')['DATA']
  
  if (is.null(raw.data)) {
    return(FALSE)
  }
  plot_type <- match.arg(plot_type)
  raw.data <- as.vector(raw.data$DATA)
  studbook.screenTitle('Litter size')
  
  #Basic statistics and plotting
  studbook.summary( raw.data,long=FALSE,title="Litter")
  values.mean(raw.data)
  print(table(raw.data))
  
  
  if (studbook.plot()) {
    studbook.openOutput('littersize')    
    distribution.initGraphs(FALSE)
    distribution.selectGraph('barplot',TRUE)
    old_type <- distribution.getType()
    distribution.setType(plot_type)
    distribution.plot(data=raw.data,main_title='Litter size',xlabel='litter size',
                      bar_breaks="Sturges",leftPoints=min(raw.data) )

    if (bootstrap) {
      studbook.bootstrap(data=raw.data,statistic=mean.fun,main_title="litter size") 
    }
    
    distribution.setType(old_type)
    studbook.closeOutput()  
  }
  
  return(TRUE)
}


###
#' @title Litter/clutch survival
#' @description Analyses neonatal survival for different litter sizes
#' @author Frank PG Princee
#' @family natural history
#' @export
#' 
studbook.litterSurvival <- function()
{
  litter.data <- litter.read(add_fields=TRUE)  # Function in studbook.seasonality.R 
  if (is.null(litter.data)) {
    return(FALSE)
  }
  
  # Strip row with sire - which is used for quantitative genetics
  if (is.element("SEX",colnames(litter.data))) {
    litter.data <- litter.data[litter.data$SEX == "female",]
  }
  
  studbook.screenTitle("Litter size and neonatal survival")
  cat("Total litters:",nrow(litter.data),"\n")

  
  litter.data <- within(litter.data,{
    SIZE   <- MALES + FEMALES + UNKNOWN 
    DEATHS <- D_MALES + D_FEMALES + D_UNKNOWN
    SURVIVED <- SIZE - DEATHS
  })
  
  survival.glm <- lme4::glmer(cbind(SURVIVED,DEATHS) ~ SIZE + F +  (1|animal),data=litter.data,family="binomial")
  print(summary(survival.glm))
  
  if (class(survival.glm)[1] != "glmerMod") {
     plot(DEATHS/SIZE ~ SIZE ,data=litter.data,type='p')
     lines(litter.data$SIZE,predict(survival.glm,type="response"),lty='dashed')  
  }
}


###
#' @title Effect of generation, age of dam, and inbreeding on litter size
#' @description Multivariate GLM or GLMM for effects of age, generation and inbreeding on litter size
#' @param fixed.effect List with fixed term(s) in regression. Enter as: \code{
#' c("fixed1","fixed2","fixed:fixed2")}  (default: "1")
#' @param random.effect List with random term(s) in regression. Enter as: \code{
#' c("(1|random1)",c("(1|random2)")} (default: NA) 
#' @param family Distribution families for GLM, \code{poisson,Gamma,gaussian,inverse.gaussian,quasi,
#' quasipoisson} and approperiate link (default:\code{poisson("log")})
#' @param do_scale Scale multivariate data when different measures 
#' @param file_name Base name of data file (default: \code{"litter_season"})
#' @author Frank PG Princee
#' @family natural history
#' @export
#' 
litterSize.effects <- function(fixed.effect="1",random.effect=NA,
                               family=poisson("log"),do_scale=FALSE,
                               file_name="litter_season")
{
  litter.data <- studbook.read(file_name) 
 
  if (is.null(litter.data)) {
    stop("No data on litter size.")
  }
  studbook.screenTitle("Effects of age of dam, generation and inbreeding on litter size")
  litter.data <- within(litter.data, SIZE   <- MALES + FEMALES + UNKNOWN) 
 
  if (!is.element("LOCATION",random.effect)) {
    litter.data$LOCATION <- as.factor(litter.data$LOCATION)
  }
  
  if (do_scale & length(fixed.effect > 1)) {
    litter.data[fixed.effect] <- lapply(litter.data[fixed.effect],scale)
  }
  
  if (length(fixed.effect > 1)) {
     effects <- paste(fixed.effect,collapse="+")
  }
  
  if (!is.na(random.effect)) {
    effects <- paste(effects,paste(random.effect,collapse="+"),sep="+")
  }
  print(effects)
  glm.terms <- paste("SIZE",effects,sep="~")
  
  
  if (is.na(random.effect)) {
    litter.glm <- glm(as.formula(glm.terms),data=litter.data,family=family)
    
  } else {
    litter.glme <- glmer(as.formula(glm.terms),data=litter.data,family=family)
  }
  print(summary(litter.glm))
  cat("\n")  
  cat("Family/link                :",family(litter.glm)[["family"]],"/",family(litter.glm)[["link"]],"\n")
  if (class(litter.glm)[1] == "glm") {
    cat("Residual deviance (df)     :",litter.glm$deviance,"(",litter.glm$df.residual,")\n")
    overdispersed <- litter.glm$deviance/litter.glm$df.residual # Crawley The R Book 
    cat("Overdispersion(*)          :",overdispersed,"\n")
  }
}
