###
#' @title Longevity 
#' @description \code{studbook.longevity} estimates and prints the oldest observed age at death.
#' 
#' @details A data frame with columns ALL, MALE and FEMALE is created from the files \code{lifespan.all.csv}, \code{lifespan.male.csv} and
#' and \code{lifespan.female.csv}. 
#' 
#' Since the oldest observed age can be an artifact, longevity is estimated from
#' data that are truncated at the tail of the distribution. For example, a truncation 
#' of 0.9 refers to the oldest observed age in 90% of the data (ordered by age). 
#' A quantile of 1.0 refers to the maximum observed age at death.   
#' 
#' @param truncate vector with different levels of truncation (default 0.9, 0.95, 0.99, 1.0)
#' @param file_name Base name of data file (default: \code{"lifespan"})
#' @author Frank PG Princee
#' @examples
#' \dontrun{
#' studbook.longevity()                       # No truncation
#' studbook.longevity(truncate=c(0.975,0.99)) # Trucations at 0.975 and 0.99 
#' }
#' @family natural history
#' @export
#' 
studbook.longevity<-function(truncate=c(.90,.95,.99,1.),file_name="lifespan")
{
  #Read data	
  all     <-studbook.read(file_name)['AGE']
  male    <-studbook.read(file_name,sex='male')['AGE']
  female  <-studbook.read(file_name,sex='female')['AGE']
  
  if (is.null(all) || is.null(male) || is.null(female)) {
    stop("No lifespan data.")
  }
  
  studbook.screenTitle("Longevity and percentiles")
  
  longevity <- data.frame("ALL"    = quantile(all[,'AGE'],truncate),
                          "MALE"   = quantile(male[,'AGE'],truncate),
                          "FEMALE" = quantile(female[,'AGE'],truncate))
  print(longevity)
  cat("\n")
}


###
#' @title  Compare distribution in tails
#' @param lifespan vector with age at death data.
#' @keywords internal
#' 
studbook.lifespanTail <- function(lifespan,main_title="all")
{
  truncate <- c(0.01,0.05,0.1)   
  pc1  <- floor(length(lifespan) * 0.01)  # One percent 
  pc5  <- floor(length(lifespan) * 0.05)  # Five percent
  pc10 <- floor(length(lifespan) * 0.1)   # Ten percent 
  
  sort(lifespan)
  
  
  cat(main_title,"\n")
  if (pc10 > 0) {
    sample <- tail(lifespan,n=pc10)
    
    # Boxplot of excluding data       
    boxplot.longevity<-list(tail(lifespan,n=pc1),tail(lifespan,n=pc5),tail(lifespan,n=pc10))
    if (pc1 == 0) {
      boxplot.longevity[1] <- NaN
    }
    
    if (pc5 == 0) {
      boxplot.longevity[2] <- NaN
    }
    
    op <- par(las=1)	
    studbook.caption(main_title,"Longevity in tail of distribution.")
    boxplot(boxplot.longevity,names=c(1,5,10),main="",
            xlab='Percentage (%) of upper ages included',ylab='Age (in days)', outline=TRUE) 
    studbook.title(main_title)         
    studbook.bootstrap(data=sample,statistic=mean.fun,main_title="longevity (10%)")
    par(op)
  }        
}


