###
#' @title Calculate mortality rates from survivorship table
#' @description \code{lifetable.qx} calculates mortality rates (\eqn{q[x]}) from
#' survivorship (\eqn{l[x]}).
#' 
#' @details The mortality rate at class \eqn{x} is calculated as \eqn{q[x]=l[x+1]/l[x]}.
#' 
#' @param lx vector with survivorship values
#' @return vector with morality rates
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.qx <- function(lx)
{
  classes <- length(lx)
  qx <- rep(0,classes)
  for (c in 1:(classes - 1))
  {
    qx[c] <- 1 - (lx[c+1]/lx[c])
  }
  qx[classes] <- NA
  return(qx)
}





###
#' @title Calculate survival rate from mortality rate
#' @description \code{mortality.px} calculates the survival rate (\eqn{p[x]})
#' from the mortality rate (\eqn{q[x]}).
#' 
#' @details The survival rate at class \eqn{x} is calculates as \eqn{p_x = 1.0 - q_x}.
#' 
#' @param qx mortality rate or vector with mortality rate
#' @return survival rate(s)
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.px <- function(qx)
{
  px <- 1.0 - qx  
  return(px)
}


###
#' @title Calculates survivorship from survival rates
#' @description \code{mortality.lx} calculates survivorship \eqn{$l_x$} from
#' survival rates (\eqn{p_x}).
#' @details Survivorship at class \eqn{x} is calculated as \eqn{l_x = l_{x-1} * p_x-1}, 
#' where \eqn{l_1 = 1.0}.
#' @param px vector with survival rates
#' @return survivorship values
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.lx <- function(px)
{    
  classes <- length(px)
  lx      <- rep(0,classes)
  lx[1]   <- 1.0
  
  for (c in 2:classes) {
    lx[c] <- lx[c-1] * px[c-1]
  }
  return(lx)
}


###
#' @title Calculates mortality from survivorship
#' @description \code{mortality.dx} calculates mortality (\eqn{d_x}) from
#' survivorship (\eqn{l_x})
#' 
#' @details Mortality at class \eqn{x} is calculated as \eqn{d_x = l_x - l_{x+1}}.
#' 
#' @param lx vector with survivorship values
#' @return mortality values
#' @family mortality
#' @keywords internal
#
mortality.dx <- function(lx)
{
  classes <- length(lx)
  dx      <- rep(0,classes)
  
  for (c in 1: classes-1)
    dx[c] <- (lx[c] - lx[c+1]) 
  return(dx)
}

###
#' @title Calculate midpoint survivorship
#' @description \code{midLx} calculates midpoint survivorship (\eqn{L_x})
#' from survivorship (\eqn{l_x})
#' 
#' @details Midpoint survivorship is estimated as \eqn{L_x=(l_x + l_{x+1})/2} for
#' age classes 1 to n-1; \eqn{L_n = l_n / 2}.
#' 
#' @param lx survivorship values
#' @return Midpoint survivorship
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.midLx <- function(lx)
{
  classes <- length(lx)
  Lx      <- rep(0,classes)
  
  for (c in 1:classes - 1)     {
    Lx[c] <- (lx[c] + lx[c+1]) / 2.0
  }
  
  Lx[classes] <- lx[classes] / 2.0
  
  return(Lx)
}


###
#' @title Calculate midpoint survival rate
#' @description \code{midLx} calculates midpoint survival rate (\eqn{P_x})
#' from survival rate (\eqn{p_x})
#' 
#' @details Midpoint survival rate is estimated as \eqn{P_x=(p_x + p_{x+1})/2} for
#' age classes 1 to n-1; \eqn{P_n = p_n / 2}.
#' 
#' @param px survival rate
#' @return Midpoint survival rates
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.midPx <- function(px)
{
  classes <- length(px)
  Px      <- rep(0,classes)
  
  for (c in 1:classes - 1)     {
    Px[c] <- (px[c] + px[c+1]) / 2.0
  }
  
  Px[classes] <- px[classes] / 2.0
  
  return(Px)
}


###
#' @title Calculate person years from midpoint survivorship
#' @description \code{mortality.Tx} calculates person years (\eqn{T_x}) from
#' midpoint survivorship (\eqn{L_x})
#' 
#' @details Person years at age \eqn{x} is calculated as \eqn{T_x= T_{x-1} - L_x-1} for
#' classes 2 to n. \eqn{T_1 = \sum{L_x}}.
#' 
#' @param Lx Midpoint survivorship
#' @return person years
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.Tx <- function(Lx)
{
  classes <- length(Lx)
  Tx      <- rep(0,classes)
  Tx[1]   <- sum(Lx)
  for (c in 2:classes)     {
    Tx[c] <- Tx[c-1] - Lx[c-1]
  }
  return(Tx)
}

###
#' @title Calculate life expectancy
#' @description \code{mortality.ex} calculates life expectancy (\eqn{e_x})
#' from person years (\eqn{T_x}) and survivorship (\eqn{l_x})
#' 
#' @details Life expectancy at age \eqn{x} is calculated as \eqn{e_x = T_x/l_x}.
#'   
#' @param Tx vector with person years
#' @param lx vecor with survivorship values
#' @return life expectancies
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.ex <- function(Tx,lx)
{
  ex <- rep(0,length(Tx))                                          # create empty vector
  ex <- Tx/lx
  return(ex)
}


###
#' @title Standard variance of qx
#' 
#' @param qx double Mortality rate
#' @param Nx integer Individuals at risk (=number of trials)
#' @return double
#' @author Frank PG Princee
#' @references Akçakaya, 2002
#' @family mortality
#' @keywords internal
#' 
qx.var <- function(qx,Nx)
{
  vars <- (qx * (1-qx)) / Nx 
  return( vars )
}  



###
#' @title Standard error of qx
#' 
#' Standard error is square root binomial variance
#' @param qx double Mortality rate
#' @param Nx integer Individuals at risk (=number of trials)
#' @return double
#' @author Frank PG Princee
#' @references Akçakaya, 2002
#' @family mortality
#' @keywords internal
#' 
qx.se <- function(qx,Nx)
{
  vars <- (qx * (1-qx)) / Nx 
  return( sqrt(vars) )
}  



###
#' @title Coefficient of variation of qx
#' 
#' Variance is binomial  (see Daw 1974...)
#' 
#' @param qx double Mortality rate \eqn{q[x]}
#' @param Nx integer Individuals at risk
#' @return double
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
qx.cv <- function(qx,Nx)
{
  svar <- qx.var(qx,Nx)
  return(cv.mean(qx,svar))
}  



###
#' @title Confidence limits of qx
#' @description \code{qx.cf} estimates 95% confidence limits of \eqn{q_x}
#' @param qx double Mortality rate \eqn{q[x]})
#' @param Nx integer Individuals at risk
#' @param significance double Significane level (default: 0.05)
#' @return lower and upper limits [0,1]
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
qx.cf <- function(qx,Nx,significance=0.05)
{
  qx.limit <- list(se=NA,lower=NA,upper=NA)
  if (Nx < 2) {
    return(qx.limit)
  }
  
  
  # The variance is not sampled, therefore use normal (z=1.96) and N=inf
  se <- sqrt(qx.var(qx,Nx))
  qx.limit <- list(se = se, lower= qx - (se * 1.96),
                   upper= qx + (se *1.96) )
  
  # Adjust to 0 - 1 range
  if (qx.limit[['lower']] < 0.0) {
    qx.limit[['lower']] <- 0.0
  }
  if (qx.limit[["upper"]] > 1.0) {
    qx.limit[["upper"]] <- 1.0
  }
  
  return(qx.limit)  
}

###
#' @title Estimate constant mortality in group of classes
#' @param lx Vector with first and last lx value
#' @param n  Number of classes between first and last i.e. first + n =  last
#' @return mortality rate
#' @author Fank PG Princee
#' @family mortality
#' @keywords internal
#' 
qx.constant <- function(lx,n)
{
  pc <- exp( (log(lx[2]) - log(lx[1]) ) / n )
  return( 1- pc)
}



###
#' @title Create mortality table (matrix) 
#' @description \code{mortality.table} imports mortality table from censored or uncensored data
#' @details The column names in PML censored data are:
#' 
#' \describe{
#' \item{OBSERVED}{observed qx}
#' \item{MEAN}{mean of bootstrapped qx}
#' \item{LOWER}{lower (2.5\%) percentile of bootstrapped qx}
#' \item{UPPER}{upper (97.5\%) percentile of botstrapped qx}
#' }
#' 
#' The adjusted bootstrap = observed - (mean - observed) is shown
#' 
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param censored TRUE use censored mortality table, FALSE use uncensored table (default: TRUE)
#' @param qx_field Column names in PML file \code{"OBSERVED","MEAN"} (default: "OBSERVED")
#' @param class_width Width of class in number of days (and decimals) (default: 365.25 = 1 year)
#' @param file_name Alternative file name for mortality data. The default file names 'mortality' for censored data
#' and 'lifespan' for uncensored data are assigned when \code{file_name} is \code{NULL} (default: NULL).
#' @author Frank PG Princee
#' @family mortality
#' @export
#' 
mortality.table <- function(sex=c("female","male","unknown","all"),censored=TRUE,
                            qx_field=c("OBSERVED","MEAN"),
                            class_width=365.25,file_name=NULL)
{
  if (studbook.empty()) {
    studbook()
  }
  
  sex <- match.arg(sex)
  qx_field <- match.arg(qx_field)
 
  if (censored) {
    mortality.rates <- table.censored(sex=sex,qx_field=qx_field,
                                      class_width=class_width,file_name=file_name)
  } else {
    mortality.rates <- table.uncensored(sex=sex,qx_field=qx_field,class_width=class_width,
                                        file_name=file_name)
  }
  return(mortality.rates)
}



###
#' @title Create mortality table of censored data (matrix) 
#' @description \code{table.censored} creates a life table from censored data.
#' @details The column names in PML censored data are:
#' 
#' Format of age-specific mortality data files:
#' \tabular{lll}{
#' \bold{Field name} \tab \bold{Type} \tab \bold{Comment} \cr
#' CLASS             \tab integer     \tab Age class starting at 0 \cr  
#' N                 \tab real        \tab Number of individuals at risk \cr
#' D                 \tab integer     \tab Number of deaths (same sex as parents is optional)\cr
#' OBSERVED          \tab real        \tab Observed fecundity rate qx (=D/N)\cr
#' --------          \tab ------      \tab ------\cr
#' MEAN              \tab real        \tab Mean qx in bootstrap samples\cr   
#' VAR               \tab real        \tab Variance of qx in bootstrap samples\cr 
#' MEDIAN            \tab real        \tab Median qx in bootstrap samples\cr
#' LOWER             \tab real        \tab 2.5 percentile of qx in bootstrap samples\cr
#' UPPER             \tab real        \tab 97.5 percentile of qx in bootstrap samples\cr
#' } 
#' 
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param class_width Width of class in number of days (and decimals) (default: 365.25 = 1 year)
#' @param qx_field Column names in PML file \code{"OBSERVED","MEAN"}. The adjusted bootstrap 
#' mean is estimated from the deviation of observed and bootstrap mean (default: "OBSERVED")
#' @param file_name Name of file with mortality data (default: \code{'mortality'}).
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
table.censored  <- function(sex=c("female","male","unknown","all"),
                            qx_field=c("OBSERVED","MEAN"),
                            class_width=365.25,file_name='mortality')
{
  if (studbook.empty()) {
    studbook()
  }
  if (is.null(file_name)) {
    file_name <- 'mortality'
  }
  sex <- match.arg(sex)
  mortality <- studbook.read(file_name,sex=sex)
  
  if (is.null(mortality)) {
    stop("No mortality data.")
  }
  
  qx_field <- match.arg(qx_field)
  
  if (!is.element("MEAN",colnames(mortality))) {  # No bootstrap data
    qx_field <- 'OBSERVED'
  } else {
    mortality <- within(mortality, MEAN <- OBSERVED - (MEAN-OBSERVED) ) # Observed - bias
  } 
  
  classes         <- nrow(mortality)
  rates           <- matrix( ncol=13,nrow=classes )
  colnames(rates) <- c("class","Nx","Dx","dx","qx","px","lx","Lx","Tx","ex","var","lower","upper")
  
  rates[,'class'] <- mortality$CLASS
  rates[,'Nx']    <- mortality$N
  rates[,'Dx']    <- mortality$D
  rates[,'qx']    <- mortality[,qx_field]      
  
  if (qx_field == 'OBSERVED') {
    for (x in 1:nrow(rates)) {
      rates[x,'var'] <- qx.var(rates[x,'qx'],rates[x,'Nx'])
      qx.limit <- qx.cf(rates[x,'qx'],rates[x,'Nx'])
      rates[x,'lower'] <- qx.limit[['lower']]
      rates[x,'upper'] <- qx.limit[['upper']]
    } 
  } else {
    rates[,'var']   <- mortality$VAR
    rates[,'lower'] <- mortality$LOWER
    rates[,'upper'] <- mortality$UPPER
  }
  
  # parameter dx cannot be calculated in raw censored table unless rescaled to a radix of e.g. 1,000
  rm(mortality)
  rates <- table.extend(rates)   # Estimate px, lx,Lx, Tx, ex
  return(rates) 
}


###
#' @title Create mortality table of uncensored data (matrix) 
#' @description \code{table.uncensored} creates a life table from uncensored data.
#' @details The column names in PML censored data are:
#' 
#' \describe{
#' \item{OBSERVED}{observed qx}
#' \item{MEAN}{mean of bootstrapped qx}
#' \item{LOWER}{lower (2.5\%) percentile of bootstrapped qx}
#' \item{UPPER}{upper (97.5\%) percentile of botstrapped qx}
#' }
#' 
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param class_width Width of class in number of days (and decimals) (default: 365.25 = 1 year)
#' @param qx_field Column names in PML file \code{"OBSERVED","MEAN"}. The adjusted mean is 
#' estimated from deviation of observed and mean offspring (default: "OBSERVED")
#' @param file_name (default: \code{'lifespan'}).
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
table.uncensored  <- function(sex=c("female","male","unknown","all"),
                              qx_field=c("OBSERVED","MEAN"),
                              class_width=365.25,file_name='lifespan')
{
  if (studbook.empty()) {
    studbook()
  }
  
  sex <- match.arg(sex)
  
  if (is.null(file_name)) {
    file_name <- 'lifespan'
  }
  
  mortality <- studbook.read(file_name,sex=sex)['AGE']
  
  if (is.null(mortality)) {
    stop("No mortality (lifespan) data.")
  }
  
  qx_field <- match.arg(qx_field) 
  
  bar_breaks    <- seq(0, max(mortality + ceiling(class_width) ),class_width)        
  age_classes   <- hist(mortality[,'AGE'],breaks=bar_breaks,plot=FALSE) 
  classes       <- length(bar_breaks) - 1                                 # Remove extra bar

  total         <- sum(age_classes$counts)                                      
  living        <- total
  
  rates          <- matrix( ncol=13,nrow=classes)
  colnames(rates) <- c("class","Nx","Dx","dx","qx","px","lx","Lx","Tx","ex","var","lower","upper")
  
  n <- 1
  for (died in age_classes$counts) {
    rates[n,'class'] <- n -1                 
    rates[n,'Nx']    <- living               
    rates[n,'Dx']    <- died                 
    rates[n,'dx']    <- died/total           
    rates[n,'qx']    <- died / living        
    living           <- living - died
    n                <- n + 1
  }
  
  
  # Do bootstrap
  if (qx_field == 'MEAN') {
    warning("Bootstrap not implemented for uncensored")
    qx_field <- 'OBSERVED'
  }
  
  # Use qx derived from age at death
  if (qx_field == 'OBSERVED') {
    
    for (x in 1:nrow(rates)) {
      rates[x,'var'] <- qx.var(rates[x,'qx'],rates[x,'Nx'])
      qx.limit <- qx.cf(rates[x,'qx'],rates[x,'Nx'])
      rates[x,'lower'] <- qx.limit[['lower']]
      rates[x,'upper'] <- qx.limit[['upper']]
    } 
    
  }
  
  rm(age_classes)
  rates <- table.extend(rates)   # Estimate px, lx,Lx, Tx, ex
  return(rates) 
}



###
#' @title Extend mortality table
#' @description \code{table.extend} estimates px, lx, Lx, Tx and ex
#' @param rates data frame or matrix with 'qx' 
#' @return rates
#' @author Frank PG Princee
#' 
table.extend <- function(rates) 
{
  rates[,'px']  <- mortality.px(rates[,'qx'])  
  rates[,'lx']  <- mortality.lx(rates[,'px']) 
  rates[,'Lx']  <- mortality.midLx(rates[,'lx']) 
  rates[,'Tx']  <- mortality.Tx(rates[,'Lx'])
  rates[,'ex']  <- mortality.ex(rates[,'Tx'],rates[,'lx'])
  
  rates[,4:10]  <- round(rates[,4:10],3)
  rates[,11]    <- round(rates[,11],5)
  rates[,12:13] <- round(rates[,12:13],3)
  
  rownames(rates) <- 0:(nrow(rates) -1 )
  return(rates)
}


###
#' @title Distribute unknown sex in life tables  
#' 
#' @details The numbers at risk of unknown sex are distributed over 
#' male and female classed according the female sex ratio \eqn{F/(M+F)} 
#' @param lifetable data frame or matrix with mortality data
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.unknownSex <- function(lifetable,sexratio=0.5)
{
  unk <- studbook.read('mortality',sex='unknown')
  if (is.null(unk)) {
    cat("No mortality data on individuals of unknown sex.")
    return(lifetable)
  }
  
  cat("Distribute unknown sex over mortality rates\n")
  for (i in 1:length(unk$N)){
    if (lifetable$N[i] < 1) {
      next
    }
    lifetable[i,'N']  <- lifetable[i,'N'] + (unk[i,'N'] * sexratio)
    lifetable[i,'D']  <- lifetable[i,'D'] + (unk[i,'D'] * sexratio)
    lifetable[i,'qx'] <- lifetable[i,'D'] / lifetable[i,'N']
    lifetable[i,c('var','lower','upper')] <- rep(NaN,3)
  }
  return(lifetable)
}


###
#' @title Analyse PML bootstrap results of mortality rates

#' @details PML stores bootstrap results for each class during each class.  
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param groups Groups e.g. classes to be analysed (default c(NA))
#' @author Frank PG Princee
#' @family mortality
#' @export
#
mortality.analyse <- function(sex=c("female","male","unknown","all"),groups=c(NA))
{
  if (studbook.empty()) {
    studbook()
  }
  
  sex       <- match.arg(sex)
  file_name <- paste("bootstrap-Qx-",sex,".csv.gz",sep='')
  
  lft <- studbook.read('mortality',sex=sex)
  
  if (is.null(lft)) {
    stop("No bootstrap data on mortality.")
  }
  rownames(lft) <- 0:(nrow(lft) -1 )
  
  studbook.openOutput('bootstrap_Qx')
  n <- lft$N
  rm(lft)
  
  rate <- expression(q[x])
  main_title = paste('Bootstrap of',sex,rate)     
  lifetable.analyseBootstrap(file_name,groups,main_title,ylabel=rate)
  studbook.closeOutput()
  
  return(TRUE)
}


###
#' @title Radix of cohort 
#' @details The radix allows to more easily compare Dx values when numbers at risk per age classes 
#' differ between samples.
#' @param Life_table Mortality (Life) table data as created by \code{\link{mortality.table}}.
#' @param cohort Size of cohort (default: 1000) 
#' @return life table data based on cohort size
#' @author Frank PG Princee
#' @family mortality
#' @keywords internal
#' 
mortality.radix<- function(life_table,cohort=1000)
{
  cohort_size <- cohort
  for (c in 1:nrow(life_table) ) {
    life_table[c,"Nx"] <- cohort
    life_table[c,"Dx"] <- round(cohort * life_table[c,"qx"])
    life_table[c,"dx"] <- round(life_table[c,"Dx"] / cohort_size,3)
    cohort             <- cohort - life_table[c,"Dx"]
  }
  return(life_table)
}


###
#' @title Compare mortality between sexes without distribution of unknown sex
#' @param class_width Width of class in number of days (and decimals) (default: 365.25 = 1 year)
#' @author Frank PG Princee
#' @family mortality 
#' @export
#' 
mortality.sexes <- function(class_width=365.25)
{
  if (studbook.empty()) {
    studbook()
  }
  main_title <- 'Survivorship between sexes'
  male       <- mortality.table(sex='male',qx_field='OBSERVED',censored=TRUE,class_width,file_name='mortality')
  female     <- mortality.table(sex='female',qx_field='OBSERVED',censored=TRUE,class_width,file_name='mortality')
  
  if (is.null(male) || is.null(female)) {
    stop("No male and/or female mortality data.")
  }
  
  male   <- male[,c('Nx','Dx','lx')]
  female <- female[,c('Nx','Dx','lx')]
  
  attr(male,"sample")   <- 'males'
  attr(female,"sample") <- 'females'
  lifetable.compare(male,female,main_title)    
  
  if (studbook.plot()) {
    lifetable.plotGroups(male[,'lx'],female[,'lx'],max_y=1,
                         main_title,legend.txt=c('males','females'),
                         ylabel=expression(l[x]),file_name='sexes_lx')
  }
  return(TRUE)
}


###
#' @title Compare results from censored data and un-censored
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param class_width Width of class in number of days (and decimals) (default: 365.25 = 1 year)
#' @author Frank PG Princee
#' @family mortality 
#' @export
#' 
mortality.methods <- function(sex=c("female","male","unknown","all"),
                              class_width=365.25)
{
  if (studbook.empty()) {
    studbook()
  }
  sex <- match.arg(sex)
  main_title <- paste('Censored and uncensored data (',sex,')')
  
  censored   <- mortality.table(sex=sex,qx_field='OBSERVED',censored=TRUE,class_width)
  uncensored <- mortality.table(sex=sex,qx_field='OBSERVED',censored=FALSE,class_width)
  
  if (is.null(uncensored) || is.null(censored)) {
    stop("Missing censored and/or uncensored mortality data.")
  }
  
  # Shrink data frame 
  censored    <- censored[,c('Nx','Dx','lx')]
  uncensored  <- uncensored[,c('Nx','Dx','lx')]
  attr(censored,"sample")   <- paste("censored",sex)
  attr(uncensored,"sample") <- paste("uncensored",sex)
  
  
  names <- list("Group"=c('censored','uncensored'),
                "Survival"=c('survived','died'))
  
  censored.survived    <- censored[1,'Nx'] - censored[1,'Dx'] 
  censored.died        <- censored[1,'Dx']
  uncensored.survived  <- uncensored[1,'Nx'] - uncensored[1,'Dx'] 
  uncensored.died      <- uncensored[1,'Dx']
  
  data_set             <- c(censored.survived,uncensored.survived,
                            censored.died,uncensored.died)
  
  studbook.twotable(data_set,names,main_title)
  cat("\n")
  
  lifetable.compare(censored,uncensored,main_title)
  
  if (studbook.plot()) {
    lifetable.plotGroups(censored[,'lx'],uncensored[,'lx'],max_y=1,
                         main_title,legend.txt=c("censored","uncensored"),
                         ylabel=expression(l[x]),file_name='method_lx')
  }
  return(TRUE)
}


###
#' @title Plot log survivorship (as radix 1000) to show shape 
#' @param sex Sex group; valid names are \code{"male","female","unknown","all"} 
#' (default: "female")
#' @param class_width Information on class width for X-axis label (default:"year")
#' @author Frank PG Princee
#' @references Pearson (1928)
#' @family mortality
#' @export
#' 
mortality.shape <- function(sex=c("female","male","unknown","all"),class_width='year')
{
  if (studbook.empty()) {
    studbook()
  }
  sex <- match.arg(sex)
  life_table   <- mortality.table(sex,censored=TRUE,file_name='mortality')
  
  if (is.null(life_table)) {
    return(FALSE)
  }
  
  life_table <- mortality.radix(life_table)
  main_title <- paste( 'Survivors (',sex,')' ) 
  xlabel <- paste('Age class (width=',class_width,')')
  
  studbook.caption(main_title,"Log survivorship in cohort of 1000.")
  plot(life_table[,c('class','Nx')],type='l',log='y',
       xlab=xlabel,ylab=expression(N[x]))
  for (y in c(50,100,200,500,1000)) {
    abline(h=y,lty='dotted')
  }
  studbook.title(main_title)
  return(TRUE)
}


###
#' @title Mortality rates
#' @description \code{studbook.mortality} imports PML mortality tables
#' @details optionally corrected for sex ratio
#' 
#' \describe{
#' \item{observed}{The observed \eqn{q_x} is presented. The variance and confidence limits are
#' estimated as a expected from a binomial distribution.}
#' \item{bootstrap}{the bootstrap mean, variance and 2.5 and 97.5 percentiles are used.}
#' }
#' @param sex Sex group; valid names are \code{"female","male","unknown","all"} 
#' (default: "female")
#' @param censored TRUE is censored data; FALSE is uncensored (age at death)
#' @param qx_field Column names in PML file \code{"OBSERVED","MEAN"} (default: "OBSERVED")
#' @param sexratio Distribution of unknown sex ratio according sex ratio F/(M+F); 
#' NA =exclude (default: NA)
#' @param confidence TRUE plot confidence interval (default:TRUE)
#' @param radix Convert Nx to radix of 1,000 or 10,000 (default: NA) 
#' @param smooth Smoothing procedure or parametric model and additional parameters. Valid options are:
#' \code{"none"},\code{"PMx"},\code{"SPARKS"},\code{"Tukey"},\code{"Siler"}, \code{"Gompertz"}
#' and \code{"Weibull"} (default: "none") 
#' @param params Optional parameters for smoothing procedure/parametric model (default:NULL)
#' @param class_width Information on class width for X-axis label (default:"year")
#' @param file_name Base name of data file (default: \code{"mortality"})
#' @author Frank PG Princee
#' @family mortality
#' @export
#'  
studbook.mortality <- function(sex=c("female","male","unknown","all"),censored=TRUE,
                               qx_field=c('OBSERVED','MEAN'),sexratio=NA,confidence=TRUE,radix=NA,
                               smooth=c("none","PMx","SPARKS","Tukey","Siler","Gompertz","Weibull"),
                               smooth_params=NULL,class_width=365.25,file_name=NULL)
{
  if (studbook.empty()) {
    studbook()
  }
  
  sex       <- match.arg(sex)
  qx_field  <- match.arg(qx_field)
  smooth    <- match.arg(smooth)
  mortality <- mortality.table(sex=sex,censored=censored,qx_field=qx_field,class_width=class_width,
                               file_name=file_name)
 
  if (is.null(mortality)) {
    stop("No mortality table data.")
  }
  
  # Adjust mortality rate for unknown sex according sex-ratio
  if(!is.na(sexratio)) {
    if (sex == 'male') {
      sexratio <-  1.0 - sexratio
    }
    mortality <- mortality.unknownSex(mortality,sexratio)
  }
  
  # Convert to radix e.g. 1,000 or 10,000
  if (!is.na(radix)) {
    mortality <- mortality.radix(mortality,radix)
  }
  
  # Smoothing method
  smooth_comment <- smooth
  if (smooth != 'none') {
    model     <- c(smooth,smooth_params)
    result    <- model.smooth(mortality[,c('Nx','qx')],c(smooth,smooth_params)) 
    mortality <- cbind(mortality,result)
    smooth_comment <- comment(result)
    colnames(mortality)[ncol(mortality)] <- "smoothed"
    confidence <- FALSE
  }
  
  attr(mortality,"sample") <- ifelse(censored,"censored","age at death")
  attr(mortality,"sex")    <- sex
  attr(mortality,"qx")     <- paste("Data column",qx_field)
  attr(mortality,'smoothed') <- smooth_comment
  if (!is.na(radix)) {
    attr(mortality,"radix") <- radix
  }
  
  if (qx_field == 'OBSERVED') {
    param_method <- 'observed' 
  } else {
    param_method <- 'bootstrap'
  }
  
  
  studbook.screenTitle(paste('Mortality rates in',sex,'(',param_method,')'))
  options(scipen=TRUE)
  print(mortality,digits=3)

  fig_name   <- 'mortality'
  main_title <- 'Mortality rates from'
  if (censored) {
     fig_name <- paste(fig_name,'censored',sep='-')
     main_title <- paste(main_title,'censored data')
  } else { 
    fig_name <- paste(fig_name,'uncensored',sep='-')
    main_title <- paste(main_title,'uncensored data')
  }
  fig_name <- paste(fig_name,sex,sep='.')
  studbook.openOutput(fig_name)
  main_title <- paste0(main_title,' (',sex,')')
  
  ylabel <- expression(q[x])
  if (smooth == 'Weibull') {
    field_name <- 'lx'
    ylabel <- expression(l[x])
  }
  lifetable.plot(mortality,field_name='qx',confidence=confidence,main_title=main_title, 
                class_width=class_width,max_y=1,ylabel=ylabel)
  studbook.closeOutput()  
  
  return(TRUE)
}




