# Define studbook options and settings

###
#' @title Default studbook settings
#' @description \code{.defaultStudbook} creates and initializes variables in the environment 
#' \code{.StudbookEnv}
#' @details This function is called when is the package \code{studbookR} is loaded. 
#' The following variables are created in the environment \code{.StudbookEnv}:
#' \describe{
#'  \item{census.interval}{Census interval; default: \code{"year"}}
#'  \item{figure}{Figure number; start set to 1.}
#'  \item{file_handle}{Generic file handle; default: \code{NULL}}
#'  \item{format}{Format of figures; default: \code{"eps"}}
#'  \item{name}{Name of studbook project; default :\code{NULL}}
#'  \item{output}{Output device; default: \code{"screen"} }
#'  \item{overview}{Generate overview; default: \code{FALSE}}
#'  \item{path}{Main path to project folders; default: \code{"~/pml/projects"}}
#'  \item{plot}{Plot graphs; default: \code{TRUE}}
#'  \item{program_mode}{Program that generates input files; default: \code{"PML"}}
#' }
#' @author Frank PG Princee
#' 
.defaultStudbook <- function()
{
  assign("census.interval","year",.StudbookEnv)
  assign("figure",1,.StudbookEnv)
  assign("file_handle",NULL,.StudbookEnv)
  assign("format","eps",.StudbookEnv)
  assign("name",NULL,.StudbookEnv)
  assign("output","screen",.StudbookEnv)
  assign("overview", FALSE, .StudbookEnv) 
  assign("path","~/pml/projects",.StudbookEnv)  # in home directory
  assign("plot",TRUE,.StudbookEnv)
  assign("program_mode","PML",.StudbookEnv)
  
  
  #load last settings
  if (file.exists("~/.studbookR.rda")) {
    load("~/.studbookR.rda",envir=studbookR:::.StudbookEnv)
  }
  cat("Studbook mode       :",studbook.get("program_mode"),"\n")
  cat("Main projects folder:",studbook.getPath(),"\n")
  cat("Last studbook used  :",studbook.getName(),"\n")
}


###
#' @title studbook tutorial
#' @description \code{tutorial} uses a demonstration data-set
#' @author Frank PG Princee
#' @export
#' 
tutorial <- function()
{
  path <- find.package("studbookR")
  path <- paste(path,"extdata/projects",sep="/")
  studbook.setMode("TUTORIAL",path)
}


###
#' @title Set studbook analyses program to PML
#' @param path Path where PML projects are stored (default: \code{"~/pml/projects"})
#' @export 
#' 
pml <- function(path=NULL)
{
  if (is.null(path)) {
      path <- "~/pml/projects"
  }
  studbook.setMode("PML",path)
}
  


###
#' @title Select studbook mode
#' @details The option \code{"TUTORIAL"} provides access to tutorial data.
#' @param program \code{"PML","SPARKS","TUTORIAL"} (default: \code{"PML"})
#' @param path Path where to find projects (default: NULL)
#' @author Frank PG Princee
#' @export
#' 
studbook.setMode <- function(mode=c("PML","SPARKS","TUTORIAL"),path=NULL)
{
  mode <- match.arg(mode)
  if (is.null(path)) {
    path <- getwd()     # Get current path
  } 
  studbook.set("program_mode",mode)
  studbook.set("path",path)
  studbook.save()
}


###
#' @title Save basic settings
#' @details Saves the variables \code{program_mode, path and name} to the file \code{"~/.studbookR.rda"}
#' @author Frank PG Princee
#' @export
#' 
studbook.save <- function()
{
  save(program_mode, path, name,file="~/.studbookR.rda",envir=studbookR:::.StudbookEnv)
}


###
#' @title Set studbook options
#' @author Frank PG Princee
#' @param option Option to set (in single or double quotes)
#' @param value  Value of option
#' @return old value
#' @export
#' 
studbook.set <- function(option,value)
{
  old <- studbook.get(option)
  assign(option,value,.StudbookEnv)
  invisible(old)
}


###
#' @title Get studbook options
#' @param option Option 
#' @return value
#' @author Frank PG Princee
#' @export
#' 
studbook.get <-  function(option)
{
  return(get(option,.StudbookEnv))
}


###
#' @title Check whether option in in range of values
#' @param option Name of option
#' @param values Possible values for option
#' @author Frank PG Princee
#' @export
#' 
studbook.isOption <- function(option,values)
{
  if (is.vector(values)) {
    invisible(is.element(studbook.get(option),values))  
  }
  return(studbook.get(option) == values)
}


###
#' @title Display studbook options and settings
#' @author Frank PG Princee
#' @export
#' 
studbook.showVars <- function()
{
  print(ls(envir=studbookR:::.StudbookEnv,all.names=TRUE))
}


###
#' @title Check whether a graphic format (device) is supported
#' @description \code{is.device} checks whether a specific graphic format
#' is available. Currently only graphic formats "eps","jpg","png","ps","tif" and
#' "svg" are supported.
#' 
#' @param device_name name of device 
#' @return TRUE if supported
#' @author Frank PG Princee
#' @keywords internal
#' 
is.device <- function(device_name=c("eps","jpg","pdf","png","ps","tif","svg"))
{
  device_name <- match.arg(device_name)
  
  if (is.element(device_name,c("eps","ps")) ) {
    device_function <- "postscript"
  } else if (device_name == "jpg") {
    device_function <- "jpeg"
  } else if (device_name == "pdf") {
    device_function <- "pdf"
  } else if (device_name == "png"){
    device_function <- "png"
  } else if (device_name == "tif"){
    device_function <- "tiff"
  } else if (device_name == "svg"){
    device_function <- "svg"
  } else {
    return(FALSE)
  }
  return(exists("device_function",mode="function")) 
}



###
#' @title Check plot graph status
#' @author Frank PG Princee
#' @return TRUE is graph will be plot
#' @keywords internal
#' 
studbook.plot <- function()
{
  return(studbook.get("plot"))
}  


###
#' @title Select studbook
#' @param name Studbook or project name
#' @author Frank PG Princee
#' @export
#' 
studbook.setName <- function(name)
{

  studbook.set('name',name)
  path <- paste(studbook.getPath(),name,sep='/')
  if (dir.exists(path)) {
      studbook.save()
  } else {
      studbook.set('name',NULL)
      cat(paste("Project",name,"does not exist.\n"))
  }
}   


###
#' @title Select studbook project
#' @details This function is wrapper around \code{studbook.setName} allowing to
#' select a studbook from a list of available projects.
#' @param name Name of studbook project. Leave blank to select from list.
#' @author Frank PG Princee
#' @examples
#' \dontrun{
#' studbook()          # Select studbook projects from list
#' studbook("species") # Select studbook project "species"
#' }
#' @export
#' 
studbook <- function(name = NULL)
{
  # No name provided
  if (is.null(name)) {
    if (interactive()) {    
      name <- studbook.projects()
    } else {
      stop("No studbook selected.\n")   
    }
  } else if (name == "?") {
    return(studbook.getName())
  }
  
  if (!is.null(name)) {
    studbook.setName(name)
    studbook.save()
  }

}


###
#' @title Get studbook name
#' @return name
#' @author Frank PG Princee
#' @export
#' 
studbook.getName<-function()
{
  return(studbook.get('name'))
}


###
#' @title Test whether no studbook has been selected
#' @author Frank PG Princee
#' @return TRUE if no  studbook
#' 
studbook.empty <- function()
{
  return(is.null(studbook.get("name")) )
}



###
#' @title Set data path to studbook projects
#' @param path Full path to studbook project
#' @author Frank PG Princee
#' @seealso \code{\link{studbook}}
#' @export
#' 
studbook.setPath<-function(path) 
{
  studbook.set('path',path)
  studbook.save()
}

###
#' @title Get data path
#' @return data path
#' @author Frank PG Princee
#' @export
#' 
studbook.getPath<-function()
{
  return(studbook.get('path'))
}

###
#' @title Set type of graphic file format
#' @param format Image format. Valid options are: \code{eps},\code{jpg},
#' \code{png},\code{tif} and \code{svg}.
#' @author Frank PG Princee 
#' @seealso \code{\link{is.device}}, \code{link{studbook.openOutput}}
#' @export
#' 
studbook.setFormat <- function(format=c("eps","jpg","pdf","png","tif","svg"))
{
  format <- match.arg(format)
  if (is.device(format)) {
    studbook.set('format',format)
  }
}   

###
#' @title Get graphics format
#' @return image format
#' @author Frank PG Princee
#' @export 
studbook.getFormat <- function()
{
  return(studbook.get('format'))
}  


###
#' @title Set output type
#' @param output output style. Valid options are \code{"draft"}, \code{"html"} 
#' \code{"publish"}, \code{"rtf"} and \code{"screen"} (default: "screen").
#' @author Frank PG Princee 
#' @export
studbook.setOutput <- function(output=c("screen","draft","html","publish","rtf"))
{
  output <- match.arg(output)
  
  studbook.set("output",output)
  
  if (output == 'draft') {
    studbook.set("format","pdf")
  } else if (is.element(output,c("html","rtf"))) {
    studbook.set("format","png")
  } else if (output == "publish") {
    studbook.set("format","eps")
  }
}


###
#' @title Get type of output 
#' @author Frank PG Princee 
#' @export
studbook.getOutput <- function()
{
  return(studbook.get('output'))
}


###
#' @title Set (reset) figure number
#' @param number Number of first figure
#' @return previous number
#' @author Frank PG Princee
#' @keywords internal
#' 
studbook.setFigure <- function(number)
{
  old <- studbook.get('figure')
  studbook.set('figure',number)
  return(old)
}

###
#' @title Set (reset) figure number
#' @return Number of figure
#' @author Frank PG Princee
#' @keywords internal
#' 
studbook.getFigure<-function()
{
  return(studbook.get('figure'))
}

###
#' @title New figure number
#' @description \code{studbook.newFigure} increases figure number with 1 
#' @return returns new number
#' @author Frank PG Princee
#' @keywords internal
#' 
studbook.newFigure<-function()
{
  tmp <- studbook.get('figure')
  
  if (!is.null(tmp)) {
     tmp <-  tmp + 1
     studbook.set('figure',tmp)	
  }
  return(tmp)
}
