###
#' @title Create file name
#' @description Create file name for saving figures
#' 
#' @details A file name with full path information is created whenever the output is
#' not directed to screen. Multiple file names of the format "<base_name>-%03d" can be
#' generated.
#' 
#' @param base_name base name of file
#' @param single_file create a single file or multiple files based base_name with sequential numbers (default = TRUE)
#' @return file name or empty string when device is screen.
#' @author Frank PG Princee
#' @note The subfolder 'graphics' must exist in the project folder.
#' @keywords internal
#
studbook.createFileName <-function(base_name,multiple_file=FALSE)
{
  if (studbook.isOption('output','screen')) {
    return("")
  }
  
  base_name <- paste(studbook.getName(),base_name,sep="-")
  if (multiple_file) {
    base_name <- paste(base_name,"%03d",sep="-")                 # base_name-%03d (=1-999)
  }
  
  if (studbook.isOption('output','html')) {
    sub_folder <- 'html'
  } else {
    sub_folder <- 'graphics'
  }    
  file_name <- paste(studbook.getPath(),studbook.getName(),sub_folder,base_name,sep='/')
  file_name <- paste(file_name,studbook.getFormat(),sep='.')
  return(file_name)
}


###
#' @title Set graph grid
#' @description \code{studbook.setFigureGrid} sets number of figures per row and/or column
#' 
#' @param rows number of rows (default =  1)
#' @param columns number of columns (default = 1) 
#' @param bycol rotate figures by column (default = TRUE)    
#' @author Frank PG Princee
#' @seealso \code{\link{par}}
#' @export
#
studbook.setFigureGrid <- function(rows=1,columns=1,bycol=TRUE)
{
  if (bycol) {
    par(mfcol=c(rows,columns) )
  } else {
    par(mfrow=c(rows,columns) )
  }
}


###
#' @title Open draft report
#' @description \code{studbook.openDraft} opens a PDF device to write figures.
#' 
#' @details The PDF report is formatted as A4 with four figures per page. The base name
#' of the PDF file is "draft".
#' 
#' @author Frank PG Princee
#' @export   
#
studbook.openDraft <- function()
{
  studbook.setFormat('pdf')
  
  file_name <- studbook.createFileName('draft',TRUE)
  print(file_name)
  if (exists("pdf",mode="function")) {
    pdf(file=file_name,family='Helvetica',onefile=TRUE,title='PML Analyses',paper="A4")
  }
  
  #Important: change settings after new device has been activated
  par(cex.main=0.90,mai=c(1.02,0.82,0.82, 0.42),mfrow=c(2,2),las=0) 
}


###
#' @title Opens graphic device
#' @details This function opens a graphic as set in the option "format", depending on the selected 
#' option "output" (in the global list \code{studbook.options}).
#' 
#' No (new) device is opened when the option "output" is "draft" or "screen". 
#' The option "screen" will change the margins of the screen device and resets
#' the number of figures to 1. Furthermore, it will activate a 
#' prompt (\code{par(ask=TRUE)}).
#' 
#' The output devices "publish" and "html" will (re-)set the graphic format to
#' encapsulated postscript ("eps") and "png", respectively. 
#' Furthermore, the option "publish" changes the margins and prints values
#' at the y--axis horizontally. 
#' 
#' The following graphic formats are currently implemented (when supported):
#' \describe{
#' \item{eps}{width and height: 4 inch ~ 10 cm}
#' \item{jpg}{width and height: 480 pixels, quality: 100\%}
#' \item{png}{width and height: 480 pixels}
#' \item{tif}{width and height: 480 pixels, compression method: "lzw"}
#' \item{svg}{width and height: 480 pixels}
#' }
#' 
#' @param analysis name of analysis which serves as base name for output file
#' @return "screen" or file name (empty when output is "draft")
#' @author Frank PG Princee
#' @seealso \code{\link{studbook.setOutput}}, \code{\link{studbook.setFormat}}
#' @export
#  
studbook.openOutput <- function(analysis="analysis")                          # Name of analyses: blank for screen devices
{
  if (!studbook.plot()) {
    return("")
  }
  
  if (studbook.isOption('output','draft')) {                                  # File already opened thru openOverview   
    return("")
  }
  
  if (studbook.isOption('output','screen')) {
    par(cex.main=0.90,mai=c(1.02,0.82,0.82,0.42),mfcol=c(1,1),las=0)
    par(ask=TRUE)
    return('screen')
  }
  
  if (studbook.isOption('output','publish')) {
    studbook.setFormat('eps')
  }
  
  if (studbook.isOption('output','html' )) {
    studbook.setFormat('png') 
    par(ask=FALSE)  
  }  
  
  file_name <- studbook.createFileName(analysis,multiple_file=TRUE)
  
  # Encapsulated postscript:file per graph
  if ( studbook.isOption('format','eps')) {
    postscript(file=file_name,onefile=FALSE,horizontal=FALSE,width=4,height=4,
               title='PML Analyses',paper="special")
  }
  
  
  if ( studbook.isOption('format','jpg')) {
    jpeg(filename=file_name,quality=100,width=480,height=480,units='px')
  }
  
  if ( studbook.isOption('format','png') ) {
    png(filename=file_name,width=480,height=480,units='px')
  } 
  
  if ( studbook.isOption('format','tif')) { 
    tiff(filename=file_name,compression='lzw',width=480,height=480,units='px')
  }    
  
  if (studbook.isOption('format','svg')) {
    svg(filename=file_name,onefile=FALSE,width=4,height=4)                   # size in inches
  } 
  
  if (studbook.isOption('output','publish')) {
     par(mai=c(1.0, 0.8, 0.1, 0.4),mfcol=c(1,1),las=1) 
  }
  return(file_name)
}



###
#' @title Close output device
#' @details Closes output devices other than opened as "screen" or "draft".
#' 
#' @return TRUE if successful
#' @author Frank PG Princee
#' @seealso \code{\link{studbook.openOutput}}
#' @export
#' 
studbook.closeOutput <- function()
{
  
  if (studbook.isOption('output',c('screen','draft'))       # screen or draft  
      || !studbook.plot()) {                                # no graph plotting       
    return(FALSE)
  }
  
  dev.off()                                                                   # Device is unique file group
  return(TRUE)
}

###
#' @title Close draft report
#' @description \code{studbook.closeDraft} closes the device used to write a draft report.
#' 
#' @details This function closes both draft report and "figures.txt" with information
#' on figures.
#' 
#' @author Frank PG Princee
#' @seealso \code{\link{studbook.openDraft}}
studbook.closeDraft <- function()
{              
  if (studbook.isOption('output','draft')) {
    dev.off()
    studbook.closeTextFile()
  }
}



###
#' @title Open text files
#' @description \code{studbook.openTextFile} opens the text file for information on
#' figures and a file to redirect screen output.
#' 
#' @details This function uses \code{sink} to redirect screen output to file. The report date, 
#' common (English) and scientific names of the studbook species, and the project name
#' are included in a header.
#' 
#' The text file "figures.txt" with information on figures is created in the subfolder "graphics"
#' as set in \code{studbook.setPath}. 
#' 
#' @param file_name Name of text file
#' @author Frank PG Princee
#' @seealso \code{\link{studbook.setPath}}
#' @export
#
studbook.openTextFile <- function(file_name) 
{
  
  text_file     <- paste(file_name,'txt',sep='.')
  text_file     <- paste(studbook.getPath(),studbook.getName(),
                         'reports',text_file,sep='/')
  figure_file   <- paste(studbook.getPath(),studbook.getName(),
                         '/graphics/figures.txt',sep='/')
  
  tmp_handle <- file(figure_file,"w")
  studbook.set("file_handle",tmp_handle)
  sink(text_file)
  biology <- studbook.biology() 
  cat("Report date    :",format(Sys.time(), "%d %b %Y %X"),"\n")
  cat("Common name    :",XML::xmlValue(biology[['COMMON_NAME']]),"\n")
  cat("Scientific name:",XML::xmlValue(biology[['LATIN_NAME']]),"\n")  
  cat("Project name   :",studbook.getName(),"\n")
  cat("------------------------------------------------------\n\n") 
}


###
#' @title Closes R sink file and file with figure information
#' @author Frank PG Princee
#' @export
#' 
studbook.closeTextFile <- function()
{
  sink()
  if (isOpen(studbook.get("file_handle"))) {
    close(studbook.get("file_handle"))
    studbook.set("file_handle",NULL)
  }
}



