###
# PARENT-OFFSPRING REGRESSION
###



###
#' @title Extract phenotypic (trait) value of studbook ID
#' @description \code{phenotype.ID} extracts phenotypic value of studbook ID 
#' @param ID text Studbook ID
#' @return value
#' @author Frank PG Princee
#' @keywords internal
#'
phenotype.ID <- function(id=NA)
{
  TRAIT <- phenotype.getActive()
  
  if (is.na(id)) {     # No ID provided
    return(NA)
  }
  id <- as.character(id)
  
  rc <- which(studbook.phenotype[,'animal'] == id)  #Get all matching records
  
  if (length(rc) == 1) {
    phenotype <- studbook.phenotype[rc,TRAIT]
  } else if (length(rc) > 1) {
    phenotype <- mean(studbook.phenotype[rc,TRAIT],na.omit=TRUE) # Mean phenotype of ID
  } else {
    phenotype <- NA
  } 
  
  return(phenotype)  
}


###
#' @title Group phenotypic data in families
#' @param family Raw family data
#' @keywords internal
#' 
phenotype.grouped <- function(family)
{
  family.group <- unique(family[,c('SIRE_ID','DAM_ID')])
  family.group <- within(family.group,{SIRE <- NA
                                       DAM  <- NA
                                       KID <- NA
                                       PARENT <- NA
                                       N <- NA})
 
  
  for (x in 1:nrow(family.group)) {
    kids.idx <- which(family$SIRE_ID == family.group[x,'SIRE_ID'] & 
                      family$DAM_ID == family.group[x,'DAM_ID'])
    
    family.group[x,'SIRE']   <- family[kids.idx[1],'SIRE']
    family.group[x,'DAM']    <- family[kids.idx[1],'DAM']
    family.group[x,'PARENT'] <- family[kids.idx[1],'PARENT']
    family.group[x,'KID']    <- mean(family[kids.idx,'KID'])
    family.group[x,'N']      <- length(kids.idx)
   
  }
  return(family.group)
}


###
#' @title Estimate heritability by parent-offspring regression
#' @description \code{phenotype.regression} carries linear regression of phenotypic trait values
#' between parent and offspring
#' @details phenotypic values refer to active trait
#' repeated traits: mean offspring - mean dam, mean sire or midparent (of means)
#' @param parent Values are parent, 'sire' or 'dam' 
#' @param offspring Sex of offspring: 'male', 'female' or 'all' (which includes 'unknown') 
#' @param grouped Logical TRUE will group offspring data per family (default: TRUE)
#' @param weighted Logical TRUE will weight for family (group) size (default: TRUE)
#' @param bc  Logical perform Box-Cox transformation analysis (default: FALSE)
#' @param lambda Box-Cox lambda value for Y^(lambda) transformation (default: 1)
#' @return data frame with sire/dam/offspring values
#' @author Frank PG Princee
#' @export
#' 
phenotype.regression <- function(parent=c('parent','dam','sire'),
                                 offspring=c('all','female','male'),
                                 grouped=TRUE, weighted=TRUE,bc=FALSE,
                                 lambda=1)
{
  
  main_title <- "Parent-offspring regression"
  studbook.screenTitle(main_title)
  parent <-  match.arg(parent)
  offspring <- match.arg(offspring)
  
  # Check whether pedigree and phenotype data are available
  
  if (is.null(studbook.pedigree)) {
    pedigree.read()
  }
  
  if (is.null(studbook.phenotype)) {
    phenotype()
  }
  
  repeatable <- phenotype.isRepeatable()
  
  parent    <- toupper(parent)     # Field names are in uppercase 
  offspring <- tolower(offspring)  # sex is in lowercase
  
  if (!grouped) {
    weighted <- FALSE
  }
  
  # Create list with unique IDs of requested sex 
  if (!is.element("SEX",names(studbook.phenotype)) | offspring == "all" ) {
    kids <- unique(studbook.phenotype[,'animal'])
  } else {
    kids <- unique(studbook.phenotype[studbook.phenotype$SEX == offspring,'animal'])
  }
  kids.n <- length(kids)
 
  TRAIT <- phenotype.getActive() 
  family <- data.frame("KID_ID"=kids,"KID"=rep(NA,kids.n),
                       "SIRE_ID"=rep(NA,kids.n),"SIRE"=rep(NA,kids.n),
                       "DAM_ID"=rep(NA,kids.n),"DAM"=rep(NA,kids.n),
                       "PARENT"=rep(NA,kids.n),"N"=rep(0,kids.n)) #N= number of kids
  
  #Loop through Phenotype.data to create family and extract phenotypic values
  for (x in 1:kids.n) {
    phenotype.idx <- match(kids[x],studbook.phenotype[,'animal'])  #Get first record that matches animal
    
    if (is.na(phenotype.idx)) {
      stop(paste("No phenotypic data on",kids[x]))
    }
    
    #Get pedigree record for individual x
    pedigree.idx <- pedigree.findID(kids[x])
    if (is.na(pedigree.idx)) {
      next
    }
    
    dam_id  <- studbook.pedigree[pedigree.idx,2]  
    sire_id <- studbook.pedigree[pedigree.idx,3]
    
    family[x,'SIRE_ID'] <- sire_id
    family[x,'DAM_ID']  <- dam_id
    family[x,'KID']     <- mean(studbook.phenotype[studbook.phenotype$animal==kids[x],TRAIT])
    family[x,'SIRE']    <- phenotype.ID(sire_id)
    family[x,'DAM']     <- phenotype.ID(dam_id)
    family[x,'N']       <- 1
    
    if ( !is.na(family[x,'SIRE']) & !is.na(family[x,'DAM']) ) {
      family[x,'PARENT'] <- sum(family[x,c('SIRE','DAM')])/2.0
    }
  } 
  
  if (grouped) {
    family <- phenotype.grouped(family) 
  }
  
  family <- family[!is.na(family[,parent]),]
  
  # Make sure that rownames and index are the same after grouping
  rownames(family) <- 1:nrow(family)
 
  if (nrow(family) == 0) {
    stop("No data: check whether phenotypes refer to single sex")
  }
    
  
  if (parent == "PARENT") {
    xlabel <- 'Mid-parent phenotype'
  } else {
    xlabel <- paste('Phenotype of',tolower(parent))
  }
    
  xlabel <- paste(xlabel," (",tolower(TRAIT),")",sep="")
  
  if (offspring == "both") {
    ylabel <- 'Phenotype of offspring'
  } else {
    ylabel <- paste("Phenotype of",offspring,"offspring")
  }
  
  if (grouped) {
    cor.rs <- weights::wtd.cor(x=family[,'SIRE'],y=family[,'DAM'],weight=family[,'N']) 
    r.tmp <- cor.rs[1]
    p.tmp <- cor.rs[4]
  } else {
    cor.rs <- cor.test(family[,'SIRE'],family[,'DAM'])
    r.tmp <- cor.rs$estimate
    p.tmp <- cor.rs$p.value
  }
  
  ylabel <- paste(ylabel," (",tolower(TRAIT),")",sep="")
  cat("Trait                 :",TRAIT,"\n")
  cat("Parent                :",xlabel,"\n")
  cat("Offspring             :",ylabel,"\n")
  cat("Offspring grouped     :",grouped,"\n")
  cat("Weighted family size  :",weighted,"\n")
  cat("Box-Cox test          :",bc,"\n")
  cat("Lambda                :",lambda,"\n")
  cat("Phenotypic correlation: ",r.tmp," (p=",p.tmp,") ",
      ifelse(grouped,"weighted","unweighted"),"\n",sep="")  #Ungrouped means a record per offspring
  
  studbook.set("confidence",FALSE)
  studbook.set("prediction",FALSE)

  # Box-Cox transformation for positive values
  # Yeo-Johnson for non-positive values
  
  if (lambda != 1) {
     bc <- FALSE
     family[,parent] <- car::yjPower(family[,parent],lambda)
     family[,'KID']  <- car::yjPower(family[,'KID'],lambda) 
  }
  
  ###
  # todo Weighted method (requires interative reweighing)
  # T = (t-b^2)/(1-t), where b= unweighted regression at start, t is intraclass coefficient
  # w[i] = (n[i] + n[i]T) / (1 + n[i]T)
  #
  data_W <- NULL
  if (weighted) {
    data_W <- family[,'N']   # Weight against number of offspring (crude method) 
  }
  
  regression.linear(data_X=family[,c(parent,"KID")],data_W=data_W,diagnostic=2,
                    main_title=main_title,xlabel=xlabel,ylabel=ylabel,
                    base_name=paste('heritability',phenotype.getActive()),
                    bc=bc) 
}



