###
# Module: pedigree
# package studbookR
# 
# Imports pedigree data from PML 
#

.defaultPedigree <- function()
{
  assign("studbook.pedigree",NULL,.GlobalEnv)    # Make globally available
  
  assign("pedigree.index",NULL,.StudbookEnv)     # Index to pedigree
  assign("pedigree.format","none",.StudbookEnv)  # format 'factor' for MCMCglmm; 'integer' for pedigreemm
  assign("pedigree.pruned",FALSE,.StudbookEnv)   # Active pedigree has been pruned   
  assign("pedigree.remarks",NULL,.StudbookEnv)
}


###
#' @title Clean pedigree
#' @description \code{pedigree.clean} empties the data.frame \code{studbook.phenotype}
#' i.e. set to \code{NULL}.
#' @author Frank PG Princee
#' @export
#' 
pedigree.clean <- function()
{
  studbook.pedigree <- NULL
}
    
    
###
#' @title Import pedigree
#' @description \code{pedigree.read} imports pedigree data from the text file \code{pedigree.csv}.
#' 
#' @details  \code{pedigree.csv} 
#' 
#' 
#' Format of import file \code{pedigree.csv}:
#' \tabular{lll}{
#' \bold{Field name} \tab \bold{Type} \tab \bold{Comment} \cr
#' id               \tab text  \tab studbook id \cr  
#' sire             \tab text  \tab studbook id of sire (NA if wild or unknown)\cr
#' dam              \tab text  \tab studbook id of dam (NA if wild or unknown)\cr 
#' }
#' 
#' must be tab delimited and include the fields
#' must include columns for id, sire and dam (in the same  order). Column headers must be included.
#' The data must be ordered by dat eof birth i.e. parents before offspring  
#'  
#' The data are converted to factors in order to distinguish between id`s that could potentially be
#' be interpreted as the same numerical value e.g. "41" and 0041".  
#' 
#' Imported data are stored as a data.frame with columns "ID", "DAM" and "SIRE" in the
#' global variable \code{Pedigree.data}.x
#' @author Frank PG Princee 
#' @seealso \code{\link{studbook.csv}}
#' @export
#' 
pedigree.read <- function()
{ 
  data <- studbook.csv('pedigree.csv',field_sep='\t')
  
  # Remove duplicate records and order pedigree
  data <- pedantics::fixPedigree(Ped=data[,1:3])   # First three columns
  
  studbook.set("pedigree.index",data[,1])
  studbook.set("pedigree.remarks",studbook.getName())
  studbook.set("pedigree.format","factor")
  
  assign("studbook.pedigree",data,.GlobalEnv)
  cat("[",nrow(data),"records imported ]\n")
}


###
#' @title Plot pedigree
#' @description Plot pedigree of individual or entire population
#' @details Wrapper function around kinship2::plot.pedigree 
#' The following data need to be provided: ID, DAM, SIRE, SEX and STATUS
#' 
#' @param font_size double Relative font size (0..1]
#' @author Frank PG Princee
#' @export
#' 
pedigree.plot <- function(font_size=0.8)
{
  pedigree <- studbook.csv('pedigree.csv',field_sep='\t')
  
  # Kinship pedigree does not accept a single parent to be NA.
  
  for (x in 1:nrow(pedigree)) {
    if (is.na(pedigree[x,'DAM']) | is.na(pedigree[x,'SIRE'])) { # Either one or both NA
      pedigree[x,c('DAM','SIRE')] <- c(NA,NA)
    }
  }
  
  pedigree <- with(pedigree,
                    kinship2::pedigree(id=ID,dadid=SIRE,momid=DAM,
                                        sex=as.integer(SEX),status=as.integer(STATUS)) )
  
  kinship2::plot.pedigree(pedigree,cex=font_size) 
}


###
#' @title Check whether IDs occur in pedigree
#' @param animals: vector with IDs (factors)
#' @author Frank PG Princee
#' @export
#' 
pedigree.check  <- function(animals)
{
  id.data <- studbook.pedigree[1]
  id.data <- as.character(id.data)
  for (animal in animals) {
    if (is.na(match(animal,id.data,nomatch=NA) ) ){
      cat("Individual# ",animal,"does not exist.\n")
    }
  }
}


###
#' @title Find row of ID in Pedigree data
#' @param id ID number
#' @return rownumber or NA
#' @keywords internal
#' 
pedigree.findID <- function(id)
{

  if (is.na(id)) {
    return(NA)
  }
  
  id  <- as.character(id)
  idx <-  match(id,studbook.pedigree[,1]) #  NA is no match is found
  return(idx)
}
  

###
#' @title Convert pedigree ID's to integers 
#' @description \code{pedigree.integer} converts pedigree ID's based on factors to 
#' integers and re-creates pedigree as \code{pedigreemm} pedigree
#' @details pedigree  (i.e. ID = label)
#' @author Frank PG Princee
#' @export
#'
pedigree.integer <- function()
{
  if (get("pedigree.format",.StudbookEnv) == 'integer') {
    return(TRUE)
  }
  
  data  <- get("studbook.pedigree",.GlobalEnv)
  
  # Check use of mdf function in package dmm 
  index <- studbook.get("pedigree.index")
  label <- match(data[,1],index)
  dam   <- match(data[,2],index)
  sire  <- match(data[,3],index)
 
  data <- pedigreemm::pedigree(sire=sire,dam=dam,label=label)
  assign("studbook.pedigree",data,.GlobalEnv)
  assign("pedigree.format","integer",.StudbookEnv)
}


###
#' @title Returns number of animals in pedigree
#' @return Number of records in global variable \code{studbook.pedigree}
#' @keywords internal
#'  
pedigree.size <- function()
{
  return(nrow(studbook.pedigree))
}


###
#' @title Set pruned status
#' @description Set status of pruning pedigree
#' @param status logical TRUE= Pedigree has been pruned
#' @author Frank PG Princee
#' @keywords internal
#' 
pedigree.setPruned <- function(status)
{
  assign("pedigree.pruned",status,.StudbookEnv)
}


###
#' @title Get pruned status
#' @description \code{isPruned} Get information whether pedigree has already been pruned
#' @return TRUE if pruned
#' @author Frank PG Princee
#' @keywords internal
#' 
pedigree.isPruned <- function()
{
  return(studbook.get("pedigree.pruned"))
}


###
#' @title Prune pedigree
#' @description \code{pedigree.prune} prunes the pedigree to relevant ancestors of
#' individuals int trait data
#' 
#' @details This function is a wrapper around the function prunePed in the package MCMCglmm.
#' @param animals vector with id's of animals (as stored in column animal) 
#' @author Franl PG Princee
#' @export 
#' 
pedigree.prune <- function(animals)
{
  animals  <- unique(animals)
  
  if (length(animals) > 0) {
    old_size <- pedigree.size()
    data <- studbook.pedigree
    data <- MCMCglmm::prunePed(data,animals)
    
    assign("studbook.pedigree",data,.GlobalEnv)
   
    new_size <- pedigree.size()
    cat("[ Reduced pedigree from",old_size,"to",new_size,"records ]\n")
    pedigree.setPruned(TRUE)
  } else {
    cat("[ No list of individuals provided ]\n")
  }
}



###
#' @title Show pedigree data
#' @export
#' 
pedigree.list <- function()
{
  print(studbook.pedigree)
}


###
#' @title Sex ratio in individual's offspring
#' @description \code{pedigree.sexratio} tests randomness of male/female sequences per 
#' individual using the runs test. 
#' @details Data in the file \code{pedigree.csv} are imported. Individuals records must be
#' sorted by dat eof birth.
#' @author Frank PG Princee
#' @export
#' 
pedigree.sexratio <- function()
{
  data <- as.data.frame(studbook.csv('pedigree.csv',field_sep='\t'))  
  
  data[data$SEX==1,'SEX'] <- "M"
  data[data$SEX==2,'SEX'] <- "F"
  data[data$SEX==3,'SEX'] <- NA
  
  dam <- unique(data[,'DAM'])
  dam <- na.omit(dam)
  
  for (x in 1:length(dam)) { 
    
    kids <-  which(data$DAM == dam[x])  
    if (is.result(kids)) {
      
      if (length(kids) < 2) {
        next
      }
      
      kids.sex <- data[kids,'SEX']
      kids.sex <- na.omit(kids.sex)
      
      females  <- length(kids.sex[kids.sex == "F"])
      males    <- length(kids.sex[kids.sex == "M"])
      sexratio <- females/(females + males)
      
      cat(dam[x],":")
      
      for (k in 1:length(kids.sex)) {
        cat(kids.sex[k],sep="")
      }
      
      binom.rs <- binom.test(x=c(females,males),p=1/2)  # Test whether equal or not
      equal <- ifelse(binom.rs$p.value < 0.05,FALSE,TRUE)
      
      kids.sex <- as.factor(kids.sex)
      if (length(levels(kids.sex)) != 2) {
        runs.rs <- NULL     # males or females only               
      } else {
        runs.rs  <- tseries::runs.test(as.factor(kids.sex))
      }
      
      
      if (is.null(runs.rs)) {
        cat("\n")
        next
      }
      if (is.nan(runs.rs[["p.value"]]) || runs.rs[["p.value"]][1] > 0.05) {
        allocated <- FALSE  
      } else {
        allocated <- TRUE
      }
      
      cat(" [",ifelse(equal,'equal','bias'),sep="")
      cat(" ",ifelse(allocated,'allocated','none'),"]\n",sep="")
    } 
  }
}

###
#' @title Sex ratio in offspring per location
#' @description \code{location.sexratio} tests randomness of male/female sequences 
#' per location using the binomial test. 
#' @details Data in the file \code{pedigree.csv} are imported
#' @author Frank PG Princee
#' @export
#' 
location.sexratio <- function()
{
  data <- as.data.frame(studbook.csv('pedigree.csv',field_sep='\t'))  
  
  data[data$SEX==1,'SEX'] <- "M"
  data[data$SEX==2,'SEX'] <- "F"
  data[data$SEX==3,'SEX'] <- NA
  
  birthsite <- unique(data[,'BIRTHSITE'])
  for (x in 1:length(birthsite)) { 
    if (is.null(birthsite[x])) {
      next
    }
    kids <-  which(data$BIRTHSITE == birthsite[x])  
    if (is.result(kids)) {
      
      if (length(kids) < 2) {
        next
      }
      
      kids.sex <- data[kids,'SEX']
      kids.sex <- na.omit(kids.sex)
      
      females  <- length(kids.sex[kids.sex == "F"])
      males    <- length(kids.sex[kids.sex == "M"])
      sexratio <- females/(females + males)
      
      cat(birthsite[x]," (",males," ",females,") : ",sexratio," [",sep="")
      
#  H0: The probability of collecting a male is equal to 0.5.   
      binom.rs <- binom.test(x=c(females,males),p=1/2)  
# p < alpha (0.05) => reject H0
      equal <- ifelse(binom.rs$p.value < 0.05,FALSE,TRUE)
      
      cat(ifelse(equal,'equal','bias'),"]\n",sep="")
    } 
  }
}