


.defaultPhenotype <- function() {
  
  assign("studbook.phenotype",NULL,.GlobalEnv)    # Phenotypes of studbook individuals
  assign("phenotype.format",NULL,.StudbookEnv)    # Format of phenotype data
  assign("traits",NULL,.StudbookEnv)              # Name of trait(s) in data set
  assign("phenotype.active",NULL,.StudbookEnv)    # Active trait in analysis
  assign("climate.field","thermal",.StudbookEnv)  # Climate field to add    
  assign("phenotype.repeats",1,.StudbookEnv)      # Minimum number of repeats
  assign("phenotype.remarks",NULL,.StudbookEnv)   # Comments regarding phenotype
  
}


###
#' @title Column names in phenotype data
#' @return vector with column names
#' @author Frank PG Princee
#' @export
#' 
phenotype.names <- function()
{
  return(colnames(studbook.phenotype))
}


###
#' @title Set active trait
#' 
#' @description \code{phenotype.setActive} sets the trait to be analysed. 
#' @param name Column name in the global data frame \code{studbook.phenotype} that
#' holds data on trait. 
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.setActive <- function(name)
{
  assign("phenotype.active",name,.StudbookEnv)
}


###
#' @title Get trait
#' 
#' @description \code{phenotype.getActive} gets the active trait name to be analysed. 
#' 
#' @return name of column with data on trait
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.getActive <- function()
{
  trait <- get("phenotype.active",.StudbookEnv)
  if (is.null(trait) ) {
    cat("Set active trait\n")
  }
  return(trait)
}


###
#' @title Set number of repeats
#' 
#' @description \code{phenotype.setRepeats} sets minimum number of repeats
#' @param repeats integer number of repeats 
#' @author Frank PG Princee
#' @export
#' 
phenotype.setRepeats <- function(number)
{
  assign("phenotype.repeats",number,.StudbookEnv)
}


###
#' @title Get number of repeats
#' 
#' @description \code{phenotype.getTrait} gets the minimum number of repeats in dataset
#' @return number
#' @author Frank PG Princee
#' @export
phenotype.getRepeats <- function()
{
  return(get("phenotype.repeats",.StudbookEnv))
}

###
#' @title Get information on phenotype
#' @return Contents of \code{remarks}
#' @author Frank PG Princee
#' @export
#' 
phenotype.remarks <- function()
{
  return(get("phenotype.remarks",.StudbookEnv))
}


###
#' @title Estimate sample mean of trait
#' @description \code{phenotype.mean} estimates the arithmetic mean of a trait
#' @return mean
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.mean <- function()
{
  TRAIT      <- phenotype.getActive()
  trait.mean <- mean(studbook.phenotype[,TRAIT],na.rm=TRUE)
  return(trait.mean)
}
  

###
#' @title Estimate sample variance of trait
#' @description \code{phenotype.variance} estimates phenotypic variance \code{VP} of a trait in the observed data.
#' @return variance
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.variance <- function()
{
  TRAIT     <- phenotype.getActive()
  trait.var <- var(studbook.phenotype[,TRAIT],na.rm=TRUE)
  return(trait.var)
}

###
#' @title Transform phenotypic data using Yeo-Johnson 
#' @param lambda Level of transformation (default: 1)
#' @keywords internal 
#' 
phenotype.transformation <- function(lambda=1) 
{
  TRAIT <- phenotype.getActive()
  data  <- get("studbook.phenotype",.GlobalEnv) 
  data[,TRAIT] <- car::yjPower(data[,TRAIT],lambda)
  assign("studbook.phenotype",data,.GlobalEnv)
}



###
#' @title Analyses trait variance per generation group
#' 
#' @description \code{phenotype.generation} estimates variances in a trait per generation group. 
#' @param sex Select sex \code{"all", "female", "male"} for first age at reproduction,
#' fitness, and lifespan (default: "all")
#' @param G.min minimum generation to be included (default: NA)
#' @param G.max maximum generation to be included (default: NA)
#' @author Frank PG Princee
#' @export
#' 
phenotype.generation <- function(sex=c("all","female","male"),G.min=NA,G.max = NA)
{
  sex <- match.arg(sex)
  
  TRAIT <- phenotype.getActive()
  
  # Check whether column SEX exists in data set
  if (is.element("SEX",colnames(studbook.phenotype))) { 
    data  <- studbook.phenotype[,c("animal","SEX","G",TRAIT)]  # Extract required columns from environment
    if (sex != "all") {
      data <- subset(data,SEX == sex) 
      data$animal <- factor(data$animal)
    }
  } else {
    data <- studbook.phenotype[,c("animal","G",TRAIT)]
  }
  
  if (is.na(G.min)) {
    G.min <- min(as.integer(data$G))
  }
  if (is.na(G.max)) {
    G.max <- max(as.integer(data$G))
  }
  
  G.N       <- (G.max - G.min) + 1
  G.results <- data.frame("G"=G.min:G.max,"n"=rep(NA,G.N),"mean"=rep(NA,G.N),
                          "var"=rep(NA,G.N),"cv"=rep(NA,G.N))
  G         <- G.min
  
  for (x in 1:nrow(G.results)) {
    G.results[x,'n']    <- length(data[data$G == G,TRAIT])
    G.results[x,'mean'] <- mean(data[data$G == G,TRAIT])
    G.results[x,'var']  <- var(data[data$G == G,TRAIT])
    G.results[x,'cv']   <- cv.mean(mean=G.results[x,'mean'],variance=G.results[x,'var'], 
                                   N=G.results[x,'n'])
    G <- G + 1
  }
  
  G.results <- G.results[!is.na(G.results$var),]
  
  print(G.results)
  
  data <- subset(data,G >= G.min & G <= G.max)
  studbook.openOutput("VP_generation")
  eqn <- formula(paste(TRAIT,"~ G"))
  boxplot(formula=eqn,data=data,main="",xlab="generation",ylab=TRAIT)
  studbook.title(paste("Distribution of",TRAIT,"in generation groups"))
  studbook.closeOutput()
  
  result <-car::leveneTest(y=data[,TRAIT],group=factor(data[,'G']))
  print(result)
  cat('[p < 0.05: reject null hypothesis that variances are equal]\n\n')
  
  
  set_exact = ifelse(is.tied(data[,TRAIT]),FALSE,TRUE)
  
  result <- pairwise.wilcox.test(data[,TRAIT],data[,'G'],
                                 p.adjust.method="holm",set_exact=set_exact)
  print(result)
  cat('[p < 0.05: reject null hypothesis that samples are the same ]\n\n')
  
  
  studbook.set("confidence",TRUE)
  studbook.set("prediction",FALSE)
 
  
  regression.linear(data_X=data[,'G'],data_Y=data[,TRAIT],diagnostic=2,
                    main_title=paste(TRAIT,"per generation"),xlabel="Generation",ylabel=TRAIT,
                    base_name=paste('generation',phenotype.getActive())) 
}


###
#' @title Check repeatibilty
#' @description \code{phenotype.isRepeatable} checks whether a trait is repeatable.
#' @details Phenotypic data are checked for multiple records of the same individual. 
#' The check is carried out on multiple occurrences of an individuals' ID in the 
#' column \code{id_field}.
#' 
#' @param id_field Name of field (column) in data frame that holds individual ID's (default animal). 
#' @return TRUE if multiple records for same individual exist.
#' @author Frank PG Princee
#' @keywords internal
#'  
phenotype.isRepeatable <- function(id_field = 'animal')
{
  ids <- studbook.phenotype[,id_field]
  return( length(unique(ids)) < length(ids) ) 
}


###
#' @title Counts number of observations per individual
#'
#' @description \code{phenotype.countRepeats} counts the number of records (repeats) for 
#' each individual.
#' 
#' @return data frame with individuals ID and repeats per ID
#' @author Frank PG Princee
#' @seealso \code{\link{phenotype.repeats}} \code{\link{phenotype.isRepeatable}}
#' @keywords internal
#' 
phenotype.countRepeats <- function()
{
  repeats <- as.data.frame(table(studbook.phenotype[,'animal']))
  names(repeats) <- c("animal","n")
  return(repeats)
}


###
#' @title Set minimum number of repeats per individual
#' @details Remove individuals with less than the requested repeated observations from 
#' study.
#' @param repeats minimum number of repeats in data. Enter NA to determine
#' distribution of repeats among individuals (default: NA).
#' @author Frank PG Princee
#' @export
#' 
phenotype.repeats <- function(repeats=NA)
{
  if (!phenotype.isRepeatable()) {
    invisible(FALSE)
  }
  if (is.na(repeats)) {
     cat("Distribution of repeated data:\n")
     repeat.id <- table(studbook.phenotype[,'animal'])
     print(table(repeat.id))
     repeats <- readline("Enter minimum number of repeats: ")
  }
  phenotype.setRepeats(repeats)
  
  id.repeats <- phenotype.countRepeats()
  id.include <- id.repeats[id.repeats$n >= repeats,'animal']
  
  data        <- studbook.phenotype
  data        <- data[data$animal %in% id.include,]
  data$animal <- factor(data$animal)
  data$ID     <- data$animal
  
  rownames(data) <- 1:nrow(data)
  assign("studbook.phenotype",data,.GlobalEnv)
  pedigree.read()   #always read pedigree file to ensure that is contains correct data
  invisible(TRUE)
}



###
#' @title Clean  phenotype data
#' @description \code{phenotype.cleanUp} removes rows of the specified field with NA values
#' @param field text Field to clean
#' @author Frank PG Princee
#' @export
#' 
phenotype.clean <- function(field)
{
  # Trap errors with lost information
  if (is.element(field,colnames(studbook.phenotype))) {
    data <- studbook.phenotype
    data <- data[!is.na(data[,field]),]  
    data$animal <- factor(data$animal)
    assign("studbook.phenotype",data,.GlobalEnv)
  }
}



###
#' @title Estimates repeatability in a trait using an ANOVA model
#' @description \code{phenotype.repeatability} estimates repeatability in the selected (active) 
#' trait.
#' @details This function is a wrapper around the function ICCest in the R package ICC
#' @param sex Select sex \code{"all","female", "male"} (default: \code{"all"})
#' @author Frank PG Princee
#' @export
#' 
phenotype.repeatability <- function(sex=c("all","female", "male"))
{
  sex <- match.arg(sex)
  
  TRAIT <- phenotype.getActive() 
  if (!phenotype.isRepeatable()) {
    cat("Phenotype",TRAIT,"is not repeatable","\n")
    return(FALSE)
  }
  
  if (is.element("SEX",colnames(studbook.phenotype))) { 
    data  <- studbook.phenotype[,c("animal","SEX",TRAIT)]  # Extract required columns from environment
    if (sex != "all") {
      data <- subset(data,SEX == sex) 
      data$animal <- factor(data$animal)
    }
  } else {
    data <- studbook.phenotype[,c("animal",TRAIT)]
  }
  
  main_title <- paste("Repeatability in",phenotype.remarks()) 
  studbook.screenTitle(main_title)
  
  trait.N       <- nrow(data) 
  trait.repeats <- phenotype.getRepeats() 
  
  cat("Sex                     :",sex,"\n")
  cat("Trait                   :",TRAIT,"\n") 
  cat("Minimum repeats         :",trait.repeats,"\n")
  digits.old <- options(digits=4)
  cat("Number of individuals   :",length(unique(data[,'animal'])),"\n") 
  cat("Number of observations  :",trait.N,"\n")
  
  # Since version ICC 2.30 TRAIT can be passed as object
  repeats <- ICC::ICCest(animal,TRAIT,data=data)   
  
  cat("Measurements per group  :",repeats$k,"\n")
  cat("Repeatability           : ",repeats$ICC,
      " [",repeats$LowerCI,", ",repeats$UpperCI,"]\n",sep="")
  
  observations <- table(data[,'animal'])
  
  hist(observations,breaks=1:max(observations),
       main="Observed repeats",xlab="repeats")
  par(ask=FALSE)
  cat(createLine(50,"-"),"\n")
  options(digits.old)
 
  return(TRUE)
}


###
#' @title Analyse distribution of phenotypes
#' 
#' @description \code{phenotype.analyse} provides information on basic statistics of a trait.
#' @param sex select sex \code{"all","female", "male"} (default: \code{"all"}).
#' @param test name of distribution (default: 'normal').
#' @author Frank PG Princee
#' @export
#' 
phenotype.analyse <- function(sex=c("all","female", "male"), test='normal')
{
 
  sex <- match.arg(sex)
 
  # Litter data have duplicated records
  if (phenotype.remarks() == "Litter") {
    sex <- "female" 
  }
  
  TRAIT <- phenotype.getActive() 
  if (is.element("SEX",colnames(studbook.phenotype))) { 
     data  <- studbook.phenotype[,c("animal","SEX",TRAIT)]  # Extract required columns from environment
     if (sex != "all") {
       data <- subset(data,SEX == sex) 
       data$animal <- factor(data$animal)
     }
  } else {
     data <- studbook.phenotype[,c("animal",TRAIT)]
  }
  
  par(ask=TRUE)
  main_title <- paste("Variance analysis of",phenotype.remarks()) 
  studbook.screenTitle(main_title)
  
  trait.N       <- nrow(data) 
  trait.mean    <- mean(data[,TRAIT],na.rm=TRUE)
  trait.median  <- median(data[,TRAIT],na.rm=TRUE)
  trait.VP      <- var(data[,TRAIT],na.rm=TRUE)
  
  cat("Sex                     :",sex,"\n")
  cat("Trait                   :",TRAIT,"\n") 
  cat("Number of observations  :",trait.N,"\n")
  digits.old <- options(digits=4)
  cat("Mean                    :",trait.mean,"\n")
  cat("Median                  :",trait.median,"\n")
  cat("Variance                :",trait.VP,"\n")
  cat("Standard deviation      :",sqrt(trait.VP),"\n")
  cat("Coefficient of variation:",cv.mean(trait.mean,trait.VP,trait.N),"%\n")
  cat(createLine(50,"-"),"\n")
  options(digits.old)
  
  distribution.initGraphs(FALSE)
  if (is.element(TRAIT,c('FITNESS','SIZE'))) {  # Fitness and litter size
    studbook.set('barplot',TRUE)
    studbook.set('histogram',FALSE)
  } else {
    studbook.set('barplot',FALSE)
    studbook.set('histogram',TRUE)
  }
  distribution.plot(data[,TRAIT],main_title=TRAIT,
                    xlabel=paste('phenotype',TRAIT),
                    leftPoints=min(data[,TRAIT]) )
  
  if (test == 'normal') {
    distribution.normal(data[,TRAIT])
    
  } else {
    distribution.analyse(data[,TRAIT],test)
  }
  par(ask=FALSE)
}


### 
#' @title Add climate data to phenotype
#' 
#' @description \code{phenotype.addClimate} adds bioClim, thermal or Koeppen-Geiger data 
#' to phenotypes. The climate field is set with climate.field in the studbook
#' environment
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.addClimate <- function()
{
   if (is.null(get('studbook.sites',.GlobalEnv))) {
      sites.load()
   }
  
  if (!is.element('LOCATION',phenotype.names()) ) {
    cat("No location data available: cannot add climate.\n")
    return(FALSE)
  }
  
  data <- studbook.phenotype
  data <- within(data,CLIMATE <- NA)

  climate <- get("climate.field",.StudbookEnv)
  for (x in 1:nrow(data)) {
    site <- data[x,'LOCATION']
    zone <- studbook.sites[studbook.sites$mnemonic==site,climate]  
    data[x,'CLIMATE'] <- ifelse(is.null(zone),NA,zone)  
  }   
  data$LOCATION <- as.factor(data$LOCATION)
  assign("studbook.phenotype",data,.GlobalEnv)
  return(TRUE)
}


###
#' @title Add column ID 
#' @description \code{phenotype.addID} adds column \code{ID} to \code{studbook.phenotype}
#' @details \code{ID} is used to estimate VI/VPCE
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.addID <- function()
{
  data <- studbook.phenotype
  data <- within(data, ID <- animal)
  assign("studbook.phenotype",data,.GlobalEnv)
}


###
#' @title Add column dom 
#' @description \code{phenotype.addDominant} adds \code{dom} to \code{studbook.phenotype}
#' @details \code{dom} is used to estimate VD
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.addDominant <- function()
{
  data <- studbook.phenotype
  data <- within(data, dom <- animal)
  assign("studbook.phenotype",data,.GlobalEnv)
}

###
#' @title Remove unknown IDS 
#' @description \code{phenotype.unknownID} removes records with animal ID = "UNK" etc.
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.unknownID <- function()
{
  data <- studbook.phenotype
  data <- subset(data,!is.element(animal,c("UNK","UNKN","UNKM")) )
  data$animal <- factor(data$animal)
  assign("studbook.phenotype",data,.GlobalEnv)
}


###
#' @title Phenotype first age at reproduction
#' @details stores first age at reproduction in data frame \code{studbook.phenotype} in studbook environment
#' @author Frank PG Princee
#' @keywords internal
#
phenotype.firstAge <- function()
{
  convert_list <- c("animal"="factor","DAM"="factor","SEX"="factor",
                    "REARING"="factor","LOCATION"="factor")
 
  data <- studbook.read('firstbreeding',col_classes=convert_list)
  assign("studbook.phenotype",data,.GlobalEnv)
  
  phenotype.unknownID()
  phenotype.addDominant()
  phenotype.addClimate()
  
  studbook.set("phenotype.remarks","First age at reproduction")
  studbook.set("traits","AGE")
  studbook.set("phenotype.active","AGE")
  studbook.set("phenotype.format","factor")    #Use with MCMCglmm 
}


###
#' @title Phenotype interbirth interval
#' @details dam,date,males,females,unknown,moon,latitude,photoperiod,hours,G,F
#' - stores in data frame \code{studbook.pheneotype} in studbook environment
#' @author Frank PG Princee
#' @keywords internal
#
phenotype.interval <- function()
{
  convert_list <- c("animal"="factor","REARING"="factor")
  data <- studbook.read('birth_interval',col_classes=convert_list,sex='female')
  assign("studbook.phenotype",data,.GlobalEnv)
  
  
  phenotype.unknownID()
  phenotype.addID()
  phenotype.addClimate()
  
  studbook.set("phenotype.remarks","Interbirth interval")
  studbook.set("traits","INTERVAL")
  studbook.set("phenotype.active","INTERVAL")
  studbook.set("phenotype.format","factor")    # Use with MCMCglmm 
}


###
#' @title Construct trait "fitness" from lifespan and litter 
#' @details dam,date,males,females,unknown,moon,latitude,photoperiod,hours,G,F
#' - stores in data frame \code{studbook.pheneotype} in studbook environment
#' @author Frank PG Princee
#' @keywords internal
#
phenotype.fitness <- function()
{
  phenotype.litter('SIZE')
  litter.data <- studbook.phenotype
  
  # Animals that died
  convert_list <- c("animal"="factor","DAM"="factor","SEX" = "factor")
  data <- studbook.read('lifespan',col_classes=convert_list)  
  
  data <- within(data,FITNESS <- 0)
                    
  for (x in 1:nrow(data)) {
    id <- data[x,'animal']
    id <- as.character(id)
    rc <- which(litter.data$animal == id )  #Get all matching records
    if (length(rc) > 0) {
      data[x,'FITNESS'] <- sum(litter.data[rc,'SIZE'])
    }
  }
  assign("studbook.phenotype",data,.GlobalEnv)
  
  phenotype.unknownID()
  
  studbook.set("phenotype.remarks","Fitnes")
  studbook.set("traits","FITNESS")
  studbook.set("phenotype.format","factor")    #Use with MCMCglmm 
  studbook.set("phenotype.active","FITNESS")
}
  


###
#' @title Phenotype litter date and size
#' @details dam,date,males,females,unknown,moon,latitude,photoperiod,hours,G,F
#' - stores in data frame \code{studbook.phenotype} in studbook environment
#' litter_trait \code{SURVIVED} refers to SIZE after 30 days
#' @param litter_trait options are \code{'DAY', 'SIZE','SEXRATIO','SURVIVED'}
#' (default: \code{"DAY"})
#' @author Frank PG Princee
#' @keywords internal
#
phenotype.litter <- function(litter_trait = "DAY")
{
  convert_list <- c("animal"="factor","LOCATION"="factor","LATITUDE"="numeric")
  
  data <- studbook.read('litter_season',col_classes=convert_list)
  
  data <- within(data, LATITUDE <- ifelse(is.nan(LATITUDE),NA,LATITUDE))
  data <- within(data, PHOTOPERIOD <- ifelse(is.nan(PHOTOPERIOD),NA,PHOTOPERIOD))
  
  data <- within(data, {   
    DATE     <- base::as.Date(DATE)
    DAY      <- tis::dayOfYear(DATE)  
    YEAR     <- as.numeric(format(DATE,"%Y"))
    SIZE     <- MALES + FEMALES + UNKNOWN
    SURVIVED <- SIZE - (D_MALES + D_FEMALES + D_UNKNOWN)
    SEXRATIO <- FEMALES / (MALES + FEMALES)
    })
  
  assign("studbook.phenotype",data,.GlobalEnv)
  
  phenotype.unknownID()
  phenotype.addID()
  phenotype.addClimate()
  
  studbook.set("phenotype.remarks","Litter")
  studbook.set("traits", c('DAY','SIZE','SEXRATIO','SURVIVED'))
  studbook.set("phenotype.format","factor")    #Use with MCMCglmm 
  studbook.set("phenotype.active",litter_trait)
}



###
#' @title Phenotype lifespan
#' @details dam,date,males,females,unknown,moon,latitude,photoperiod,hours,G,F
#' - stores in data frame \code{studbook.pheneotype} in studbook environment
#' @author Frank PG Princee
#' @keywords internal
#' 
phenotype.lifespan <- function()
{
  convert_list <- c("animal"="factor","DAM"="factor",
                    "SEX"="factor","REARING"="factor") 
  
  data <- studbook.read('lifespan',col_classes=convert_list)  
  
  assign("studbook.phenotype",data,.GlobalEnv)
  
  phenotype.unknownID()
  phenotype.addDominant()
  phenotype.addClimate()
  
  studbook.set("phenotype.remarks","Lifespan (age at death)")
  studbook.set("traits","AGE")
  studbook.set("phenotype.active","AGE")
  studbook.set("phenotype.format","factor")    #Use with MCMCglmm 
}


###
#' @title Lifespan of adults
#' @description \code{phenotype.adult} limits the lifespan data to adults 
#' @details Individuals >= age
#' @param age Age in days when adult stage starts (default: 365)
#' @author Frank PG Princee
#' @export
#' 
phenotype.adult <- function(age=365) 
{
  if (min(studbook.phenotype[,"AGE"]) > age) { # Data may already be reduced
    phenotype.lifespan()  # Re-read data
    if (pedigree.isPruned()) { 
      pedigree.read()
    }
  }
  data <-  subset(studbook.phenotype, AGE >= age)
  assign("studbook.phenotype",data,.GlobalEnv)
  
  if (pedigree.size() > 0) {
    pedigree.prune(studbook.phenotype[,"animal"])
  }
}

###
#' @title Shift phenotypic season backwards
#' @details The variance of seasons that cross the new year will be too high.
#' This problem can besolved by setting the first day of the season to 1.
#' @param day1 Day of the year (1-366) that is set to 1 (default:1)
#' @author Frank PG Princee
#' @export
#' 
phenotype.shiftSeason <- function(day1=1)
{
   data <- get("studbook.phenotype",.GlobalEnv)
   data <- within(data, DAY <- ifelse(DAY < day1,DAY + 366, DAY ))
   data[,'DAY'] <- data[,'DAY'] - (day1-1)
   assign("studbook.phenotype",data,.GlobalEnv)
   data <- get("studbook.phenotype",.GlobalEnv)
}

####
#' @title Show phenotypic data
#' @param scope Scope is \code{"all", "head","tail"} (default: \code{"all"})
#' @author Frank PG Princee
#' @export
#
phenotype.list <- function(scope=c("all","head","tail"))
{
  scope <- match.arg(scope)
  if (scope == "head") {
    print(head( get("studbook.phenotype",.GlobalEnv)))
  } else if (scope == "tail") {
    print(tail( get("studbook.phenotype",.GlobalEnv)))
  } else {
    print(get("studbook.phenotype",.GlobalEnv)) 
  }
}


###
#' @title Convert factor animal to integer index
#' @description \code{phenotype.integer} converts pedigrees to integer indices for
#' use in the package pedigreemm
#' @author Frank PG Princee
#' @export
#'
phenotype.integer <- function()
{
  ped.index   <- studbook.get("pedigree.index")       # Index to unpruned factor pedigree
  trait.data  <- get("studbook.phenotype",.GlobalEnv) 
  
  ids <- match(trait.data[,"animal"],ped.index)
  trait.data$animal <- as.integer(ids)
  trait.data$ID     <- as.factor(trait.data$animal)
  
  
  assign("studbook.phenotype",trait.data,.GlobalEnv)
  assign("phenotype.format","integer",.StudbookEnv) 
}
  

###
#' @title Select available life traits from menu
#' @author Frank PG Princee
#' @export 
# 
phenotype <- function()
{
  
  menu_options <- c("First age at reproduction",
                    "Fitness",
                    "Inter-birth interval",
                    "Lifespan",
                    "Litter day", 
                    "Litter size", 
                    "Litter sex-ratio",
                    "Litter survived")
  
  switch(menu(menu_options),
         phenotype.firstAge(),
         phenotype.fitness(),
         phenotype.interval(),
         phenotype.lifespan(),
         phenotype.litter("DAY"),
         phenotype.litter("SIZE"),
         phenotype.litter("SEXRATIO"),
         phenotype.litter("SURVIVED")) 
}


