###
# Functions to set Bayesian priors
#


###
#' @title Creates a prior for MCMCglmm
#' 
#' @description \code{prior.create} creates prior based on variances and 
#' their distribution patterns. 
#' 
#' @param prior.var prior based on provided variance (default NA). 
#' @param pG distribution of additive (G1...) and residual variances (R).
#' @param random.vars number of random effects must be provided if pG is NULL.
#' @param belief degree of belief in prior (default: 1).
#' @author Frank PG Princee
#' @family prior
#' @export
#' 
prior.create <- function(prior.var,pG=NULL,random.vars,belief=1)
{
  
  if (is.null(pG)) {
    effects <-  random.vars
    pG <- rep(1/(effects+1),effects)        # Equal variance for G and R
  } else {
    effects <- length(pG)
  }
  
  pR  = 1.0 - sum(pG)
  
  # Create R structure
  R.list <- list( V = matrix(prior.var * pR),n=belief)
  
  #Create G structure with additive random effects
  G.list <- list(G1 = list(V=matrix(prior.var * pG[1]),n=belief))
  
  if (effects > 1) {
    for (x in 2:effects) {
      G.list[[x]] <- list(V=matrix(prior.var * pG[x]),n=belief)
      names(G.list)[x] <- paste('G',x,sep='')
    }
  }
  invisible( list(G = G.list,R = R.list) )  
}



###
#' @title Create flat prior
#' @details Hadfield tutorial 2010 
#' @param effects Number of random effects in model 
#' @family prior
#' @export
#' 
prior.flat <- function(effects)
{
  
  R.list <- list(V=1,nu=0)
  G.list <- list(G1 = list(V=1,nu=0))
  
  if (effects > 1) {
    
    for (x in 2:effects) {
      G.list[[x]] <- list(V=1,nu=0)
      names(G.list)[x] <- paste('G',x,sep='')
    }
    
  }
  invisible( list(G = G.list,R = R.list) )
}


###
#' @title Create `Fisher` prior
#' @details Example in tutorial of Pierre de Villemereuil (2012)
#' @param effects Number of random effects in model 
#' @family prior
#' @export
#'
prior.Fisher <- function(effects)
{
  
  R.list <- list(V=1,nu=1)
  G.list <- list(G1 = list(V=1,nu=1,alpha.mu=0,alpha.V=1000))
  
  if (effects > 1) {
    
    for (x in 2:effects) {
      G.list[[x]] <- list(V=1,nu=1,alpha.mu=0,alpha.V=1000)
      names(G.list)[x] <- paste('G',x,sep='')
    }
    
  }
 invisible( list(G = G.list,R = R.list) )
}


###
#' @title Set residual variance to fixed
#' @param effects Number of random effects
#' @references Hadfield (2010)
#' @family prior 
#' @export
#' 
prior.fixed <- function(effects)
{
  R.list <- list(V = 10, fix = 1)
  G.list <- list(G1 = list(V=1,nu=1,alpha.mu=0,alpha.V=1000))
  
  if (effects > 1) {
    for (x in 2:effects) {
      G.list[[x]] <- list(V=1,nu=1,alpha.mu=0,alpha.V=1000)
      names(G.list)[x] <- paste('G',x,sep='')
    }
    
  }
  invisible( list(G = G.list,R = R.list) )
}


###
#' @title Create weak prior
#' 
#' @description \code{prior.weak} creates an inverse-gamma prior 
#' with V = 1 and nu = 0.002
#' @param effects Number of random effects in model 
#' @family prior 
#' @export
#' 
prior.weak <-  function(effects)
{
 
  R.list <- list( V = 1, nu=0.002)
  G.list <- list( G1 = list(V=1,nu=0.002))
  
  if (effects > 1) {
    
    for (x in 2:effects) {
      G.list[[x]] <- list(V=1,nu=0.002)
      names(G.list)[x] <- paste('G',x,sep='')
    }
    
  }
 invisible( list(G = G.list, R=R.list) )
}
