###
# Different methods of population projection
###


###
#' @title Population projection based on the geometric mean of lambda and census data
#' @description \code{projection.census}
#' @inheritParams projection.arima
#' 
#' @param sex  Sex \code{"all","male","female"}. Multiple selections possible, must be in
#' same order (to match columns in census.data)
#' @param first First year in census. NULL is oldest year in data set (default: NULL) 
#' @param last  Las Last year in census. NULL is recentyear in data set (default: NULL)
#' @param project.years Number of years to project
#' @param conf.int Estimate prediction interval for projection (default:FALSE)
#' @param alpha Signficance level (default: 0.05)
#' @author Frank Princee
#' @family population projection
#' @export
#'  
projection.census <- function(sex=c('all','male','female'),first=NULL,last=NULL,
                              project.years=25,conf.int=FALSE,alpha=0.05)
{
  sex <- match.arg(sex,several.ok=TRUE)
  sex <- toupper(sex)
  sex <- ifelse(sex == "ALL",sex,paste(sex,"S",sep=""))
  sex.n <- length(sex)
  
  census.ts    <- census.read("census",first,last)
  census.years <- nrow(census.ts)
  
  
  if (is.null(first) | is.null(last)) {
    first <- start(census.ts)[1]
    last  <- end(census.ts)[1]
  }
  
  
  # mean = geometric mean, lower and upper geometric mean
  geomean.sex <- matrix(ncol=3,nrow=sex.n)
  rownames(geomean.sex) <- sex
  colnames(geomean.sex) <- c("MEAN","LOWER","UPPER")
  
  
  idx <- last - first 
  idx <- census.years - idx + 1
  cat("idx:",idx,"\n")
  
  
  for (x in 1:sex.n) {
    lambda <- census.lambda(census.ts[,sex[x]])
    geomean <- census.geometricMean(rates.data=lambda,alpha=alpha,interval="confidence")  
    geomean.sex[x,1:3] <- as.numeric(geomean[idx,1:3])
  }
  print(geomean.sex)
  
  # Should be easier way to add years to mts model
  total.years <- census.years + project.years
  project.ts<- matrix(ncol=sex.n,nrow=total.years)
  colnames(project.ts) <- sex
  project.ts  <- ts(project.ts,start=first,frequency=1)
  
  # Add column to project ts 
  
  conf.lower <- rep("",sex.n)
  conf.upper <- rep("",sex.n)
  
  for (x in 1 : sex.n) {
    conf.lower[x] <- paste(sex[x],"LOWER",sep=".")
    conf.upper[x] <- paste(sex[x],"UPPER",sep=".")
  }
  
  conf.matrix <- matrix(ncol=(2 * sex.n),nrow=total.years)
  project.ts <- cbind(project.ts,conf.matrix)
  colnames(project.ts) <- c(sex,conf.lower,conf.upper)
  
  # Fill projection census years with data
  project.ts[1:census.years,sex]      <- census.ts[1:census.years,sex] 
  project.ts[census.years,conf.lower] <- census.ts[census.years,sex]
  project.ts[census.years,conf.upper] <- census.ts[census.years,sex]
  
  
  for (year in (census.years + 1):total.years ) {
    project.ts[year,sex] <- project.ts[year-1,sex] * geomean.sex[1:sex.n,'MEAN']
    project.ts[year,conf.lower] <- project.ts[year-1,conf.lower] * geomean.sex[1:sex.n,'LOWER']
    project.ts[year,conf.upper] <- project.ts[year-1,conf.upper] * geomean.sex[1:sex.n,'UPPER']
  }
  
  if (studbook.plot()) {
    census.plot(project.ts,census_type="projection",project.year=last,conf.int=conf.int)
  }
}




###
#' @title Population projection based on arima method
#' @description \code{projection.arima}
#' @details The arima analysis the data from first year to census year
#' @param sex  Sex \code{"all","male","female"}
#' @param first First (census) year in analysis. NULL isfirst census in data set (default:NULL)
#' @param last Last census year in analysis. NULL is last census in data set (default: NULL)
#' @param years integer Number of years to project 
#' @author Frank Princee
#' @family population projection
#' @export
#'  
projection.arima <- function(sex=c('all','male','female'),first=NULL,last=NULL,project.years=25)
{
  
  sex <- match.arg(sex)
  ylab <- sex
  sex <- toupper(sex)
  
  sex <- ifelse(sex == "ALL",sex,paste(sex,"S",sep=""))
  
  screen.title <- paste("Time series analysis of",sex)
  studbook.screenTitle(screen.title)
  census.ts <- census.read("census")
  start <- start(census.ts)[1]
  end   <- end(census.ts)[1]
  
  census.fit <- forecast::auto.arima(window(census.ts[,sex],start=first,end=last),seasonal=FALSE)
  print(census.fit)
  census.projection <- forecast::forecast(census.fit,h=project.years)
  cat("\n")
  cat("[Forecast accuracy]\n")
  print(forecast::accuracy(census.projection))
  cat("-------------------------------------------------\n")
  cat("ME  : mean error                      RMSE: root mean squared error\n") 
  cat("MAE : mean absolute error             MPE : mean percentage error\n") 
  cat("MAPE: mean absolute percentage error  MASE: mean absolute scaled error\n")
  cat("ACF1: autocorrelation at lag 1\n")
  
  census.projection$mean[census.projection$mean < 0] <- 0
  census.projection$lower[census.projection$lower[,1] < 0,1] <- 0
  census.projection$lower[census.projection$lower[,2] < 0,2] <- 0
  
  if (studbook.plot()) {
    studbook.openOutput("arima")
    ylab <- tolower(ylab)
    ylab <- firstCap(ylab)
    plot(census.projection,shadecols=c('grey80','grey95'),plot.conf=TRUE,
         fcol='black',main="",xlab='Year',ylab=ylab)
    abline(v=last,lty='dotted')
    lines(window(census.ts[,sex],start=last))
    studbook.title("Future projection (arima)")
    studbook.closeOutput()
  }
  
  digits.old <- options(digits=4)
  forecast::Acf(window(census.ts[,sex],start=first,end=last),main="Autocorrelation")
  options(digits.old)
}


###
#' @title Create stage matrix from from age classes
#' @description \code{create.stages} 
#' @details This function supports stages for which start and end can be defined by age. 
#' stage.list=list(infant=0,juvenile=1,adult=3:9,senescent=10)   
#' @param stage.list List with in which component names refer to stages and age class boundaries are
#' provided as either single or multiple classes e.g. list(infant=0,juvenile=1:2).
#' @param ages A matrix or data frame with age class per row and count(s) in column(s)
#' @return matrix 
#' @author Frank Princee
#' @family population projection
#' @export
#' 
create.stages <- function(stage.list,ages)
{ 
  # Prepare matrix
  classes <- nrow(ages)
  stage.matrix <- matrix(0,nrow=length(stage.list),ncol=ncol(ages))
  rownames(stage.matrix) <- names(stage.list)
  colnames(stage.matrix) <- colnames(ages)
  
  for (x in 1:length(stage.list)) {
    # Add last classes to last stage if only start has been defined
    if (x == length(stage.list) & length(stage.list[[x]]) < 2) {
      stage.list[[x]] <- stage.list[[x]]:(classes - 1)
    }
    stage.list[[x]] <- stage.list[[x]] + 1                 # Adjust age class to row index
    
    if (length(stage.list[[x]]) < 2) {
      stage.matrix[x,] <- ages[stage.list[[x]],]
    } else {
      stage.matrix[x,] <- colSums(ages[stage.list[[x]],])  # Sum columns for defined rows
    }
  }
  return(stage.matrix)
}


###
#' @title Stochastic population projections based on Leslie matrix
#' @description \code{projection.stochastisc}
#' @details If number of years is entered then the the timespan will be 0:25 
#' Age classes are grouped in stage groups for graphic purposes i.e. the population
#' projection is based on age classes. 
#' 
#' @param sex Sex group; valid names are \code{"female","male","unknown","all"} 
#' (default: "female")
#' @param field text Data field for \eqn{q_x} and \eqn{m_x}: \code{"OBSERVED","MEAN","LOWER","UPPER"} as
#' used in PML output (default: "OBSERVED").
#' @param season Type of birth season: \code{"pulse"} or \code{"flow"} (default: \code{"pulse"})
#' @param census Type of census" \code{"post"} or \code{"pre"}-breeding (default: \code{"post"})
#' @param date Date of census in "YYYYMMDD" format; or \code{"first","last", "?"} (default: \code{"last"})
#' @param project.years integer sequence of years e.g. 2014:2024 or total years e.g. 20, The first year
#' is the current population. 
#' @param observed TRUE is plot census data when available (default:FALSE) 
#' @param varF matrix Variance in fecundity when litter size > 1. (default: NULL)
#' 
#' @param runs integer Number of replications (default: 100)
#' @param alpha double Level of significance 'translated' into percentiles; 
#' No percentiles are estimated when alpha=0 (default alpha: 0.05) 
#' @author Frank PG Princee
#' @family population projection
#' @export
#' 
projection.stochastic <- function(sex=c("female","male","unknown","all"),
                                  field=c("OBSERVED","MEAN","LOWER","UPPER"),
                                  season=c("pulse","flow"),census=c("post","pre"),
                                  date="last",project.years=25,observed=FALSE,
                                  varF=NULL,runs=100,alpha=0.05) 
{
  
  if (studbook.empty()) {
    studbook()
  }
  
  sex    <- match.arg(sex)
  field  <- match.arg(field)
  season <- match.arg(season)
  census <- match.arg(census)
  
  leslie.matrix <- studbook.leslieMatrix(sex=sex,season=season,census=census,
                                         show_info=FALSE,sensitivity=FALSE)
  
  if (observed) {
    date <- "?"
  }
  
  if (is.element(date,c("first","last","?"))) {
    date <- age.file(date,sex)
  }
  
  population <- studbook.read(paste("age",date,sep="-"),sex)
  first.year <- as.integer(substr(date,1,4))   #Date is yyyymmdd format
  
  # Get year of last age distribution
  if (length(project.years) < 2) {
    first.year <- as.integer(substr(date,1,4))   #Date is yyyymmdd format
    project.years <- first.year:(first.year + project.years)
  }
  timespan <- length(project.years)
  
  
  if (observed) {
    census.ts <- census.read("census")
    
    if ( start(census.ts)[1] <= project.years[timespan] &
         end(census.ts)[1] > project.years[1] ) {
      census.sex <- ifelse(sex == "ALL",sex,paste(sex,"S",sep=""))
      census.sex <- toupper(census.sex)
      census.ts <- window( census.ts[,census.sex],start=project.years[1],
                           end=project.years[timespan])
    } else {
      observed <- FALSE
    }
    
  }
  
  
  # Adjust age to matrix
  n.pop    <- nrow(population)
  n.matrix <- nrow(leslie.matrix)
  
  if (n.pop > n.matrix) {
    population[n.matrix,'Y'] <- sum(population[n.matrix:n.pop,'Y'])
    population <- population[1:n.matrix,]
  } else if (n.pop < n.matrix) {
    n.dif <- n.matrix - n.pop    
    extra.classes <- data.frame("X"=(n.pop+1):n.matrix,"Y"=rep(0,n.dif))
    population <- rbind(population,extra.classes)
  }
  
  population <- as.matrix(population[,'Y'])
  leslie.split <- popbio::splitA(leslie.matrix)
  
  tmax <- timespan                  # length of the trajectories first element for current population
  totalpop <- matrix(0,tmax,runs)   # initializes totalpop matrix to store trajectories   
  nzero <- population               # starting population size
  
  N0 <- sum(population)
  totalpop[1,] <- rep(N0,runs)
  for (j in 1:runs) {
    n <- nzero
    
    for (i in 2:tmax) {
      n <- popbio::multiresultm(n,leslie.split$T,leslie.split$F,varF=varF) 
      n[is.na(n)] <- 0   # For in case no births occured
      totalpop[i,j] <- sum(n)
    } 
  } 
  
  population.mean <- rowMeans(totalpop)
  minimum <- apply(totalpop,1,min)   # 1: apply by row
  maximum <- apply(totalpop,1,max)
  overview <- cbind(mean=population.mean,min=minimum,max=maximum)
  
  minimum <- apply(totalpop,1,min)
  if (alpha > 0) {
    percentiles <- c(alpha/2,1-(alpha/2))
    population.cf   <- apply(totalpop,1,quantile,probs=percentiles)
    overview <- cbind(overview,lower=population.cf[1,],upper=population.cf[2,])
  } 
  rownames(overview) <- project.years
  digits.old <- options(digits=0)
  print(overview)
  options(digits.old)
  
  
  
  min.x <- project.years[1]
  max.x <- project.years[timespan] 
  min.y <- min(totalpop,na.rm=TRUE)
  max.y <- max(totalpop,na.rm=TRUE)
  
  if (observed) {
    min.x <- min(min.x,start(census.ts)[1],na.rm=TRUE)
    max.x <- max(max.x,end(census.ts)[1],na.rm=TRUE)
    min.y <- min(min.y,census.ts,na.rm=TRUE)
    max.y <- max(max.y,census.ts,na.rm=TRUE)
  }
  
  ylabel <- paste(sex,"population")
  ylabel <- tolower(ylabel)
  ylabel <- firstCap(ylabel)
  
  studbook.openOutput('future_random')
  digits.old <- options(digits=0)
 
  matplot(x=project.years,y=totalpop,
          xlim=c(min.x,max.x),ylim=c(min.y,max.y),
          type = 'l',lty='dotted',col='black',
          xlab = 'Year', ylab = ylabel)
  matlines(x=project.years,y=population.mean,type="l",col='black',lty='solid',lwd=2)
  
  if (alpha > 0) {
    matlines(x=project.years,y=population.cf[1,],col='black',lty='dashed',lwd=2)
    matlines(x=project.years,y=population.cf[2,],col='black',lty='dashed',lwd=2)
  }
  
  if (observed) {
    matlines(x=start(census.ts)[1]:end(census.ts)[1],
             y=census.ts,type="b",lty=4,lwd=1,pch=20)
    abline(v=first.year,lty='dotted',lwd=1)
  }
  
  studbook.title(paste('Stochastic projection of',sex))
  studbook.closeOutput()
  options(digits.old)
}



###
#' @title Population projection based on Leslie matrix
#' @details If data for timespan is provided then the timespan will be 0:25 
#' Age classes are grouped in stage groups for graphic purposes i.e. the population
#' projection is based on age classes, but results re-grouped in stages. 
#' @param sex Sex group; valid names are \code{"female","male","unknown","all"} 
#' (default: "female")
#' @param field text Data field for \eqn{q_x} and \eqn{m_x}: \code{"OBSERVED","MEAN","LOWER","UPPER"} as
#' used in PML output (default: "OBSERVED").
#' @param season Type of birth season: \code{"pulse"} or \code{"flow"} (default: \code{"pulse"})
#' @param census Type of census" \code{"post"} or \code{"pre"}-breeding (default: \code{"post"})
#' @param date Date of census with age distribution data in yyyymmdd format; alternatively
#' enter "first" or "last" available census data; "?" to select list. 
#' @param project.years integer sequence of years e.g. 2014:2024 or total years e.g. 20
#' @param stage.list List with stage definitions 
#' @param show_info Show matrix info (default: TRUE)
#' @param sensitivity Sensitivity and elasticity analyses (default: FALSE)
#' @param do_print Print matrix (default: FALSE)
#' @seealso create.stages
#' @author Frank PG Princee
#' @examples 
#' \dontrun{
#' stage.list <- list(infant=0,juvenile=1,adult=2:10,senescent=11)
#' future.matrix(sex="female",field='MEAN',timespan=50,stage.list=stage.list,do_print=FALSE)
#' }
#' @family population projection
#' @export
#'               
projection.leslie <- function(sex=c("female","male","unknown","all"),field=c("OBSERVED","MEAN","LOWER","UPPER"),
                              season=c("pulse","flow"),census=c("post","pre"),
                              date="last",project.years=25,stage.list=NULL,
                              show_info=TRUE,sensitivity=FALSE,do_print=FALSE)
{
  
  if (studbook.empty()) {
    studbook()
  }
  
  sex    <- match.arg(sex)
  field  <- match.arg(field)
  season <- match.arg(season)
  census <- match.arg(census)
  
  
  leslie.matrix <- studbook.leslieMatrix(sex=sex,season=season,census=census,
                                         show_info=show_info,
                                         sensitivity=sensitivity)
  
  if (is.element(date,c("first","last","?"))) {
    date <- age.file(date,sex)
  }
  population <- studbook.read(paste("age",date,sep="-"),sex)
  
  
  # Get year of last age distribution
  if (length(project.years) < 2) {
    print(date)
    first.year <- as.integer(substr(date,1,4))   #Date is yyyymmdd format
    project.years <- first.year:(first.year + project.years)
  }
  timespan <- length(project.years)
  
  # Adjust age to matrix
  n.pop    <- nrow(population)
  n.matrix <- nrow(leslie.matrix)
  if (n.pop > n.matrix) {
    population[n.matrix,'Y'] <- sum(population[n.matrix:n.pop,'Y'])
    population <- population[1:n.matrix,]
  } else if (n.pop < n.matrix) {
    n.dif <- n.matrix - n.pop    
    extra.classes <- data.frame("X"=(n.pop+1):n.matrix,"Y"=rep(0,n.dif))
    population <- rbind(population,extra.classes)
    
  }
  
  projection <- popbio::pop.projection(leslie.matrix,population[,'Y'],timespan)
  
  stages <- projection$stage.vectors # Grouping i.e. age class or stage as defined in Leslie matrix
  
  if (length(stage.list) > 0) {
    stages <- create.stages(stage.list,stages)   #Create pseudo stages based on age
  }
  colnames(stages) <- project.years
  if (do_print) {
    print(stages)
  }
  
  # Expected population growth based on lambda
  growth.rate <- popbio::lambda(leslie.matrix)
  pop.expected <- rep(0,timespan)
  pop.expected[1] <- projection$pop.sizes[1]  
  
  for (year in 2:timespan) {
    pop.expected[year] <- pop.expected[year-1] * growth.rate
    if (pop.expected[year] < 0) {
      pop.expected[year] <- 0
    }
  }
  
  #Plot data
  studbook.openOutput(paste("projection_matrix",sex,sep="."))
  
  y.min <- min(pop.expected,projection$pop.sizes)
  y.max <- max(pop.expected,projection$pop.sizes)
  
  # Population size (dashed is expected from matrix lambda)
  plot(project.years,projection$pop.sizes, ylim=c(y.min,y.max),type='b',pch=20,
       xlab="Year",ylab=paste("Number of ",sex,"s",sep="") )
  lines(project.years,pop.expected,type='l',lty='dashed',lwd=1)
  
  studbook.title(paste("Projection of",sex,"population"))
  
  # Age distribution (stacked bars with number of individuals per age class) 
  oldmar <- par('mar')
  par(mar=c(3.825, 3.075, 3.075, 3.825))  
  graphics::barplot(stages,space=0,names.arg=colnames(stages),
                    xlab="Year",ylab="Females",
                    legend.text=rownames(stages),
                    args.legend=list(x=ncol(stages),cex=0.6,bg='white',bty = "y"))
  
  par('mar'=oldmar)
  studbook.title(paste(sex,"age distribution per year"))
  
  
  # Damping: (annual) growth rate (lambda)
  plot(project.years[1]:project.years[timespan-1],projection$pop.changes,type='l',
       lwd=1, xlab="Year",ylab=expression(lambda))
  
  studbook.title(paste("Projected lambda in",sex),
                 paste("lambda",round(growth.rate,3),sep="="))
  abline(h=growth.rate,lty='dashed')
  studbook.closeOutput()
}