###
# Global variables
#

.defaultRaster <- function() {
  assign("studbook.raster",NULL,.StudbookEnv) # Active spatial raster
  assign("raster.palette",NULL,.StudbookEnv)  # Raster palette in use
}

###
#' @title Load raster file
#' @description \code{raster.load} loads raster file 
#' @details File \code{raster_file} is loaded into the global variable
#' \code{studbook_raster} using package \code{raster}. 
#' @param raster_file Name of raster file
#' @author Frank PG Princee
#' @export
#' 
raster.load <- function(raster_file)
{
  if (is.null(raster_file) ) {
    stop("No raster file defined")
  }
  cat("Loading raster data...\n")
  assign("studbook.raster",raster::raster(raster_file),.StudbookEnv)
}


###
#' @title Modify raster values
#' @description \code{raster.modify} modifies raster values with a provided function.
#' @param func function to be used to change values.
#' @author Frank PG Princee
#' @export
#' 
raster.modify <- function(func)
{
  cat('Modifying values....\n')
  raster.tmp <- get("studbook.raster",.StudbookEnv)
  names.old <- names(raster.tmp)
                     
  raster.tmp <- raster::calc(raster.tmp,func)
  names(raster.tmp) <- names.old
  assign("studbook.raster",raster.tmp,.StudbookEnv)
}


###
#' @title Get names from studbook.raster
#' @description \code{raster.names} returns names that are stored in 
#' \code{studbook.raster}.
#' @return names
#' @author Frank PG Princee
#' @export
#' 
raster.names <-  function()
{
  return(names(get("studbook.raster",.StudbookEnv)))
}


###
#' @title Create raster palette
#' @description \code{create.palette} creates a colour palette for use in a raster
#' @details The palette will assigned to global variable \code{raster.palette}. 
#' @param colours number of different colours to be created.
#' @param palette name of ColorBrewer palette or palette vector.
#' @param reverse create palette in reverse order.
#' @author Frank PG Princee
#' @export
#'   
create.palette <- function(colours=9,palette='Greys',reverse=FALSE)
{
  if (length(palette) ==  1) {
     assign("raster.palette",RColorBrewer::brewer.pal(colours,palette),
            .StudbookEnv)
  } else {
     assign("raster.palette",palette,.StudbookEnv)
  }
  
  if (reverse) {
     assign("raster.palette",base::rev(get("raster.palette",.StudbookEnv)),
            .StudbookEnv)
  }
}


###
#' @title Plot raster
#' @description \code{raster.plot} plots the raster store in \code{studbook.raster}
#' with the palette stored in \code{raster.palette}.
#' @param main_title Raster title
#' @author Frank PG Princee
#' @export
#' 
raster.plot <- function(main_title="Raster")
{
  if ( is.null(get("studbook.raster",.StudbookEnv)) ) {
    stop("No raster has been defined.")
  }
  plot(get("studbook.raster",.StudbookEnv),
       col=get("raster.palette",.StudbookEnv))
  title(main=main_title,xlab="longitude",ylab="latitude")
}


###
#' @title Extract raster value for points
#' @description \code{raster.extract} extracts value in raster for given longitude and latitude.
#' @param lonlat data frame with longitude and latitude data (in this order).
#' @param buffer radius (in metres) to extract data when simple and bilinear yield no data.
#' @return raster values
#' @author Frank PG Princee
#' @seealso raster.nearest
#' @export
#' 
raster.extract <- function(lonlat,buffer=5000)
{
  
  if (is.null(get("studbook.raster",.StudbookEnv)) ) {
     stop("No raster loaded")
  }
  
  raster.file <- get("studbook.raster",.StudbookEnv)
 
  xy <- sp::SpatialPoints(lonlat)
  values <- raster::extract(raster.file,xy)
  
  for (x in 1:nrow(values)) {
    if (is.na(values[x])) {
      # Start with bilinear
      value <- raster::extract(raster.file,xy[x],method='bilinear') # interpolation from four nearest points
      if (is.na(value)) {
        value <- raster::extract(raster.file,xy[x],method='simple',buffer=buffer)
      }
      values[x] <- value
    }
  }
  return(values)
}



