###
# Package : studbookR
# 
# Copyright (c) 2011 Frank PG Princee, All rights reserved
#


.defaultRegression <- function()
{
  assign("confidence",TRUE,.StudbookEnv)
  assign("prediction",FALSE,.StudbookEnv)
}

###
#' @title Confidence intervals for geeglm regression
#' @family studbook regression
#' @keywords internal
#' 
confint.geeglm <- function(object, parm, level = 0.95, ...) 
{
  cc <- coef(summary(object))
  mult <- qnorm((1+level)/2)
  citab <- with(as.data.frame(cc),
                cbind(lwr=Estimate-mult*Std.err,
                      upr=Estimate+mult*Std.err))
  rownames(citab) <- rownames(cc)
  citab[parm,]
}

###
#' @title Confidence intervals for glmRob regression
#' @family studbook regression
#' @keywords internal
#' 
confint.glmRob <- function(object, parm, level = 0.95, ...) 
{
  cc <- coef(summary(object))
  mult <- qnorm((1+level)/2)
  citab <- with(as.data.frame(cc),
                cbind(lwr=Estimate-mult*cc[,"Std. Error"],
                      upr=Estimate+mult*cc[,"Std. Error"]))
  rownames(citab) <- rownames(cc)
  citab[parm,]
}




###
#' @title Find range in data sets with multiple columns
#' @param data_set Data set with multiple columns
#' @param column column in frame to be considered (default: 2)
#' @return minimum and maximum of values
#' @author Frank PG Princee
#' @family studbook regression
#' @keywords internal
#' 
regression.findRange <- function(data_set,column=2)
{ 
  data.range = range(data_set[,column])
  min_val = data.range[1]
  max_val = data.range[2]
  
  for (col in 2:length(data_set) ) {
    data.range = range(data_set[,col])
    min_val = min(min_val,data.range[1])
    max_val = max(max_val,data.range[2])
  }
  return(c(min_val,max_val))
}



###
#' @title  Plot regression
#' @author Frank PG Princee
#' @family studbook regression
#' @keywords internal
#' 
regression.plot<-function(data_set,main_title,xlabel='X',ylabel='Y',
                          base_name='analyses') 
{
  
  if (!studbook.plot()) {
    return(NULL)
  }
  studbook.openOutput(paste(base_name,'regression',sep='-'))
  mgp.old <- par('mgp')
  par(mgp=c(3,0.5,0))
  ylimit = regression.findRange(data_set)
  comment <- "Dashed lines are confidence intervals; dotted lines are predictions."
 
  if (!(is.expression(xlabel))) {
    xlabel <- firstCap(xlabel)
  }
  
  if (!(is.expression(ylabel))) {
    ylabel <- firstCap(ylabel)
  }
  
  studbook.caption(main_title,comment)
  
  plot(data_set[,1],data_set$original,main="",xlab=xlabel,ylab=ylabel,
       type='p',pch=21,cex=0.75,col=gray(.2),ylim=ylimit)
  studbook.title(main_title)
  
  #Fitted line, confidence and future
  legend <- NULL
  lines(data_set[,1],data_set$fit,lwd=1,lty='solid')
  if (studbook.get('confidence')) {
    lines(data_set[,1],data_set$confidence.lower,lty='dashed')
    lines(data_set[,1],data_set$confidence.upper,lty='dashed')
    line_style <- 'dashed'
  }
  
  if (studbook.get('prediction')) {
    lines(data_set[,1],data_set$prediction.lower,lty='dotted')
    lines(data_set[,1],data_set$prediction.upper,lty='dotted')
    if ( is.null(legend) ){
      line_style <- 'dotted'
    } else {
      line_style <- c(line_style,'dotted')
    }
  }
  par(mgp=mgp.old)
  studbook.closeOutput()
}

###
#' @title Create data frame of original data, confidence and prediction limits
#' @family studbook regression
#' @keywords internal
#' 
regression.matrix <- function(data_set,rg_object,log="")  
{
  new.data <-  data.frame("x"=data_set[,1], "original"=data_set[,2],"fit"=fitted(rg_object))
  
  if (grepl("x",log)) {
    if (min(new.data[,'x']) > 0) {
      new.data[,'x'] <- exp(new.data[,'x'])
    }
  }
  
  if (grepl("y",log)) {
    if (min(new.data[,'original']) > 0) {
      new.data[,"original"] <- exp(new.data[,'original'])
      new.data[,"fit"]      <- exp(new.data[,'fit'])
    } 
  }
 
  if (studbook.get('confidence')) {
    confidence <- predict(rg_object,interval="confidence",se.fit=TRUE)             # Confidence limits
    
    if (grepl("y",log)) {
        confidence$fit[,"upr"] <- exp(confidence$fit[,"upr"])
        confidence$fit[,"lwr"] <- exp(confidence$fit[,"lwr"])
    }
    
    new.data <- data.frame(new.data,
                           "confidence.upper" = confidence$fit[,"upr"],
                           "confidence.lower" = confidence$fit[,"lwr"])
    
   
  }
  
  
  if (studbook.get('prediction')) {
    prediction <- predict(rg_object,interval="prediction",se.fit=TRUE)             # Prediction limits
    new.data <- data.frame(new.data, 
                           "prediction.upper" = prediction$fit[,"upr"],
                           "prediction.lower" = prediction$fit[,"lwr"])
    
    if (grepl("y",log)) {
      prediction$fit[,"upr"] <- exp(prediction$fit[,"upr"])
      prediction$fit[,"lwr"] <- exp(prediction$fit[,"lwr"])
    } 
  }
  return(new.data)
}   


###
#' @title Diagnostics of regression
#' @param data_fit Fitted regression data
#' @author Frank PG Princee
#' @family studbook regression
#' @keywords internal
#' 
regression.diagnostics <- function(data.fit)
{

  
  rs <- shapiro.test(residuals(data.fit))     # Normal distribution of residuals
  print(rs)
  cat("[p < 0.05: non-normal distribution\n\n")
  
  cat("Durbin-Watson test for non-independence\n")
  rs <- car::durbinWatsonTest(data.fit) # non-independence test (time-series)
  print(rs)
  cat("[p < 0.05: no correlation between residuals]\n\n")
  
  rs <- car::ncvTest(data.fit)          # Breusch-Pagan non-constant error variance (heteroscedasticity)
  print(rs)
  cat("[p < 0.05: residuals are heteroscedastic]\n\n")
  
  cat("\nBonferonni outlier test\n")
  rs <- car::outlierTest(data.fit)      # Bonferonni p-value for most extreme obs
  print(rs)
  cat("[p < 0.05: residual point is outlier]\n\n")
  rs <- influence.measures(data.fit)
  summary(rs)
  cat("\n")
}

###
#' @title Diagnostics of regression residuals
#' @param data.fit Fitted linear regression data.
#' @param diagnostic Type of of diagnostic (default: c(1:3,5) ).
#' @author Frank PG Princee
#' @family studbook regression
#' @keywords internal
#' 
regression.residual <- function(data.fit,diagnostic=c(1:3,5), main_title='Residuals',
                                base_name='analyses')
{
   studbook.openOutput(paste(base_name,'residuals',sep='-'))
   if (studbook.getOutput() == "publish") {
     plot(data.fit,which=diagnostic,caption=NULL)
   } else {
     plot(data.fit,which=diagnostic)
   }
   studbook.closeOutput()
}


###
#' @title Linear regression with diagnostics and plot of regression line
#' @description \code{regression.linear} is a wrapper function that combines linear regression, 
#' plotting of regression, and diagnosis of residuals.
#' @details Linear regression can optionally be weighted. 
#' The Yeo-Johnson data transformation is used:
#' \itemize{
#' \item {\eqn{\lambda = 1.00}} {no transformation needed; produces results identical to original data}
#' \item {\eqn{\lambda = 0.50}} {square root transformation}
#' \item {\eqn{\lambda = 0.33}} {cube root transformation}
#' \item {\eqn{\lambda = 0.25}} {fourth root transformation}
#' \item {\eqn{\lambda = 0.00}} {natural log transformation}
#' \item {\eqn{\lambda = -0.50}} {reciprocal square root transformation} 
#' \item {\eqn{\lambda = -1.00}} {reciprocal (inverse) transformation}
#' }
#'  
#' @param data_X data frame with X,Y or vector with independent X data 
#' @param data_Y response data or NULL when data_X is a data frame
#' @param data_W vector with weights
#' @param model response model (default: "Y~X")
#' @param log Transform x and/or y data to logarithmic data \code{"","x","y" "xy"} (default: "")
#' @param diagnostic Diagnostic plots of residuals in the range 1:6 (or 0 for none) 
#' (default: \code{c(1:3,5)})
#' @param main_title Main title (defualt: \code{"Linear regression"})
#' @param xlabel Label for x-axis (default: \code{"X"})
#' @param ylabel Label for y-axis (default: \code{"Y"})
#' @param base_name Base name of graphic output file (default: \code{"linear"})
#' @param bc Logical  Perform  Box-Cox transformation analysis (default: FALSE)
#' @author Frank Princee
#' @family studbook regression
#' @export
#' 
regression.linear <- function(data_X,data_Y=NULL,data_W=NULL,model="Y~X",
                            log="",diagnostic=c(1:3,5),
                            main_title='Linear regression',
                            xlabel='X',ylabel='Y',base_name='linear',
                            bc=FALSE)
{
  if (is.data.frame(data_X) ){                                
    data_set <- data_X
  } else {
    if (is.ts(data_X)) {
      data_Y <- as.numeric(data_X)
      data_X <- as.numeric(time(data_X))
    }
    data_set  <- data.frame(data_X,data_Y)
  }
  colnames(data_set) <- c('X','Y')  
  
  if (grepl("x",log)) {
    if (min(data_set[,"X"]) > 0) {
      data_set[,"X"] <- log(data_set[,"X"])
    } else {
      log <- gsub("x", "", log)
    }
  }
  
  if (grepl("y",log)) {
    if (min(data_set[,"Y"]) > 0) {
      data_set[,"Y"] <- log(data_set[,"Y"])
    } else {
      log <- gsub("y", "", log)
    }
  }
  
  if (bc) {
    car::boxCox(Y~X,data=data_set,family="yjPower")
  }

  fit.lm  <- lm(Y ~ X,data=data_set,weights=data_W) 
  
  new.data <- regression.matrix(data_set,fit.lm,log=log) 
  
  print(summary(fit.lm)) 
  print(confint(fit.lm))
  lK <- logLik(fit.lm)
  cat('logLik:',lK,'\tAIC:',AIC(fit.lm),'(df:',attr(lK,'df'),')\n')
  studbook.set("fitted",fit.lm)
  if (studbook.plot()) {
    regression.plot(new.data,main_title,xlabel,ylabel,base_name)
    if (diagnostic[1] > 0) {
      regression.residual(data.fit=fit.lm,diagnostic=diagnostic,
                          main_title=main_title,base_name=base_name)
    }
    
  }
  regression.diagnostics(fit.lm)
}



###
#' @title Segmented regression
#' @details Uses functions in package \code{segmented}
#' @param data_set XY data set
#' @param breakpoints list with at least one suggested breakpoints.
#' @author Frank PH Princee
#' @family studbook regression
#' @export
#' 
regression.segmented <-function(data_set,breakpoints,
                                main_title='Segmented linear regression',
                                xlabel='X',ylabel='Y',base_name='analyses')
{
  # Implemented broken-stick or segmented regression
  if (length(breakpoints) == 0) {
    stop("You must provide at least one breakpoint")
  }
  
  if (is.list(breakpoints)) {
    print("Is list")
  }
  
  if (!studbook.plot()) {
    return(NULL)
  }
  studbook.openOutput(paste(base_name,'segmented',sep='-'))
  attach(data_set)
  fit.lm  <- lm(Y ~ X)
  fit.seg <- segmented::segmented(fit.lm,seg.Z=~X, psi=list(X = breakpoints ) )
  
  cat("[SEGMENTED REGRESSION]\n",main_title,"\n")
  print(summary(fit.seg))
  print(slope(fit.seg))
 # print(broken.line(fit_seg,linkinv=TRUE))
  studbook.caption(main_title)
  plot(fit.seg,lty='dashed')
  points(data_set)
  detach(data_set)
  studbook.title(main_title)
  studbook.closeOutput()
  
}


###
#' @title Regression of orthogonal polynomials
#' @family studbook regression 
#' @export
#
regression.polynomial <- function(data_X,data_Y=NULL,data_W=NULL,degree=1,main_title='Polynomial regression',
                                  xlabel='X',ylabel='Y',base_name='analyses')
{
  
  if (is.data.frame(data_X)) {
    data_set <- data_X
  } else {
    if (is.ts(data_X)) {
      data_Y <- as.numeric(data_X)
      data_X <- as.numeric(time(data_X))
    }
    data_set  <- data.frame(data_X,data_Y)
  }
  colnames(data_set) <- c('X','Y')
  
  if (is.null(data_W)) {
    fit.lm <- lm(Y ~ poly(X,degree),data=data_set)
  } else {
    fit.lm <- lm(Y ~ poly(X,degree),weights=data_W,data=data_set)
  }
  
  print(summary(fit.lm))  
  cat('AIC:',AIC(fit.lm),'\n')
  fitted.data <- regression.matrix(data_set,fit.lm)
  
  regression.plot(fitted.data,main_title,xlabel,ylabel,base_name)
}


###
#' @title Find best fitted polynomial model based on Pr(|t|)
#' @family studbook regression 
#' @export 
#  
regression.findPolynomial <- function(data_X,data_Y,data_W=NULL)
{
  if (is.data.frame(data_X)) {
    data_set <- data_X
  } else {
    if (is.ts(data_X)) {
      data_Y <- as.numeric(data_X)
      data_X <- as.numeric(time(data_X))
    }
    data_set  <- data.frame(data_X,data_Y)
  }
  colnames(data_set) <- c('X','Y')
  
  max_degrees <- length(unique(data_set$Y)) - 1
  
  r2 <- 0 #r squared
  for (degree in c(1:max_degrees) ) {
    if (is.null(data_W)) {
      test.lm <- lm(Y ~ poly(X,degree),data=data_set)
    } else {
      test.lm <- lm(Y ~ poly(X,degree),weights=data_W,data=data_set)
    }
    test.summary <- summary(test.lm)
    prob <- test.summary$coefficients[degree+1,4]  #row 1  = intercept, column 4 = Pr(>|t|)
    
    if (prob > 0.05) {  
       degree <- degree -  1     #previous degree
       break;
    }
  }
 
  return(degree)
}
