###
# Package : studbookR
# 
# Copyright (c) 2011 Frank PG Princee, All rights reserved
#
.defaultSeason <- function()
{
  assign("rad_per_day",(2*pi)/366,.StudbookEnv)    # radians per day
  assign("rad_per_month", (2*pi)/12,.StudbookEnv)  # radians per month
  assign("save_season",NULL,.StudbookEnv)
}


############ helper functions ############

###
#' @title Converts angle to day number
#' @description \code{angle2day} converts an angle to day number
#' @param angle in radians
#' @return day number
#' @author Frank PG Princee
#' @family seasonality
#' @keywords internal
#' 
angle2day <- function(angle) {
  day <- angle/studbook.get("rad_per_year")
  return(day)
}

###
#' @title Estimate Rayleigh's probability
#' @description \code{rayleigh.p} estimates Rayleigh's probability. 
#' @details Based on code in RayleighTestRad in package circular.
#' @param r resultant.
#' @param N sample size.
#' @param p value.
#' @author original code by Claudio Agostinelli
#' @family seasonality
#' @export
#' 
rayleigh.p <- function(r,N)
{
  z <- (N * r^2)  # Rayleig z-score
  p.value <- exp(-z)
  if (N < 50) {
    temp <- 1 + (2 * z - z^2)/(4 * N) - 
      (24 * z - 132 * z^2 + 76 * z^3 - 9 * z^4)/(288 * N^2)
  } else {
    temp <- 1
  }
  p.value <- min(max(p.value * temp, 0), 1)
  return(p.value)
}


###
#' @title Create circular object from seasonality data in radians
#' @param season.day vector with days of year.
#' @param names names of the data.
#' @author Frank PG Princee
#' @return circular object
#' @family seasonality
#' @keywords internal
#' 
circular.data <- function(season.day,names='')
{
  rad_per_day <- studbook.get("rad_per_day")
  season.day  <- circular::circular(season.day * rad_per_day,
                                   type="angles",units='radians',
                                   template='none',names=names)
  return(season.day)
}


###
#' @title Post-hoc Rao test
#' @description \code{pairwise.rao.test} applies a posthoc Rao test
#' @details the \code{rao.test} in the package \code{circular} is applied 
#' @param x angle in radians
#' @param g groups
#' @param p.adjust.method post-hoc method (default: p.adjust.methods[1] = 'holm')
#' @param test test for \code{'mean', 'dispersion'} (default: 'mean')
#' @note based on source code pairwise.wilcox.test
#' @export
#' 
pairwise.rao.test <- function (x, g,p.adjust.method = p.adjust.methods,
                               test=c('mean','dispersion'),...) 
{
  test <- match.arg(test)
  if (test == 'mean') {   # Rao test for mean (1) and dispersion (2)
    idx <- 1   
    test_method <- "Rao test for equal mean"
  } else {
    idx <- 2
    test_method <- "Rao test for equal dispersion"
  }
  
  DNAME <- paste(deparse(substitute(x)), "and", deparse(substitute(g)))
  g <- factor(g)
  p.adjust.method <- match.arg(p.adjust.method)
  
  # convert x after deparsedx
  if (!circular::is.circular(x)) {
    x <- circular.data(x)
  }
  
  # local function
  compare.levels <- function(i, j) {
    xi <- x[as.integer(g) == i]
    xj <- x[as.integer(g) == j]
    result <- circular::rao.test(xi,xj,alpha=0)$p.value[idx]
  }
  
  p.values <- pairwise.table(compare.levels, levels(g), p.adjust.method)
  ans <- list(method = test_method, data.name = DNAME, p.value = p.values, 
                 p.adjust.method = p.adjust.method)
  class(ans) <- "pairwise.htest"
  ans
}



##############

###
#' @title Weighted circular mean
#' 
#' @author Frank PG Princee; 
#' @param season.day day of year. 
#' @param weights  weights for each date.
#' @family seasonality 
#' @keywords internal
#'  
weighted.circular <- function(season.day,weights)
{
  rad_per_day <- studbook.get("rad_per_day")
  
  season.day <- season.day * rad_per_day
  N <- length(season.day)
  
  sinr <- sum(weights * sin(season.day)) 
  cosr <- sum(weights * cos(season.day)) 
  angle <- atan2(sinr, cosr) # Arc-tangent to calculate mean angle
  
  r  <- (sqrt(sinr^2 + cosr^2))/N
  p.value <- rayleigh.p(r,N) 
  return(list('statistic'=r,'p.value'=p.value,'angle'=angle,'n'= N))   
}


###
#' @title Read seasonal litter data into data frame
#' @details Optionally The field SIZE, DEATHS and SURVIVED are added.
#' @param add_fields TRUE add fields for total SIZE, DEATHS and SURVIVED (default:FALSE).
#' @return data frame
#' @author Frank PG Princee
#' @family seasonality
#' @keywords internal
#' 
litter.read <- function(add_fields=FALSE)
{
  litter.data <- studbook.read('litter_season') 
  
  if (is.null(litter.data)) {
    return(FALSE)
  }
  
  # Strip row with sire - which is used for quantitative genetics
  if (is.element("SEX",colnames(litter.data))) {
    litter.data <- subset(litter.data, SEX == "female")
  }
  
  if (add_fields) {
    litter.data <- within(litter.data,{
      SIZE   <- MALES + FEMALES + UNKNOWN 
      DEATHS <- D_MALES + D_FEMALES + D_UNKNOWN
      SURVIVED <- SIZE - DEATHS
    })
  }
  
  return(litter.data)
}

###
#' @title Filter data from northern or souther hemisphere
#' @param hemisphere \code{"world","north", "south"}.
#' @return modified data-set
#' @author Frank PG Princee
#' @family seasonality
#' @keywords internal
#' 
season.hemisphere <- function(season.data,hemisphere=c("world","north","south"))
{
  if (hemisphere == 'north') {
    season.data <- subset(season.data,LATITUDE >=0)
  } else if (hemisphere == 'south') {
    season.data <- subset(season.data,LATITUDE < 0)
  } else {
    #nothing
  }
  return(season.data)
}


###
#' @title Linear regression of seasonality effect
#' @param season.data data frame with first column day of years.
#' @return aov results
#' @author Frank PG Princee
#' @family seasonality
#' @keywords internal
#' 
season.lm <- function(season.data)
{
  cat("[ Circular regression ]\n")
  season.circular <- circular.data(season.data[,1],'')
  rs <- circular::lm.circular(type="c-l",y=season.circular,x=season.data[,2],
                              init=c(0))
  return(rs)
}



###
#' @title Plot seasonal data 
#' @description \code{season.plot} plots circular graph of seasonal data. 
#' @param season.day vector with days of the year.
#' @param segment date segments. Options are \code{"month", "solstice" "north_season" or
#' "south_season"}. (default: \code{"month"}).
#' @param plot_arrow plot arrow in the direction of mean day of the season (default: TRUE). 
#' @author Frank PG Princee
#' @family seasonality
#' @keywords internal
#' 
season.plot <-function(season.day,
                       main_title='Seasonality',
                       segment=c("month","north_season","south_season","solstice"),
                       plot_arrow=TRUE)
{ 
  if (!studbook.plot()) {
    return(NULL)
  }
  
  
  segment     <- match.arg(segment)
  rad_per_day <- studbook.get("rad_per_day")
 
  season.day <- na.omit(season.day)
  season.circular <- circular.data(season.day,names=main_title)
  
  
  #Create circular axis with ticks based on days per month (February=29)
  days_per_month <- c(31,29,31,30,31,30,31,31,30,31,30,31)                     
  season.axis    <- rep(0,12)  
 
  for (i in 2:12) {
    season.axis[i] <- season.axis[i-1] + (days_per_month[i - 1] * rad_per_day)
  }
  season.axis <- circular::circular(season.axis,units='radians')
  
  
  solstice.names   <- c('Mar equinox','Jun solstice','Sep equinox','Dec soltice')
  solstice.seasons <- c('spring','summer','autumn','winter')
  solstice.axis    <- rep(0,4)                                                # Approximations of solstices and equinox
  solstice.axis[1] <- (  79 * rad_per_day )                                   # 20 March 2009 equinox
  solstice.axis[2] <- ( 172 * rad_per_day )                                   # 21 June 2009 solstice 
  solstice.axis[3] <- ( 265 * rad_per_day )                                   # 22 September 2009 equinox
  solstice.axis[4] <- ( 355 * rad_per_day )                                   # 21 December 2009 solstice 
  
  solstice.axis <- circular::circular(solstice.axis, units='radians')
  
  studbook.caption(main_title)
  circular::rose.diag(season.circular,cex=0.5,col='gray',axes=FALSE,ticks=TRUE,bins=12,
                      main="",shrink=1.25)
  points(season.circular,cex=0.5,col='gray10',stack=TRUE,bins=366,
       main="",shrink=1.25)
 

  # carry out Rayleigh test for plotting arrow at mean direction
  result <- circular::rayleigh.test(season.circular)
  
  if (plot_arrow & result$p.value < 0.05) {
    circular::arrows.circular(circular::mean.circular(season.circular),
                              shrink=1,code=2,lwd=1)
  }
  
  if (segment == 'month') {
    circular::axis.circular(at=season.axis,labels=month.abb,cex=0.75)
  } else if (segment == 'north_season') {
    circular::axis.circular(at=solstice.axis,labels=solstice.seasons,cex=0.75)
  } else if (segment == 'south_season') {
    solstice.seasons <- c(solstice.seasons[3:4],soltice.seasons[1:2])   #Start with autumn
    circular::axis.circular(at=solstice.axis,labels=solstice.seasons,cex=0.75)
  } else if (segment == 'solstice') {
    circular::axis.circular(at=solstice.axis,labels=solstice.names,cex=0.75)
  } else {
    circular::axis.circular(at=season.axis,labels=month.abb,cex=0.75)
  }
    
  studbook.title(main_title)
  return(TRUE)
}


###
#' @title Test effects on seasonality
#' @family seasonality
#' @keywords internal
#' 
season.effects <- function(season.data,effects=c("age","climate","generation","latitude"))
{

  if (is.element("age",effects)) {
    season.age(season.data)
  }
  
  if (is.element("climate",effects)) {
    season.climate(season.data)
  }
  
  
  if (is.element("generation",effects)) {
    season.generation(season.data)
  }
  
  if (is.element("latitude",effects)) {
    season.latitude(season.data)
  }
  
}



###
#' @title Test whether maternal age has effect on date of birth
#' @note Date must be provided as day of year
#' @family seasonality
#' @keywords internal
#
season.age <- function(season.data,
                       main_title='Effect of maternal age on date of birth',
                       significance=0.05)
{
  if (!is.element("DAM_AGE",colnames(season.data)) ) {
    return(FALSE)
  }
  
  
  season.data <- subset(season.data, DAM_AGE > 0)
  
  if ( is.result(grep("UNK|MULT|WILD",season.data$DAM)) ) {
    # Remove them from data frame by -grep
    season.data <- season.data[-grep("UNK|MULT|WILD",season.data$DAM),]
  }
  
  if (nrow(season.data) == 0) {
    stop('No data')
  }
  
  cat("[",main_title,"]\n")
  if (studbook.plot()) {
    plot(DAY ~ DAM_AGE,data=season.data,main="",
         xlab='Age of dam',ylab='Day of year')
    studbook.title(main_title)
  }
  rs <- season.lm(season.data[,c('DAY','DAM_AGE')])  
  lines(season.data$DAM_AGE,fitted(rs))
  print(rs)
  return(TRUE)
}


###
#' @title Test whether climate has effect on date of birth
#' @note Date must be provided as day of year
#' @family seasonality
#' @keywords internal
#' 
season.climate <- function(season.data,
                           main_title='Effect of climate on date of birth',
                           significance=0.05)
{
  #Add location
  if (is.null(studbook.sites)) {
    sites.load()
  }
  
  season.data <- within(season.data, CLIMATE <- NA)
  #Add zone to location
  for (x in 1:nrow(season.data)) {
    site <- season.data[x,'LOCATION']  
    zone <- studbook.sites[studbook.sites$mnemonic==site,'thermal']  
    if (length(zone) == 1) {
      if (!inThermal(zone)) {
        next
      }
      
      zone <- studbook.get("thermal.zone")[as.integer(zone)]
      season.data[x,'CLIMATE'] <- zone
    }
  }    
  
  cat("[",main_title,"]\n")
  cat("Table climate\n")
  print(table(season.data$CLIMATE))
  result <- pairwise.rao.test(x=season.data$DAY,g=season.data$CLIMATE,
                                 p.adjust.method="holm",test='mean')
  print(result)
  
  result <- pairwise.rao.test(x=season.data$DAY,g=season.data$CLIMATE,
                              p.adjust.method="holm",test='dispersion')
  print(result)
  
  cat('[p <',significance,'indicate significant differences between groups]\n\n')
  if (studbook.plot()) {
    plot(DAY ~ as.factor(CLIMATE), data=season.data,main="",xlab='Climatic zone',
         ylab='Day of year',type='p',na.rm=TRUE)
    studbook.title(main_title)
  }
}


###
#' @title Test effect of generation
#' @family seasonality
#' @keywords internal
#
season.generation <- function(season.data,
                              main_title='Seasonality in births among maternal generations',
                              significance=0.05) 
{
  
  cat("[",main_title,"]\n")
  cat("Table generation groups\n")
  print(table(season.data$G))
  result <- pairwise.rao.test(season.data$DAY,season.data$G,
                              p.adjust.method="holm",test='mean')
  print(result)
  
  result <- pairwise.rao.test(season.data$DAY,season.data$G,
                              p.adjust.method="holm",test='dispersion')
  print(result)
  
  cat('[p <',significance,'indicate significant differences between groups]\n\n')
  
  if (studbook.plot()) {
     plot(DAY ~ G,data=season.data,main="",xlab='Generation group of dam',
         ylab='Day of year')
    studbook.title(main_title)
  }
}


###
#' @title Test effect of latitude on seasonality
#' @family seasonality
#' @keywords internal
#
season.latitude <- function(season.data)
{
  
  if (studbook.plot()) {
    
    if (studbook.isOption('output','screen') ) {
      op<-par(mfrow=c(2,2))
    }
    
    if (length(season.data$LATITUDE[!is.na(season.data$LATITUDE)]) <= 0 ) {
      return(FALSE)
    }
    
    plot(DAY ~ LATITUDE,data=season.data,type='p',main="",
         xlab='Latitude (degrees)',ylab='Day of year')
    studbook.title('Birth day and latitude ')
    
    if (length(season.data$PHOTOPERIOD[!is.na(season.data$PHOTOPERIOD)]) > 0 ) {
      plot(PHOTOPERIOD ~ LATITUDE,data=season.data,main="",type='p', 
           xlab='Latitude (degrees)',ylab='Day length (hours)')
      studbook.title('Photoperiod at day of birth\n')
    }
    
    
    
    if (length(season.data$HOURS[!is.na(season.data$HOURS)]) > 0 ) {
      plot(HOURS ~ LATITUDE,data=season.data,main="",type='p',
           xlab='Latitude (degrees)',ylab='Total hours')  
      studbook.title('Cumulative photoperiod since conception/implantation at latitude\n')
    }
    
    if (studbook.isOption('output','screen')) {
      par(op) 
    } 
    
  }
   
  cat("Effect of latitude on day of birth\n")
  rs <- season.lm(season.data[,c('DAY','LATITUDE')])
  print(rs)
  
  #Day length
  cat("Latitude and photoperiod at date of birth\n")
  if (length(season.data$PHOTOPERIOD[!is.na(season.data$PHOTOPERIOD)]) > 0 ) {
    print(anova(lm(PHOTOPERIOD ~ as.factor(LATITUDE),data=season.data)))
  }
  
  
  #Cumulative PHOTOPERIOD
 cat("Latitude and cumultative photoperiod at date of birth\n")
  if (length(season.data$HOURS[!is.na(season.data$HOURS)]) > 0 ) {
    print(anova(lm(HOURS - LATITUDE,data=season.data)))
  }
  cat('\n')
  return(TRUE)
  
}


###
#' @title Seasonality in litter size
#' @family seasonality
#' @keywords internal
#
season.litterSize <- function(season.data,
                              main_title='Seasonality litter size',
                              significance=0.05) 
{ 
  cat(main_title,"\n")
  
  if (studbook.plot()) {
    plot(SIZE ~ DAY,data=season.data,main="",xlab='Day of year',
         ylab='litter size')
    studbook.title(main_title)
  }
 
  cat("Table litter size\n")
  print(table(season.data$SIZE))
  result <- pairwise.rao.test(season.data$DAY,season.data$SIZE,
                              p.adjust.method="holm",test='mean')
  print(result)
  
  result <- pairwise.rao.test(season.data$DAY,season.data$SIZE,
                              p.adjust.method="holm",test='dispersion')
  print(result)
  
  rs <- season.lm(season.data[,c('SIZE','DAY')])
  print(rs)
}


###
#' @title Print Rayleigh's test results
#' @param result Rayleigh's test results
#' @param n Sample size
#' @author Frank PG Princee
#' @family seasonality
#' @keywords internal
#' 
rayleigh.print <- function(result,n)
{
  cat("[Rayleigh's test]\n")
  cat("Group        :", comment(result),"(n =",n,")\n")
  cat("Resultant r  :", result$statistic,"\n")
  cat("Rayleigh's R :", n * result$statistic,"\n")
  cat("p            :", result$p.value,"\n")
  cat("H0: uniform distribution is ")
  cat(ifelse(result$p.value < 0.05,"rejected","not rejected"),"\n\n")
  
}


###
#' @title Seasonality tests
#' @details Circular statistics is used to test H0: data is from an uniform distribution  
#' @param season.data Seasonality in day of year
#' @param test One or more circular tests. Options are \code{"rayleigh", "kuiper", "rao", "watson"}  (default:\code{"rayleigh"})
#' @param main_title Main title for graphics
#' @author Frank PG Princee
#' @references circular
#' @family seasonality
#' @keywords internal
#' 
season.test <- function(season.data,
                        test=c("rayleigh","kuiper","rao","watson"),
                        main_title='Seasonality')
{
  test <- match.arg(test,several.ok=TRUE)
  rad_per_day <- studbook.get("rad_per_day")
  season.data <- na.omit(season.data)
  
  season.circular <- circular::circular(season.data * rad_per_day,
                                       type="angles",units='radians',
                                       template='none',names=main_title)
  
  season.mean  <- circular::mean.circular(season.circular)
  season.day   <- (season.mean / rad_per_day)
  season.var   <- circular::var(season.circular)
  season.ci    <- circular::mle.vonmises.bootstrap.ci(season.circular, alpha=.05)
  
  season.dateOfDay <- date::date.mdy(date::as.date(season.day)) 
  season.dateLower <- date::date.mdy(date::as.date(season.ci$mu.ci['2.5%']/rad_per_day))
  season.dateUpper <- date::date.mdy(date::as.date(season.ci$mu.ci['97.5%']/rad_per_day))  
 
  n <- length(season.data)
  
  # Percentiles
  pc <- circular::quantile.circular(season.circular,probs=c(0,0.005,0.025,0.5,0.975,0.995,1.0))
  pc <- (pc/rad_per_day)                                                  
  pc <- date::date.mdy(date::as.date(pc))
  
  cat(main_title," [ n=",n, "]\n")
  cat("[Range]\n")
  cat("Season       :", paste(pc$day[1],month.abb[pc$month[1]],"-",pc$day[7],month.abb[pc$month[7]],"\n"))
  cat("Season (99%) :", paste(pc$day[2],month.abb[pc$month[2]],"-",pc$day[6],month.abb[pc$month[6]],"\n"))
  cat("Season (95%) :", paste(pc$day[3],month.abb[pc$month[3]],"-",pc$day[5],month.abb[pc$month[5]],"\n"))
  cat("Median season:", paste(pc$day[4],month.abb[pc$month[4]],"\n\n"))
  cat("Mean angle   :", circular::conversion.circular(season.mean,"degrees") %% 360,"degrees (",season.mean,"radians)\n" )
  cat("Variance     :", season.var,"\n")
  cat("Mean day     :", paste(season.dateOfDay$day,month.abb[season.dateOfDay$month]),paste(" [",season.day,"]",sep=""),"\n")
  cat("CI 2.5%      :", paste(season.dateLower$day,month.abb[season.dateLower$month]),"\n")
  cat("CI 97.5%     :", paste(season.dateUpper$day,month.abb[season.dateUpper$month]),"\n\n")
  
  
  if (is.element("rayleigh",test)) {
    # Nonparametric
    rayleigh.result <- circular::rayleigh.test(season.circular)
    comment(rayleigh.result) <- main_title
    rayleigh.print(rayleigh.result,n=n)
  }
  
  
  if (is.element("rao",test)) {
    rao.result      <- circular::rao.spacing.test(season.circular,alpha=0.05)
    print(rao.result)
  }
  
  if (is.element("kuiper",test)) {
    kuiper.result   <- circular::kuiper.test(season.circular,alpha=0.05)
    print(kuiper.result)
  }
  
  if (is.element("watson",test)) {
    watson.uniform  <- circular::watson.test(season.circular,dis="uniform",alpha=0.05)
    print(watson.uniform)
    
    watson.mises    <- circular::watson.test(season.circular,dist="vonmises",alpha=0.05)
    print(watson.mises)
  }
}


###
#' @title Two sample tests for differences between seasons
#' @param sample1 First sample in radians
#' @param sample2 Second sample in radians
#' @author Frank PG Princee
#' @family seasonality
#' @keywords internal
#' 
season.twoSample <- function(sample1,sample2)
{
  sample1  <- na.omit(sample1)
  sample2  <- na.omit(sample2)
  
  set1           <- circular::circular(sample1)
  set2           <- circular::circular(sample2)
  result.watson  <- circular::watson.two.test(set1,set2)     # non-parametric
  result.rao     <- circular::rao.test(set1,set2,alpha=0.05) # non-parametric
  
  cat("[H0: two samples came from the same population]\n")
  print(result.watson)
  print(result.rao)
  cat('\n')
}

###
#' @title Seasonality in individual births
#' @description \code{season.birth} analyses birth dates.
#' @param sex single or multiple sex groups. Valid codes are \code{"all","male","female","unknown"}
#' @param effects variables that may have effect on date of birth. Multiple options can be selected:
#' \code{"age", "climate", "generation", "latitude"} (default: all). Use "none" when no effect is tested. 
#' @param hemisphere options are \code{"world","north","south"} (default:\code{"world"})
#' @param segment date segments. Options are \code{"month", "solstice" "north_season" or
#' "south_season"}. (default: \code{"month"}).
#' @param test select one or more circular tests. Options are \code{"rayleigh", "kuiper", "rao", "watson"}
#' (default:\code{"rayleigh"}).
#' @param plot_arrow plot arrow for mean day (default: TRUE).
#' @param delayed delayed implantation in days. Subtracted from date of birth (default: 0).
#' @author Frank PG Princee
#' @family seasonality
#' @export
#
season.birth <- function(sex=c("all","male","female","unknown"),
                         effects=c("age","climate","generation","latitude","none"),
                         hemisphere=c("world","north","south"),
                         segment=c("month","north_season","south_season","solstice"),
                         test=c("rayleigh","kuiper","rao","watson"),
                         plot_arrow=TRUE,
                         delayed=0)
{  
  if (studbook.empty()) {
    studbook()
  }
  sex <- match.arg(sex,several.ok=TRUE)
  effects <- match.arg(effects,several.ok=TRUE)
  hemisphere <- match.arg(hemisphere)
  segment <- match.arg(segment)
  test    <- match.arg(test,several.ok=TRUE)

  studbook.screenTitle("Seasonality in births")
  studbook.openOutput('birth_season')
  
  for (group in sex) {
    season.data <- studbook.read('birth_season',group) 
    
    season.data <- within(season.data, 
                          DAY <- ( tis::dayOfYear(DATE) - delayed) )
   
    #Replace NaN with NA
    season.data$LATITUDE[is.nan(season.data$LATITUDE)] <- NA
    season.data$PHOTOPERIOD[is.nan(season.data$PHOTOPERIOD)] <- NA
    season.data$HOURS[is.nan(season.data$HOURS)] <- NA
    
    season.data <- season.hemisphere(season.data,hemisphere)
    
    season.test(season.data$DAY,test=test,main_title=group)
    main_title <- paste("Seasonality in births (",group,")",sep="")
    season.plot(season.data$DAY,main_title,segment=segment,
                plot_arrow=plot_arrow)
  
    if (effects[1] != "none") {
      season.effects(season.data,effects)
    }
   
    if (group == "male") {
      season.male <- season.data
    }
    if (group == "female") {
      season.female <- season.data
    }
    
  }
  
  # Compare male and female seasonality in births
  if ( is.element("male",sex) && is.element("female",sex) ){
    cat("Test for differences in male & female birth season\n")    
    season.twoSample(season.male$DAY,season.female$DAY)    
  }
  studbook.closeOutput()
}





###
#' @title Seasonality in deaths
#' @description \code{season.death} analyses death dates.
#' @inheritParams season.birth
#' @param test One or more circular tests. Options are \code{"rayleigh"},\code{"kuiper"},\code{"rao"},
#' and \code{"watson"}  (default:\code{"rayleigh"})
#' @param plot_arrow Plot arrow for mean day (default: TRUE)
#' @author Frank PG Princee
#' @family seasonality
#' @export
#' 
season.death <- function(sex=c("all","male","female","unknown"),
                         hemisphere=c("world","north","south"),
                         segment=c("month","north_season","south_season","solstice"),
                         test=c("rayleigh","kuiper","rao","watson"),
                         plot_arrow=TRUE)
{  
  if (studbook.empty()) {
    studbook()
  }
  studbook.screenTitle("Seasonality in deaths")
  studbook.openOutput('death_season')
  
  sex <- match.arg(sex,several.ok=TRUE)
  hemisphere <- match.arg(hemisphere)
  segment <- match.arg(segment)
  test <- match.arg(test,several.ok=TRUE)
  
  for (group in sex) {
      season.data <- studbook.read('death_season',sex=group)
      if (is.null(season.data)) {
        warning("No seasonality data")
        next
      }
      season.data <- within(season.data,DAY <- tis::dayOfYear(DATE) )
      
      season.test(season.data[,'DAY'],test=test,main_title=group)
      main_title <- paste("Seasonality in deaths (",group,")",sep="")
      season.plot(season.data[,'DAY'],main_title,segment=segment,
                  plot_arrow=plot_arrow)
      
      if (group == "male") {
        season.male <- season.data
      }
      if (group == "female") {
        season.female <- season.data
      }
  }
  
  # Compare male and female seasonality in death
  if (is.element("male",sex) && is.element("female",sex)) {
    cat("Test for differences in male and female death season\n")
    season.twoSample(season.male$DAY,season.female$DAY)
  }
  studbook.closeOutput()
}



###
#' @title Seasonality in litter
#' @description \code{season.litter} analyses litter dates.
#' @param effects Variables that may have effect on date of birth. Multiple options can be selected:
#' \code{"age", "climate", "generation", "latitude"} (default: all). Use "none" to de-select all 
#' @param segment Date segments. Options are \code{"month", "solstice" "north_season" or
#' "south_season"} (default: \code{"month"})
#' @param test Circular tests. Options are \code{"rayleigh", "kuiper", "rao", "watson"}  
#' (default:\code{"rayleigh"})
#' @param plot_arrow Plot arrow for mean day (default: TRUE)
#' @param delayed Delayed implantation in days. Subtracted from day of birth (default: 0)
#' @author Frank PG Princee
#' @family seasonality
#' @export
#'  
season.litter <- function(effects=c("age","climate","generation","latitude","none"),
                          hemisphere=c("world","north","south"),
                          segment=c("month","north_season","south_season","solstice"),
                          test=c("rayleigh","kuiper","rao","watson"),
                          plot_arrow=TRUE,
                          delayed=0)
{  
  
  if (studbook.empty() ) {
    studbook()
  }
  
  effects    <- match.arg(effects,several.ok=TRUE) 
  hemisphere <- match.arg(hemisphere)
  segment    <- match.arg(segment)
  test       <- match.arg(test,several.ok=TRUE)
  
  season.data <- litter.read(add_fields=FALSE)
  if (is.null(season.data)) {
    stop("No data on litter season.")
  }
 
  x <- which(colnames(season.data) == "animal")
  if (is.result(x)) {
    colnames(season.data)[x] <- "DAM"
  }  
  
  studbook.screenTitle("Seasonality in litters")
  season.data <- within(season.data, {
    DAY   <- tis::dayOfYear(DATE)
    MONTH <- tis::month(DATE) 
    SIZE <- MALES + FEMALES + UNKNOWN
  }) 
  
  
  studbook.openOutput('litter_season')
  
  #Clear up file
  season.data$LATITUDE[is.nan(season.data$LATITUDE)] <- NA
  season.data$PHOTOPERIOD[is.nan(season.data$PHOTOPERIOD)] <- NA
  season.data$HOURS[is.nan(season.data$HOURS)] <- NA
  
  season.data <- season.hemisphere(season.data,hemisphere)
 
  season.test(season.data$DAY,test=test,main_title='Litters')
  season.plot(season.data$DAY,'Seasonality in litters (all)',segment=segment,
              plot_arrow=plot_arrow)
 
  
  # Weighted analysis of sex-bias in litters and seasonality
  season.males   <- weighted.circular(season.data$DAY,season.data$MALES)
  season.females <- weighted.circular(season.data$DAY,season.data$FEMALES)
  season.unknown <- weighted.circular(season.data$DAY,season.data$UNKNOWN)
  
  comment(season.males)   <- "Males in litter"
  comment(season.females) <- "Females in litter"
  comment(season.unknown) <- "Unknown in litter"
  rayleigh.print(season.males,sum(season.data$MALES))
  rayleigh.print(season.females,sum(season.data$FEMALES))
  rayleigh.print(season.unknown,sum(season.data$UNKNOWN))
  
  if (effects[1] != "none") {
    season.effects(season.data,effects)
  }
  
  studbook.closeOutput()
  
}


###
#' @title Seasonality in births and deaths
#' @description \code{studbook.season} is a wrapper function to analyse seasonality.
#' @details Seasonality in births, deaths and litters can be analysed. 
#' 
#' @param analysis Select type of analysis \code{"birth"},\code{"death"} or \code{"litter"} 
#' (default: \code{"birth"}).
#' @param test Circular test. Options are \code{"raleigh"},\code{"kuiper"},\code{"rao"},
#' and \code{"watson"}  (default:\code{"rayleigh"})
#' @param plot_arrow Plot arrow for mean day (default: TRUE)
#' @param delayed Delayed implantation in days (birth and litter only). Subtracted from date of birth (default: 0)
#' @inheritParams season.birth
#' @author Frank PG Princee
#' @family seasonality
#' @export
#' 
studbook.season <- function(analysis=c('birth','death','litter'),
                            effects=c("age","climate","generation",
                                      "latitude","none"),
                            sex=c("all","male","female","unknown"),
                            hemisphere=c("world","north","south"),
                            segment=c("month","north_season",
                                      "south_season","solstice"),
                            test=c("rayleigh","kuiper","rao","watson"),
                            plot_arrow = TRUE,
                            delayed=0)
{
  if (studbook.empty()) {
    studbook()
  }
  analysis <- match.arg(analysis)
  sex <- match.arg(sex,several.ok=TRUE)
  hemisphere <- match.arg(hemisphere)
  segment <- match.arg(segment)
  test <- match.arg(test)
  
  # to do: link segment to hemisphere
  
  if (analysis == 'birth')  {
    season.birth(sex=sex,effects=effects,hemisphere=hemisphere,segment=segment,test=test,plot_arrow=plot_arrow,delayed=delayed)
  } else if (analysis == 'litter') {
    season.litter(effects=effects,hemisphere=hemisphere,segment=segment,test=test,plot_arrow=plot_arrow,delayed=delayed)
  } else if (analysis == 'death')  {
    season.death(sex=sex,hemisphere=hemisphere,segment=segment,plot_arrow=plot_arrow,test=test)
  } else {
    cat("No analysis defined: pass `birth`, `litter` or `death`") # Should be caught by match.arg anyway
  } 
  
}

