###
# Global variables
#

###
#' @title Default sites settings
#' @description \code{.defaultSites} creates and initializes variables in the environment 
#' \code{.StudbookEnv}
#' @details This function is called when is the package \code{studbookR} is loaded. 
#' The variable \code{studbook.sites} is created in the global environment (and set to \code{NULL}).
#' The following variables are created in the environment \code{.StudbookEnv}:
#' \describe{
#'  \item{sites.connect}{connnect to SQlite3 database with location data}
#'  \item{sparks.db}{Full path to SQLite3 version of SPARKS locations}
#'  \item{studbook.sites}{Data frame with climate and lon/lat information on studbook locations}
#' }
#' @author Frank PG Princee
#' 
.defaultSites <- function()
{
  assign("studbook.sites",NULL,.GlobalEnv)   # Easier to use across modules
  assign("sites.connect",NULL,.StudbookEnv)  #

  OS <- Sys.info()['sysname']
  
  if (OS == "Darwin") {
    assign("sparks.db","/Applications/PML.app/Contents/Resources/db/sparks.sqlite",.StudbookEnv)
  } else if (OS == "Windows") {
    assign("sparks.db","C:/Program Files/PMLapp/db/sparks.sqlite")
  } else if (OS == 'Linux') {
     assign("sparks.db","/usr/local/share/pml/db/sparks.sqlite",.StudbookEnv)
  }
}


###
#' @title Import studbook sites from SQLite database
#' @author Frank PG Princee
#' @export
#' 
sites.import <- function()
{    
  if (!file.exists(get("sparks.db",.StudbookEnv))) {
    stop(paste("Database",get("sparks.db",.StudbookEnv),"does not exist"))
  }
  
  if (!is.null(studbook.sites)) {
    stop("Already sites frame loaded. Remove before new import")
  }
  
  driver <- DBI::dbDriver("SQLite")
  assign("db.connect", DBI::dbConnect(driver,dbname=get("db",.StudbookEnv)),
         .StudbookEnv)   # Connect to global database
  
  query <- "SELECT mnemonic,latitude,longitude FROM location WHERE latitude IS NOT NULL"
  studbook.sites <<- DBI::dbGetQuery(get("connect",.StudbookEnv),query)
}


###
#' @title Load studbook sites in R format
#' @details the data are loaded to the variable \code{sites} in the user's workspace
#' 
#' @author Frank PG Princee
#' @export
#' 
sites.load <- function()
{
  cat("Loading spatial data on studbook sites\n")
  data("studbook.sites",package="studbookR")
}



###
#' @title Add column to sites dataframe
#' @param field Name of column
#' @author Frank PG Princee
#' @export
#' 
sites.addField <- function(field)
{
  sites <- get("studbook.sites",envir=.GlobalEnv)
  sites <- within(sites,field=NA)
  
  #Change column to field name
  fields <- names(sites)
  n <- length(fields)
  fields[n] <- field
  names(sites) <- fields
  assign("studbook.sites",sites)
}


###
#' @title Update field in sites data frame with raster values
#' @details A new column will be created if the requested field does not exist
#' @param field Name of existing or new field
#' @param buffer Radius of buffer around point in meters (default: 5000)
#' @author Frank PG Princee
#' @export
#' 
sites.update <- function(field='climate',buffer=5000)
{
  #Add column if field does not exist
  sites <- get("studbook.sites",envir=.GlobalEnv)
  
  if (!is.element(field,names(sites)) ){
    sites.addField(field)
    sites <- get("studbook.sites",envir=.GlobalEnv)
  }
  
  
  for (s in 1:nrow(studbook.sites)){
    lat <- sites[s,"latitude"]
    lon <- sites[s,"longitude"]
    if (is.na(lat)) {
      next
    }
    
    if (!is.na(sites[s,field])) {
      next
    }
    
    lonlat <- data.frame(lon=lon,lat=lat)
   
    value <- raster.extract(lonlat,buffer)              #extracts from active raster!
    cat(s,sites[s,'mnemonic'],lon,lat,value,"\n")
    
    if (!is.na(value)) {
      sites[s,field] <- value
    }
  }
  #assign("studbook.sites",sites)
}


###
#' @title Save studbook sites in R format
#' @author Frank PG Princee
#' @export
#' 
sites.save <- function()
{
  studbook.sites <- get("studbook.sites",envir=.GlobalEnv)
  if (is.null(studbook.sites)) {
    stop("No sites defined.")
  }
  save(studbook.sites,file=studbook.get("sites.rda"))
}


###
#' @title Swap longitude, latitude values
#' @details Correct errors in entering longitude and latitude in wrong order
#' @param id Row number in data frame sites
#' @author Frank PG Princee
#' @export
#' 
swap.coordinates <- function(id=NA)
{
  if (is.na(id)) {
    return(FALSE)
  }
  sites <- get("studbook.sites",envir=.GlobalEnv)
  lonlat <- c(NA,NA)
  lonlat <- sites[id,c('latitude','longitude')]
  sites[id,c('longitude','latitude')] <- lonlat
  assign("studbook.sites",sites)
}


###
#' @title Plot table with temperature and precipitation at location
#' @param location name
#' @param data_type Data type "tmean", "tmin", "tmax", or "prec"
#' @author Frank PG Princee
#' @export
#' 
sites.plotMonth <- function(location,data_type=c("tmean","tmax","tmin","prec"))
{
  data_type <- match.arg(data_type)
  fields <- rep(NA,12)
  for (x in 1:12) { 
    fields[x] <- paste(data_type,x,sep='_') 
  }
  if (data_type == "tmean") {
    ylabel <- "Mean temperature (Celcius)"
  } else if (data_type == "tmin") {
    ylabel <- "Minimum temperature (Celcius)"
  } else if (data_type == "tmax") {
    ylabel <- "Maximum temperature (Celcius)"
  } else if (data_type == "prec") {
    ylabel <- "Precipitation (mm)"
  }
  
  sites <- get("studbook.sites",envir=.GlobalEnv)
  
  months <- sites[sites$mnemonic==location,fields]
  months <- as.matrix(months)
  barplot(height=months,names.arg=month.abb,ylab=ylabel)
  print(months)
}


###
#' @title Get temperature or precipitation at location for month
#' @param location name
#' @param month Month number [1..12]
#' @param data_type Data type "tmean", "tmin", "tmax", or "prec"
#' @return temperature or precipitation. NA when no data
#' @author Frank PG Princee
#' @export
#' 
sites.getMonth <- function(location,month,data_type=c("tmean","tmax","tmin","prec"))
{
  data_type <- match.arg(data_type)
  if (is.element(month,1:12)) { 
    month <- paste(data_type,month,sep="_")
    sites <- get("studbook.sites",envir=.GlobalEnv)
    return(sites[sites$mnemonic == location,month])
  }
  return(NA) 
}

