###
#' @title Life table smoothing and parametric models
#' @description \code{model.smooth} selects smoothing and modelling procedures for lifetables.
#' 
#' @details This function is used to select smoothing or modelling procedures for life table data. The following
#' smoothing procedures and models are included:
#' \describe{
#' \item{SPARKS}{running mean that can be repeated <times>, recursively: \code{model=c("SPARKS",<times>)}.}
#' \item{PMx}{running median that can be repeated <times>, recursively: \code{model=c("PMx",<times>)}.}
#' \item{Tukey}{Tukey's smoothing models that can be varied by selecting <model> and <endrule>: 
#' \code{model=c("Tukey",<model>,<endrule>})}.
#' \item{Siler}{Siler model: \code{model=c("Siler").}}
#' \item{Gompertz}{Gompertz-Makeham model: \code{model=c("Gompertz").}}
#' \item{Weibull}{Weibull mode: \code{model=c("Weibull").}}
#' \item{Hadwiger}{Hadwiger: \code{model=c("Hadwiger", "first|last|range|none").}}
#' }
#' 
#' @param data_set lifetable data.
#' @param model vector with model settings (default: c("PMx",1))
#' @return results of model with name of procedure as comment
#' @author Frank PG Princee
#' @family smoothing
#' @keywords internal
#' 
model.smooth <- function(data_set,model=c('PMx',1))
{
  field_name <- colnames(data_set)[2]
  studbook.screenSection('Smoothed data')
  if (model[1] == 'SPARKS') {
    result <- smooth.SPARKS(data_set[,field_name],model[2])
    comment(result) <- paste("Smoothing of mean as in SPARKS (",model[2],"times).")
  } else if (model[1] == 'PMx') {
    result <- smooth.PMx(data_set[,field_name],model[2])
    comment(result) <- paste("Smoothing of median as in PMx (",model[2],"times).")
  } else if (model[1] == 'Tukey') {
    result  <- smooth.Tukey(data_set[,field_name],model=model[2],rule=model[3])
    comment(result) <- paste("Smoothing using Tukey model",model[2],"with endrule",model[3],".")
  } else if (model[1] == 'Siler') {
    result <- model.Siler(data_set[,field_name])
    comment(result) <- "Siler curve"
  } else if (model[1] == 'Gompertz') {
    result <- model.Gompertz(data_set[,field_name]) 
    comment(result) <- "Gompertz - Makeham curve"
  } else if (model[1] == 'Weibull') {
    lx <- mortality.lx(1-data_set[,field_name])
    result <- model.Weibull(lx)
    comment(result) <- "Weibull curve"
  } else if (model[1] == 'Hadwiger') {
   
    if (is.na(model[2])) {
      model[2] <- "none"
    }
    mx.range <- fecundity.range(data_set[,field_name])
  
    if (model[2] == "first") {         #first:end
      mx.range[2] <- nrow(data_set)
    } else if (model[2] == "last") {
      mx.range[1] <- 1                 #1:last
    } else if (model[2] == "none") { 
      mx.range <- c(1,nrow(data_set))
    }
            
    mx <- data_set[mx.range[1]:mx.range[2],field_name]
    result <- model.Hadwiger(mx)
    result <- c( rep(0,mx.range[1]-1),result,rep(0,nrow(data_set)-mx.range[2]) )
    comment(result) <- "Hadwiger curve"
  } else {
    result <- data_set[,field_name]
    comment(result) <- "No model selected."
  }
  return(result)
}


###
#' @title SPARKS smoothing 
#' @description \code{model.SPARKS} is based on the smoothing algorithm as implemented in SPARKS.
#' @details SPARKS implements smoothing of running means.
#' The following rules for \eqn{n} classes are applied:
#' \describe{ 
#' \item{x=0}{no change}
#' \item{x=1}{\eqn{(data[1] + data[2])/2}}
#' \item{1 < x < n}{\eqn{(data[x-1] + data[x] + data[x-1])/3}} 
#' \item{x=n}{\eqn{data[n-1]+data[n])/3}} 
#' }
#' 
#' @param data_set vector with mortality or fecundity rates.
#' @return vector of \eqn{n} classes with smoothed values.
#' @author Frank PG Princee 
#' @family smoothing
#' @keywords internal
#'  
model.SPARKS <- function(data_set) 
{
    n = length(data_set)                  # Length of original dataset
    data_set[n+1] <- 0                    # Add extra element for calculating class
    smoothed <- data_set                  # smoothed[1] stays untouched
    smoothed[2] = mean(data_set[2:3])
    for (d in c(3:n) ) {
        smoothed[d] = mean(data_set[ ((d-1):(d+1)) ]) 
    }     
    return(smoothed[1:n]) #Return original number of elements                             
}

###
#' @title PMx smoothing
#' @description \code{model.PMx} is based on the smoothing procedure in PMx.
#' 
#' @details PMx implements a variant of the running median algorithm based on Tukey (1977). 
#' The following rules for \eqn{n} classes are applied (Ballou, pers. comm):
#' \describe{
#' \item{x=0}{No change}
#' \item{0 < x < n}{median(data[x-1] + data[x],+data[x+1])}
#' \item{x=n}{median(data[n-1] + data[n] + (3*data[n-1] - 2 * data[n-2]))}
#' }
#'   
#' @param data_set vector with mortality or fecundity rates
#' @return vector of \eqn{n} classes with smoothed values
#' 
#' @author Frank PG Princee
#' @family smoothing
#' @references Tukey, JW. 1977. Exploratory Data Analysis, Addison Wesley, Reading MA.
#' @keywords internal 
#' 
model.PMx <- function(data_set) 
{
    n = length(data_set)                                                        #Length of original dataset
    smoothed <- data_set 
    
    data_set[n+1] <- (3 * data_set[n-1]) - (2 * data_set[n-2])                  #Add extra element for calculating class
    smoothed <- runmed(data_set,k=3,'keep')                                     #Keeps first and last (added) one
    
    return(smoothed[1:n])                                                       #Return original number of elements      
}


### 
#' @title SPARKS recursive smoothing 
#' @description This function is used to repeat the SPARKS smoothing model recursively.
#' @param data_set vector with mortality or fecundity rates
#' @param times number of times of recursive smoothing
#' @return vector with smoothed values
#' 
#' @author Frank PG Princee
#' @family smoothing
#' @keywords internal 
#' 
smooth.SPARKS <- function(data_set,times=1)
{
    for (t in 1:times) {
        data_set <- model.SPARKS(data_set)        
    }
    return(data_set)
}


### 
#' @title PMx recursive smoothing 
#' @description This function is used to repeat the PMx smoothing model, recursively.
#' 
#' @param data_set vector with mortality or fecundity rates
#' @param times number of times of recursive smoothing
#' @return vector with smoothed values
#' @author Frank PG Princee
#' @family smoothing
#' @keywords internal 
#' 
smooth.PMx <- function(data_set,times=1)
{
    for (t in times) {
        data_set <- model.PMx(data_set)                        
    }
    return(data_set)
}

###
#' @title Tukey smoothing 
#' @description \code{smooth.Tukey} uses Tukey's smoothing algorithms.
#' 
#' @details This function is wrapper around the function \code{\link{smooth}}.
#' 
#' @param data_set vector with mortality or fecundity rates (or other data)
#' @param model Kind of Tukey smoother (default model 3)
#' @param copy end rule of smoother i.e. "Tukey" or "copy" (default)
#' @return smoothed vector
#' @author Frank PG Princee
#' @family smoothing
#' @keywords internal 
#'  
smooth.Tukey <- function(data_set,model='3',rule='copy')
{
    return(smooth(data_set,kind=model,endrule=rule) )
}

###
# Mortality models
###


###
#' @title Siler curve
#' @description \code{model.Siler} fits a Siler curve of mortality rates.
#' 
#' @details This function is a wrapper around the functions \code{fitSiler} and
#' \code{Siler} in the package \code{fmsb}.
#' 
#' q(x) = a1*exp(-b1*x)+a2+a3*exp(b3*x) 
#' 
#' The parameters \code{a1}, \code{b1}, \code{a2}, \code{a3} and \code{b3}
#' are solved by \code{fitSiler}.
#' 
#' @param qx vector with mortality rates
#' @return vector with Siler model values
#' @author Frank PG Princee
#' @references Taylor et al. (1995)
#' @family smoothing
#' @keywords internal 
#' 
model.Siler <- function(qx) 
{
    params <- fmsb::fitSiler(data=qx)  
    FLAG <- params[7]
    while (FLAG > 0) {
        params <- fmsb::fitSiler(params[1:5],data=qx)
        FLAG <- params[7]
    }
    cat("[Siler]\n")
    cat("a1 = ",params[1],", b1 = ",params[2],", a2 = ",params[3],
        "a3 = ",params[4],", b3 = ",params[5],", convergence = ",params[7],sep="")
    cat("\n\n")
    
    max_c <- length(qx) -1 
    qx.siler <- fmsb::Siler(a1=params[1],b1=params[2],a2=params[3],a3=params[4],
                            b3=params[5],t=c(0:max_c))

    return(qx.siler)
}


###
#' @title Gompertz-Makeham mortality model
#' 
#' @description \code{model.Gompertz} fits a Gompertz-Makeham curve from mortality rates.
#' @details This function is a wrapper around the functions \code{fitGM} and
#' \code{GompertzMakeham} in the package \code{fmsb}. 
#' 
#'
#' u(x)=A*expB*x + C
#'
#' parameters \code{A}, \code{B} and \code{C} are estimated using \code{fitGM}. 
#' u(x) is converted to q(x) by \code{GompertzMakeham}.
#'    
#' @param qx mortality rates
#' @return vector with Gompertz model values
#' @author Frank PG Princee
#' @seealso \code{\link{GompertzMakeham}}
#' @references  Gompertz B. (1825) On the nature of the function expressive of the law of human mortality, and on a new mode of determining the value of life contingencies. 
#' \emph{Philosophical Transactions of the Royal Society of London}, \bold{115}, 513–583.
#' @family smoothing
#' @keywords internal 
#' 
model.Gompertz <- function(qx)
{
    params <- fmsb::fitGM(data=qx)  
    
    FLAG <- params[5]
    while (FLAG > 0) {
        params <- fmsb::fitGM(params[1:3],data=qx)
        FLAG <- params[5]
    }
    max_c <- length(qx) -1
    cat("[Gompertz-Makeham]\n")
    cat("A = ",params[1],", B = ",params[2],", C = ",params[3],
        ", convergence = ",params[5],sep="")
    cat("\n\n")
    
    qx.gompertz <- fmsb::GompertzMakeham(A=params[1],B=params[2],C=params[3],t=c(0:max_c))
    return(qx.gompertz)
    
}


###
#' @title Weibull model
#' 
#' @description \code{model.Weibull} fits a Weibull curve from survivorship (lx).
#' @details This function is a wrapper around the function \code{fitdistr} 
#' in the package \code{MASS}. 
#' 
#'
#' u(x)=A*expB*x + C
#'
#' parameters \code{A}, \code{B} and \code{C} are estimated using \code{fitGM}. 
#' u(x) is converted to q(x) by \code{GompertzMakeham}.
#'    
#' @param lx survivorship
#' @return vector with Weiburg values
#' @author Frank PG Princee
#' @family smoothing
#' @references Pinder III JE, Wiener JG and Smith MH. 1978. 
#' The Weibull distribution: 
#' A new method of summarizing survivorship data. 
#' \emph{Ecology}, \bold{59(1)}, 175–179, 1978.
#' @keywords internal 
#' 
model.Weibull <- function(lx)
{
  lx.fit <- MASS::fitdistr(lx,"weibull")  
  shape <- as.numeric(lx.fit$estimate["shape"])
  scale <- as.numeric(lx.fit$estimate["scale"])
  cat("Weibull\n")
  
  cat("Shape=",shape,"\n")
  cat("Scale=",scale,"\n\n")
  
  #Create weibull: WRONG
  lx.weibull <- pweibull(lx,shape=shape,scale=scale,lower.tail=TRUE)
  
  return(lx.weibull)
}


###
#' @title Fits Hadwiger model to fecundity data
#' @param mx fecundity rates
#' @return vector with fitted fecundity rates
#' @family smoothing
#' @keywords internal 
#' 
model.Hadwiger <- function(mx)
{
  
  params <- fitHadwiger(data=mx)

  FLAG <- params[5]
  while (FLAG > 0) {
    params <- fitHadwiger(params[1:3],data=mx)
    FLAG <- params[5]
  }
  cat("[Hadwiger function]\n")
  cat("a = ",params[1],", b = ",params[2],", c = ",params[3],
      ", convergence = ",params[5],sep="")
  cat("\n\n")
  
  mx.Hadwiger <- studbook.Hadwiger(params[1],params[2],params[3],length(mx))
  
  return(mx.Hadwiger) 
}


###
#' @title Fits Brass polynomial to fecundity data
#' @param mx fecundity data.
#' @return vector with fitted fecundity rates
#' @family smoothing
#' @keywords internal
#' 
model.Brass <- function(mx)
{
  
  params <- fitBrass(data=mx)
  FLAG <- params[5]
  while (FLAG > 0) {
    params <- fitBrass(params[1],data=mx)
    FLAG <- params[5]
  }
  cat("Brass polynomial: c=",params[1],"B=",params[2],",C=",params[3],
      "convergence =",params[5],sep="")
  cat("\n\n")
  mx.Brass <- studbook.Brass(params[1])
  
  return(mx.Brass) 
}

