###
# Package : studbookR
# 17-05-2014: Added cohort
# 25-08-2013: Added minimum and maximum age criteria
# Copyright (c) 2011,2014 Frank PG Princee, All rights reserved
#
# Kaplan-Meier estimator and Cox Proportional Hazard regression  
# to analyse survival in studbooks

# Optionally implement survplot functions in package rms


###
#' @title Create list of grouped cohorts
#' @param census range min and maximum period e.g. c(1979,1989)
#' @param period period e.g. 2 or 5 years
#' @return list with periods
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#' 
equalCohorts <- function(census, period)
{
  groups <- floor((census[2]-census[1])/period)
  cohort <- rep(NA,groups)
  end_cohort <- census[2]
  start_cohort <- end_cohort - period +1 
  for (x in groups:1) {
    cohort[x] <- list(start_cohort:end_cohort)
    end_cohort <-  start_cohort - 1
    start_cohort <- end_cohort - period + 1
    if (start_cohort < census[1]) {
      start_cohort <-  census[1]
    }
  }
  return(cohort)
}

###
#' @title Create litters from survival data
#' @param survival.data Survival data
#' @details EVENT in survival.data must have been adapted to neonatal mortality.
#' @return litter.data
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#' 
survival.litters <- function(survival.data)
{
  survival.data <- subset(survival.data,LITTER_ID > -1)
  litter <- unique(survival.data$LITTER_ID)
  n <- length(litter)
  litter.data <- data.frame("LITTER_ID"=litter,
                            "BIRTHDATE"=rep(0,n),"BIRTHSITE"=rep(0,n),
                            "DAM"=rep(0,n),"DAM_AGE"=rep(0,n),
                            "BIRTHS"=rep(0,n),"DEATHS"=rep(0,n),
                            "F"=rep(0,n),"G"=rep(0,n))
  for (x in 1:n) {
    litter.mates <- which(survival.data$LITTER_ID == litter[x])
    
    litter.data[x,'BIRTHDATE'] <- survival.data[litter.mates[1],'BIRTHDATE']
    litter.data[x,'BIRTHSITE'] <- survival.data[litter.mates[1],'BIRTHSITE']
    litter.data[x,'DAM']       <- survival.data[litter.mates[1],'DAM']
    litter.data[x,'DAM_AGE']   <- survival.data[litter.mates[1],'DAM_AGE']
    litter.data[x,'DAM_F']     <- survival.data[litter.mates[1],'DAM_F']
    litter.data[x,'SIRE_F']    <- survival.data[litter.mates[1],'SIRE_F']
    litter.data[x,'F']         <- survival.data[litter.mates[1],'F']
    litter.data[x,'G']         <- survival.data[litter.mates[1],'G']
    litter.data[x,'BIRTHS']    <- length(litter.mates)
    litter.data[x,'DEATHS']    <- sum(survival.data[litter.mates,'EVENT'])  # died = 1
  }
  litter.data <- within(litter.data, SURVIVED <- BIRTHS-DEATHS)
  return(litter.data)
}

###
#' @title Check whether survival data are recurrent
#' @details This function checks whether the column \code{ID} has been defined 
#' and whether duplicate data exists in this column
#' @param survival.data Frame work with survival data
#' @return TRUE of FALSE 
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#'
is.recurrent <- function(survival.data)
{
  recurrent <- FALSE
  if (is.element("ID",colnames(survival.data)) ) {
    recurrent <- ifelse(anyDuplicated(survival.data$ID) == 0,FALSE,TRUE)
  }
  return(recurrent)
}


###
#' @title Check whether only singleton litters exist
#' @details This function checks whether the column \code{LITTER_ID} has been defined 
#' and whether duplicate data exists in this column
#' @param survival.data Frame work with survival data
#' @return TRUE of FALSE 
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#'
is.singleton <- function(survival.data)
{ 
  singleton <- FALSE
  if (is.element("LITTER_ID",colnames(survival.data)) ) {
    singleton <- ifelse(anyDuplicated(survival.data$LITTER_ID) == 0,TRUE,FALSE)
  }
  return(singleton)
}


###
#' @title Right-censor individuals older than provided age
#' @details This function can be used to study 30 day survival 
#' @param survival.data Frame with data
#' @param end_age Maximum age to analyse, older ages are censored
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#'
survival.censorAge <- function(survival.data,end_age=Inf)
{
  # Individuals that die after end_age are right-censored at age = end_age
  if (is.finite(end_age)) {
    for (x in 1:nrow(survival.data)) {
      if (survival.data[x,'TIME'] > end_age) {
        survival.data[x,c('TIME','EVENT')] <- c(end_age,0)
      }
    }
  }
  
  return(survival.data)
}


###
#' @title Distribute unknown sex over males and females
#' @details Unknwon sex is divided equally. First is female
#' @param survival.data Frame with survival data
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#'  
survival.unknownSex <-  function(survival.data)
{
  # Distribute equally over males/females 
  sex <- "female"
  for (x in 1:nrow(survival.data)) {
    if (survival.data[x,'SEX'] == "unknown") {
      survival.data[x,'SEX'] <- sex
      sex <- ifelse(sex == "male","female","male")
    }
  }
  return(survival.data)
}  

###
#' @title Inbreeding class in survival
#' @param survival.data  Survival data
#' @return survival.data
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#'
survival.inbred <- function(survival.data)
{
  if (!is.element("INBRED",colnames(survival.data))) {
    survival.data <- within(survival.data,INBRED <- NA)
  }
  
  for (x in 1:nrow(survival.data)) {
    survival.data[x,'INBRED'] <- inbreeding.level(survival.data[x,"F"])
  }
  survival.data$INBRED <- as.factor(survival.data$INBRED)
  return(survival.data)
}


###
#' @title Create group filter
#' @description \code{survival.filter} creates list with filter settings on group.
#' @param param string parameter (mostly used with BioClim data) (default:NULL)
#' @param location string group of locations to compare \code{"BIRTHSITE","LOCATION", 
#' NULL} (default:NULL)
#' @param condition string filter for selected parameter (default:NULL)
#' @param condition vector with prediction values or percentiles for Cox model with numerical values
#' @return list with param and filter
#' @author Frank PG Princee
#' @family survival
#' @export
#' 
survival.filter <- function(param=NULL,location=NULL,condition=NULL,prediction=NULL)
{
  return(list(param=param,location=location,condition=condition,prediction=prediction))
}


###
#' @title Add FAO thermal climatic zone to survival.data
#' @description \code{survival.thermal} adds FAO thermal zone at location 
#' to survival record
#' @param survival.data Frame with survival data
#' @param location "BIRTHSITE" or "LOCATION"
#' @return survival.data
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#'
survival.thermal <- function(survival.data,location)
{
  if (is.null(studbook.sites)) {
    sites.load() 
  }
  
  if (!is.element("CLIMATE",colnames(survival.data))) {
    survival.data <- within(survival.data, CLIMATE <- NA)
  }
  
  studbook.sites$selected <<- FALSE
  
  # Add climatic zone to location
  for (x in 1:nrow(survival.data)) {
    site <- survival.data[x,location]  
    climate  <- studbook.sites[studbook.sites$mnemonic==site,'thermal']
    
    studbook.sites[studbook.sites$mnemonic==site,'selected'] <<- TRUE
    if (length(climate) == 1) {
      if (!inThermal(climate)) {
        next
      } 
      climate <- studbook.get("thermal.zone")[as.integer(climate)]
      survival.data[x,'CLIMATE']  <- climate     # Use as integer number
    }
  }    
  survival.data <- survival.data[!is.na(survival.data$CLIMATE),]
  survival.data$CLIMATE <- as.factor(survival.data$CLIMATE)
  return(survival.data)
}


###
#' @title Add Koeppen-Geiger climatic zone to survival.data
#' @param survival.data Frame with survival data
#' @param location \code{"BIRTHSITE" or "LOCATION"}
#' @return survival.data
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#' 
survival.koeppen <- function(survival.data,location)
{
  if (is.null(studbook.sites)) {
    sites.load() 
  }
  
  if (!is.element("CLIMATE",colnames(survival.data))) {
    survival.data <- within(survival.data, CLIMATE <- NA)
  }
  
  studbook.sites$selected <- FALSE
  
  # Add climatic zone to location
  for (x in 1:nrow(survival.data)) {
    site <- survival.data[x,location]  
    climate  <- studbook.sites[studbook.sites$mnemonic==site,'koeppen']
    
    studbook.sites[studbook.sites$mnemonic==site,'selected'] <- TRUE
    if (length(climate) == 1) {
      if (!inKoeppen(climate)) {
        next
      } 
      climate <- studbook.get("koeppen.zone")[as.integer(climate)]
      survival.data[x,'CLIMATE']  <- climate     # Use as integer number
    }
  }    
  survival.data <- subset(survival.data, !is.na(CLIMATE))  # Remove NA records
  survival.data$CLIMATE <- as.factor(survival.data$CLIMATE)
  return(survival.data)
}




###
###
#' @title Add WorldClim data to survival.data
#' @param survival.data Frame with survival data
#' @param location \code{"BIRTHSITE","LOCATION"}
#' @param filter Filter list with params and conditions
#' @return survival.data
#' @author Frank PG Princee
#' @keywords internal
#' @family survival
#' 
survival.worldclim <- function(survival.data,filter)
{
  if (is.null(studbook.sites)) {
    sites.load() 
  }
  
  studbook.sites$selected <- FALSE
  
  is.scale <- FALSE
  width <- 0
  
  if (is.null(filter[['condition']])) {
    biovar.expr <- NULL
  } else {
    if (filter[['condition']] =='scale' ) {  # Trewartha
        biovar.expr <- NULL                            # No expression filter
        is.scale <- TRUE
    } else if (length(grep("width",filter[['condition']])) > 0) { 
      biovar.expr <- NULL
      split_txt <- strsplit(filter[['condition']],"=")
      width <- as.integer(split_txt[[1]][2])
    } else {
      biovar.expr <- parse(text=filter[['condition']])   # Expression text
    }
  }
  
  climate_field <- "TEMPERATURE"
  
  # Add month when tmean prec data are used
  
  if (length(grep("tmax|tmean|tmin|prec",filter[['param']])) > 0 ) {
    addMonth <- TRUE
    if (filter[['param']] == "prec") {
      climate_field<- "PRECIPITATION"
    } else {
      climate_field <- "TEMPERATURE"
    }
  } else {
    addMonth <- FALSE
    if (is.element(filter[['param']],c("bio1","bio5","bio6","bio8","bio9","bio10","bio11")) ){
      climate_field <- "TEMPERATURE"
    } else {
      climate_field <- "PRECIPITATION"
    }
  }
  
  if (!is.element(climate_field,colnames(survival.data))) {
    if (climate_field == "TEMPERATURE") {
      survival.data <- within(survival.data, TEMPERATURE <- NA)
    } else {
      survival.data <- within(survival.data, PRECIPITATION <- NA)
    }
  }
  
  # Add climatic zone to location
  for (x in 1:nrow(survival.data)) {
    site <- survival.data[x,filter[['location']]]  
  
    if (addMonth) { 
      birthmonth <- tis::month(survival.data[x,'BIRTHDATE'])
      field <- paste(filter[['param']],birthmonth,sep="_")  #tmean1..9
    } else {
      field <- filter[['param']]
    }
    biovar  <- studbook.sites[studbook.sites$mnemonic==site,field]
    
    
    #Check whether bioclimate data are available
    if (length(biovar) > 0) {
      
      survival.data[x,climate_field] <- biovar
      studbook.sites[studbook.sites$mnemonic==site,'selected'] <- TRUE
      
      
      if (is.scale) {
        survival.data[x,climate_field] <- thermal.getScale(biovar)
      } else if (width > 0) {
        survival.data[x,climate_field] <- floor(biovar/width) * width
      } else {
        # Apply condition 
        if (!is.null(biovar.expr)) {
          result <- eval(biovar.expr)
          survival.data[x,climate_field] <- ifelse(result == 1,"yes","no")
        }
      }      
    }
  }    
  survival.data <- survival.data[!is.na(survival.data[,climate_field]),]
  if (is.scale) {
    survival.data[,climate_field] <- as.factor(survival.data[,climate_field])
  }
  return(survival.data)
}



###
# Manipulation of results
###

###
#' @title Extract survivorship values from strata
#' @title \code{survival.getStratum} extract survivorship values from individual strata
#' in a survfit object.
#' @param survival.fit \code{survfit} object 
#' @param stratum Number of stratum to extract (default=1)
#' @return data frime with time, numbers at risk and survivorship for stratum
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#' 
survival.getStratum <- function(survival.fit,stratum=1)
{
  if (class(survival.fit)[1] == "survfit") { 
    
    if (is.null(survival.fit$strata)) {
      stratum <- 1
    } 
    
    if (stratum ==  1) {
      start <- 1
    } else {
      start <- 1 + sum(survival.fit$strata[1:(stratum-1)])
    }
    
    end <- start + survival.fit$strata[stratum] - 1
    stratum.fit <- data.frame("time"  =survival.fit$time[start:end],
                              "n.risk"=survival.fit$n.risk[start:end],
                              "surv"  =survival.fit$surv[start:end])
    
  } else if (class(survival.fit)[1] == "survfit.cox") {
    stratum.fit <- data.frame("time" = survival.fit$time,n.risk=survival.fit$n.risk,
                              "surv" = survival.fit$surv[,stratum])
  } else {
    return(NULL)
  }
  
  return(stratum.fit)
}


###
#' @title Compute cumulative hazard rates from Kaplan-Meier Survival curve
#' @description \code{survival.cumulativeHazard} computes the cumulative hazard rate 
#' for each survival time in a Kaplan-Meier survival curve.
#' @details Computation is Ht = -log(St)
#' @param survival.data frame with time and survival
#' @author Frank PG Princee
#' @family survival
#' @keywords internal 
#
survival.cumulativeHazard <- function(survival.data)
{
  hazard <- data.frame("time"=survival.data$time,"H" = -log(survival.data$S))
  return(hazard)
}


###
# 
survival.weibull <- function()
{
  #see survreg
}

###
#' @title Display last data 
#' @param survival.fit \code{survfit} object
#' @param data_name Name of data e.g. survival, firstbirth
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#' 
survival.lastRecord <- function(survival.fit,data_name)
{
  if (is.element(data_name,c('firstbirth','fecundity') ) ){
    cat("Proportion of non-breeders\n")
  } else { 
    cat("Proportion survived\n")
  }
  cat("--------------------------\n")
  
  # Kaplan-Meier survfit
  if (class(survival.fit[1]) == "survfit") {
    if (is.null(survival.fit$strata)) {
      N <- length(survival.fit$time)
      cat(survival.fit$n.risk[1],survival.fit$n.risk[N],survival.fit$surv[N],"\n",sep="\t")
    } else {
      for (x in 1:length(survival.fit$strata)) {
        stratum.fit <- survival.getStratum(survival.fit,x)
        N <- nrow(stratum.fit)
        cat(attr(survival.fit$strata[x],"names"),
            stratum.fit[1,'n.risk'],stratum.fit[N,'n.risk'],
            stratum.fit[N,'surv'],"\n",sep="\t")
      }
    }
  }
}


##
#' @title Creates list with survival parameter settings
#' @param analysis type of analysis: \code{'survival', 'fecundity', 'firstbirth'}
#' (default: \code{'survival'}).
#' @details A list with parameter settings specific for survival, fecundity and 
#' age at first breeding is created.
#' @param method \code{'kaplan'}: Kaplan-Meier estimator, 
#' \code{'cox'}: Cox proportional hazards regression (default: \code{'kaplan'}) 
#' @return list
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#' 
survival.settings <- function(analysis=c('survival','fecundity','firstbirth'),
                              method=c('kaplan','cox'))
{

  # Check type of analysis
  if (analysis == "survival") {
    data_file  <- "kmsurvival"
    main_title <- "survival"
    sub_title  <- "Survival"   
    ylabel     <- "Survivorship"   
    birthsite  <- NA
  } else if (analysis == "fecundity") {
    data_file  <- "kmfecundity"
    main_title <- "fecundity."
    sub_title  <- "Fecundity"
    ylabel     <- "Proportion non-breeders"
    birthsite  <- "LOCATION"
  } else if (analysis == "firstbirth" ) {
    data_file  <- "kmfecundity"
    main_title <- "first age at reproduction."
    sub_title  <- "Fecundity"
    ylabel     <- "Proportion non-breeders"
    birthsite  <- "LOCATION"
  }
  
  if (method == "kaplan") {
    main_title <- paste("Kaplan-Meier estimator for",main_title)
    Qtest <- TRUE
  } else if (method == "cox") {
    main_title <- paste("Cox proportional hazard regression on",main_title)
    Qtest <- FALSE
  }
    
  
  names <- list(analysis=analysis,data_file=data_file,main_title=main_title,sub_title=sub_title,
                ylabel=ylabel,birthsite=birthsite,Qtest=Qtest)
  return(names)
}


###
#' @title Kaplan Meier Estimator and Cox regression for survival and fecundity
#' @description \code{studbook.survival} estimates survival and fecundity using the 
#' Kaplan-Meier Product Estimator and/or Cox's Proportional Hazards Regression Model
#'  
#' @details 
#' 
#' Age interval is inclusive i.e. [\code{age[1]},\code{age[2]}]
#' Cox automatically adds cluster LITTER_ID when twin, triplets etc. exist
#' 
#' The following data groups can optionally be analysed: 
#' \tabular{lllll}{
#'                 \tab \strong{filter}\cr
#' \strong{group}  \tab \strong{param}  \tab \strong{condition} \tab \strong{location} \tab \strong{predictions} \cr
#' "ALTITUDE"      \tab \cr
#' "BREEDER"       \tab \cr
#' "CLIMATE"       \tab "koeppen", "thermal" \tab \tab "BIRTHSITE", "LOCATION"\cr
#' "COHORT"        \tab                      \tab empty: each year; list(c(1978,1988),c(1989,2008)) \cr
#' "INBRED"        \tab empty: F > 0; = F; \tab empty; \tab \cr
#' "G"             \tab \cr
#' "LITTER_SIZE"   \tab \cr
#' "PRECIPITATION" \tab \cr
#' "REARING"       \tab \cr
#' "SEX"           \tab \cr 
#' "TAXON"         \tab \cr
#' "TEMPERATURE"   \tab \cr 
#'}
#' 
#' @param analysis Type of data to be analysed. Options are \code{"survival", "firstbirth", "fecundity"} (default: \code{"survival"}.
#' @param method Analysis method to be used: \code{"cox", "kaplan","recurrent"} (default: \code{"kaplan"})
#' @param sex Sex group to analyse. Options are \code{"all", "female", "male", "unknown"} (default: \code{"all"}). 
#' @param unknown text. Possible values are \code{"exclude","equal","include"} to
#' exclude individuals of unknown sex; distribute equally over 
#' males and females or include as unknown (default:\code{"exclude"}). 
#' @param group text Name of group(s) to be compared. The value "1" means no groups are compared (default: "1")
#' @param filter list with parameter and filter conditions. (default: param=NA,location='BIRTHSITE', 
#' condition=NA). 
#' @param stratum Field to stratify e.g. "BIRTHSITE" (default:NA)
#' @param legend.title Title for legend (default: NULL) 
#' @param plot.conf Plot confidence interval (up to 2 strata) (default:TRUE)
#' @param age Range of  monitoring ages (in days). Living individuals beyond the last age 
#' are censored. A single value refer to the start age. (default: c(0,Inf) )
#' @param cluster Field to cluster in Cox PH analysis. Options are \code{"BIRTHSITE", "DAM","ID", 
#' "LITTER_ID", "NONE"} (default: \code{"LITTER_ID"} when twins are observed.). 
#' @param rho The \code{rho}value in the G-rho family of Harington and Fleming (1982) test.
#' \code{rho=0} is the log-rank or Mantel-Haenszel test, \code{rho=1} is the Peto & Peto 
#' modification of the Gehan-Wilcoxon test. The latter is more approperiate for early deaths.
#' (default: 0)
#' @param alpha Significance level (default: 0.05)
#' @param distribution Assumed distribution of the y value (=age)
#' @param ymin minimum of Y-scale (default: 0)
#' @param percent \code{TRUE}: plot survival as percentage
#' @param summary logical TRUE: print summary 
#' @param verbose logical TRUE: show results
#' @note The function survdiff does not accept left truncated data
#' 
#' @author Frank Princee
#' @family survival
#' @export
#' 
studbook.survival <- function(analysis=c("survival","fecundity","firstbirth"),
                              method=c('kaplan','cox','recurrent'),
                              sex=c('all','female','male','unknown'),
                              unknown=c("exclude","equal","include"),
                              group="1",filter=list(param=NA,location='BIRTHSITE',
                                                    condition=NULL,prediction=NULL),
                              stratum=NA,
                              legend.title=NULL,
                              plot.conf =TRUE,
                              age=c(0,Inf),cluster=c("LITTER_ID","BIRTHSITE","DAM","ID","NONE"),
                              rho=0,alpha=0.05,ymin=0,percent=FALSE,summary=FALSE,verbose=TRUE)
{  
  
  analysis  <- match.arg(analysis)
  method    <- match.arg(method)
  sex       <- match.arg(sex)
  unknown   <- match.arg(unknown)
  cluster   <- match.arg(cluster)
  
  # Check type of analysis and retrieve pre-settings
  setting <- survival.settings(analysis,method)
  
  if (is.na(filter[['param']])) {
    setting[['group']] <- tolower(group)
  } else {
    setting[['group']] <- paste(filter[['param']],tolower(group),sep="-")
  }
  
  if (is.null(legend.title)) {
    setting[['legend']] <- setting[['group']]
  } else {
    setting[['legend']] <- legend.title
  }
  
  setting[['plot_conf']] <- plot.conf 
  setting[['sex']]   <- sex   
  
  subset <- NULL
  
  # All data are required when distribution of unknown
  if (is.element(unknown, c("equal","include"))) {
    import_sex <- "all"
  } else {
    import_sex <- sex
  }
  
  survival.data <- studbook.read(setting[['data_file']],sex=import_sex) 
  
  if (is.null(survival.data)) {
    stop("No data")
  }
  
  if (is.element(method,c("fecundity","firstbirth")) & 
      is.element(cluster,c("DAM","LITTER_ID")) ) {
      cluster <- "NONE"
  }
  
  survival.data <- unique(survival.data)                       # Prevent double records
  survival.data$BIRTHDATE <- as.Date(survival.data$BIRTHDATE)  # To use year and month
  
  group <- toupper(group)
  
  if (is.element("COHORT",group)) {
    survival.data <- within(survival.data, COHORT <- tis::year(BIRTHDATE))
  }
  
  # Extract first birth or non-breeder from fecundity (KIDS= -1 refers to censored).  
  if (analysis == "firstbirth") {
    survival.data <- subset(survival.data,KIDS %in% 0:1)  
  }
  
  # Singeltons
  if (is.element("LITTER_ID",colnames(survival.data))) {
    singleton <- ifelse(anyDuplicated(survival.data$LITTER_ID) == 0,TRUE,FALSE)
    
    if (singleton) {
      if (cluster == "LITTER_ID") {
        cluster <- "NONE"
      }
    }
    if (!singleton & cluster == "LITTER_ID") {
      survival.data <- subset(survival.data,LITTER_ID > -1)       # Remove unknown litters
      survival.data$LITTER_ID <- factor(survival.data$LITTER_ID)
    }
  }
  
  if (cluster == 'DAM') {
    survival.data <- subset(survival.data,DAM_AGE > 0)
    survival.data$DAM <- as.factor(survival.data$DAM)
  }
  
  ###
  # Some screen information on settings (especially when automated scenarios)
  #
  studbook.screenTitle(setting[['main_title']])
  
  
  # Validate monitor age
  if (length(age) == 1) {
    age <- c(age,Inf)
  }
  
  if (is.finite(age[1]) && age[2] < age[1]) {
    age[2] <- max(survival.data$TIME)
  }
  
  
  ###
  # MODIFICATION OF DATA
  ###
  
  survival.data <- survival.censorAge(survival.data,age[2]) # Censor age
  
  if (age[1] > 0) {
    survival.data <- subset(survival.data,TIME >= age[1]) 
    #survival.data$TIME <- survival.data$TIME - age[1]
  }
  
  ###
  # inbreeding coefficient/class
  #
  if (is.element("INBRED",group)) {
    if (is.na(filter[['param']])) {
        survival.data <- within(survival.data,INBRED <- ifelse(F > 0,"yes","no"))
        survival.data$INBRED <- as.factor(survival.data$INBRED)
    } else if (filter[['param']] == 'F') {
      survival.data <- within(survival.data,INBRED <- F)
    } else if (filter[['param']] == 'scale') {
      survival.data <- survival.inbred(survival.data)
    }
     
    if (!is.null(filter[['condition']])) {
        survival.data <- inbreeding.parents(survival.data,filter[['condition']])
    }
    term <- "INBRED"

  }
  
  ###
  # Sex
  #
  
  cat("Sex      :",sex,"\n") 
  cat("Unknown  : ")
  if (unknown == "equal")  {
    n_unknown <- nrow(survival.data[survival.data$SEX =='unknown',])
    cat(ceiling(n_unknown/2),"assumed to be",sex,"\n")
    survival.data <- survival.unknownSex(survival.data)
    
    # Include required sex 
    if (is.element(sex,c('male','female'))) {      
      survival.data <- subset(survival.data,SEX ==sex)
    }
  } else {
    cat(unknown,"\n")
  }
  
  
  if (unknown == "exclude" & import_sex == "all") {
    survival.data <- subset(survival.data,SEX != 'unknown')
  }
  
  ###
  # Altitude (from WorldClim data)
  #
  if (is.element("ALTITUDE",group)) {
    cat("Group    :",studbook.get("worldclim.files")[[filter[['param']]]],"at",
                     filter[['location']],"\n")
    cat("- filter :",filter[['condition']],"\n")
    survival.data <- survival.worldclim(survival.data,filter)
    stop("Not fully implemented")
  }
  
  ###
  # Climate style
  #
  if (is.element("CLIMATE",group)) {
    if (filter[['param']] == 'thermal') { 
      cat("Group    : thermal climate at",filter[['location']],"\n")
      survival.data <- survival.thermal(survival.data,filter[['location']]) #FAO thermal climate
    } else if (filter[['param']] == 'koeppen') {
      cat("Group    : Koeppen-Geiger climate at",filter[['location']],"\n")
      survival.data <- survival.koeppen(survival.data,filter[['location']]) #Koeppen-Geiger climate
    } 
    
    
    # Create table with locations per zone
    # climate.table(survival.data)
    
    climate <- unique(survival.data$CLIMATE)
    site_table <- data.frame(climate,rep(0,length(climate)),rep(0,length(climate)) )
    names(site_table) <- c('CLIMATE',filter[['location']],'individuals')
    
    
    if (is.element(getColumnType(survival.data,"CLIMATE"),c("character","factor")) ) {
      for(x in 1:length(climate)) {

        if (filter[['location']] == 'BIRTHSITE') {
          locations <- unique(survival.data$BIRTHSITE[survival.data$CLIMATE == climate[x] ])          
        } else {
          locations <- unique(survival.data$LOCATION[survival.data$CLIMATE == climate[x] ])
        }
        site_table[x,filter[['location']]] <- length(locations)
        animals   <- nrow(survival.data[survival.data$CLIMATE == climate[x],])
        site_table[x,'individuals'] <- animals
      }
      cat("\n")
      cat("Number of locations per climate zone:\n")
      cat("-------------------------------------------\n")
      print(site_table)
      cat("--------------------------------------------\n\n")
    }
    # studbook locations
    locations <- unique(survival.data$BIRTHSITE)
    locations <- c(locations, unique(survival.data$LOCATION))
    
  } # END OF CLIMATE
  
  
  if (is.element("TEMPERATURE",group)) {
    cat("Group    :",studbook.get("worldclim.files")[[filter[['param']]]],"at",
        filter[['location']],"\n")
    cat("- filter :",filter[['condition']],"\n")
    survival.data <- survival.worldclim(survival.data,filter)
  }
  
  if (is.element("PRECIPITATION",group)) {
    cat("Group    :",studbook.get("worldclim.files")[[filter[['param']]]],"at",
        filter[['location']],"\n")
    cat("- filter :",filter[['condition']],"\n")
    survival.data <- survival.worldclim(survival.data,filter)
  }
  
  cat("Age range: [",age[1],"-",age[2],"] (days)\n",sep="")  
  
  if (is.element("SEX",group)) {   
    cat("Group    : sex\n")
    survival.data$SEX <- factor(survival.data$SEX)
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between sexes.")
    
  } else if (is.element('COHORT',group)) {
    cat("Group    : cohorts")
    
    if (is.null(filter[['condition']])) {
      tmp <- range(survival.data$COHORT)  # Change code later to use year(birth)
      cohort_filter <- c(tmp[1]:tmp[2]) 
      cat("- filter : ",cohort_filter,"\n")
    } else if (is.list(filter[['condition']])) {
      # Create filter of periods of time e.g. 1977-1986, 1987-1996
      cohort_filter <- rep(NA,length(filter[['condition']]))   
      c <- 1
      for (period in filter[['condition']]) {
        cohort_filter[c] <- paste("[",min(period),"-",max(period),"]",sep="")
        survival.data[is.element(survival.data$COHORT,period),'COHORT'] <- cohort_filter[c]
        c <- c + 1
      }
    } else {
      cohort_filter <- filter[['condition']]
      cat("- filter : ",cohort_filter,"\n")
    }
    survival.data <- subset(survival.data,is.element(COHORT,cohort_filter))
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between cohorts.")
  } else if (is.element('DAM_AGE',group)) {
    cat("Group    : age of dam\n")
    survival.data <- subset(survival.data,DAM_AGE > 0)   
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between age of dam.")
    
    
  } else if (is.element("G",group)) {
    cat("Group    : generations",filter[['condition']],"\n")
    if (is.null(filter[['condition']])) {
      tmp <- range(survival.data$G)
      filter[['condition']] <- c(tmp[1]:tmp[2]) 
    }
    survival.data <- subset(survival.data,is.element(G,filter[['condition']]))
    if ( !is.na(filter[['param']]) && filter[['param']] == "factor") {
       survival.data$G <- factor(survival.data$G)
    }
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between generation groups.")
    
  } else if (is.element("INBRED",group)) {
    cat("Group    : inbreeding\n")
    
    if ( !is.null(filter[['condition']]) ) {
       cat("Parents  :",filter[['condition']],"inbred\n")
    }
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between inbred and non-inbred groups.")
    
  } else if (is.element('LITTER_SIZE',group)) {
    cat("Group    : litter size\n")
    survival.data <- subset(survival.data,LITTER_SIZE > 0)
    survival.data$LITTER_SIZE <- survival.data$LITTER_SIZE
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between litter sizes.")  
    
  } else if (is.element('REARING',group)) {
    cat("Group    : parent versus hand-rearing\n")
    survival.data$REARING <- as.factor(survival.data$REARING)
    subset <- (survival.data$TIME >= age[1] & (survival.data$REARING == 'parent' | survival.data$REARING == 'hand'))
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between parent and hand-reared offspring.")
    
  } else if (is.element('TAXON',group))  {
    cat("Group    : subspecies\n")
    survival.data$TAXON <- factor(survival.data$TAXON)
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between sub-species.")
    
  } else if (is.element('CLIMATE',group))  {
    
    colours   <- 1:length(unique(survival.data$CLIMATE))
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between climatic zones.")
    
    # colours   <- 1:2
    # subset    <- (survival.data$TIME >= age[1] & !is.na(survival.data$BIOCLIM))
    # sub_title <- paste(sub_title,"differences between climatic zones.")
    # formula <- formula(survival.object ~ BIOCLIM)
    
  } else if (is.element("TEMPERATURE",group)) {
    setting[['sub_title']] <- paste(setting[['sub_title']],"effects of temperature.")
    
  } else if (is.element("PRECIPITATION",group)) {
    setting[['sub_title']] <- paste(setting[['sub_title']],"effects of precipitation.")
    
  } else if (is.element("BREEDER",group))  { 
    cat("Group    : breeder versus non-breeders\n")
    survival.data$TAXON <- factor(survival.data$BREEDER)
    setting[['sub_title']] <- paste(setting[['sub_title']],"differences between breeders and non-breeders.")
    
  } else {
    group[1] = "1"
    legend.text <-c('median','95% CI')                                                # Default legend when no groups 
    setting[['sub_title']] <- paste(setting[['sub_title']],": Mean and 95% confidence limits.") 
  }
  
  if (is.element("DAM", colnames(survival.data))) {
     survival.data$DAM <- factor(survival.data$DAM)
  }
  
  if (!is.na(stratum) && is.character(survival.data[,stratum]) ){
    if ( !is.factor(survival.data[,stratum]) ) {
      survival.data[,stratum] <- factor(survival.data[,stratum])
    }
  }
  
  
  if (nrow(survival.data) < 1) {
    stop("No survival data")
  } 
  
  # Final analysis
 
  term <-  paste(group,collapse=" + ")  # Create term1 + term2 +... 
  if (is.recurrent(survival.data)) {
    method <- 'recurrent'
  }
  
  
  if (method == 'kaplan') {
    kaplan.fit <- kaplan.meier(survival.data,subset,term,stratum,rho)
    
    if (is.null(kaplan.fit$strata)) {
      setting[["Qtest"]] <- FALSE
    }
    if (studbook.plot()) {
      survival.plot(survival.fit=kaplan.fit,ymin=ymin,setting=setting)
    }
  } else if (method == "cox") {
    
    cox.rs <- cox.regression(survival.data,subset,term,stratum,cluster)
    
    cox.result(cox.rs)
    
    if (studbook.plot() & length(group) < 2) {
      if (class(cox.rs)[1] == "coxph.penal") {  # Frailty has been used
        cox.fit <- survfit(cox.rs)
      } else {
        cox.fit <- cox.predict(survival.data,cox.rs,filter[['prediction']]) 
      }
      if (studbook.plot()) {
        survival.plot(survival.fit=cox.fit,ymin=ymin,setting=setting)
      }
    }
  } else if (method == 'recurrent') {
      recurrent.fit <- survival.recurrent(survival.data,subset,term)
    
    if (studbook.plot()) {
      survival.plot(survival.fit=recurrent.fit,ymin=ymin,setting=setting)
    }
  }
}





