###
#' @title Extract legend text from survfit object
#' @param kaplan.fit survfit object
#' @return legend
#' @author Frank PG Princee
#' @keywords internal
#' 
survival.legend <- function(survival.fit)
{

  if (class(survival.fit)[1] == "survfit.cox") {
    return(colnames(survival.fit$surv))
  }
  if (is.null(survival.fit$strata)) {
    return(NULL)
  }

  legend.text <- names(survival.fit$strata)
  temp  <- legend.text
  for (x in 1:(length(legend.text))) {
    temp[x] <- strsplit(legend.text[x],split="=")
    temp[x] <- temp[[x]][2]
  }      
  legend.text <- temp
  
  return(legend.text)
}


###
#' @title Plot fitted Kaplan-Meier estimator Cox Proportional Hazard Regression
#' @param survival.fit Object of class \code{survfit, survfit.cox, survfitr}
#' @param ymin Change scale from 0-1 to ymin-1 (default:0)
#' @param setting Settings for analysis
#' @author Frank PG Princee
#' @family survival
#' @keywords internal
#' 
survival.plot <- function(survival.fit,ymin=0,setting)
{

  if (studbook.isOption('output','publish')) {
    colours <- "black"
  }
  
  if (is.na(ymin)) {
    ymin <- min(survival.fit$surv,na.omit=TRUE)
  }
  
  age <- range(survival.fit$time) 
  age.info <- lifetable.ageScale(age[2])

  xlabel <- paste("Age in",age.info[['name']])
  
  ylabel <- setting[['ylabel']]
  ylabel <- firstCap(ylabel)
  
  
  # Default colour and line settings
  colours <- "black" 
  lstyle <- c(1,3,3)
  wstyle <- c(1,1.5,1.5)
  
  
  studbook.openOutput(paste("KM",setting[['analysis']],setting[['sex']],
                            setting[['group']],sep='_'))
  
  oldpar <- par(las=1)
  if (is.element(class(survival.fit)[1],c('survfit','survfit.cox') ) ){
    
    if (class(survival.fit)[1] == 'survfit.cox') {
      strata <- as.integer(dim(survival.fit$surv)[2])
    } else {
      strata <- length(survival.fit$strata)
    }
    
    
    conf <- TRUE
    conf.cox <- FALSE
    if (is.empty(strata)) {
      strata <- 0
      lstyle <- c('solid','dotted','dotted')
      legend.line <- lstyle
      conf.cox <- FALSE
    } else if (strata  == 2) {                  
      lstyle <- c('solid','dashed')
      legend.lstyle <- c('solid','dashed')
      wstyle <- 1.5
      conf <- FALSE
    } else if (strata > 2) {
      lstyle <- 1:strata
      legend.lstyle <- lstyle
      conf <- FALSE
    } 
    
    if (conf & class(survival.fit)[1] == 'survfit.cox') {
      conf <- FALSE
      conf.cox <- ifelse(strata > 0,TRUE,FALSE)
    }
    
    # Overruled by user
    if (!setting[["plot_conf"]]) {
      conf <- FALSE
      conf.cox <- FALSE
    }
      
    plot(survival.fit,main="",xlab=xlabel,ylab=ylabel,mark.time=FALSE,
         firstx=age[1]/age.info[['scale']],ymin=ymin,
         conf.int=conf,
         col=colours,lty=lstyle,lwd=wstyle,cex=1,
         xscale=age.info[["scale"]],yscale=1)
    
    
    if (conf.cox & class(survival.fit)[1] == 'survfit.cox') {
      xx <- c(survival.fit$time/365.25,rev(survival.fit$time/365.25))
      for (s in 1:as.integer(strata)) {
          yy <- c(survival.fit$upper[,s],rev(survival.fit$lower[,s]))
          polygon(xx,yy,col='grey',density=40,angle=90,border='grey40')
      }
    }
    
    if (ymin < 0.5) {
       abline(h=0.5,lty=4) #median 
    }
    
    # Lines for Q test Sx = 0.2, Sx = 0.6
    if (setting[["Qtest"]]) {
      
      if (ymin < 0.2) {
        abline(h=0.2,lty='dotted')
      }
      
      if(ymin < 0.6) {
        abline(h=0.6,lty='dotted')
      }
    }
    
    if (strata > 1) {
      legend.text <- survival.legend(survival.fit)
      if (!is.null(legend.text)) {
        legend.pos <- "bottomleft"
        if (max(survival.fit$surv) < 0.5) {
          legend.pos <- "topright"
        }
        legend(legend.pos,legend=legend.text,title=setting[['legend']],
               col=colours,lty=legend.lstyle,lwd=wstyle,cex=0.6,bty='n')  
      }
    }
    
  } else if (class(survival.fit) == 'survfitr') {  
    strata <- attr(survival.fit,"strata")
    if (!is.null(strata)) {                  
      lstyle <- 1:strata
      wstyle <- 1.5
    } 
    
    conf <- ifelse(is.null(strata),TRUE,FALSE)
    plot(survival.fit,ymin=ymin,conf.int=conf,lty=lstyle,lwd=wstyle,cex=.5)
    if (ymin < 0.5) {
      abline(h=0.5,lty=4) #median 
    }
  } else {
    # Nothing
  }
  
  par(oldpar)
  studbook.title(setting[['main_title']],setting[['sub_title']])
  studbook.closeOutput()  
}