###
# Functions to handle FAO thermal climate zones
# Thermal climate zones (FAO) 
# Global Agro--ecological Assessment for Agriculture in the 21st Century: Methodology and Results
# Fisher et al. 2002)
#


.defaultThermal <- function()
{
  assign("thermal.zone",c("Tropics","Subtropics (summer rain)","Subtropics (winter rain)",
                          "Temperate (oceanic)","Temperate (sub-continental)","Temperate (continental)",
                          "Boreal (oceanic)","Boreal (sub-continental)","Boreal (continental)",
                          "Artic","Inland water bodies"),.StudbookEnv)
  
  assign("thermal.palette",c("#FF0000","#FFAA00","#FFD37E","#D4FFBD","#55FF00","#3AFF00",
                             "#FFBEE7","#00C5FF","#004DA9","#B2B2B2","#BEEAFF"), .StudbookEnv)
  
  
  assign("thermal.selected",1:8,.StudbookEnv)
  assign("thermal.file","~/pml/climate/thermal/w001001.adf",.StudbookEnv)
}

###
#' @title Set thermal climate file
#' @param file_name Full path name of file
#' @author Frank PG Princee
#' @family thermal climate
#' @export
#' 
thermal.setFile <- function(file_name="")
{
  assign("thermal.file",file_name,.StudbookEnv)
}

###
#' @title Select thermal climate zones
#' 
#' @description \code{climate.select} sets the thermal climatic zones to be included
#' @details The numerical values of thermal climatic zones are entered as a
#' numerical sequence in the range 1-8. The following zones are recognized:
#' \enumerate{
#' \item  1. Tropics
#' \item  2. Subtropics (summer rain)
#' \item  3. Subtropics (winter rain)
#' \item  4. Temperate (oceanic)
#' \item  5. Temperate (sub-continental)
#' \item  6. Temperate (continental)
#' \item  7. Boreal (oceanic)
#' \item  8. Boreal (sub-continental)
#' \item  9. Boreal (continental)
#' \item 10. Artic
#' \item 11. Inland water bodies
#'}
#'The selected range is stored in variable \code{selected} of environment \code{.StudbookEnv}.
#'
#' @param zones Numerical value of zones (default 1:8) 
#' @author Frank PG Princee#' 
#' @family thermal climate
#' @export
#' 
thermal.select <- function(zones=1:8) 
{
  if (zones[1] == "?") {
    print(get("thermal.zone",.StudbookEnv) )   
  } else {
    assign("thermal.selected",zones,.StudbookEnv)
  }
}
  
###
#' @title Check whether thermal zone is selected
#' @description \code{inThermal} checks whether thermal zone is selected
#' @author Frank PG Princee
#' @family thermal climate
#' @keywords internal
#' 
inThermal <- function(zone)
{
  return( is.element(zone,get("thermal.selected",.StudbookEnv) ) )
}  
  

### 
#' @title Load FAO thermal climate raster from file
#' @note The FAO thermal climate raster mat need to be downloaded from
#' @family thermal climate
#' @export
#' 
thermal.load <- function()
{
  raster.load(get("thermal.file",.StudbookEnv))
  create.palette(palette = get("thermal.palette",.StudbookEnv))
}

###
#' @title Plot thermal climate raster
#' @author Frank PG Princee
#' @family thermal climate
#' @export 
#' 
thermal.plot <- function()
{
  raster.plot('Thermal climate zones')
}


###
#' @title Update sites frame with Thermal climate data
#' @param max_delta Number of steps to extract nearest value
#' @author Frank PG Princee
#' @family thermal climate
#' @export
#' 
sites.thermal <- function(max_delta=10)
{
  thermal.load() 
  sites.update('thermal',max_delta)
}
