###
#' @title Print summary of basic statistics on sample
#' @description \code{studbook.summary} prints a summary of basic statistics 
#' @param sample vector with data
#' @param long TRUE: long summary with variance, and standard error (default: TRUE)
#' @param CV TRUE: compute coefficient of variation as part of long summary (default:TRUE)
#' @param title title referring to analysis (default: "StudbookR Analysis")
#' @author Frank Princee
#' @keywords internal
#' 
studbook.summary <- function(sample,long=TRUE,CV=TRUE,title="StudbookR Analysis")
{
  #sample <- na.omit(sample)
  if (is.ts(sample)) {
    sample <- tseries::na.remove(sample)
  }
  cat(title," [ n = ",length(sample),"]\n")
  cat("----------------------------------------------------\n")
  print( summary(sample) )      
  cat("----------------------------------------------------\n\n")
  if (long) {
    cat("Percentiles 2.5-97.5%   :", quantile(sample,c(0.025,0.975),na.rm=TRUE),"\n")
    cat("Sampling variance       :", var(sample,na.rm=TRUE),"\n")
    cat("Standard deviation      :", sd(sample,na.rm=TRUE),"\n")
    cat("Standard error of mean  :", sample.se(sample),"\n")
    
    # Test whether no negative data occur: CV not valid
    
    if ( (min(sample,na.rm=TRUE) < 0) ) {
      CV <- FALSE
    }
    
    if (CV) {
      cat("Coefficient of variation:", sample.cv(sample),"%\n\n")
    }
  }
}


###
#' @title Create line with sample size and summary
#' @param sample vector or tseries with one column
#' @return vector with sample size and data summary
#' @keywords internal
#' 
summary.line <- function(sample)
{
  if (is.ts(sample)) {
    sample <- tseries::na.remove(sample)
  }
  result <- c(length(sample),as.vector(summary(sample)))
  return(result)  
}
  


###
#' @title Standard error of mean from sample
#' 
#' @description \code{sample.se} estimates the standard error \code{se} as 
#' \code{se = sd/sqrt(n)} from raw data.
#' 
#' @param sample numeric vector
#' @return standard error
#' @author Frank PG Princee
#' @examples
#' data_set <- sample(x=1:6,size=100,replace=TRUE)
#' studbookR:::sample.se(data_set)
#' @keywords internal
#' 
sample.se <- function(sample)
{
  sample <- na.omit(sample)
  n      <- length(sample)
  se     <- sd(sample,na.rm=TRUE)/sqrt(n)
  return(se)
}


###
#' @title Coefficient of variation from data sample
#'
#' @description \code{sample.cv} estimates the coefficient of variation \code{cv} as \code{cv = (sd * 100)/mean}.
#' 
#' @param sample numeric vector
#' @return coefficient of variation as percentage
#' @author Frank PG Princee
#' @examples
#' data_set <- sample(x=1:6,size=100,replace=TRUE)
#' studbookR:::sample.cv(data_set) #56.23757
#' @keywords internal
#' 
sample.cv <- function(sample)
{
  sample <- na.omit(sample)
  cv     <- (sd(sample,na.rm=TRUE) * 100.0)/mean(sample,na.rm=TRUE)
  return(cv)
}



###
#' @title Standard error of mean 
#' @description \code{se.mean} estimates the standard error \code{se} as 
#' \code{se = sd/sqrt(n) = sqrt(var/n)} or 
#' 
#' @param variance numeric Variance
#' @param N   integer Sample size
#' @return standard error
#' @author Frank PG Princee
#' @keywords internal
#' 
se.mean <- function(variance,N)
{
  return(sqrt(variance/N))
}


###
#' @title Confidence or prediction interval of mean
#' @description \code{interval.mean} estimates confidence and prediction interval of mean
#' @details The Student's t distribution is applied for N < 100, 
#' and the normal distribution for N > 100 whens ample variances are used (Sokal and Rohlf, 2012). 
#' This selection can be overwritten by selecting "normal" or "student" 
#' @param mean double Mean value
#' @param var double Variance
#' @param N integer sample size 
#' @param alpha Signficance level (default: 0.05)
#' @param interval Type of interval: \code{"confidence" or "prediction"} 
#' (default: \code{"prediction"})
#' @param distrib Distribution for confidence limits \code{"auto", "normal", "student"} 
#' (default: \code{"auto"}) 
#' @return vector with se, lower and upper limit 
#' @author Frank PG Princee
#' @keywords internal
#' 
interval.mean <- function(mean,variance,N,alpha=0.05,
                    interval=c("confidence","prediction"),
                    distrib=c("auto","normal","student"))
{
  interval <- match.arg(interval)
  distrib  <- match.arg(distrib)
  
  significance <- 1.0 - alpha/2.0 # qt and pnorm provide one side quantile

  if (distrib == "auto") {
    if (N > 100) {
      distrib <- "normal"
    } else {
      distrib <- "student"
    }  
  }
  
  if (distrib == "normal") {
    value <- qnorm(significance) # Normal
  } else {
    if (N == 1) {
      return(rep(NA,3))
    }
    value <- qt(significance,N-1) # Student t' distribution p
  }
  
  if (is.infinite(N)) { 
    se <- sqrt(variance)
  } else {  
    if (interval== "confidence") {
      se <- sqrt(variance/N)
    } else {
      se <- sqrt(variance) * sqrt(1 + 1/N)
    }
  }
  
  lower <- mean - (se * value)
  upper <- mean + (se * value)
  
  result <- c(se=se,lower=lower,upper=upper)
  attr(result,"interval") <- interval
  return(result)
}



###
#' @title Coefficient of variation from variance and mean  
#' @description \code{cv.mean} estimates the coefficient of variation \code{cv} as \code{cv = (sd * 100)/mean}.
#' @details The coefficient of variation is corrected for sample size if
#' \code{N} is provided: CV*= (1 + 1/(4*N)) * CV
#' 
#' @param mean double Mean of sample
#' @param variance double Variance of sample \code{s^2}
#' @param N integer Sample size if correction to be applied (default: NA)
#' @return coefficient of variation as percentage
#' @author Frank PG Princee
#' @examples
#' studbookR:::cv.mean(4.352941,5.992647,N=10)  
#' @references Sokald and Rohlf (2012, p55-57)
#' @keywords internal
#' 
cv.mean <- function(mean,variance,N=NA)
{
   cv <- (sqrt(variance) * 100) / mean
   if (!is.na(N)) {
     cv <- (1 + 1/(4*N)) * cv 
   }
   return(cv)
}


###
#' @title Test whether tied data occurs
#'
#' @description \code{is.tied} tests whether data in vector are tied
#' 
#' @param data_set data vector
#' @return TRUE or FALSE
#' @author Frank PG Princee
#' @examples
#' studbookR:::is.tied(1:10)     # FALSE
#' studbookR:::is.tied(c(1:9,9)) # TRUE
#' @keywords internal
#'  
is.tied<-function(data_set)
{
  return(anyDuplicated(data_set,incomparables=NA) > 0)	
}


###
#' @title Truncate data set
#'
#' @description \code{sample.truncate} can truncate data at either head, tail or both sides of a data set. 
#' @param sample vector with data
#' @param truncate proportion of data to be truncated from one or both sides
#' @param side which part(s) of the data set needs to be truncated 
#' refers to \code{head}, \code{tail} or \code{both}.
#' @return truncated data set
#' @author Frank PG Princee
#' @note data must be sorted
#' @examples
#' data_set <- 1:10
#' studbookR:::sample.truncate(data_set,0.1,'head') 
#' studbookR:::sample.truncate(data_set,0.1,'tail')
#' studbookR:::sample.truncate(data_set,0.1,'both')
#' 
#' @keywords internal
#' 
sample.truncate <- function(sample,truncate=0.0,side='tail')
{
  if (truncate == 0.0) {
    return(sample)
  }
  sample <- na.omit(sample)
  sample <- sort(sample)
  n  <- length(sample)
  s  <- floor( n * truncate) 
  
  if (side == 'tail') {
    return( sample[1:(n-s)] )
  } else if (side == 'head')  {
    return( sample[s:n])
  } else {
    return( sample[s:(n-s)] )
  }
}

###
#' @title Create confidence interval
#' @param sample sample vector with data.
#' @param cf Confidence limit (default:0.95).
#' @author Haleh Yasrebi
#' @keywords internal
#' @note From package survJamda 1.1.3 
#' 
geometric.CI <- function(sample,cf=0.95)
{
  p <- cf + (1 - cf)/2
  z <- qnorm(p)
  n <- length(sample)
  
  tmp.mean  <- mean(log(sample), na.rm = TRUE)
  lower  <- exp(tmp.mean - ( z * (sd(log(sample), na.rm = TRUE) /sqrt(n))))
  upper  <- exp(tmp.mean + ( z * (sd(log(sample), na.rm = TRUE) /sqrt(n))))
  return (c(lower=lower,upper=upper))
}


###
#' @title  Geometric mean (weighted and unweighted)
#' 
#' @description \code{geometric.Mean} estimates the weighted or unweighted geometric mean of a sample
#' 
#' @param sample vector with data
#' @param weights vector with weights per data element (default NULL - unweighted)
#' @param alpha Signficance level (default: 0.05)
#' @param interval Type of interval: \code{"confidence" or "prediction"} 
#' (default: \code{"prediction"})
#' @param distrib Distribution for confidence limits \code{"auto", "normal", "student"} 
#' (default: \code{"auto"}) 
#' @return geometric mean
#' @author Frank PG Princee; 
#' confidence limits from package survJamda 1.1.3, author Haleh Yasrebi
#' @seealso \code{\link{mean.values}}
#' @note data_set and weights should not have NA values
#' 
#' @examples
#' data_set <- sample(x=1:6,size=100,replace=TRUE)
#' studbookR:::geometric.Mean(data_set)
#' @keywords internal
#' 
geometric.Mean<-function(sample,weights=NULL,alpha=0.05,
                         interval=c("confidence","prediction"),
                         distrib=c("auto","normal","student"))
{
  interval <- match.arg(interval)
  distrib  <- match.arg(distrib)
  
  N <- length(sample)
  significance <- 1.0 - alpha/2.0 # qt and pnorm provide one side quantile
  
  if (distrib == "auto") {
    if (N > 100) {
      distrib <- "normal"
    } else {
      distrib <- "student"
    }  
  }
  
  geo.lower <- NA
  geo.upper <- NA
  
  if (distrib == "normal") {
    value <- qnorm(significance) # Normal
  } else {
    if (N == 1) {
      return(rep(NA,3))
    }
    value <- qt(significance,N-1) # Student t' distribution p
  }
 
  if (!length(weights)) {
    geo.base  <- mean(log(sample), na.rm = TRUE)  
    geo.mean  <- exp(geo.base )              # is the same as prod(sample)^(1/n)
    
    sd.log <- sd(log(sample),na.rm = TRUE)
    if (interval == "confidence") {
       se <-  sd.log / sqrt(N)
    } else {
       se <-  sd.log * sqrt(1-1/N)
    }
    geo.lower <- exp(geo.base - (value * se))
    geo.upper <- exp(geo.base + (value * se)) 
  } else {
    geo.mean <- 0;
    for (i in 1:length(sample)) {
      if (is.na(sample[i]) | sample[i] <= 0) {
        weights[i] <- 0                                                 #Do not include weight in sum
        next
      }
      geo.mean <- geo.mean + (weights[i] * log(sample[i]) ) 
    }
    geo.mean <- exp(geo.mean / sum (weights))
  }
  result <- list(mean=geo.mean,lower=geo.lower,upper=geo.upper)
  attr(result,"interval") <- interval  
  
  return ( result)
}


###
#' @title Arithmetic and geometric mean and median
#' @description \code{values.mean} estimates weighted and unweighted arithimetic mean, 
#' geometric mean and median in a sample. Results are printed on screen.
#' @param sample vector with data
#' @param weights vector with weights (default NULL - no weighting )
#' @param txt comment of weighing
#' @author Frank PG Princee
#' 
#' @seealso \code{\link{geometric.Mean}} \code{\link{mean}} \code{\link{median}} 
#' \code{\link{wtd.mean}} \code{\link{wtd.quantile}}
#' @export
#' @examples
#' data_set <- sample(x=1:6,size=100,replace=TRUE)
#' studbookR:::values.mean(data_set)
#' @keywords internal
#' 
values.mean <- function(sample,weights=NULL,txt=NULL)
{
  main_title      <- ifelse(!length(weights),'Unweighted','Weighted')
  mean.value      <- Hmisc::wtd.mean(sample,weights,na.rm=TRUE)
  sd.value        <- sqrt(Hmisc::wtd.var(sample,weights,na.rm=TRUE) )
  median.value    <- Hmisc::wtd.quantile(sample,weights,probs=0.5,na.rm=TRUE)  
  geometric.value <- geometric.Mean(sample,weights)  
  
  cat(main_title," [ n = ",length(sample),"]\n")        
  cat("----------------------------------------------------\n")
  cat("Arithmetic mean          :",mean.value,' [sd =',sd.value,']\n')
  cat("Median                   :",median.value,'\n')
  cat("Geometric mean           :",geometric.value[["mean"]]," [",
      geometric.value[["lower"]],",",geometric.value[["upper"]],"]\n")
  cat("----------------------------------------------------\n")
  if (is.ts(sample)) {
    cat("- Longest consecutive stretch of non-missing values: ") 
    cat(start(sample)[1],"-",end(sample)[1],"\n",sep="")
  }
  if (!is.null(txt)) {
    cat("- Weighted against",txt,"(zero values excluded).\n")
  }
  cat('\n')
  
}


###
#' @title Wilcoxon test 
#'
#' @description \code{wilconon.test} is a wrapper around the function \code{wilcox.test}.
#' 
#' @details This function tests whether one or both data sets are tied. 
#' If tied no exact test can be carried out. The result of the test is evaluated.
#' 
#' @param sample1 vector with numerical data
#' @param sample2 vector with numerical data
#' @param main_title Screen title
#' @param alpha significance level (default = 0.05)
#' @author Frank PG Princee 
#' @seealso \code{\link{wilcox.test}} \code{\link{is.tied}} 
#' @examples
#' sample1 <- sample(x=1:6,size=100,replace=TRUE)
#' sample2 <- sample(x=1:6,size=100,replace=TRUE)
#' studbookR:::wilcoxon.test(sample1,sample2)
#' @keywords internal
#' 
wilcoxon.test <- function(sample1,sample2,main_title='Compare samples',alpha=0.05)
{
  # Test whether data--sets are tied; in which case no excact test can be carried out
  
  set_exact <- ( !is.tied(sample1) && !is.tied(sample2) ) 
  
  # alternative = two.sided (default) 
  # reject: p < alpha e.g. 0.05 or 0.01 
  cat("")
  studbook.screenSection(main_title)
  result <- wilcox.test(sample1,sample2,paired=FALSE,exact=set_exact)
  print( result)
  cat("H0: equal distributions is ")  
  cat(ifelse(result$p.value < alpha,"rejected","not rejected")) 
  cat("\n\n")
}

###
#' @title Creates vector with symbol size per point
#'
#' @description \code{weighted.points} creates a vector with symbol size per point weighted using a power.
#' 
#'  @param point_data vector with original values 
#'  @param pow power (default = 0.25) 
#'  @return vector with cex values
#'  @author Frank PG Princee
#'  @keywords internal
#'  
weighted.points <- function(point_data,pow=0.25)
{
  points <- rep(0,length(point_data))
  
  for (d in 1: length(point_data)) {
    points[d] <- point_data[d]^pow
  }
  return(points)
}


###
#' @title Wald test
#' 
#' @description \code{Wald.test} estimates the Wald statistic for two samples of sizes \code{N1} and \code{N2}
#' with variances \code{V1} and \code{V2}, respectively. 
#' 
#' @param N1 Sample size of data set 1
#' @param V1 Variance in data set 1
#' @param N2 Sample size of data set 2
#' @param V2 Variance in data set 2
#' @author Frank PG Princee
#' @export 
#' 
Wald.test <- function(N1,var1,N2,var2)
{
  N.diff  <- N2 - N1
  se.diff <- sqrt(var1^2 + var2^2)
  z       <- abs(N.diff)/se.diff
  P       <- (1 - pnorm(z)) * 2
  CI      <- N.diff + 1.96 * se.diff * c(-1,1)
  
  studbook.screenTitle("Wald test")
  cat("N1    :", N1," var:",var1,"\n")
  cat("N2    :", N2," var:",var2,"\n")
  cat("change:", N2 - N1,"\n")    
  cat("P     :", P,"\n")
  cat("95% CI:", CI[1],"-",CI[2],"\n")
}


####
#' @title Sturges rule
#' @param R data range
#' @param N sample size
#' @return Number of breaks
#' @author Frank PG Princee
#' @references Sturges H.A. 1926 The choice of class interval. 
#' Journal of the American Statistical Association 21:65–66.
#' @keywords internal
#' 
sturges.rule <- function(R,N)
{
  return( R/(1+3.322*log(N)) )
}


###
#' @title Value is within range
#' @details Test whether value is within range
#' @param x Value to be tested
#' @param data_range min and max 
#' @param inclusive TRUE = [m,m], FALSE = (a,b)
#' @author Frank PG Princee
#' @keywords internal
#' 
is.between <- function(x,data_range,inclusive=TRUE)
{
  if(inclusive) {
    result <- ( x >= data_range[1] & x <= data_range[2] )
  } else {
    result <- ( x >  data_range[1] & x <  data_range[2] )
  }
  return(result)
}


###
#' @title Calculate slope between points
#' @description The function \code{graph.slope} calculates the slope between
#' successive points.
#' @param x numeric x coordinates of points
#' @param y numeric y coordinates of points
#' @return vector with slopes between points
#' @author Frank Princee
#' @keywords internal
#' 
graph.slope <-  function(x,y) 
{
  slope <- rep(NaN,length(x))
  for (i in 1:length(x)-1) {
    slope[i] <- (y[i] - y[i+1]) / (x[i] - x[i+1]) 
  } 
  return(slope)
}

###
#' @title Find sample size for binomial data
#' @param N sample size.
#' @param p0 population proportion
#' @param p1 true p  
#' @export
#' 
power.bin <- function(N,p0,p1)
{
   k     <- qbinom(0.05, N, p0) - 1
   alpha <- pbinom(k, N, p0)
   power <- pbinom(k, N, p1)
   power.values <- data.frame(N, k, alpha, power)
   print(power.values)
}

###
#' @title Test whether result vector is not empty
#' @param result Vector
#' @return TRUE if not empty
#' @author Frank PG Princee
#' @keywords internal
#'  
is.result <- function(result) 
{
  return(length(result) > 0)
}


###
#' @title Test whether result vector is empty
#' @param result Vector
#' @return TRUE if empty
#' @author Frank PG Princee
#' @keywords internal
#'  
is.empty <- function(result) 
{
  return(length(result) == 0)
}



###
#' @title Get type of data frame column
#' @param data_set data frame
#' @param column_name Column name
#' @return column class
#' @author Frank PG Princee
#' @keywords internal
#' 
getColumnType <- function(data_set,column_name)
{
  column_types <- sapply(data_set,class)
  column_nr    <- which(names(column_types) == column_name)
  return(as.character(column_types[column_nr]))
}


###
#' @title Test whether residuals overdispersed
#' @param Linear (mixed) model
#' @note From http://glmm.wikidot.com/faq. works for lme4
#' @keywords internal
#
overdispersion <- function(model) {
  ## number of variance parameters in 
  ##   an n-by-n variance-covariance matrix
  vpars <- function(m) {
    nrow(m)*(nrow(m)+1)/2
  }
  model.df <- sum(sapply(VarCorr(model),vpars))+length(fixef(model))
  rdf <- nrow(model.frame(model))-model.df
  rp <- residuals(model,type="pearson")
  Pearson.chisq <- sum(rp^2)
  prat <- Pearson.chisq/rdf
  pval <- pchisq(Pearson.chisq, df=rdf, lower.tail=FALSE)
  return(c(chisq=Pearson.chisq,ratio=prat,rdf=rdf,p=pval))
}


###
#'@title Days to age in years, months, day format
#'@param days Number of days since beginning of year
#'@details Uses 365 and 30 days for year and month, respectively.
#'@return vector with age in years, months and days
#'@author Frank PG Princee
#'@keywords internal
#' 
days2age <- function(days)
{
  years <- trunc(days / 365);
  months <- trunc((days %% 365) / 30)
  rest.day <- trunc((days %% 365) %% 30)
  return(c('years'=years,'months'=months,'days'=rest.day))                  
}


###
#' @title Trims leading and/ trailing space
#' @param text Text to be trimmed 
#' @param type "l"= left trim/leading, "r= right trim/trailing", "b"=both (default: \code{"b")})
#' @return string w/o leading/trailing whitespace
#' @keywords internal
#' 
trim.text <- function(text,type="b")
{
  if (type == "l") {
    text <- sub("^\\s+", "", text)
  } else if (type =="r") {
    text <- sub("\\s+$"," ",text)
  } else {
    text <- gsub("^\\s+|\\s+$", "", text)
  }
  invisible(text)
}
  
 



